/************************************************************************
*
* get_datim
*
* Queries the remote's date and time. If 'rbv' is NULL, we use the
* local time, in which case 'waitwarp' may be used to step back in
* time a little bit.
*
************************************************************************/
int get_datim(void *rbv, long *pdat, long *ptim, long waitwarp)
{
    int result = 0;
    long val1, val2;
    char *buf = malloc(MAX_BUFSIZ);

    if(rbv) {
        if(rio_transact(rbv, "DATE", buf, waitwarp))
            result = -1;
        else if(buf[0] != '1' || parse_datim(buf, pdat, ptim)) {
            syslog(LOG_INFO, "Warning: %s", buf);
            if((result = fake_date(buf, 3600)) == 0)
                result = parse_datim(buf, pdat, ptim);
        }
    }
    else if((result = fake_date(buf, waitwarp)) == 0)
        result = parse_datim(buf, pdat, ptim);

    free(buf);

    return(result);
}

/************************************************************************
*
* parse_datim
*
* Parses the date response and sets two longs, date and time. In
* case of bad format we preserve the original long values.
*
************************************************************************/
static int parse_datim(char *buf, long *pdat, long *ptim)
{
    int result = -1;
    long ltim, ldat;

    if(strlen(buf) >= 18) {
        ltim = atol(buf + 12);
        buf[12] = '\0';
        if((ldat = atol(buf + 6)) != 0) {
            *ptim = ltim;
            *pdat = ldat;
            result = 0;
        }
    }
    return(result);
}

/************************************************************************
*
* fake_date
*
* Fake a server's date response.
*
************************************************************************/
static int fake_date(char *buf, long warp)
{
    struct tm *gmt;
    time_t now;

    time(&now);
    now -= warp;

    if((gmt = gmtime(&now)) == NULL) {
        syslog(LOG_ERR, "Can't get local date and time");
        return(-1);
    }
    sprintf(buf, "111 %04.4d%02.2d%02.2d%02.2d%02.2d%02.2d",
            gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday,
            gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

    return(0);
}
