
typedef int (_System CUPINIT)();
typedef int (_System CUPEXIT)();
typedef int (_System CUPCONSUMEARTICLE)();
typedef int (_System CUPGETPOST)();
typedef int (_System CUPPOSTED)();

typedef struct {
    char    *name;
#ifdef __OS2__
    HMODULE hmod;
#else
    HINSTANCE hmod;
#endif
    CUPINIT           *pCupInit;
    CUPEXIT           *pCupExit;
    CUPCONSUMEARTICLE *pCupConsumeArticle;
    CUPGETPOST        *pCupGetPost;
    CUPPOSTED         *pCupPosted;
} PLUGIN;

/************************************************************************
*
*
************************************************************************/
void *CupLoad(char *name)
{
    int result = 0;
    PLUGIN pip = calloc(1, sizeof(PLUGIN));

    pip->name = strdup(name);

#ifdef __OS2__

    if((result = DosLoadModule(NULL, 0, name, &pip->hmod)) == 0) {
        if((rc = DosQueryProcAddr(pip->hmod, 0, "CupInit", &pip->pCupInit)) != 0)
            result = -1;
        else if((rc = DosQueryProcAddr(pip->hmod, 0, "CupExit", &pip->pCupExit)) != 0)
            result = -1;
        else if((rc = DosQueryProcAddr(pip->hmod, 0, "CupConsumeArticle", &pip->pCupConsumeArticle)) != 0)
            result = -1;
        else if((rc = DosQueryProcAddr(pip->hmod, 0, "CupGetPost", &pip->pCupGetPost)) != 0)
            result = -1;
        else if((rc = DosQueryProcAddr(pip->hmod, 0, "CupPosted", &pip->pCupPosted)) != 0)
            result = -1;
        if(result)
            DosFreeModule(pip->hmod);
    }
    else
        result = -1;

#else

    if((DWORD)(pip->hmod = LoadLibrary(name)) != HINSTANCE_ERROR) {
        if((pip->pCupInit = (CUPINIT *)GetProcAddress(pip->hmod, "CupInit")) == NULL)
            result = -1;
        else if((pip->pCupExit = (CUPEXIT *)GetProcAddress(pip->hmod, "CupExit")) == NULL)
            result = -1;
        else if((pip->pCupConsumeArticle = (CUPCONSUMEARTICLE *)GetProcAddress(pip->hmod, "CupConsumeArticle")) == NULL)
            result = -1;
        else if((pip->pCupGetPost = (CUPGETPOST *)GetProcAddress(pip->hmod, "CupGetPost")) == NULL)
            result = -1;
        else if((pip->pCupPosted = (CUPPOSTED *)GetProcAddress(pip->hmod, "CupPosted")) == NULL)
            result = -1;
        if(result)
            FreeLibrary(pip->hmod);
    }
    else
        result = -1;

#endif

    if(result) {
        CupUnload(pip);
        pip = NULL;
    }

    return(pip);
}

/************************************************************************
*
* CupUnload
*
************************************************************************/
void CupUnload(void *handle)
{
    PLUGIN pip = handle;

    free(pip->name);
    free(pip);
}

/************************************************************************
*
* CupInit
*
* Should return the supported level. Currently we are at level 0.
* Returns -1 in case of any error.
*
************************************************************************/
int CupInit(void *handle)
{
    PLUGIN pip = handle;

    return((pCupInit)());
}

/************************************************************************
*
* CupExit
*
* Clean up.
*
************************************************************************/
int CupExit(void *handle)
{
    PLUGIN pip = handle;

    return((pCupExit)());
}

/************************************************************************
*
* CupConsumeArticle
*
* Process the given article.
*
************************************************************************/
int CupConsumeArticle(void *handle)
{
    PLUGIN pip = handle;

}


/************************************************************************
*
* CupGetPost
*
* Queries the next post waiting for a given remote.
*
* CupGetPost and CupPosted will be called strictly sequential. These
* routines need not to be reentrant.
*
************************************************************************/
int CupGetPost(void *handle)
{
    PLUGIN pip = handle;

}


/************************************************************************
*
* CupPosted
*
* Informs us wether the post has been successful or not. The exact
* nntp error code is supplied, so that we can decide, wether we
* will remove that post or keep it for later retries.
*
************************************************************************/
int CupPosted(void *handle, int result)
{
    PLUGIN pip = handle;

}
