#include <stdlib.h>
#include <stdio.h>

#include <syslog.h>
#include <sostream.h>
#include <chlimits.h>

#include "config.h"

/*
 * This module adds a layer to sostream in order to be prepared
 * to send authentication information to the remote host at any
 * time.
 */


int rio_transact(void *rbv, const char *question, char *answer, long maxsecs)
{
    int result = -1;

    while((result = SoStreamTransact(rbv, question, answer, maxsecs)) == 0) {
        if(atoi(answer) != 480)
            break;
        if(authenticate(rbv, answer)) {
            result = -1;
            break;
        }
    }
    return(result);
}


int *rio_connect(void)
{
    void *rbv;
    int sock;
    char *buf = malloc(MAX_BUFSIZ);

    /*
     * Connect to remote
     */
    if ((sock = SoStreamConnect(cfg.remoteaddr, cfg.remoteport, 119)) < 0) {
        syslog(LOG_ERR, "Could not connect to %s", cfg.remoteaddr);
        free(buf);
        return(NULL);
    }

    rbv = SoStreamBufCreate(sock);
    if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, cfg.remotewait) <= 0) {
        SoStreamClose(sock);
        SoStreamBufDestroy(rbv);
        free(buf);
        return(NULL);
    }

    if (buf[0] != '2') {
        syslog(LOG_ERR, "sock error: %s", buf);
        SoStreamClose(sock);
        SoStreamBufDestroy(rbv);
        free(buf);
        return(NULL);
    }

    /*
     * Set posting ability flag
     */
    cfg.canpost = (atoi(buf) != 201);

    /*
     * Tell them we're a slave process
     */
    if(DOMOD(MOD_SLAVE)) {
        for(;;) {
            if(SoStreamTransact(rbv, "SLAVE", buf, cfg.remotewait)) {
                SoStreamClose(sock);
                SoStreamBufDestroy(rbv);
                free(buf);
                return(NULL);
            }
            if(atoi(buf) != 480)
                break;
            if(authenticate(rbv, buf)) {
                SoStreamClose(sock);
                SoStreamBufDestroy(rbv);
                free(buf);
                return(NULL);
            }
        }
        if (buf[0] != '2')
            syslog(LOG_NOTICE, "Warning: %s", buf);
    }

    /*
     * Mode reader may be used on INN servers
     * to invoke NNRPD, the newsreader server.
     */
    if(DOMOD(MOD_READER)) {
        for(;;) {
            if(SoStreamTransact(rbv, "MODE READER", buf, cfg.remotewait)) {
                SoStreamClose(sock);
                SoStreamBufDestroy(rbv);
                free(buf);
                return(NULL);
            }
            if(atoi(buf) != 480)
                break;
            if(authenticate(rbv, buf)) {
                SoStreamClose(sock);
                SoStreamBufDestroy(rbv);
                free(buf);
                return(NULL);
            }
        }
        if (buf[0] != '2')
            syslog(LOG_NOTICE, "Warning: %s", buf);
    }

    free(buf);
    return(rbv);
}
