#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <xfile.h>
#include <domnpath.h>
#include <loadtext.h>

static char wmfile[_MAX_PATH];  /* Name of the watermark file */
static char *groups;            /* Contents of the watermark file */
static long *wmarks;            /* Numerical watermarks */
static int  nmarks;             /* Number of entries in groups/wmarks */
static int  cmark;              /* Index to next entry to be processed */

/************************************************************************
*
* init_watermark
*
* Loads the watermark file for a given host.
*
* Returns 0 on success, -1 in case of any error.
*
* This routine is not reentrant.
*
************************************************************************/
int init_watermark(char *remotehost, char *basedir)
{
    char **lpp;
    char *lp;
    int idx = 0;

    strcpy(wmfile, basedir);
    domain_path(wmfile, remotehost, ".watermark");
    if((nmarks = load_textfile(wmfile, &groups)) > 0) {
        wmarks = calloc(nmarks, sizeof(long));
        for(lpp = groups; (lp = *lpp++) != NULL; idx++) {
            for(cp = lp; *cp != ':' && *cp; cp++)
                continue;
            if(*cp != ':')
                continue;
            *cp++ = 0;
            strlwr(lp);
            if((wmarks[idx] = atol(cp + 1)) == 0)
                wmarks[idx] = 1;
        }
    }
    return(nmarks);
}

/************************************************************************
*
* exit_watermark
*
* Safely writes back an update watermark file.
*
* Returns 0 on success, -1 in case of any error. The old watermark file
* will be backupped.
*
* This routine is not reentrant.
*
************************************************************************/
int exit_watermark(void)
{
    return(0);
}

/************************************************************************
*
* next_watermark
*
* Queries the next watermark entry. If all groups had been processed,
* it will return -1. Otherwise this routine will set a pointer to the
* name of the group, set the watermark and return a zero based index
* to be used to call set_indexmark later on.
*
* This routine is reentrant and protected by a mutual exclusion sema4.
*
************************************************************************/
int next_watermark(char **grpp, long *markp)
{
    for(; cmark < nmark; cmark++) {
        if((*markp = wmarks[cmark]) != 0) {
            *grpp = groups[cmark];
            return(cmark++);
        }
    }
    return(-1);
}

/************************************************************************
*
* set_indexmark
*
* Updates the watermark for a given index into our list. The index
* must have been returned by a previous call to next_watermark.
*
* No need for mutual exclusion, because it will be called by the
* thread which did the next_watermark call only.
*
************************************************************************/
void set_indexmark(int idx, long mark)
{
    wmarks[idx] = mark;
}

/************************************************************************
*
* check_wmcross
*
* This routine will be used to prevent downloading crossposts. The
* given grouplist will have been extracted from an Xref line by the
* caller. Here we simply check if any of the groups in this list
* has been fetched by one of the sucking threads previously.
*
* Life would be much easier, if the watermark list had been sorted.
* This however would disable the user from giving preferences to
* some groups by listing them in front of others and let them
* process first, so that in case of timeouts we will loose the
* less important groups only.
*
* No mutual exclusion, because we won't touch anything. It won't
* even hurt if cmark is incremented while we are looping.
*
************************************************************************/
int check_wmcross(char **grplist, int ngroups)
{
    int result = 0;
    int i, j;
    char c;

    for(i = 0; i < cmark; i++) {
        c = groups[i][0];
        for(j = 0; j < ngroups; j++)
            if(grplist[j][0] == c && strcmp(grplist[j], groups[i]) == 0) {
                result++;
                break;
            }
    }

    return(result);
}
