/*
 * This program was hacked by Harald Kipp using a source
 * written by Eric S. Raymond.
 *
 * Bug reports related to THIS modified version should be sent to
 *
 *  harald@os2point.ping.de
 *
 * You may freely copy or redistribute this software. However,
 * this may not apply to any part of it, if otherwise noted.
 */

const char chanco_rcsid[] = "$Id: chanco.c,v 1.1 1997/02/13 11:25:36 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>
#include <time.h>
#include <ctype.h>

#include <version.h>
#include <chlimits.h>
#include <syslog.h>
#include <getopt.h>
#include <sostream.h>
#include <sodtgram.h>
#include <chanlib.h>

#include <fcntl.h>
#include <io.h>

#include "config.h"

typedef struct {
    char *cmd;
    int  args;
    char letter;
    int  glue;
    long lswait;
} METHOD;

static METHOD methods[] = {
    { "addhist"    , 5, 'a', 1, 30L }, /* addhist id arr exp post path...: Add history line                 */
    { "allow"      , 1, 'D', 1, 30L }, /* allow reason...                : Allow remote connections         */
    { "begin"      , 1, 'b', 0, 30L }, /* begin site                     : Start newly-added site           */
    { "cancel"     , 1, 'c', 0, 30L }, /* cancel id                      : Cancel message locally           */
    { "changegroup", 2, 'u', 0, 30L }, /* changegroup group rest         : Change mode of group             */
    { "checkfile"  , 0, 'd', 0, 30L }, /* checkfile                      : Check syntax of newsfeeds file   */
    { "disconnect" , 1, 'E', 0, 30L }, /* disconnect client              : Kick out a client                */
    { "drop"       , 1, 'e', 0, 30L }, /* drop site                      : Stop feeding site                */
    { "flush"      , 1, 'f', 0, 30L }, /* flush site                     : Flush feed for site*             */
    { "flushlogs"  , 0, 'g', 0, 30L }, /* flushlogs                      : Flush log files                  */
    { "flushserver", 1, 'F', 1, 30L }, /* flushserver reason...          : Flush history and active         */
    { "go"         , 1, 'h', 1, 30L }, /* go reason...                   : Restart after pause/throttle     */
    { "groupentry" , 1, 'G', 0, 30L }, /* groupentry index               : Print group by index             */
    { "hangup"     , 1, 'i', 0, 30L }, /* hangup channel                 : Hangup specified connection      */
    { "hostentry"  , 1, 'H', 0, 30L }, /* hostentry index                : Print host by index              */
    { "mode"       , 0, 's', 0, 30L }, /* mode                           : Print operating mode             */
    { "name"       , 1, 'j', 0, 30L }, /* name nnn                       : Print name of specified channel* */
    { "newgroup"   , 3, 'k', 0, 30L }, /* newgroup group rest creator    : Create new group                 */
    { "oversync"   , 1, 'Y', 0, 30L }, /* oversync priority              : Update overview                  */
    { "param"      , 2, 'l', 0, 30L }, /* param letter value             : Change command-line params       */
    { "pause"      , 1, 'm', 1, 30L }, /* pause reason...                : Short-term pause                 */
    { "readers"    , 2, 'v', 1, 30L }, /* readers flag text...           : Enable/disable newsreading       */
    { "refile"     , 2, 't', 0, 30L }, /* refile path group              : Refile an article                */
    { "reject"     , 1, 'C', 1, 30L }, /* reject reason...               : Reject remote connections        */
    { "rejecting"  , 0, 'S', 0, 30L }, /* rejecting                      : Print reject reason              */
    { "reload"     , 2, 'o', 1, 30L }, /* reload what reason...          : Re-read config files*            */
    { "renumber"   , 1, 'n', 0, 30L }, /* renumber group                 : Renumber the active file*        */
    { "reserve"    , 1, 'z', 1, 30L }, /* reserve reason...              : Reserve next pause/throttle      */
    { "resource"   , 1, 'R', 0, 30L }, /* resource type                  : Query system resource            */
    { "rmgroup"    , 1, 'p', 0, 30L }, /* rmgroup group                  : Remove named group               */
    { "send"       , 2, 'A', 1, 30L }, /* send feed text...              : Send text to exploder feed       */
    { "shutdown"   , 1, 'q', 1,  0L }, /* shutdown reason...             : Shut down server                 */
    { "throttle"   , 1, 'r', 1, 30L }, /* throttle reason...             : Stop accepting articles          */
    { "trace"      , 1, 'w', 0, 30L }, /* trace flag                     : Turn tracing on or off           */
    { "userentry"  , 1, 'U', 0, 30L }, /* userentry index                : Print user by index              */
    { "xabort"     , 1, 'x', 1,  0L }, /* xabort text...                 : Abort the server                 */
};

#define NMETHODS (sizeof(methods) / sizeof(methods[0]))

int prog_result = 0;

static int syslog_opt = LOG_CONS | LOG_NDELAY | LOG_NOWAIT;

static void usage(void);

/************************************************************************/
/*                                                                      */
/*  CHANCO                                                              */
/*                                                                      */
/*  This program handles incoming control messages.                     */
/*                                                                      */
/************************************************************************/
int main(int argc, char **argv)
{
    char *cp;
    int option;
    time_t t_proc;
    int i;
    int m;
    char *line;

    _tzset();
    t_proc = time(NULL);

    /*
     * Initialize socket interface.
     */
    if(SoInitialize()) {
        fprintf(stderr, "FATAL ERROR: TCP/IP initialization failed\n");
        return(128);
    }

    /*
     * Set up initial configuration
     */
    init_cfg();

    /*
     * Process command line options
     */
    while((option = getopt(argc, argv, "?d:h:p:u:vW:")) != EOF) {
        switch(option) {
        case 'd':
            cp = optarg;
            while(*cp) {
                if(*cp == 'C')
                    cfg.logflg |= LOG_CONFIG;
                else if(*cp == 'o')
                    syslog_opt |= LOG_PERROR;
                else if(*cp == 'p')
                    syslog_opt |= LOG_PID;
                else
                    fprintf(stderr, "Unknown option -d%c ignored", *cp);
                cp++;
            }
            break;
        case 'h':
            strcpy(cfg.controlhost, optarg);
            break;
        case 'p':
            strcpy(cfg.controlport, optarg);
            break;
        case 'u':
            strcpy(cfg.logname, optarg);
            break;
        case 'v':
            cfg.verbose++;
            break;
        case 'W':
            cfg.maxwait = atol(optarg);
            break;
        default:
        case '?':
            usage();
            return(3);
        }
    }

    /*
     * All command line options are processed, check if one
     * argument is left to give us an article file name.
     */
    argc -= optind;
    argv += optind;
    if(!argc) {
        usage();
        return(3);
    }

    /*
     * Display banner and open log
     */
    openlog("chanco", syslog_opt, LOG_NEWS);
    syslog(LOG_INFO, "Version %s-%s up and running...", version, __DATE__);

    if(!validate_cfg())
        return(3);

    for (m = 0; m < NMETHODS; ++m)
        if (!stricmp(methods[m].cmd, argv[0]))
            break;

    if (m >= NMETHODS) {
        syslog(LOG_ERR, "Unknown command '%s'", argv[0]);
        usage();
        return(3);
    }
    argc--;
    argv++;

    /*
     * Glue extra words together
     */
    if(argc > methods[m].args && methods[m].glue) {
        int len = 0;
        char *narg;

        for(i = methods[m].args - 1; argv[i]; i++)
            len += strlen(argv[i]) + 1;
        narg = cp = malloc(len);
        for(i = methods[m].args - 1; argv[i]; i++) {
            cp = strchr(strcpy(cp, argv[i]), '\0');
            *cp++ = ' ';
        }
        *--cp = '\0';
        argv[methods[m].args - 1] = narg;
        argv[methods[m].args] = NULL;
    }
    else if(argc != methods[m].args) {
        syslog(LOG_ERR, "Wrong number of arguments");
        usage();
        return(3);
    }

    line = malloc(MAX_BUFSIZ);
    strcpy(line, cfg.logname);
    cp = line + strlen(line);
    *cp++ = '\001';
    *cp++ = methods[m].letter;
    *cp++ = '\001';
    for(i = 0; argv[i]; i++) {
        cp = strchr(strcpy(cp, argv[i]), '\0');
        *cp++ = '\001';
    }
    *--cp = '\0';
    if(cfg.maxwait == 0L)
        cfg.maxwait = methods[m].lswait;
    if(SoDtgramTransact(cfg.controlhost, cfg.controlport, 119,
                        line, line, cfg.maxwait) != -1) {
        if(cfg.verbose)
            printf("%s\n", line);
        if((prog_result = atoi(line)) != 0)
            syslog(LOG_NOTICE, "%s", line);
    }
    else {
        if(cfg.verbose)
            printf("4 Unable to connect %s\n", cfg.controlhost);
        prog_result = 4;
    }
    free(line);

    if((t_proc = time(NULL) - t_proc) < 1)
        t_proc = 1;
    if(prog_result && prog_result != 2)
        syslog(LOG_NOTICE, "Failed to run %s, error %d",
               methods[m].cmd, prog_result);
    else
        syslog(LOG_INFO, "Processed %s in %ld second%s",
               methods[m].cmd, t_proc, (t_proc == 1) ? "" : "s");

    return(prog_result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static void usage(void)
{
   fprintf(stderr, "Chanco %s - %s %s\n\n", version, __DATE__, __TIME__);
   fputs("usage: chanco [options] <command>\n\n"
         "commands:\n"
         "  addhist id arr exp post path...  Add history line\n"
         "  allow reason...                  Allow remote connections\n"
#if 0
         "  begin site                       Start newly-added site\n"
#endif
         "  cancel id                        Cancel message locally\n"
         "  changegroup group rest           Change mode of group\n"
#if 0
         "  checkfile                        Check syntax of newsfeeds file\n"
#endif
         "  drop site                        Stop feeding site\n"
         "  flush site                       Flush feed for site*\n"
         "  flushlogs                        Flush log files\n"
         "  flushserver reason...            Flush history and active\n"
         "  go reason...                     Restart after pause/throttle\n"
#if 0
         "  hangup channel                   Hangup specified connection\n"
#endif
         "  mode                             Print operating mode\n"
#if 0
         "  name nnn                         Print name of specified channel*\n"
#endif
         "  newgroup group rest creator      Create new group\n"
         "  param letter value               Change command-line params\n"
         "  pause reason...                  Short-term pause\n"
#if 0
         "  readers flag text...             Enable/disable newsreading\n"
         "  refile path group                Refile an article\n"
#endif
         "  reject reason...                 Reject remote connections\n"
#if 0
         "  reload what reason...            Re-read config files*\n"
         "  renumber group                   Renumber the active file*\n"
         "  reserve reason...                Reserve next pause/throttle\n"
#endif
         "  rmgroup group                    Remove named group\n"
#if 0
         "  send feed text...                Send text to exploder feed\n"
#endif
         "  shutdown reason...               Shut down server\n"
         "  throttle reason...               Stop accepting articles\n"
#if 0
         "  trace flag                       Turn tracing on or off\n"
#endif
         "  xabort text...                   Abort the server\n"
         "  oversync speed                   Update overview normal/slow/fast/stop\n"
         "  hostentry index                  Print host by index\n"
         "  userentry index                  Print user by index\n"
         "  resource type                    Query memory/news/overview/history/sysload\n"
         "  disconnect client                Kick out a host/user@host\n"
         "  groupentry index                 Print group by index\n"
         "  rejecting                        Print reject reason\n\n"
         "options:\n"
         "  -dC          dump config to chanco.log\n"
         "  -do          log to console\n"
         "  -dp          log with pid\n"
         "  -h<hostname> host to control   (127.0.0.1)\n"
         "  -p<service>  udp service port  (119)\n"
         "  -u<username> user name         (unknown)\n"
         "  -v           verbose\n"
         "  -W<seconds>  timeout           (30)\n", stderr);
}

/*
 * $Log: chanco.c,v $
 * Revision 1.1  1997/02/13 11:25:36  oliver
 * initial revision
 *
 */
