/*
 * This part was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 * This module contains routines to read the configuration file.
 *
 */

const char config_rcsid[] = "$Id: config.c,v 1.1 1997/02/13 11:25:37 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <direct.h>

#include <version.h>

#include <syslog.h>
#include <readconf.h>

#include "config.h"

CONFIG cfg;

static ENVTABLE et[] = {
    { cfg.workdir          , "CHANGIWORKDIR"          , "", 0 },
    { cfg.logname          , "CHANGILOGNAME"          , "USER", 4 },
    { cfg.controlport      , "CHANGICONTROLPORT"      , "119", 2 },
    { cfg.controlhost      , "CHANGILOCALHOST"        , "127.0.0.1", 2 },
    { NULL                 , NULL                     , "", 0 }
};

static int dump_cfg(void);

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
void init_cfg(void)
{
    read_conf(et);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
int validate_cfg(void)
{
    int result = 1;

    if(cfg.logname[0] == '\0') {
        if(cfg.verbose)
            fputs("No user specified\n", stderr);
        syslog(LOG_NOTICE, "No user specified");
        strcpy(cfg.logname, "unknown");
    }

    if(DOLOG(LOG_CONFIG))
        dump_cfg();

    return(result);
}

/************************************************************************/
/*                                                                      */
/* Dump configuration                                                   */
/*                                                                      */
/************************************************************************/
static int dump_cfg(void)
{
    int result = -1;
    FILE *fp;
    ENVTABLE *ep;
    time_t t;

    if((fp = fopen("chanco.log", "a")) != NULL) {
        time(&t);
        fprintf(fp, "REM %s", ctime(&t));
        fprintf(fp, "REM Chanco Version %s\n", version);
        fprintf(fp, "REM Compiled %s\n", __DATE__);
        for(ep = et; ep->cnfg; ep++)
            fprintf(fp, "SET %s=%s&\n", ep->ename, ep->cnfg);
        fputc('\n', fp);
        if(fclose(fp))
            syslog(LOG_ERR, "%s", _strerror("Config dump"));
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "%s", _strerror("Config dump"));

    return(result);
}

/*
 * $Log: config.c,v $
 * Revision 1.1  1997/02/13 11:25:37  oliver
 * initial revision
 *
 */
