/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains the routines dealing with the account structure
 */
/*
 * $Log$
 */

const char account_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <syslog.h>
#include <sysdep.h>
#include <mutex.h>

#include "config.h"
#include "chanx.h"
#include "account.h"

ACCNT_INFO acci;

static long lck_watch = -1;

/************************************************************************/
/*  clr_account                                                         */
/*                                                                      */
/*  Clears the accounting structure.                                    */
/*                                                                      */
/************************************************************************/
void clr_account(void)
{
    memset(&acci, 0, sizeof(acci));
    acci.t_proc = time(NULL);
    lck_watch = create_mutex();
}

/************************************************************************/
/*  This is the main thread during article retrieval.                   */
/*                                                                      */
/*                                                                      */
/************************************************************************/
int show_account(void)
{
    int result = 0;
    int i;
    int arts2get = 0;   /* Set if article retrieval started */
    int watch_result = 0;
    int alldone = 0;    /* Set if we are finished */

    fprintf(stderr, "Total Good  Bad");
    for(i = 0; i < cfg.numthreads && i < 12; i++)
        fprintf(stderr, "  T%02u", i + 1);
    fprintf(stderr, "\n");

    for(;;) {
        alldone = 1;
        fprintf(stderr, "\r");

        /*
         * Process id collection status.
         */
        switch(acci.id_state) {
        case STA_BROKEN:
            fprintf(stderr, " FAIL%5u%5u", acci.num_got,
                                           acci.num_dropped +
                                           acci.num_dupes +
                                           acci.num_miss);
            result = -1;
            break;
        case STA_STARTED:
            alldone = 0;
            fprintf(stderr, " CONN          ");
            break;
        case STA_CONNECTED:
            alldone = 0;
            if(acci.num_new || acci.num_dupes || acci.num_dropped) {
                fprintf(stderr, "%5u%5u%5u", acci.num_new +
                                             acci.num_dupes,
                                             acci.num_got,
                                             acci.num_dropped +
                                             acci.num_dupes +
                                             acci.num_miss);
                /*
                 * If we got new ids and haven't started article
                 * retrieval, then let's start it now.
                 */
                if(!arts2get && acci.num_new) {
                    collect_arts();
                    arts2get = 1;
                }
            }
            else
                fprintf(stderr, " WAIT          ");
            break;
        case STA_DONE:
            fprintf(stderr, "%5u%5u%5u", acci.num_new +
                                         acci.num_dupes,
                                         acci.num_got,
                                         acci.num_dropped +
                                         acci.num_dupes +
                                         acci.num_miss);
            /*
             * If we got new ids and haven't started article
             * retrieval, then let's start it now.
             */
            if(!arts2get && acci.num_new) {
                collect_arts();
                arts2get = 1;
            }
            break;
        }

        /*
         * Process article retrieval status.
         */
        if(arts2get) {
            int num_broken = 0;

            for(i = 0; i < cfg.numthreads; i++) {
                switch(acci.acct[i].state) {
                case STA_BROKEN:
                    num_broken++;
                    if(i < 12)
                        fprintf(stderr, " FAIL");
                    break;
                case STA_STARTED:
                    alldone = 0;
                    if(i < 12)
                        fprintf(stderr, " CONN");
                    break;
                case STA_CONNECTED:
                    alldone = 0;
                    if(i < 12) {
                        if(acci.acct[i].num_got)
                            fprintf(stderr, "%5u", acci.acct[i].num_got);
                        else
                            fprintf(stderr, " WAIT");
                    }
                    break;
                case STA_DONE:
                    if(i < 12)
                        fprintf(stderr, "%5u", acci.acct[i].num_got);
                    break;
                }
            }
            if(num_broken == cfg.numthreads) {
                result = -1;
                break;
            }
        }
        if((watch_result = watch_limits(0)) != 0) {
            result = -1;
            break;
        }
        if(alldone)
            break;
        ms_sleep(500);
    }
    fprintf(stderr, "\n");
    switch(watch_result) {
    case 1:
        fprintf(stderr, "Byte limit reached\n");
        break;
    case 2:
        fprintf(stderr, "Time limit reached\n");
        break;
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int show_xaccount(void)
{
    int result = 0;
    int i;
    int arts2get = 0;   /* Set if article retrieval started */
    int watch_result = 0;
    int alldone = 0;    /* Set if we are finished */

    fprintf(stderr, "Total Good  Bad");
    for(i = 0; i < cfg.numthreads && i < 12; i++)
        fprintf(stderr, "  T%02u", i + 1);
    fprintf(stderr, "\n");

    for(;;) {
        alldone = 1;
        fprintf(stderr, "\r");

        /*
         * Process id collection status.
         */
        switch(acci.id_state) {
        case STA_BROKEN:
            fprintf(stderr, " FAIL%5u%5u", acci.num_got,
                                           acci.num_dropped +
                                           acci.num_dupes +
                                           acci.num_miss);
            result = -1;
            break;
        case STA_STARTED:
            alldone = 0;
            fprintf(stderr, " CONN          ");
            break;
        case STA_CONNECTED:
            alldone = 0;
            if(acci.num_new || acci.num_dupes || acci.num_dropped) {
                fprintf(stderr, "%5u%5u%5u", acci.num_new +
                                             acci.num_dupes,
                                             acci.num_got,
                                             acci.num_dropped +
                                             acci.num_dupes +
                                             acci.num_miss);
                /*
                 * If we got new ids and haven't started article
                 * retrieval, then let's start it now.
                 */
                if(!arts2get && acci.num_new) {
                    collect_xarts();
                    arts2get = 1;
                }
            }
            else
                fprintf(stderr, " WAIT          ");
            break;
        case STA_DONE:
            fprintf(stderr, "%5u%5u%5u", acci.num_new +
                                         acci.num_dupes,
                                         acci.num_got,
                                         acci.num_dropped +
                                         acci.num_dupes +
                                         acci.num_miss);
            /*
             * If we got new ids and haven't started article
             * retrieval, then let's start it now.
             */
            if(!arts2get && acci.num_new) {
                collect_xarts();
                arts2get = 1;
            }
            break;
        }

        /*
         * Process article retrieval status.
         */
        if(arts2get) {
            int num_broken = 0;

            for(i = 0; i < cfg.numthreads; i++) {
                switch(acci.acct[i].state) {
                case STA_BROKEN:
                    num_broken++;
                    if(i < 12)
                        fprintf(stderr, " FAIL");
                    break;
                case STA_STARTED:
                    alldone = 0;
                    if(i < 12)
                        fprintf(stderr, " CONN");
                    break;
                case STA_CONNECTED:
                    alldone = 0;
                    if(i < 12) {
                        if(acci.acct[i].num_got)
                            fprintf(stderr, "%5u", acci.acct[i].num_got);
                        else
                            fprintf(stderr, " WAIT");
                    }
                    break;
                case STA_DONE:
                    if(i < 12)
                        fprintf(stderr, "%5u", acci.acct[i].num_got);
                    break;
                }
            }
            if(num_broken == cfg.numthreads) {
                result = -1;
                break;
            }
        }
        if((watch_result = watch_limits(0)) != 0) {
            result = -1;
            break;
        }
        if(alldone)
            break;
        ms_sleep(500);
    }
    fprintf(stderr, "\n");
    switch(watch_result) {
    case 1:
        fprintf(stderr, "Byte limit reached\n");
        break;
    case 2:
        fprintf(stderr, "Time limit reached\n");
        break;
    }
    return(result);
}

/************************************************************************/
/*  watch_limits                                                        */
/*                                                                      */
/*  Adds 'nbytes' to the total number of bytes received and checks      */
/*  if either the byte limit or time limit is reached.                  */
/*                                                                      */
/*  Returns 1 if the byte limit is reached, 2 if the time limit is      */
/*  reached or 0 otherwise.                                             */
/*                                                                      */
/************************************************************************/
int watch_limits(long nbytes)
{
    int result = 0;

    if(request_mutex(&lck_watch, 120000L) == 0) {
        acci.num_bytes += nbytes;
        if(cfg.sizelimit && acci.num_bytes >= cfg.sizelimit) {
            syslog(LOG_NOTICE, "Byte limit reached");
            result = 1;
        }
        if(cfg.timelimit && time(NULL) >= cfg.timelimit) {
            syslog(LOG_NOTICE, "Time limit reached");
            result = 2;
        }
        release_mutex(lck_watch);
    }
    return(result);
}

/************************************************************************/
/*  log_account                                                         */
/*                                                                      */
/*  Writes the accounting results to the log file.                      */
/*                                                                      */
/************************************************************************/
void log_account(void)
{
    if(acci.t_proc == 0)
        acci.t_proc = 1;
    else if((acci.t_proc = time(NULL) - acci.t_proc) <= 0)
        acci.t_proc = 1;
    if(acci.num_posted)
        syslog(LOG_INFO, "Articles posted  : %d", acci.num_posted);
    if(acci.num_new)
        syslog(LOG_INFO, "New ids          : %d", acci.num_new);
    if(acci.num_dupes)
        syslog(LOG_INFO, "Duplicate ids    : %d", acci.num_dupes);
    if(acci.num_got)
        syslog(LOG_INFO, "Articles received: %d", acci.num_got);
    if(acci.num_dropped)
        syslog(LOG_INFO, "Articles dropped : %d", acci.num_dropped);
    if(acci.num_miss)
        syslog(LOG_INFO, "Articles missing : %d", acci.num_miss);
    if(acci.num_bytes)
        syslog(LOG_INFO, "Bytes received   : %ld (%ld cps)", acci.num_bytes,
                acci.num_bytes / acci.t_proc);
    syslog(LOG_INFO, "Chanx processed %d article%s in %lu second%s",
           (acci.num_posted + acci.num_got),
           ((acci.num_posted + acci.num_got) == 1) ? "" : "s",
           acci.t_proc, (acci.t_proc == 1) ? "" : "s");
}
