/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

const char artpool_rcsid[] = "$Id$";

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <syslog.h>

#include <sysdep.h>
#include <chlimits.h>
#include <mutex.h>

#include "config.h"
#include "account.h"
#include "chanx.h"

#define MUTEXWAIT   (124000 + __LINE__)

typedef struct {
    void *next;
    char item[1];     /* either a group name, article number or 0 */
} ITEM_LINK;

static ITEM_LINK *artbot = NULL;
static ITEM_LINK *arttop = NULL;

static long lck_artpool = -1;

/************************************************************************/
/*                                                                      */
/************************************************************************/
int init_artpool(void)
{
    if((lck_artpool = create_mutex()) == -1)
        return(-1);

    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void exit_artpool(void)
{
    destroy_mutex(lck_artpool);
}

/************************************************************************/
/*                                                                      */
/* add_artpool                                                          */
/*                                                                      */
/* Adds an item to the article pool.                                    */
/*                                                                      */
/************************************************************************/
void add_artpool(char *item)
{
    if(request_mutex(&lck_artpool, MUTEXWAIT))
        return;

    /*
     * No more items available. Mark the end of the list.
     */
    if(item == NULL) {
        if(arttop) {
            arttop->next = malloc(sizeof(ITEM_LINK));
            arttop = arttop->next;
        }
        else
            artbot = arttop = malloc(sizeof(ITEM_LINK));
        arttop->next = NULL;
        arttop->item[0] = '\0';
    }

    /*
     * Add an item to the list.
     */
    else {
        size_t itemlen = strlen(item);

        if(arttop) {
            arttop->next = malloc(sizeof(ITEM_LINK) + itemlen);
            arttop = arttop->next;
        }
        else
            artbot = arttop = malloc(sizeof(ITEM_LINK) + itemlen);
        arttop->next = NULL;
        memcpy(arttop->item, item, itemlen + 1);
        if(isdigit(*item))
            acci.num_new++;
    }
    release_mutex(lck_artpool);
}

/************************************************************************/
/*                                                                      */
/* get_artpool                                                          */
/*                                                                      */
/* Return value -1 if no more items are available, 0 otherwise          */
/*                                                                      */
/************************************************************************/
int get_artpool(char *group, long *artnum)
{
    int result = 0;
    int waiting = 1;
    static char *lastgroup = NULL;

    while(waiting) {
        if(request_mutex(&lck_artpool, MUTEXWAIT) == 0) {
            if(artbot) {
                *artnum = atol(artbot->item);
                if(artbot->item[0]) {
                    void *lnk = artbot;

                    if(*artnum == 0) {
                        if(lastgroup == NULL)
                            lastgroup = malloc(MAX_GROUPNAME);
                        strcpy(lastgroup, artbot->item);
                    }
                    else if(lastgroup) {
                        strcpy(group, lastgroup);
                        waiting = 0;
                    }
                    if((artbot = artbot->next) == NULL)
                        arttop = NULL;
                    free(lnk);
                }
                else {
                    if(lastgroup) {
                        free(lastgroup);
                        lastgroup = NULL;
                    }
                    result = -1;
                    waiting = 0;
                }
            }
            release_mutex(lck_artpool);
        }
        if(waiting)
            ms_sleep(100);
    }
    return(result);
}
