/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/
/*
 * $Log$
 */

const char artspew_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#ifdef __OS2__
#include <pipe.h>
#endif
#include <xfile.h>
#include <domnpath.h>
#include <mutex.h>

#include "config.h"
#include "account.h"
#include "chanx.h"

#define MUTEXWAIT   (122000 + __LINE__)

static long lck_artspew = -1;
static FILE *fpbatch;
static FILE *fpdrop;
static char batchfile[_MAX_PATH];
static char dropfile[_MAX_PATH];

#ifndef __OS2__
    char *tname;
    char rnewscall[_MAX_PATH];
#endif

/************************************************************************/
/*                                                                      */
/************************************************************************/
int init_artspew(void)
{
    /*
     * Create mutex semaphore to control output channel
     * access.
     */
    if((lck_artspew = create_mutex()) == -1)
        return(0);

    /*
     * Open output channel
     */
    if(DOMOD(MOD_TYPE))
        fpbatch = freopen("", "wb", stdout);
    else if(DOMOD(MOD_BATCH)) {
        strcpy(batchfile, cfg.workdir);
        domain_path(batchfile, cfg.remotehost, ".batch");
        if((fpbatch = xfopen(batchfile, "ab")) == NULL) {
            syslog(LOG_ERR, "%s", _strerror(batchfile));
            return(0);
        }
    }
#ifdef __OS2__
    else if((fpbatch = popen(cfg.rnewscall, "wb")) == NULL) {
        syslog(LOG_ERR, "Failed to run %s", cfg.rnewscall);
        return(0);
    }
#else
    else {
        tname = tempnam(cfg.workdir, "RNP");
        if((fpbatch = fopen(tname, "wb")) == NULL) {
            perror(tname);
            syslog(LOG_ERR, "%s", _strerror(tname));
            return(-1);
        }
    }
#endif

    /*
     * Open drop file
     */
    if(DOMOD(MOD_DROPSAVE)) {
        strcpy(dropfile, cfg.workdir);
        domain_path(dropfile, cfg.remotehost, ".drop");
        if((fpdrop = xfopen(dropfile, "a")) == NULL)
            syslog(LOG_ERR, "%s", _strerror(dropfile));
    }

    return(1);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void exit_artspew(void)
{
    /*
     * Close output channel
     */
    if(fpbatch) {
        if(DOMOD(MOD_BATCH)) {
            if(fclose(fpbatch)) {
                syslog(LOG_ERR, "%s", _strerror(batchfile));
                cfg.omitupdate++;
            }
        }
        else if(!DOMOD(MOD_TYPE)) {
            unsigned int rc;
#ifdef __OS2__
            if((rc = pclose(fpbatch)) != 0) {
                syslog(LOG_ERR, "%s returned error %d", cfg.rnewscall, rc >> 8);
                cfg.omitupdate++;
            }
#else
            sprintf(rnewscall, "%s < %s", cfg.rnewscall, tname);
            if(fclose(fpbatch)) {
                syslog(LOG_ERR, "%s", _strerror(tname));
                cfg.omitupdate++;
            }
            else if((rc = system(rnewscall)) != 0) {
                syslog(LOG_ERR, "'%s' returns error %d", rnewscall, rc);
                cfg.omitupdate++;
            }
            if(unlink(tname))
                syslog(LOG_WARNING, "%s", _strerror(tname));
            free(tname);
#endif

        }
    }
    if(DOMOD(MOD_DROPSAVE) && fpdrop) {
        if(fclose(fpdrop))
            syslog(LOG_ERR, "%s", _strerror(dropfile));
    }
}

/************************************************************************/
/*  spew_block                                                          */
/*                                                                      */
/************************************************************************/
int spew_block(char *buf, long bufsiz)
{
    int result = -1;

    if(bufsiz && request_mutex(&lck_artspew, MUTEXWAIT) == 0) {
        if(fprintf(fpbatch, "#! rnews %ld\n", bufsiz) == -1)
            syslog(LOG_ERR, "Can't write article batch");
        else if(fputs(buf, fpbatch) == -1)
            syslog(LOG_ERR, "Can't write article batch");
        else {
            acci.num_got++;
            result = 0;
        }
        release_mutex(lck_artspew);
    }
    return(result);
}

/************************************************************************/
/*  spew_hb                                                             */
/*                                                                      */
/************************************************************************/
int spew_hb(char *headline[], long hsize, char *body, long bsize)
{
    int result = -1;
    int i;

    if(hsize && request_mutex(&lck_artspew, MUTEXWAIT) == 0) {
        hsize++;
        if(DOMOD(MOD_DOSFILE))
            hsize++;
        if(fprintf(fpbatch, "#! rnews %ld\n", hsize + bsize) == -1)
            syslog(LOG_ERR, "Can't write article batch");
        else {
            result = 0;
            for(i = 0; headline[i]; i++) {
                if(fputs(headline[i], fpbatch) == -1) {
                    syslog(LOG_ERR, "Can't write article batch");
                    result = -1;
                    break;
                }
                if(fputc('\n', fpbatch) == EOF) {
                    syslog(LOG_ERR, "Can't write article batch");
                    result = -1;
                    break;
                }
            }
            if(result == 0) {
                if(fputc('\n', fpbatch) == EOF) {
                    syslog(LOG_ERR, "Can't write article batch");
                    result = -1;
                }
                else if(bsize && fputs(body, fpbatch) == -1) {
                    syslog(LOG_ERR, "Can't write article batch");
                    result = -1;
                }
                else
                    acci.num_got++;
            }
        }
        release_mutex(lck_artspew);
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static char *beautify(char *s)
{
    char *cp;

    while(*s == ' ')
        s++;
    for(cp = s; *cp; cp++)
        if(*cp == '\t')
            *cp = ' ';

    return(s);
}

/************************************************************************/
/*                                                                      */
/* drop_msgid                                                           */
/*                                                                      */
/* This routine will be called to drop an article header which has      */
/* been retrieved by message id.                                        */
/*                                                                      */
/************************************************************************/
void drop_msgid(char *headline[])
{
    char *msgid = NULL;
    char *subject = " ";
    char *from = " ";
    char *lines = " ";
    char *cp;
    int i;

    acci.num_dropped++;
    if(fpdrop == NULL)
        return;
    for(i = 0; headline[i]; i++) {
        if((cp = strchr(headline[i], ':')) != NULL) {
            *cp = '\0';
            if(stricmp(headline[i], "From") == 0)
                from = beautify(cp + 1);
            else if(stricmp(headline[i], "Lines") == 0)
                lines = beautify(cp + 1);
            else if(stricmp(headline[i], "Message-ID") == 0)
                msgid = beautify(cp + 1);
            else if(stricmp(headline[i], "Subject") == 0)
                subject = beautify(cp + 1);
        }
    }
    if(msgid) {
        if(request_mutex(&lck_artspew, MUTEXWAIT) == 0) {
            fprintf(fpdrop, "%s\t%s\t%s\t%d\n", msgid, subject, from, atoi(lines));
            release_mutex(lck_artspew);
        }
    }
}

/************************************************************************/
/* drop_artnum                                                          */
/*                                                                      */
/* This routine will be called to drop an article header which has      */
/* been retrieved by group:number.                                      */
/*                                                                      */
/************************************************************************/
void drop_artnum(char *group, long artnum, char *headline[])
{
    char *subject = " ";
    char *from = " ";
    char *lines = " ";
    char *cp;
    int i;

    if(fpdrop == NULL)
        return;

    for(i = 0; headline[i]; i++) {
        if((cp = strchr(headline[i], ':')) != NULL) {
            *cp = '\0';
            if(stricmp(headline[i], "From") == 0)
                from = beautify(cp + 1);
            else if(stricmp(headline[i], "Lines") == 0)
                lines = beautify(cp + 1);
            else if(stricmp(headline[i], "Subject") == 0)
                subject = beautify(cp + 1);
        }
        fprintf(fpdrop, "%s:%lu\t%s\t%s\t%d\n", group, artnum, subject, from, atoi(lines));
    }
}

/************************************************************************/
/* drop_overview                                                        */
/*                                                                      */
/************************************************************************/
void drop_overview(char *group, char *overline)
{
    if(fpdrop == NULL)
        return;

    fprintf(fpdrop, "%s:%s\n", group, overline);
}
