/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */
const char authent_rcsid[] = "$Id$";


#include <stdlib.h>
#include <stdio.h>

#include <chlimits.h>
#include <sostream.h>

#include <syslog.h>

#include "config.h"
#include "chanx.h"

/************************************************************************/
/*                                                                      */
/* authenticate                                                         */
/*                                                                      */
/* Send original authentication to the server.                          */
/*                                                                      */
/* Returns 0 on success, -1 otherwise.                                  */
/*                                                                      */
/************************************************************************/
int authenticate(void *rbv, char *buf)
{
    if(cfg.auth_user[0] == '\0' || cfg.auth_pass[0] == '\0') {
        syslog(LOG_ERR, "%s", buf);
        fprintf(stderr, "%s\n", buf);
        return(-1);
    }

    /*
     * 15.03.98 harald
     * Do not send leading exclamation mark, because this
     * is an option for immediate active authentication.
     */
    sprintf(buf, "AUTHINFO USER %s", cfg.auth_user + (cfg.auth_user[0] == '!'));
    if(SoStreamTransact(rbv, buf, buf, cfg.localwait) < 0)
        return(-1);
    if(atoi(buf) != 381) {
        syslog(LOG_ERR, "%s", buf);
        fprintf(stderr, "%s\n", buf);
        return(-1);
    }
    sprintf(buf, "AUTHINFO PASS %s", cfg.auth_pass);
    if(SoStreamTransact(rbv, buf, buf, cfg.localwait) < 0)
        return(-1);
    if(atoi(buf) != 281) {
        syslog(LOG_ERR, "%s", buf);
        fprintf(stderr, "%s\n", buf);
        return(-1);
    }
    return(0);
}
