const char catchup_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <chlimits.h>
#include <syslog.h>
#include <sostream.h>
#include <pipe.h>
#include <xfile.h>

#include "config.h"
#include "account.h"
#include "rexxif.h"
#include "chanx.h"


/************************************************************************/
/*                                                                      */
/************************************************************************/
int catchup(void *rbv)
{
    int  result = 0;
    int  broken = 0;
    int  keepwm = 1;
    FILE *fpw;      /* watermark file     */
    FILE *fpn;      /* new watermark file */
    char *buf;
    char *ngtag;
    int  count;
    long lo, hi;
    long wm, nwm;


    /*
     * Open list of groups
     */
    if((fpw = xfopen(cfg.watermarklist, "r")) == NULL) {
        perror(cfg.watermarklist);
        syslog(LOG_ERR, "%s", _strerror(cfg.watermarklist));
        return(-1);
    }

    /*
     * Open new watermark file
     */
    if((fpn = xfopen(cfg.watermarktemp, "w")) == NULL) {
        perror(cfg.watermarktemp);
        syslog(LOG_ERR, "%s", _strerror(cfg.watermarktemp));
        fclose(fpw);
        return(-1);
    }

    /*
     * Loop for each group
     */
    buf = malloc(MAX_BUFSIZ);
    ngtag = malloc(MAX_GROUPNAME);
    while(fgets(buf, MAX_GROUPNAME, fpw)) {
        char *cps = buf;
        char *cpd = ngtag;

        if(broken) {
            fputs(buf, fpn);
            continue;
        }


        /*
         * Interpret watermark line
         */
        while(*cps && *cps != ' ' && *cps != '\t' && *cps != '\n' && *cps != ':')
            *cpd++ = *cps++;
        *cpd = '\0';
        if(*cps != ':' || *ngtag == '\0') {
            fputs(buf, fpn);
            continue;
        }
        if(*cps)
            wm = atol(cps + 1) + 1;
        else
            wm = 1;
        nwm = wm;

        /*
         * Get group information from remote
         */
        for(;;) {
            sprintf(buf, "GROUP %s", ngtag);
            if(SoStreamTransact(rbv, buf, buf, cfg.remotewait)) {
                broken = 1;
                break;
            }
            if(atoi(buf) != 480 && (DOMOD(MOD_AUTHSIMPLE) || atoi(buf) != 450))
                break;
            if(authenticate(rbv, buf)) {
                broken = 1;
                break;
            }
        }
        if(broken)
            continue;
        if (buf[0] != '2') {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        if(sscanf(buf + 4, "%d %ld %ld", &count, &lo, &hi) != 3) {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        if(lo < wm)
            lo = wm;

        nwm = hi;
        keepwm = 0;

        if(cfg.verbose) {
            fputs(ngtag, stderr);
            fputc('\n', stderr);
        }

        fprintf(fpn, "%s: %lu\n", ngtag, nwm);
    } /* next group */

    free(buf);
    free(ngtag);

    fclose(fpw);
    fclose(fpn);

    if(keepwm)
        cfg.omitupdate++;

    return(result);
}
