/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */
const char chanx_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <process.h>
#include <io.h>

#include <version.h>
#include <syslog.h>
#include <sostream.h>
#include <getopt.h>
#include <xfile.h>
#include <domnpath.h>

#include "config.h"
#include "account.h"
#include "rexxif.h"
#include "chanx.h"

static int syslog_opt = LOG_CONS | LOG_NDELAY | LOG_NOWAIT;
static FILE *fppid = NULL;

static void usage(char *myname);
static int parse_args(int argc, char **argv);
static void parsedbgopt(char *opt);
static void parselstopt(char *opt);
static void parsemodopt(char *opt);
static void cleanup(int sig);


/************************************************************************/
/*  CHANX                                                               */
/*                                                                      */
/*  Connect to a remote site, send news to and get news from it.        */
/*                                                                      */
/************************************************************************/
int main(int argc, char **argv)
{
    int *rbv;
    char *progname = (progname = strrchr(*argv, '\\')) ? progname + 1 : *argv;

    /*
     * Switch off buffering status display
     */
    setvbuf(stderr, NULL, _IONBF, 0);

    /*
     * Initiate local time
     */
    tzset();

    /*
     * Initialize socket interface.
     */
    if(SoInitialize()) {
        fprintf(stderr, "FATAL ERROR: TCP/IP initialization failed\n");
        return(128);
    }

    /*
     * Set up initial default configuration
     */
    init_cfg();

    /*
     * Init interupt handling
     */
    signal(SIGABRT, cleanup);
    signal(SIGBREAK, cleanup);
    signal(SIGINT, cleanup);
    signal(SIGSEGV, cleanup);
    signal(SIGTERM, cleanup);

    /*
     * Display banner
     */
    fprintf(stderr, "Chanx %s (Duemesnil 2) - %s %s\n", version, __DATE__, __TIME__);

    /*
     * Parse command line arguments
     */
    if(parse_args(argc, argv)) {
        usage(progname);
        return(3);
    }

    /*
     * Open syslog channel
     *
     * 08/28/1996 harald
     * Moved this down (after 'parse_args') to allow 'syslog_opt'
     * to be set.
     */
    openlog("chanx", syslog_opt, LOG_NEWS);
    syslog(LOG_INFO, "Version %s-%s up and running...", version, __DATE__);
    clr_account();

    /*
     * Do not allow multiple instances.
     *
     * 12/01/96 harald
     * Create pid file and keep it open as a lock against
     * duplicate instances.
     *
     * 03/23/97 harald
     * Moved down, because the name of the pidfile requires
     * the remote host read from the command line. Changed
     * fopen to xfopen, because we may now land in non existing
     * directories. Added a syslog message on failures, because
     * syslog has been initialized at this point.
     */
    if(cfg.pidfile[0] != '\0') {
        if((fppid = xfopen(cfg.pidfile, "w")) == NULL) {
            perror(cfg.pidfile);
            syslog(LOG_ERR, "%s", _strerror(cfg.pidfile));
            return(8);
        }
        fprintf(fppid, "%u\n", _getpid());
        fflush(fppid);
    }

    /*
     * Validate configuration
     */
    if(!validate_cfg())
        return(3);

    /*
     * Read filter
     */
    if(read_drop() == 0) {
        cfg.filtering = 1;
        cfg.usefilterfile = 1;
    }
    if(DOMOD(MOD_XOVER)) {
        if(init_filterover() == 0) {
            cfg.filtering = 1;
            cfg.usefilterover = 1;
        }
    }
    else {
        if(init_filtermacro() == 0) {
            cfg.filtering = 1;
            cfg.usefiltermacro = 1;
        }
    }

    /*
     * Query remote for new articles
     */
    if((rbv = remote_connect()) != NULL) {
        if(get_time(rbv)) {
            fprintf(stderr, "Main connection broken\n");
            return(4);
        }
        if(send_posts(rbv)) {
            fprintf(stderr, "Main connection broken\n");
            return(4);
        }
        if(get_lists(rbv)) {
            fprintf(stderr, "Main connection broken\n");
            return(4);
        }

        /*
         * Posting only, do not query anything.
         */
        if(DOMOD(MOD_POST)) {
            SoStreamClose(*rbv);
            SoStreamBufDestroy(rbv);
        }

        /*
         * Catch-up with remote.
         */
        else if(DOMOD(MOD_CATCHUP)) {
            catchup(rbv);
            SoStreamClose(*rbv);
            SoStreamBufDestroy(rbv);
        }

        /*
         * Use preferred XOVER method.
         */
        else if(DOMOD(MOD_XOVER)) {
            SoStreamClose(*rbv);
            SoStreamBufDestroy(rbv);
            if(init_artspew()) {
                if(init_artpool() == 0) {
                    collect_overview();

                    /*
                     * 03/23/97 harald
                     * Do not set omitupdate on broken connections.
                     */
                    show_xaccount();
                    exit_artpool();
                }
                else {
                    if(strcmp(cfg.localaddr, "127.0.0.1"))
                        fprintf(stderr, "Connecting server at %s failed\n", cfg.localaddr);
                    else
                        fprintf(stderr, "Need Changi running on local machine\n");
                    cfg.omitupdate++;
                }
                exit_artspew();
            }
        }

        /*
         * Use historical NONEWNEWS
         */
        else if(DOMOD(MOD_NONEWNEWS)) {
            if(init_artspew()) {
                getnnn(rbv);
                exit_artspew();
            }
            SoStreamClose(*rbv);
            SoStreamBufDestroy(rbv);
        }

        /*
         * Use NEWNEWS
         */
        else {
            SoStreamClose(*rbv);
            SoStreamBufDestroy(rbv);
            if(init_artspew()) {
                if(init_idpool() == 0) {
                    collect_ids();
                    if(show_account())
                        cfg.omitupdate++;
                    exit_idpool();
                }
                else {
                    if(strcmp(cfg.localaddr, "127.0.0.1"))
                        fprintf(stderr, "Connecting server at %s failed\n", cfg.localaddr);
                    else
                        fprintf(stderr, "Need Changi running on local machine\n");
                    cfg.omitupdate++;
                }
                exit_artspew();
            }
            else
                cfg.omitupdate++;
        }
    }
    else {
        fprintf(stderr, "Can't connect %s\n", cfg.remoteaddr);
        return(4);
    }

    /*
     * Spooling received batch
     */
    if(DOMOD(MOD_BATCH) && acci.num_got > 0) {
        char *cmdbuf = malloc(_MAX_PATH);
        char *batchfile = malloc(_MAX_PATH);

        strcpy(batchfile, cfg.workdir);
        domain_path(batchfile, cfg.remotehost, ".batch");
        fprintf(stderr, "Spooling...\n");
        sprintf(cmdbuf, "%s < %s", cfg.rnewscall, batchfile);
        if (system(cmdbuf))
            syslog(LOG_ERR, "%s", _strerror(cfg.rnewscall));
        else if(xremove(batchfile))
            syslog(LOG_ERR, "%s", _strerror(batchfile));
        free(cmdbuf);
        free(batchfile);
    }

    /*
     * Update timestamp file
     */
    if(!DOMOD(MOD_OMITUPD)) {
        if(DOMOD(MOD_XOVER)) {
            if(access(cfg.watermarktemp, 0))
                cfg.omitupdate++;
            if(cfg.omitupdate) {
                syslog(LOG_ERR, "%s not updated", cfg.watermarklist);
                fprintf(stderr, "%s not updated\n", cfg.watermarklist);
            }
            else {
                if(xremove(cfg.watermarklist))
                    syslog(LOG_ERR, "%s", _strerror(cfg.watermarklist));
                if(xrename(cfg.watermarktemp, cfg.watermarklist))
                    syslog(LOG_ERR, "%s", _strerror(cfg.watermarktemp));
            }
        }
        else if(!DOMOD(MOD_NONEWNEWS)) {
            if(cfg.omitupdate) {
                syslog(LOG_ERR, "%s not updated", cfg.datefile);
                fprintf(stderr, "%s not updated\n", cfg.datefile);
            }
            else {
                FILE *dtfile;

                if((dtfile = xfopen(cfg.datefile, "w")) == NULL) {
                    syslog(LOG_ERR, "%s", _strerror(cfg.datefile));
                    perror(cfg.datefile);
                }
                else {
                    fprintf(dtfile, "%s %06lu %06lu %s\n", cfg.grouplist,
                                                           cfg.nextdate,
                                                           cfg.nexttime,
                                                           cfg.distribution);
                    if(fclose(dtfile)) {
                        syslog(LOG_ERR, "%s", _strerror(cfg.datefile));
                        perror(cfg.datefile);
                    }
                }
            }
        }
    }

    /*
     * Terminate
     */
    log_account();
    closelog();
    if(fppid) {
        fclose(fppid);
        unlink(cfg.pidfile);
    }

    return (acci.num_got ? 0 : 1);
}

/************************************************************************/
/*  Process command line options                                        */
/*                                                                      */
/*  Parses command line arguments and returns 0 if accepted, -1 other-  */
/*  wise.                                                               */
/*                                                                      */
/************************************************************************/
static int parse_args(int argc, char **argv)
{
    int  option;
    FILE *fp;

    /*------------------------------------------*/
    /* Process options                          */
    /*------------------------------------------*/
    while((option = getopt(argc, argv, "?a:d:h:k:l:m:n:p:r:s:t:vw:W:")) != EOF) {
        switch(option) {
        case 'a':
            strcpy(cfg.authentication, optarg);
            break;
        case 'd':
            parsedbgopt(optarg);
            break;
        case 'h':
            strcpy(cfg.remoteaddr, optarg);
            break;
        case 'k':
            cfg.sizelimit = atol(optarg) * 1024L;
            break;
        case 'l':
            parselstopt(optarg);
            break;
        case 'm':
            parsemodopt(optarg);
            break;
        case 'n':
            cfg.numthreads = atoi(optarg);
            break;
        case 'p':
            strcpy(cfg.remoteport, optarg);
            break;
        case 'r':
            strcpy(cfg.rnewscall, optarg);
            break;
        case 's':
            strcpy(cfg.spooldir, optarg);
            break;
        case 't':
            time(&cfg.timelimit);
            cfg.timelimit += atol(optarg) * 60L;
            break;
        case 'v':
            cfg.verbose++;                  /* 09/07/97 harald */
            break;
        case 'w':
            cfg.remotewait = atol(optarg);
            break;
        case 'W':
            cfg.localwait = atol(optarg);
            break;

        default:
            fprintf(stderr, "Unknown option -%c\n", option);
        case '?':
            return(-1);
        }
    }

    /*------------------------------------------*/
    /* Process remaining command line arguments */
    /*------------------------------------------*/
    argc -= optind;
    argv += optind;

    if(argc != 1) {
        /*
         * If our remote won't accept NEWNEWS, we won't
         * accept related parameters either.
         */
        if(DOMOD(MOD_NONEWNEWS))
            return(-1);

        /*
         * We don't accept partial NEWNEWS parameters. If
         * a grouplist is specified, then last date and
         * time must be specified too. Only the distribution
         * parameter is optional.
         */
        if(argc != 4 && argc != 5)
            return(-1);
    }
    strcpy(cfg.remotehost, argv[0]);

    /*
     * Build all remote depended pathnames
     */
    strcpy(cfg.datefile, cfg.workdir);
    domain_path(cfg.datefile, cfg.remotehost, ".last");
    strcpy(cfg.iwantfile, cfg.workdir);
    domain_path(cfg.iwantfile, cfg.remotehost, ".iwant");
    strcpy(cfg.watermarklist, cfg.workdir);
    domain_path(cfg.watermarklist, cfg.remotehost, ".watermark");
    strcpy(cfg.watermarktemp, cfg.workdir);
    domain_path(cfg.watermarktemp, cfg.remotehost, ".tempmark");
    strcpy(cfg.newgroupstime, cfg.workdir);
    domain_path(cfg.newgroupstime, cfg.remotehost, ".newgroupstime");

    /*
     * 03/23/97 harald
     * Put pidfile in domainpath to allow concurrent connections
     * to different hosts.
     */
    strcpy(cfg.pidfile, cfg.workdir);
    domain_path(cfg.pidfile, cfg.remotehost, ".pid");

    /*
     * Process NEWNEWS parameters
     */
    if (argc > 1) {
        if(access(cfg.datefile, 0)) {
            /*
             * If we were called with newnews parameters and
             * we can't find a previous datefile then this is
             * an initial connect to the specified host.
             */
            syslog(LOG_NOTICE, "Virgin connect to %s", cfg.remotehost);
        }
        else {
            /*
             * If we were called with newnews parameters then
             * we ignore an existing datefile and won't update
             * it later.
             */
            cfg.omitupdate++;
            syslog(LOG_NOTICE, "Out-of-sequence connect to %s", cfg.remotehost);
        }
        strcpy(cfg.grouplist, argv[1]);
        cfg.nextdate = cfg.lastdate = atol(argv[2]);
        cfg.nexttime = cfg.lasttime = atol(argv[3]);
        if (argc > 4)
            strcpy(cfg.distribution, argv[4]);
    }
    else if(!DOMOD(MOD_NONEWNEWS) && !DOMOD(MOD_POST)) {
        /*
         * The behaviour on these cases has been changed to
         * avoid unintended large downloads of bloody
         * beginners refusing to read the manual. If we
         * aren't called with newnews options and we failed
         * to read the datefile, then we simply terminate.
         * Previous versions used to query all newsgroups
         * of the current day.
         */
        if((fp = xfopen(cfg.datefile, "r")) == NULL) {
            syslog(LOG_ERR, "%s", _strerror(cfg.datefile));
            fprintf(stderr, "Can't open %s\n", cfg.datefile);
            fprintf(stderr, "Looks like your first connection to %s\n", cfg.remotehost);
            fprintf(stderr, "in which case additional parameters are needed\n");
            return(-1);
        }
        else {
            if(fscanf(fp, "%s %ld %ld %s",
                      cfg.grouplist,
                      &cfg.lastdate,
                      &cfg.lasttime,
                      cfg.distribution) < 3) {
                fclose(fp);
                syslog(LOG_ERR, "%s has invalid format", cfg.datefile);
                fprintf(stderr, "Invalid format in %s\n", cfg.datefile);
                return(-1);
            }
            cfg.nextdate = cfg.lastdate;
            cfg.nexttime = cfg.lasttime;
            fclose(fp);
        }
    }

    /*
     * Version 0.9m did not recognize the newsrc file. This
     * is the correction.
     */
    if(strchr(cfg.grouplist, '/') || strchr(cfg.grouplist, '\\'))
        strcpy(cfg.newsrc, cfg.grouplist);

    return(0);
}

/************************************************************************/
/*                                                                      */
/* Parse debug options                                                  */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsedbgopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
         /*
          * Log drop
          */
         case 'C':
             cfg.logflg |= LOG_CONFIG;
             break;
         case 'f':
             cfg.logflg |= LOG_DROP;
             break;

         case 'o':
             syslog_opt |= LOG_PERROR;
             break;
         /*
          * Log posting
          */
         case 'p':
             cfg.logflg |= LOG_POST;
             break;

         /*
          * Log telegrams
          */
         case 'r':
         case 's':
         case 't':
             SoStreamDebug(1);
             break;

         default:
             fprintf(stderr, "Unknown option -d%c ignored\n", *opt);
             break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Parse list options                                                   */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parselstopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
         case 'a':
             cfg.modflg |= MOD_ACTLIST;
             break;
         case 'd':
             /*
              * Store ids of dropped articles in host.drop
              */
             cfg.modflg |= MOD_DROPSAVE;
             break;
         case 'g':
             cfg.modflg |= MOD_NEWGROUPS;
             break;
         case 'j':
             cfg.modflg |= MOD_DSTLIST;
             break;
         case 'n':
             cfg.modflg |= MOD_NGLIST;
             break;
         case 'u':
             cfg.modflg |= MOD_SUBLIST;
             break;
         default:
             fprintf(stderr, "Unknown option -l%c\n", *opt);
             break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Parse mode options                                                   */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsemodopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
         case 'a':
             cfg.modflg |= MOD_AUTHSIMPLE;
             break;
         case 'b':
             cfg.modflg &= (unsigned long)(~MOD_TYPE);
             cfg.modflg |= MOD_BATCH;
             break;
         case 'C':                      /* catch-up with remote */
             cfg.modflg |= (MOD_CATCHUP | MOD_XOVER);
             break;
         case 'd':
             cfg.modflg |= MOD_DOSFILE;  /* dos file <cr><lf> */
             break;
         case 'l':
             cfg.modflg |= MOD_DTLOCAL;  /* undocumented hack */
             break;
         case 'o':
             cfg.modflg |= MOD_OMITUPD;
             break;
         case 'p':
             cfg.modflg |= MOD_POST;     /* do posting only */
             cfg.omitupdate++;
             break;
         case 'q':
             cfg.modflg |= MOD_QUERY;    /* no posting */
             break;
         case 'r':
             cfg.modflg |= MOD_READER;
             break;
         case 's':
             cfg.modflg |= MOD_SLAVE;
             break;
         case 't':
             cfg.modflg &= (unsigned long)(~MOD_BATCH);
             cfg.modflg |= MOD_TYPE;     /* use stdout */
             break;
         case 'T':
             cfg.modflg |= MOD_TEST;    /* no dupe check */
             break;
         case 'x':
             cfg.modflg |= MOD_XOVER;
         case 'w':
             cfg.modflg |= MOD_NONEWNEWS;
             break;
         case 'g':
         case 'j':
         case 'n':
             fprintf(stderr, "Outdated option -m%c ignored, use -l%c\n", *opt, *opt);
             break;
         case 'u':
         case 'c':
         case 'e':
         case 'i':
             fprintf(stderr, "Outdated option -m%c ignored\n", *opt);
             break;
         default:
             fprintf(stderr, "Unknown option -m%c\n", *opt);
             break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Usage display                                                        */
/*                                                                      */
/************************************************************************/
static void usage(char *myname)
{
    fputs("Report bugs to harald@egnite.com\n", stderr);
    fprintf(stderr, "Usage: %s [options] host [groups YYMMDD HHMMSS [<dist>]]\n", myname);
    fputs("options:\n"
          "  -a<user:passwd>  authentication\n"
          "  -d<debug-flags>  debug switches               (all off)\n"
          "  -h<remotehost>   remote host to connect       (host)\n"
          "  -k<size>         stop after <size> kBytes     (unlimited)\n"
          "  -l<list-flags>   list retrieval flags\n"
          "  -m<mode-flags>   operation mode flags\n"
          "  -n<threads>      number of concurrent queries (3)\n"
          "  -p<port>         port number to connect       (nntp)\n"
          "  -r<rnewsprogram> rnews program call           (rnews)\n"
          "  -s<spooldir>     spool directory              (spool)\n"
          "  -t<minutes>      stop after <minutes> minutes (unlimited)\n"
          "  -v               verbose\n"
          "  -w<seconds>      remote connection timeout    (600)\n"
          "  -W<seconds>      local connection timeout     (60)\n\n"
          "mode flags:                   debug flags:\n"
          "  b  batch mode                 f  article filtering\n"
          "  C  catch-up                   t  nntp telegrams\n"
          "  o  omit datefile update       p  posting\n"
          "  p  posting only, no query     o  screen output\n"
          "  q  query only, no posting   list flags:\n"
          "  r  send MODE READER           a  active file\n"
          "  s  send SLAVE                 d  save dropped\n"
          "  t  type to console            g  new newsgroups\n"
          "  w  without newnews            j  distributions\n"
          "  x  use xover                  n  newsgroup descriptions\n"
          "                                u  subscriptions", stderr);
}


/************************************************************************/
/*                                                                      */
/* Signal handler                                                       */
/*                                                                      */
/************************************************************************/
static void cleanup(int sig)
{
    char *msg = "Unknown signal";
    int level = LOG_INFO;

    if(fppid)
        fclose(fppid);
    signal(sig, SIG_IGN);
    switch(sig) {
    case SIGSEGV:
        level = LOG_ERR;
        msg = "internal error";
        break;
    case SIGTERM:
        msg = "killed";
        unlink(cfg.pidfile);
        break;
    case SIGABRT:
        level = LOG_ERR;
        msg = "Aborted";
        break;
    case SIGBREAK:
    case SIGINT:
        msg = "Interrupted";
        unlink(cfg.pidfile);
        break;
    }
    syslog(level, msg);
    fputs("\nChanx ", stderr);
    fputs(msg, stderr);
    fputc('\n', stderr);
    unlink(cfg.pidfile);
    signal(SIGABRT, SIG_DFL);
    abort();
}
