/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module initializes and checks the configuration.
 */
/*
 * $Log$
 */

const char config_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <time.h>

#include <syslog.h>

#include <version.h>
#include <readconf.h>

#include "config.h"

static ENVTABLE et[] = {
    { cfg.workdir          , "CHANGIWORKDIR"          , "", 0 },

    { cfg.pidfile          , "CHANXPIDFILE"           , "", 2 },
    { cfg.spooldir         , "CHANGISPOOLDIR"         , "spool", 1 },

    { cfg.rnewscall        , "CHANGIRNEWSCALL"        , "rnews", 2 },

    { cfg.filterfile       , "CHANXFILTERFILE"        , "filter.cf", 3 },
    { cfg.filtermacro      , "CHANXFILTERMACRO"       , "cxdrop.cmd", 3 },
    { cfg.filterover       , "CHANXFILTEROVER"        , "overfilt.cmd", 3 },

    { cfg.remoteport       , "CHANXNEWSPORT"          , "nntp", 2 },
    { cfg.localaddr        , "CHANGILOCALHOST"        , "127.0.0.1", 2 },
    { cfg.localport        , "CHANGINEWSPORT"         , "nntp", 2 },

    { NULL                 , NULL                     , "", 0 }
};



CONFIG cfg;

static int dump_cfg(void);

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
void init_cfg(void)
{
    read_conf(et);
    cfg.numthreads = 3;
    cfg.remotewait = 300;
    cfg.localwait = 60;
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
int validate_cfg(void)
{
    int result = 1;
    char *cp;

    if(cfg.remoteaddr[0] == '\0')
        strcpy(cfg.remoteaddr, cfg.remotehost);

    if(cfg.authentication[0]) {
        if((cp = strchr(cfg.authentication, ':')) == NULL) {
            syslog(LOG_ERR, "Fatal error, missing password");
            result = -1;
        }
        else {
            *cp = '\0';
            strcpy(cfg.auth_user, cfg.authentication);
            strcpy(cfg.auth_pass, cp + 1);
            *cp = ':';
        }
    }

    if(DOLOG(LOG_CONFIG))
        dump_cfg();

    return(result);
}

/************************************************************************/
/*                                                                      */
/* Dump configuration                                                   */
/*                                                                      */
/************************************************************************/
static int dump_cfg(void)
{
    int result = -1;
    FILE *fp;
    ENVTABLE *ep;
    time_t t;

    if((fp = fopen("chanx.log", "a")) != NULL) {
        time(&t);
        fprintf(fp, "REM %s", ctime(&t));
        fprintf(fp, "REM Chanx Version %s\n", version);
        fprintf(fp, "REM Compiled %s\n", __DATE__);
        for(ep = et; ep->cnfg; ep++)
            fprintf(fp, "SET %s=%s&\n", ep->ename, ep->cnfg);
        fputc('\n', fp);
        if(fclose(fp))
            syslog(LOG_ERR, "%s", _strerror("Config dump"));
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "%s", _strerror("Config dump"));

    return(result);
}
