#ifndef CONFIG_H
#define CONFIG_H

/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*  Configuration                                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Id$
 */

typedef struct {
    char workdir[_MAX_PATH];
    char logfile[_MAX_PATH];
    char pidfile[_MAX_PATH];
    char spooldir[_MAX_PATH];
    char datefile[_MAX_PATH];
    char newgroupstime[_MAX_PATH];  /* added 08/12/97 */
    char iwantfile[_MAX_PATH];
    char rnewscall[_MAX_PATH];
    char filterfile[_MAX_PATH];
    char filtermacro[_MAX_PATH];
    char filterover[_MAX_PATH];
    char watermarklist[_MAX_PATH];
    char watermarktemp[_MAX_PATH];
    char remotehost[128];
    char remoteaddr[128];
    char remoteport[128];
    long remotewait;
    char localaddr[128];
    char localport[128];
    long localwait;
    unsigned long logflg;
    unsigned long modflg;
    long sizelimit;
    long timelimit;
    int  linelimit;
    int  numthreads;
    int  omitupdate;
    int  filtering;
    int  verbose;           /* added 09/07/97 harald */
    long lastdate;
    long lasttime;
    long nextdate;
    long nexttime;
    char distribution[128];
    char newsrc[_MAX_PATH];
    char grouplist[4000];
    int  canpost;
    int  usefilterfile;
    int  usefiltermacro;
    int  usefilterover;
    char authentication[128]; /* user and password */
    char auth_user[128];    /* user id */
    char auth_pass[128];    /* password */
} CONFIG;

extern CONFIG cfg;


extern void init_cfg(void);
extern int validate_cfg(void);

/*----------------------------------------------------------------------*/
/*  Logging flags                                                       */
/*----------------------------------------------------------------------*/
#define LOG_TELE        0x00000001L
#define LOG_INTER       0x00000002L
#define LOG_DROP        0x00000004L
#define LOG_POST        0x00000008L
#define LOG_CONFIG      0x00000010L

#define DOLOG(x)    ((cfg.logflg & (unsigned long)(x)) != 0)

/*----------------------------------------------------------------------*/
/*  Mode flags                                                          */
/*----------------------------------------------------------------------*/
#define MOD_NGLIST      0x00000001
#define MOD_POST        0x00000002
#define MOD_QUERY       0x00000004
#define MOD_READER      0x00000008
#define MOD_SLAVE       0x00000010
#define MOD_BATCH       0x00000080
#define MOD_OMITUPD     0x00000100
#define MOD_DTLOCAL     0x00000200
#define MOD_TYPE        0x00000400
#define MOD_DOSFILE     0x00000800
#define MOD_ACTLIST     0x00001000
#define MOD_DSTLIST     0x00002000
#define MOD_SUBLIST     0x00004000
#define MOD_NEWGROUPS   0x00008000
#define MOD_NONEWNEWS   0x00010000
#define MOD_DROPSAVE    0x00020000
#define MOD_FILTER      0x00040000
#define MOD_XOVER       0x00080000
#define MOD_TEST        0x00100000
#define MOD_CATCHUP     0x00200000

#define MOD_AUTHSIMPLE  0x00400000

#define DOMOD(x)    ((cfg.modflg & (unsigned long)(x)) != 0)

#endif
