/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */


const char getarts_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>

#include "config.h"
#include "account.h"
#include "rexxif.h"
#include "chanx.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int send_request(int *rbv, char *buf, char *cmd, char *msg_id)
{
    sprintf(buf, "%s %s\r\n", cmd, msg_id);
    if(SoStreamPuts(*rbv, buf) < 0)
        return(-1);
    return(0);
}

/************************************************************************/
/*  send_newid                                                          */
/*                                                                      */
/*  Gets the next id from the pool and sends it to the server.          */
/*                                                                      */
/*  Returns                                                             */
/*      0 if the article should be retrieved unfiltered                 */
/*      1 if the article's header should be filtered before retrieval   */
/*     -1 if no more ids are left in the pool                           */
/*     -2 if the connection to the server is broken                     */
/*                                                                      */
/************************************************************************/
static int send_newid(int *rbv, char *buf, char *msg_id)
{
    int result;

    if((result = get_id(msg_id)) != -1 &&
       send_request(rbv, buf, result ? "HEAD" : "ARTICLE", msg_id))
        result = -2;

    return(result);
}

/************************************************************************/
/*  art_thread                                                          */
/*                                                                      */
/*  This routine may run in multiple instances. It queries message ids  */
/*  from the id pool, queries the corresponding articles from a remote  */
/*  server, determines wether to drop or not to drop the article and    */
/*  adds the article contained in a linked list to the art pool.        */
/*                                                                      */
/************************************************************************/
static void _Optlink art_thread(void *pv)
{
    ACCNT_TINF *acct = pv;
    int filter;
    int wantbody = 0;
    long nbytes;
    long bbytes;
    int *rbv;
    char *msg_id;
    char *buf;
    int at_limit = 0;

    size_t bodysiz = 0;
    char *bodyptr = NULL;

    size_t headsiz = 0;
    char *headptr = NULL;
    char *headline[MAX_HEADLINES];

    /*
     * Connect to remote
     */
    if((rbv = remote_connect()) == NULL) {
        acct->state = STA_BROKEN;
        return;
    }

    buf = malloc(MAX_BUFSIZ);
    msg_id = malloc(MAX_MSGID);
    acct->state = STA_CONNECTED;

    /*
     * Send one id in advance, if result is -1, we are
     * out of ids. Result -2 indicates a broken connection.
     */
    filter = send_newid(rbv, buf, msg_id);

    /*
     * Enter a loop to retrieve articles
     */
    while(filter >= 0 && !at_limit) {

        /*
         * If the id we sent in advance requires filtering (result
         * of send_newid was 1) then we get the header now.
         */
        if(filter) {
            if((nbytes = receive_header(rbv, &headptr, &headsiz, headline)) < 0) {
                filter = -2;
                break;
            }
            at_limit = watch_limits(nbytes);
            if(nbytes) {
                acct->num_got++;
                wantbody = 1;
                if(cfg.usefilterfile)
                    wantbody = !new_drop(headline);
                if(wantbody != 0 && cfg.usefiltermacro)
                    wantbody = !run_filtermacro(headline);
                if(wantbody) {
                    if(send_request(rbv, buf, "BODY", msg_id)) {
                        filter = -2;
                        break;
                    }
                }
            }
            else {
                acci.num_miss++;
                wantbody = 0;
            }

            /*
             * Send a new id in advance before retrieving the body
             */
            if((filter = send_newid(rbv, buf, msg_id)) == -2)
                break;

            /*
             * Now get the body we queried and store the article
             */
            if(wantbody) {
                if((bbytes = receive_block(rbv, &bodyptr, &bodysiz)) < 0) {
                    filter = -2;
                    break;
                }
                at_limit = watch_limits(bbytes);
                if(spew_hb(headline, nbytes, bodyptr, bbytes)) {
                    filter = -2;
                    break;
                }
            }
            else
                drop_msgid(headline);
        }
        else {
            /*
             * The article of our last id wasn't filtered. Send
             * another id in advance before retrieving the article
             */
            if((filter = send_newid(rbv, buf, msg_id)) == -2)
                break;

            /*
             * Get the waiting article and store it
             */
            if((nbytes = receive_block(rbv, &bodyptr, &bodysiz)) < 0) {
                filter = -2;
                break;
            }
            at_limit = watch_limits(nbytes);
            if(nbytes) {
                if(spew_block(bodyptr, nbytes)) {
                    filter = -2;
                    break;
                }
                acct->num_got++;
            }
            else
                acci.num_miss++;
        }
    }
    SoStreamClose(*rbv);
    SoStreamBufDestroy(rbv);

    free(msg_id);
    free(buf);
    if(headptr)
        free(headptr);
    if(bodyptr)
        free(bodyptr);


    if(filter == -1)
        acct->state = STA_DONE;
    else
        acct->state = STA_BROKEN;
}

/************************************************************************/
/*  collect_arts                                                        */
/*                                                                      */
/*  Starts all article collection threads.                              */
/*                                                                      */
/************************************************************************/
void collect_arts(void)
{
    int i;

    for(i = 0; i < cfg.numthreads; i++) {
        _beginthread(art_thread, NULL, 32736, &(acci.acct[i]));
    }
}
