/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains routines to retrieve various lists from
 * a remote NNTP server.
 */
/*
 * $Log$
 */

const char getlists_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <xfile.h>
#include <domnpath.h>

#include "config.h"
#include "chanx.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int query_list(void *rbv, char *filename, char *query, char *mode)
{
    int items = 0;
    char *buf;
    FILE *fp;

    if((fp = xfopen(filename, mode)) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(filename));
        return(-1);
    }

    buf = malloc(MAX_BUFSIZ);
    for(;;) {
        if(SoStreamTransact(rbv, query, buf, cfg.remotewait)) {
            free(buf);
            return(-1);
        }
        if(atoi(buf) != 480 && (DOMOD(MOD_AUTHSIMPLE) || atoi(buf) != 450))
            break;
        if(authenticate(rbv, buf)) {
            free(buf);
            return(-1);
        }
    }
    if (buf[0] != '2') {
        syslog(LOG_ERR, "Server error: %s", buf);
        free(buf);
        return(-1);
    }

    for (;;) {
        if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, cfg.remotewait) <= 0) {
            items = -1;
            break;
        }

	if (!strcmp(buf, "."))
            break;
        items++;
        if(fprintf(fp, "%s\n", buf) <= 0) {
            syslog(LOG_ERR, "%s", _strerror(filename));
            items = 0;
            break;
        }
        if((items % 100) == 0)
            fprintf(stderr, ".");
    }
    fclose(fp);
    free(buf);

    return(items);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int get_lists(void *rbv)
{
    int items;
    char *listfile = malloc(_MAX_PATH);
    char *cp;

    /* 08/25/1996 harald
     * Because the following line was missing, Chanx crashed
     * while trying to retrieve any list.
     *
     * 08/29/1996 harald
     * Did it wrong again. The domainpath appends a string
     * to 'listfile', making it impossible to retrieve more
     * than one list. Added pointer 'cp' to cut the string.
     */
    strcpy(listfile, cfg.workdir);
    cp = listfile + strlen(listfile);

    if(DOMOD(MOD_NGLIST)) {
        fprintf(stderr, "Retrieving newsgroup descriptions ");
        domain_path(listfile, cfg.remotehost, ".newsgroups");
        if((items = query_list(rbv, listfile, "LIST NEWSGROUPS", "w")) >= 0)
            fprintf(stderr, "%u line(s)\n", items);
        else
            fprintf(stderr, "failed\n");
    }
    if(DOMOD(MOD_ACTLIST)) {
        fprintf(stderr, "Retrieving active newsgroup list ");
        *cp = '\0';
        domain_path(listfile, cfg.remotehost, ".active");
        if((items = query_list(rbv, listfile, "LIST ACTIVE", "w")) >= 0)
            fprintf(stderr, "%u line(s)\n", items);
        else
            fprintf(stderr, "failed\n");
    }
    if(DOMOD(MOD_DSTLIST)) {
        fprintf(stderr, "Retrieving distribution list ");
        *cp = '\0';
        domain_path(listfile, cfg.remotehost, ".distributions");
        if((items = query_list(rbv, listfile, "LIST DISTRIBUTIONS", "w")) >= 0)
            fprintf(stderr, "%u line(s)\n", items);
        else
            fprintf(stderr, "failed\n");
    }
    if(DOMOD(MOD_SUBLIST)) {
        fprintf(stderr, "Retrieving subscription list ");
        *cp = '\0';
        domain_path(listfile, cfg.remotehost, ".subscriptions");
        if((items = query_list(rbv, listfile, "LIST SUBSCRIPTIONS", "w")) >= 0)
            fprintf(stderr, "%u line(s)\n", items);
        else
            fprintf(stderr, "failed\n");
    }
    if(DOMOD(MOD_NEWGROUPS)) {
        FILE *dtfile;
        char *qbuf = malloc(MAX_CMDSIZE);

        /*
         * 08/12/97 harald
         * In case we are not using NEWNEWS, our dates are zero
         * and NEWGROUPS will fail. We added a new file to store
         * date and time.
         */
        if(cfg.lastdate == 0) {
            if((dtfile = xfopen(cfg.newgroupstime, "r")) == NULL) {
                cfg.lasttime = cfg.nexttime;
                cfg.lastdate = cfg.nextdate;
            }
            else {
                if(fscanf(dtfile, "%ld %ld", &cfg.lastdate, &cfg.lasttime) != 2) {
                    syslog(LOG_ERR, "%s has bad format", cfg.newgroupstime);
                    fprintf(stderr, "Bad format in %s\n", cfg.newgroupstime);
                    cfg.lasttime = cfg.nexttime;
                    cfg.lastdate = cfg.nextdate;
                }
                fclose(dtfile);
            }
        }
        sprintf(qbuf, "NEWGROUPS %06lu %06lu GMT", cfg.lastdate, cfg.lasttime);
        fprintf(stderr, "Retrieving list of new newsgroups ");
        *cp = '\0';
        domain_path(listfile, cfg.remotehost, ".newgroups");
        if((items = query_list(rbv, listfile, qbuf, "a")) >= 0) {
            fprintf(stderr, "%u line(s)\n", items);

            /*
             * Store time for our next query.
             */
            if((dtfile = xfopen(cfg.newgroupstime, "w")) == NULL) {
                syslog(LOG_ERR, "%s", _strerror(cfg.newgroupstime));
                perror(cfg.newgroupstime);
            }
            else {
                fprintf(dtfile, "%06lu %06lu\n", cfg.nextdate, cfg.nexttime);
                if(fclose(dtfile)) {
                    syslog(LOG_ERR, "%s", _strerror(cfg.newgroupstime));
                    perror(cfg.newgroupstime);
                }
            }
        }
        else
            fprintf(stderr, "failed\n");


        free(qbuf);
    }

    free(listfile);

    return(0);
}
