/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char getover_rcsid[] = "$Id$";

/*
 * $Log$
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>
#include <ctype.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <xfile.h>

#include "config.h"
#include "account.h"
#include "idtree.h"
#include "rexxif.h"
#include "chanx.h"

/************************************************************************/
/*  add_olddrops                                                        */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static void add_olddrops(char *fname)
{
    FILE *fp;

    if((fp = xfopen(fname, "r")) != NULL) {
        char *cp;
        char *article;
        char *line = malloc(MAX_BUFSIZ);
        char *lastgroup = malloc(MAX_GROUPNAME);

        *lastgroup = '\0';
        while(fgets(line, MAX_BUFSIZ, fp)) {
            if(*line == '<')
                continue;
            if((cp = strchr(line, ':')) == NULL)
                continue;
            *cp++ = '\0';
            for(article = cp; ; cp++) {
                if(*cp == ' ' || *cp == '\t' || *cp == '\n' || *cp == '\r') {
                    *cp = '\0';
                    break;
                }
                if(!isdigit(*cp))
                    break;
            }
            if(*cp != '\0' && !isdigit(*cp))
                continue;

            if(stricmp(line, lastgroup)) {
                add_artpool(line);
                strcpy(lastgroup, line);
            }
            add_artpool(article);
        }

        free(line);
        free(lastgroup);
        fclose(fp);
    }
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static void _Optlink over_thread(void *pv)
{
    int *rbv;
    FILE *fpw, *fpn;
    char *buf;
    char *ngtag;
    int broken = 0;
    long nwm, lo, hi;
    int count;
    long nbytes;
    int firstone;
    NODE *idroot = NULL;
    char *msgid;

    /*
     * Add previously dropped articles.
     */
    if(cfg.iwantfile[0])
        add_olddrops(cfg.iwantfile);

    /*
     * Connect to remote
     */
    if((rbv = remote_connect()) == NULL) {
        acci.id_state = STA_BROKEN;
        return;
    }

    acci.id_state = STA_CONNECTED;

    /*
     * Open last watermark.
     */
    if((fpw = xfopen(cfg.watermarklist, "r")) == NULL) {
        perror(cfg.watermarklist);
        syslog(LOG_ERR, "%s", _strerror(cfg.watermarklist));
        add_artpool(NULL);
        SoStreamClose(*rbv);
        SoStreamBufDestroy(rbv);
        acci.id_state = STA_BROKEN;
        return;
    }

    /*
     * Open new watermark.
     */
    if((fpn = xfopen(cfg.watermarktemp, "w")) == NULL) {
        perror(cfg.watermarktemp);
        syslog(LOG_ERR, "%s", _strerror(cfg.watermarktemp));
        fclose(fpw);
        add_artpool(NULL);
        SoStreamClose(*rbv);
        SoStreamBufDestroy(rbv);
        acci.id_state = STA_BROKEN;
        return;
    }

    /*
     * Loop for each group
     */
    buf = malloc(MAX_BUFSIZ);
    ngtag = malloc(MAX_GROUPNAME);
    msgid = malloc(MAX_MSGID);
    while(fgets(buf, MAX_GROUPNAME, fpw)) {
        char *cps = buf;
        char *cpd = ngtag;

        /*
         * If broken, copy all remaining watermark lines.
         */
        if(broken) {
            fputs(buf, fpn);
            continue;
        }

        /*
         * Interpret watermark line.
         */
        while(*cps && *cps != ' ' && *cps != '\t' && *cps != '\n' && *cps != ':')
            *cpd++ = *cps++;
        *cpd = '\0';
        if(*cps != ':' || *ngtag == '\0') {
            fputs(buf, fpn);
            continue;
        }
        if((nwm = atol(cps + 1)) == 0)
            nwm = 1;

        /*
         * Switch remote to this group. Support authentication.
         */
        for(;;) {
            strcpy(buf, "GROUP ");
            strcat(buf, ngtag);
            if(SoStreamTransact(rbv, buf, buf, cfg.remotewait)) {
                broken = 1;
                break;
            }
            if(atoi(buf) != 480 && (DOMOD(MOD_AUTHSIMPLE) || atoi(buf) != 450))
                break;
            if(authenticate(rbv, buf)) {
                broken = 1;
                break;
            }
        }

        /*
         * If the connection is broken, save the watermark line
         * and continue with the next group (which in fact will
         * copy the rest of the old watermark file to out new one.
         */
        if(broken) {
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }
        if(buf[0] != '2') {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }
        if(sscanf(buf + 4, "%d %ld %ld", &count, &lo, &hi) != 3) {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        /*
         * Check if new articles arrived.
         */
        if(lo <= nwm)
            lo = nwm + 1;
        if(lo > hi) {
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        /*
         * Send XOVER command.
         */
        for(;;) {
            sprintf(buf, "XOVER %ld-%ld", lo, hi);
            if(SoStreamTransact(rbv, buf, buf, cfg.remotewait)) {
                broken = 1;
                break;
            }
            if(atoi(buf) != 480 && (DOMOD(MOD_AUTHSIMPLE) || atoi(buf) != 450))
                break;
            if(authenticate(rbv, buf)) {
                broken = 1;
                break;
            }
        }

        /*
         * Connection broken. Save this watermark and copy the
         * rest.
         */
        if(broken) {
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }
        if(buf[0] != '2') {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        /*
         * The remote sends overview lines and terminates
         * the transmission by a line containing a single dot.
         */
        for(firstone = 1;;) {
            long n;

            if((nbytes = SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, cfg.remotewait)) <= 0) {
                broken = 1;
                break;
            }
            if(*buf == '.')
                break;
            if(watch_limits(nbytes)) {
                broken = 1;
                break;
            }
            for(cps = buf; ; cps++)
                if(!isdigit(*cps))
                    break;
            if((n = atol(buf)) != 0) {
                /*
                 * Filter crossposts
                 */
                char *cp1 = buf;
                char *cp2 = msgid;
                int  ntab = 0;

                for(; *cp1; cp1++)
                    if(*cp1 == '\t' && ++ntab == 4)
                        break;
                cp1++;
                while(*cp1 && *cp1 != '\t')
                    *cp2++ = *cp1++;

                *cp2 = '\0';

                if(ntab == 4 && *msgid && id_tree_insert(&idroot, msgid)) {
                    /*
                     * Filter unwanted articles
                     */
                    if(cfg.usefilterfile && over_drop(ngtag, buf)) {
                        drop_overview(ngtag, buf);
                        acci.num_dropped++;
                    }
                    else if(cfg.usefilterover && run_filterover(ngtag, buf)) {
                        drop_overview(ngtag, buf);
                        acci.num_dropped++;
                    }
                    else {
                        if(firstone) {
                            add_artpool(ngtag);
                            firstone = 0;
                        }
                        *cps = '\0';
                        add_artpool(buf);
                    }
                    if(n > nwm)
                        nwm = n;
                }
            }
        }

        /*
         * Save the new watermark.
         */
        fprintf(fpn, "%s: %lu\n", ngtag, nwm);
    }
    free(buf);
    free(ngtag);

    free(msgid);
    if(idroot)
        id_tree_destroy(idroot);

    /*
     * Add end marker, close connection and flag
     * the main thread that we've finished.
     */
    add_artpool(NULL);
    SoStreamClose(*rbv);
    SoStreamBufDestroy(rbv);

    fclose(fpw);
    fclose(fpn);


    if(broken)
        acci.id_state = STA_BROKEN;
    else
        acci.id_state = STA_DONE;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void collect_overview(void)
{
    _beginthread(over_thread, NULL, 32736, NULL);
}
