/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains the routine to retrieve current time and date
 * from a remote NNTP server.
 */
/*
 * $Log$
 */

const char gettime_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>

#include "config.h"
#include "chanx.h"

static int fake_date(char *buf);

/************************************************************************/
/* Query server's gmt                                                   */
/************************************************************************/
/*
 * 08/30/1996 harald
 * Date faking added for hosts which doesn't support DATE
 */
int get_time(void *rbv)
{
    int result = 0;
    long val1, val2;
    char *buf = malloc(MAX_BUFSIZ);

    if(DOMOD(MOD_DTLOCAL)) {
        if(fake_date(buf))
            result = -1;
    }
    else {
        for(;;) {
            if(SoStreamTransact(rbv, "DATE", buf, cfg.remotewait))
                result = -1;
            else if(atoi(buf) == 480 || (DOMOD(MOD_AUTHSIMPLE) && atoi(buf) == 450)) {
                if(authenticate(rbv, buf))
                    result = -1;
                else
                    continue;
            }
            else if(buf[0] != '1') {
                syslog(LOG_INFO, "Warning: %s", buf);
                if(fake_date(buf))
                    result = -1;
            }
            break;
        }
    }
    if(result == 0) {
        if((val2 = atol(&buf[12])) != 0) {
            buf[12] = '\0';
            if((val1 = atol(&buf[6])) != 0) {
                cfg.nexttime = val2;
                cfg.nextdate = val1;
            }
        }
    }
    free(buf);

    return(result);
}

/************************************************************************/
/* Fake servers time response                                           */
/*                                                                      */
/* Added 08/30/1996                                                     */
/************************************************************************/
static int fake_date(char *buf)
{
    struct tm *gmt;
    time_t now;

    time(&now);
    now -= 3600;    /* 1 hour time warp */

    if((gmt = gmtime(&now)) == NULL) {
        syslog(LOG_ERR, "Can't get local date and time");
        return(-1);
    }
    sprintf(buf, "111 %04.4d%02.2d%02.2d%02.2d%02.2d%02.2d\r\n",
            gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday,
            gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

    return(0);
}
