/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */
const char getxarts_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>

#include "config.h"
#include "account.h"
#include "chanx.h"

/************************************************************************/
/*  xart_thread                                                         */
/*                                                                      */
/*  This routine may run in multiple instances. It queries article      */
/*  numbers from the article pool, queries the corresponding articles   */
/*  from a remote server, and adds the article contained in a linked    */
/*  list to the art pool.                                               */
/*                                                                      */
/************************************************************************/
static void _Optlink xart_thread(void *pv)
{
    int *rbv;
    ACCNT_TINF *acct = pv;
    size_t bodysiz = 0;
    char *bodyptr = NULL;
    char *buf;
    char *group, *lastgroup;
    long artnum;
    long nbytes;

    /*
     * Connect to remote
     */
    if((rbv = remote_connect()) == NULL) {
        acct->state = STA_BROKEN;
        return;
    }

    buf = malloc(MAX_BUFSIZ);
    group = malloc(MAX_GROUPNAME);
    lastgroup = malloc(MAX_GROUPNAME);
    acct->state = STA_CONNECTED;

    while(get_artpool(group, &artnum) == 0) {
        if(strcmp(group, lastgroup)) {
            for(;;) {
                strcpy(buf, "GROUP ");
                strcat(buf, group);
                if(SoStreamTransact(rbv, buf, buf, cfg.remotewait)) {
                    acct->state = STA_BROKEN;
                    break;
                }
                if(atoi(buf) != 480 && (DOMOD(MOD_AUTHSIMPLE) || atoi(buf) != 450))
                    break;
                if(authenticate(rbv, buf)) {
                    acct->state = STA_BROKEN;
                    break;
                }
            }
            if(acct->state == STA_BROKEN)
                break;
            if(buf[0] != '2') {
                syslog(LOG_NOTICE, "%s: %s", group, buf);
                acct->state = STA_BROKEN;
                break;
            }
            strcpy(lastgroup, group);
        }

        if(SoStreamPrintf(*rbv, "ARTICLE %ld\r\n", artnum) < 0) {
            acct->state = STA_BROKEN;
            break;
        }

        /*
         * Get the article and store it
         */
        if((nbytes = receive_block(rbv, &bodyptr, &bodysiz)) < 0) {
            acct->state = STA_BROKEN;
            break;
        }
        if(nbytes) {
            if(spew_block(bodyptr, nbytes)) {
                acct->state = STA_BROKEN;
                break;
            }
            acct->num_got++;
        }
        else
            acci.num_miss++;
        if(watch_limits(nbytes)) {
            acct->state = STA_BROKEN;
            break;
        }
    }
    SoStreamClose(*rbv);
    SoStreamBufDestroy(rbv);

    free(buf);
    free(group);
    free(lastgroup);
    if(bodyptr)
        free(bodyptr);

    if(acct->state == STA_CONNECTED)
        acct->state = STA_DONE;
}

/************************************************************************/
/*  collect_xarts                                                       */
/*                                                                      */
/*  Starts all article collection threads.                              */
/*                                                                      */
/************************************************************************/
void collect_xarts(void)
{
    int i;

    for(i = 0; i < cfg.numthreads; i++) {
        _beginthread(xart_thread, NULL, 32736, &(acci.acct[i]));
    }
}
