/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains routines to send local posts to a remote
 * server.
 */
/*
 * $Log$
 */

const char ntjobs_rcsid[] = "$Id$";

#include <windows.h>

#include <stdio.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <domnpath.h>
#include <xfile.h>

#include "config.h"
#include "chanx.h"

#define DIRBUFSIZ   8192

extern char **cmdfiles;
extern char **jobfiles;
extern char **artfiles;

/************************************************************************/
/*                                                                      */
/************************************************************************/
int scan_jobs(char *sysname)
{
    HANDLE hdir;
    WIN32_FIND_DATA ffb;
    BOOL foundnext = TRUE;
    int num_jobs = 0;
    int num_got = 0;
    char *tpl = malloc(_MAX_PATH);
    char *line = malloc(MAX_BUFSIZ);
    char name[80];
    char typ;
    FILE *fp;
    char *cp;
    char *jobdir = malloc(_MAX_PATH);
    char *remotenode;

    strcpy(jobdir, cfg.spooldir);
    domain_path(jobdir, sysname, NULL);
    if((cp = strrchr(jobdir, '\\')) == NULL)
        remotenode = strdup(sysname);
    else {
        *cp++ = '\0';
        remotenode = strdup(cp);
    }
    sprintf(tpl, "%s\\C.%.8s*", jobdir, remotenode);

    if(DOLOG(LOG_POST))
        syslog(LOG_DEBUG, "Scanning '%s' for posts", tpl);
    /*
     * First loop will count entries
     */
    if( (hdir = FindFirstFile(tpl, &ffb)) != INVALID_HANDLE_VALUE ) {
        /*
         * 07/24/97 harald
         * Corrected a bug. Chanx won't post with one job
         * only, because we forgot the set 'num_jobs' to 1
         * after 'FindFirstFile'.
         */
        num_jobs = 1;
        while ( FindNextFile(hdir, &ffb) )
            num_jobs++;
        FindClose(hdir);
        if(DOLOG(LOG_POST))
            syslog(LOG_DEBUG, "Found %d posting jobs", num_jobs);

        if(num_jobs) {
            cmdfiles = malloc((size_t)(num_jobs + 1) * sizeof(char *));
            jobfiles = malloc((size_t)(num_jobs + 1) * sizeof(char *));
            artfiles = malloc((size_t)(num_jobs + 1) * sizeof(char *));

            /*
             * Second loop will build array of article filenames
             */
            if( (hdir = FindFirstFile(tpl, &ffb)) != INVALID_HANDLE_VALUE ) {
                while (foundnext && num_got < num_jobs) {
                    sprintf(tpl, "%s\\%s", jobdir, &ffb.cFileName);
                    if(DOLOG(LOG_POST))
                        syslog(LOG_DEBUG, "Processing job %s", tpl);

                    if((fp = xfopen(tpl, "r")) != NULL) {
                        cmdfiles[num_got] = strdup(tpl);
                        jobfiles[num_got] = NULL;
                        if(fgets(line, MAX_BUFSIZ, fp)) {
                            if(sscanf(line, "%c %s\n", &typ, name) == 2 && typ == 'S') {
                                sprintf(line, "%s\\%s", jobdir, name);
                                artfiles[num_got++] = strdup(line);
                            }
                        }
                        else if(DOLOG(LOG_POST))
                            syslog(LOG_DEBUG, "Failed to read 1st line of jobfile");
                        if(fgets(line, MAX_BUFSIZ, fp)) {
                            if(sscanf(line, "%c %s\n", &typ, name) == 2 && typ == 'S') {
                                sprintf(line, "%s\\%s", jobdir, name);
                                jobfiles[num_got - 1] = strdup(line);
                            }
                        }
                        else if(DOLOG(LOG_POST))
                            syslog(LOG_DEBUG, "Failed to read 2nd line of jobfile");
                        fclose(fp);
                    }
                    else
                        syslog(LOG_ERR, "%s", _strerror(tpl));


                    foundnext = FindNextFile(hdir, &ffb);
                }
                FindClose(hdir);
            }
            cmdfiles[num_got] = NULL;
            jobfiles[num_got] = NULL;
            artfiles[num_got] = NULL;
        }
    }
    else if(DOLOG(LOG_POST))
        syslog(LOG_DEBUG, "No postings found");

    free(line);
    free(tpl);
    free(jobdir);
    free(remotenode);

    return(num_got);
}
