/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains routines to send local posts to a remote
 * server.
 */
/*
 * $Log: post.c,v $
 */

const char post_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <domnpath.h>
#include <xfile.h>

#include "config.h"
#include "account.h"
#include "chanx.h"

#define DIRBUFSIZ   8192

char **cmdfiles;
char **jobfiles;
char **artfiles;

int scan_jobs(char *sysname);

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
static int post(void *rbv, char *path)
{
    int  result = 0;
    int  in_header = 1;
    char *buf = malloc(MAX_BUFSIZ);
    FILE *fp;
    char *cp;
    int  sock = *((int *)rbv);

    /*
     * Send command and get response
     */
    for(;;) {
        if(SoStreamTransact(rbv, "POST", buf, cfg.remotewait)) {
            free(buf);
            return(-1);
        }
        if(atoi(buf) != 480 && (DOMOD(MOD_AUTHSIMPLE) || atoi(buf) != 450))
            break;
        if(authenticate(rbv, buf)) {
            free(buf);
            return(-1);
        }
    }
    if (buf[0] != '3') {
        fprintf(stderr, "%s\n", buf);
        syslog(LOG_NOTICE, "Server error: %s", buf);
        free(buf);
        return(-1);
    }

    if((fp = xfopen(path, "r")) != NULL) {
        *buf = '.';
        result = 1;
        while(fgets(buf + 1, MAX_BUFSIZ - 2, fp)) {
            /*
             * Remove header lines which will cause inn to reject
             * this post.
             */
            if(in_header) {
                switch(buf[1]) {
                case '\n':
                    in_header = 0;
                    break;
                case 'D':
                    if(strncmp(buf + 1, "Date-Received:", sizeof("Date-Received:") - 1) == 0)
                        continue;
                    break;
                case 'N':
                    if(strncmp(buf + 1, "NNTP-Posting-Host:", sizeof("NNTP-Posting-Host:") - 1) == 0)
                        continue;
                    break;
                case 'P':
                    if(strncmp(buf + 1, "Posted:", sizeof("Posted:") - 1) == 0)
                        continue;
                    if(strncmp(buf + 1, "Posting-Version:", sizeof("Posting-Version:") - 1) == 0)
                        continue;
                    break;
                case 'R':
                    if(strncmp(buf + 1, "Received:", sizeof("Received:") - 1) == 0)
                        continue;
                    if(strncmp(buf + 1, "Relay-Version:", sizeof("Relay-Version:") - 1) == 0)
                        continue;
                    break;
                case 'X':
                    if(strncmp(buf + 1, "Xref:", sizeof("Xref:") - 1) == 0)
                        continue;
                    break;
                }
            }
            if((cp = strchr(buf + 1, '\n')) != NULL) {
                *cp++ = '\r';
                *cp++ = '\n';
                *cp = '\0';
            }
            if(SoStreamPuts(sock, buf + (buf[1] != '.')) < 0) {
                result = -1;
                break;
            }
        }
        fclose(fp);

        if(result == 1) {
            if(SoStreamTransact(rbv, ".", buf, cfg.remotewait))
                result = -1;
            else {
                fprintf(stderr, "%s\n", buf);
                if (buf[0] != '2') {
                    syslog(LOG_NOTICE, "Server error: %s", buf);
                    result = -1;
                }
                else
                    acci.num_posted++;
            }
        }
    }
    else
        syslog(LOG_ERR, "%s", _strerror(path));

    free(buf);

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int send_posts(void *rbv)
{
    if(!DOMOD(MOD_QUERY)) {
        if(cfg.canpost) {
            if(scan_jobs(cfg.remotehost)) {
                int i;

                fprintf(stderr, "Posting to %s...\n", cfg.remotehost);
                for(i = 0; artfiles[i]; i++) {
                    if(post(rbv, artfiles[i]) == 1) {
                        if(cmdfiles[i] != NULL && xremove(cmdfiles[i]))
                            syslog(LOG_ERR, "%s", _strerror(cmdfiles[i]));
                        if(jobfiles[i] != NULL && xremove(jobfiles[i]))
                            syslog(LOG_ERR, "%s", _strerror(jobfiles[i]));
                        if(xremove(artfiles[i]))
                            syslog(LOG_ERR, "%s", _strerror(artfiles[i]));
                    }
                    else
                        fprintf(stderr, "%s not posted\n", artfiles[i]);
                    if(cmdfiles[i])
                        free(cmdfiles[i]);
                    if(jobfiles[i])
                        free(jobfiles[i]);
                    free(artfiles[i]);
                }
                free(cmdfiles);
                free(jobfiles);
                free(artfiles);
            }
            else
                fprintf(stderr, "Nothing to post to %s\n", cfg.remotehost);
        }
        else
            fprintf(stderr, "Server doesn't allow us to post\n");
    }
    return(0);
}
