/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

const char rexxif_rcsid[] = "$Id$";

#ifdef __OS2__
#define INCL_REXXSAA
#include <rexxsaa.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>

#include "config.h"
#include "rexxif.h"

#define CXMACRONAME  "CXFILTER"

/************************************************************************/
/*                                                                      */
/************************************************************************/
int init_filtermacro(void)
{
#ifdef __OS2__
    APIRET rc;
    rc = RexxAddMacro(CXMACRONAME, cfg.filtermacro, RXMACRO_SEARCH_BEFORE);
    return(rc == 0 ? 0 : -1);
#else
    return 0;
#endif
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int init_filterover(void)
{
#ifdef __OS2__
    APIRET rc;
    rc = RexxAddMacro(CXMACRONAME, cfg.filterover, RXMACRO_SEARCH_BEFORE);
    return(rc == 0 ? 0 : -1);
#else
    return 0;
#endif
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int run_filtermacro(char *headline[])
{
#ifdef __OS2__
    int dropit = 0;
    APIRET rc;
    PRXSTRING arglist;
    ULONG numargs;
    RXSTRING instore[2];
    RXSTRING result;
    SHORT retcode;
    char buf[128];
    int i;

    for(numargs = 0, i = 0; headline[i]; i++)
        numargs++;
    arglist = malloc((numargs + 1) * sizeof(RXSTRING));
    for(i = 0; headline[i]; i++) {
        arglist[i].strlength = strlen(headline[i]);
        arglist[i].strptr = headline[i];
    }
    arglist[i].strlength = 0;
    arglist[i].strptr = NULL;

    instore[0].strptr = NULL;
    instore[0].strlength = 0;
    instore[1].strptr = NULL;
    instore[1].strlength = 0;

    result.strptr    = buf;
    result.strlength = sizeof(buf);

    rc = RexxStart(numargs, arglist, CXMACRONAME, instore, "CMD", RXFUNCTION,
                   NULL, &retcode, &result);
    if(rc == 0 && retcode == 1)
        dropit = 1;

    return(dropit);
#else
    return 0;
#endif
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int run_filterover(char *group, char *overline)
{
#ifdef __OS2__
    int dropit = 0;
    APIRET rc;
    PRXSTRING arglist;
    RXSTRING instore[2];
    RXSTRING result;
    SHORT retcode;
    char buf[128];

    arglist = malloc(3 * sizeof(RXSTRING));
    arglist[0].strlength = strlen(group);
    arglist[0].strptr = group;
    arglist[1].strlength = strlen(overline);
    arglist[1].strptr = overline;
    arglist[2].strlength = 0;
    arglist[2].strptr = NULL;

    instore[0].strptr = NULL;
    instore[0].strlength = 0;
    instore[1].strptr = NULL;
    instore[1].strlength = 0;

    result.strptr    = buf;
    result.strlength = sizeof(buf);

    rc = RexxStart(2, arglist, CXMACRONAME, instore, "CMD", RXFUNCTION,
                   NULL, &retcode, &result);
    if(rc == 0 && retcode == 1)
        dropit = 1;

    return(dropit);
#else
    return 0;
#endif
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int exit_filtermacro(void)
{
#ifdef __OS2__
    APIRET rc;
    rc = RexxClearMacroSpace();
    return(rc == 0 ? 0 : -1);
#else
    return 0;
#endif
}
