/*
 * This program was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 * harald@os2point.ping.de
 */

const char config_rcsid[]="$Id: config.c,v 1.1 1997/02/13 11:21:42 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>

#include <syslog.h>
#include <sysdep.h>
#include <readconf.h>

#include <chlimits.h>
#include <nactive.h>
#include "expire.h"
#include "config.h"

CONFIG cfg;

static ENVTABLE et[] = {
    { cfg.workdir          , "CHANGIWORKDIR"          , "", 0 },

    { cfg.pidfile          , "CHANGIEXPIREPIDFILE"    , "expire.pid", 1 },
    { cfg.newsdir          , "CHANGINEWSDIR"          , "news", 1 },
    { cfg.overdir          , "CHANGIOVERDIR"          , "news", 1 },
    { cfg.activefile       , "CHANGIACTIVEFILE"       , "active", 1 },
    { cfg.historyfile      , "CHANGIHISTORY"          , "history", 1 },

    { cfg.tempdir          , "CHANGITEMPDIR"          , "TMP", 4 },

    { cfg.expireconf       , "CHANGIEXPIRECONF"       , "expire.conf", 3 },
    { cfg.summary          , "CHANGIEXPIRESUMMARY"    , "expire.sum", 3 },

    { cfg.zapfile          , "CHANGIEXPIREZAPFILE"    , "", 0 },

    { cfg.logname          , "CHANGILOGNAME"          , "USER", 4 },
    { cfg.sortcall         , "CHANGISORTCALL"         , "exsort +1n >", 2 },
    { cfg.controlport      , "CHANGICONTROLPORT"      , "119", 2 },
    { cfg.controlhost      , "CHANGILOCALHOST"        , "127.0.0.1", 2 },
    { cfg.expreason        , "CHANGIEXPIREREASON"     , "Expiring", 2 },

    { NULL                 , NULL                     , "", 0 }
};

/************************************************************************/
/*                                                                      */
/************************************************************************/
void init_cfg(void)
{
    read_conf(et);
    cfg.expiresecs = 7 * 86400;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int load_expcnf(char *name, char ***lines)
{
    register char *cp;
    register char *lp;
    char **result;
    int  i;
    char *buf;

    /*
     * Read in the file
     */
    if((buf = load_file(name)) == NULL)
        return(-1);

    /*
     * Get a gros count of the number of lines in the file
     */
    for(cp = buf, i = 0; (cp = strchr(cp, '\n')) != NULL; cp++, i++)
        continue;

    /*
     * Allocate enough storage for a line pointer array
     * to contain all lines.
     * Set the first pointer to the beginning of the
     * buffer. This enables the caller to free the buffer,
     * even if the first valid line is not at the beginning
     * of the file.
     */
    result = malloc((size_t)(i + 1) * sizeof(char *));
    result[0] = buf;

    /*
     * Scan the file
     */
    for(i = 0, cp = buf; *cp; ) {
        for(lp = cp; *cp; cp++) {
            if(*cp == '\r' || *cp == '\n') {
                *cp++ = '\0';
                break;
    }
        }
        /*
         * Ignore comments and empty lines
         */
        if(*lp == '#' || *lp == '\0')
            continue;
        result[++i] = lp;
    }
    result[i + 1] = NULL;   /* double stiched */
    *lines = result;

    return(i);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int read_expcnf(char *name)
{
    int result = 0;
    int ncnf_lines;
    int i;
    char **cnf_lines = NULL;
    char *cp;
    long minkeep;

    /*
     * Read entries into a static array of line pointers
     */
    if((ncnf_lines = load_expcnf(name, &cnf_lines)) < 0)
        return(-1);

    /*
     * Parse entries
     */
    for(i = 1; i <= ncnf_lines; i++) {
        cp = cnf_lines[i];

        /*
         * Skip pattern
         */
        while(*cp && *cp != ' ' && *cp != '\t')
            cp++;
        if(*cp == '\0') {
            syslog(LOG_INFO, "Format error in %s: %s", name, cnf_lines[i]);
            result = -1;
            break;
        }
        *cp++ = '\0';

        /*
         * Next field is minimum days to keep
         */
        while(*cp == ' ' || *cp == '\t')
            cp++;
        if(*cp == '\0') {
            syslog(LOG_INFO, "Format error in %s: %s", name, cnf_lines[i]);
            result = -1;
            break;
        }
        minkeep = convtime(cp, 'd');

        /*
         * Next field is maximum days to keep
         */
        while(*cp && *cp != ' ' && *cp != '\t')
            cp++;
        while(*cp == ' ' || *cp == '\t')
            cp++;

        /*
         * Set min and max for all groups matching
         * our patten list.
         */
        set_expiration(cnf_lines[i], minkeep, convtime(cp, 'd'));
    }

    /*
     * Free the buffer containing the file
     */
    free(cnf_lines[0]);

    /*
     * Free the line pointer array
     */
    free(cnf_lines);

    return(result);
}

/*
 * $Log: config.c,v $
 * Revision 1.1  1997/02/13 11:21:42  oliver
 * initial revision
 *
 */
