/*
 * This program was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 */

const char histhead_rcsid[]="$Id: histhead.c,v 1.1 1997/02/13 11:21:46 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys\types.h>
#include <sys\stat.h>

#include <syslog.h>
#include <chlimits.h>
#include <xfile.h>

#include "config.h"
#include "histhead.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
static FILE *openlink(const char *path)
{
    FILE *fp, *fplink;
    register char *cp;
    register char *ip;

    if((fp = xfopen((char *)path, "r")) != NULL) {
        char *buf = malloc(MAX_BUFSIZ);

        if(fgets(buf, MAX_BUFSIZ, fp)) {
            if(*buf == 'X' && strncmp(buf, "X-Linked:", 9) == 0) {
                char *link = malloc(_MAX_PATH);

                cp = strchr(strcpy(link, cfg.newsdir), '\0');
                *cp++ = '\\';
                for(ip = buf + 9; *ip == ' '; ip++)
                    continue;
                for(; *ip && *ip != ' ' && *ip != '\n'; ip++)
                    if(*ip == '.' || *ip == ':')
                        *cp++ = '\\';
                    else
                        *cp++ = *ip;
                *cp = '\0';
                if((fplink = xfopen(link, "r")) != NULL) {
                    fclose(fp);
                    fp = fplink;
                }
                else {
                    if(DOLOG(LOG_EXPIRE))
                        syslog(LOG_DEBUG, "Missing %s", buf);
                    fclose(fp);
                    fp = NULL;
                }
                free(link);
            }
            else
                rewind(fp);
        }
        free(buf);
    }
    return(fp);
}

/************************************************************************/
/*                                                                      */
/* read_histheader                                                      */
/*                                                                      */
/* This routine reads all header lines needed for expiration of this    */
/* article. It will also return the article files creation date and     */
/* its size.                                                            */
/*                                                                      */
/*                                                                      */
/************************************************************************/
int read_histheader(char *artpath, char headitem[HDI_LAST][MAX_BUFSIZ], long *arrived, long *artsize)
{
    FILE *fp;
    struct stat ast;
    char *line;
    char *cp;
    char *colon;
    int nitems = 0;
    int i;

    if((fp = openlink(artpath)) == NULL) {
        if(DOLOG(LOG_EXPIRE))
            syslog(LOG_DEBUG, "%s", _strerror(artpath));
        return(-1);
    }

    if(fstat(fileno(fp), &ast)) {
        if(DOLOG(LOG_EXPIRE))
            syslog(LOG_DEBUG, "%s", _strerror(artpath));
        fclose(fp);
        return(-1);
    }

    /*
     * Optionally use last write instead of file's creation date
     * to determine the arrival time.
     * 07/15/1996 harald
     */
    if(cfg.uselastwrite)
        *arrived = ast.st_mtime;
    else
        *arrived = ast.st_ctime;
    *artsize = ast.st_size;
    line = malloc(MAX_BUFSIZ);
    for(i = 0; i < HDI_LAST; i++)
        headitem[i][0] = '\0';
    while(fgets(line, MAX_BUFSIZ, fp)) {
        if(*line == '\n')
            break;
        if((colon = strchr(line, ':')) == NULL)
            continue;
        *colon = '\0';
        if((cp = strchr(colon + 1, '\n')) == NULL)
            continue;
        *cp = '\0';
        cp = colon + 1;
        while(*cp == ' ' || *cp == '\t')
            cp++;
        switch(*line) {
        case 'D':
            if(strcmp(line, "Date") == 0) {
                strcpy(headitem[HDI_Date], cp);
                nitems++;
            }
            break;
        case 'E':
            if(strcmp(line, "Expires") == 0) {
                strcpy(headitem[HDI_Expires], cp);
                nitems++;
            }
            break;
        case 'M':
            if(strcmp(line, "Message-ID") == 0) {
                strcpy(headitem[HDI_Message_ID], cp);
                nitems++;
            }
            break;
        case 'X':
            if(strcmp(line, "Xref") == 0) {
                strcpy(headitem[HDI_Xref], cp);
                nitems++;
            }
            break;
        }
        if(nitems >= HDI_LAST)
            break;
    }
    free(line);
    fclose(fp);

    if(headitem[HDI_Message_ID][0] == '\0') {
        if(DOLOG(LOG_EXPIRE))
            syslog(LOG_DEBUG, "Message-ID missing in %s", artpath);
        return(-1);
    }
    if(headitem[HDI_Date][0] == '\0') {
        if(DOLOG(LOG_EXPIRE))
            syslog(LOG_DEBUG, "Date missing in %s", artpath);
        return(-1);
    }

    return(0);
}

/*
 * $Log: histhead.c,v $
 * Revision 1.1  1997/02/13 11:21:46  oliver
 * initial revision
 *
 */

