/*
 * This program was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 * This module contains routines to purge article by scanning
 * the history file.
 *
 */

const char purgebyc_rcsid[] = "$Id: purgebyc.c,v 1.1 1997/02/13 11:21:51 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include <sys\types.h>
#include <sys\stat.h>

#include <syslog.h>
#include <chlimits.h>
#include <split.h>
#include <xfile.h>
#include <arpatime.h>

#include "config.h"
#include "rebuild.h"
#include "histhead.h"
#include "expire.h"

static char headitem[HDI_LAST][MAX_BUFSIZ];

static char line[MAX_HISTORYLINE];
static char *grouplist = NULL;
static long arrived = 0;
static long expires = 0;
static long posted = 0;
static long artsize = 0;

/************************************************************************/
/*                                                                      */
/*  purge_by_cancel                                                     */
/*                                                                      */
/*  Sequentially reads the history file to purges outdated articles.    */
/*  The article-ids are simply marked canceled in the history file      */
/*  so there's no need to rebuild the index.                            */
/*                                                                      */
/************************************************************************/
int purge_by_cancel(void)
{
    int result;
    FILE *fp;
    char *filename = malloc(strlen(cfg.historyfile) + 5);
    char pathname[_MAX_PATH];
    int skipped = 0;
    char **groups;
    char *cp;
    long pos, npos;
    char *wipe;

    /*
     * Open an existing history file for reading
     */
    strcat(strcpy(filename, cfg.historyfile), ".pag");
    if((fp = xfopen(filename, "r+b")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(filename));
        free(filename);
        return(-1);
    }

    /*
     * Loop for all lines in the history file
     */
    if(cfg.verbose)
        fputs("Scanned Expired Remembr    Kept Bad Age\n", stderr);
    for(;;) {
        if(cfg.verbose  && !((ids_processed + old_kept) % 100))
            fprintf(stderr, "\r%7u %7u %7u %7u %3u%4d  ", ids_processed,
                    files_deleted, old_kept, ids_kept, skipped,
                    (now - arrived) / 86400);
        if((pos = ftell(fp)) == -1) {
            syslog(LOG_ERR, "Tell %s", _strerror(filename));
            break;
        }
        if(fgets(line, MAX_HISTORYLINE, fp) == NULL)
            break;

        /*
         * Leave broken lines untouched
         */
        if((result = parse_historyline(line, &arrived, &expires, &posted,
                                       &artsize, &grouplist, &wipe)) == -1) {
            if(++skipped >= cfg.quitbad && cfg.quitbad)
                break;
            result = 0;
            continue;
        }

        /*
         * Leave cancels with old and new format untouched
         */
        if(result) {
            old_kept++;
            result = 0;
            continue;
        }

        else {
            ids_processed++;

            /*
             * Check any group of crossposts
             */
            groups = split_list(grouplist, ',');

            /*
             * If posting date is 0 then we probably got an
             * old style entry. In that case we must get
             * posting and expiration date from the article
             * file itself.
             */
            if(posted == 0) {
                if(grouppath(pathname, cfg.newsdir, *groups) != NULL) {
                    if((cp = strrchr(pathname, ':')) != NULL) {
                        *cp = '\\';
                        if(read_histheader(pathname, headitem, &arrived, &artsize) == 0) {
                            posted = getadate(headitem[HDI_Date]);
                            expires = getadate(headitem[HDI_Expires]);
                        }
                    }
                }
                if(posted == 0)
                    posted = arrived;
            }

            if(is_expired(groups, now + cfg.timewarp, arrived, expires, posted)) {
                /*
                 * Delete article
                 */
                ids_expired++;
                remove_art(groups, artsize);

                /*
                 * Update line in history file
                 */
                old_kept++;
                ids_expired++;
                if(!cfg.noupdate) {
                    if((npos = ftell(fp)) == -1) {
                        syslog(LOG_ERR, "Tell %s", _strerror(filename));
                        break;
                    }
                    if(fseek(fp, pos, SEEK_SET)) {
                        syslog(LOG_ERR, "Seek %s", _strerror(filename));
                        break;
                    }
                    if(fgets(line, MAX_HISTORYLINE, fp) == NULL) {
                        syslog(LOG_ERR, "2-Read %s", _strerror(filename));
                        break;
                    }
                    if(fseek(fp, pos, SEEK_SET)) {
                        syslog(LOG_ERR, "Seek %s", _strerror(filename));
                        break;
                    }
                    while(*wipe && *wipe != '\r' && *wipe != '\n')
                        *wipe++ = ' ';
                    *wipe = '\0';
                    if(fseek(fp, pos, SEEK_SET)) {
                        syslog(LOG_ERR, "Seek %s", _strerror(filename));
                        break;
                    }
                    /*
                     * Ignore errors at this point, rtlib returns crazy
                     * errorcode.
                     */
                    fwrite(line, strlen(line), 1, fp);
                    if(fseek(fp, npos, SEEK_SET)) {
                        syslog(LOG_ERR, "Seek %s", _strerror(filename));
                        break;
                    }
                }
            }
            else
                /*
                 * Keep article
                 */
                ids_kept++;

            free(groups);
        }
    } /* Loop history lines */

    if(cfg.verbose)
        fprintf(stderr, "\r%7u %7u %7u %7u %3u%4d\n", ids_processed,
                files_deleted, old_kept, ids_kept, skipped,
                (now - arrived) / 86400);

    if(fclose(fp)) {
        syslog(LOG_ERR, "%s", _strerror("Closing history"));
        return(-1);
    }

    if(!result && files_deleted) {
        if(cfg.verbose)
            fputs("Remember to rebuild history from time to time\n", stderr);
    }

    return(result);
}

/*
 * $Log: purgebyc.c,v $
 * Revision 1.1  1997/02/13 11:21:51  oliver
 * initial revision
 *
 */
