const char rebuild_rcsid[] = "$Id$";
/*
 * This program was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 * This module contains routines to rebuild the history
 * database, either in new (hash index) or old format.
 *
 */
/*
 * $Log$
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include <sys\types.h>
#include <sys\stat.h>

#include <syslog.h>
#include <chlimits.h>
#include <xfile.h>

#include <ndbm.h>
#include "dbm.h"

#include "config.h"
#include "rebuild.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
int rebuild_history(char *seqhist)
{
    NDBM *db;
    register char *cp;
    register char *cp1;
    char *line;
    char *last;
    char *lswp;
    char *name;
    FILE *fp;
    FILE *fpt;
    int  addit = (*seqhist == '+');

    /*
     * Before removing the current history try to
     * open the new one.
     */
    if((fp = xfopen(seqhist + addit, "r")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(seqhist));
        return(-1);
    }

    /*
     * Truncate the history file and remove it's index
     */
    if(!addit) {
        strcpy(name = malloc(strlen(cfg.historyfile) + 5), cfg.historyfile);
        cp = strrchr(name, '\0');
        strcpy(cp, ".pag");
        if((fpt = xfopen(name, "w")) == NULL)
            syslog(LOG_ERR, "Truncate %s", _strerror(name));
        else
            fclose(fpt);
        strcpy(cp + 1, "idx");
        if(xremove(name))
            syslog(LOG_ERR, "Delete %s", _strerror(name));
        free(name);
    }

    if((db = ndbm_open(cfg.historyfile, 0)) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(cfg.historyfile));
        fclose(fp);
        return(-1);
    }

    line = malloc(MAX_HISTORYLINE);
    last = malloc(MAX_HISTORYLINE);
    memset(last, 0, MAX_HISTORYLINE);
    while(fgets(line, MAX_HISTORYLINE, fp)) {
        cp = line;
        if(*cp != '<') {
            if(DOLOG(LOG_EXPIRE))
                syslog(LOG_DEBUG, "Missing <, skipped bad line");
            continue;
        }
        while(*cp && *cp != '>')
            cp++;
        if(*cp == '\0') {
            if(DOLOG(LOG_EXPIRE))
                syslog(LOG_DEBUG, "Missing >, skipped bad line");
            continue;
        }
        cp1 = ++cp;
        *cp1 = '\0';
        if(line[1] == last[1] && strcmp(line, last) == 0)
            continue;
        *cp1 = ' ';
        while(*cp && *cp != '\n')
            cp++;
        if(*cp == '\0') {
            if(DOLOG(LOG_EXPIRE))
                syslog(LOG_DEBUG, "Skipped long line");
            continue;
        }
        *cp = '\0';
        ndbm_store(db, line);
        *cp1 = '\0';
        lswp = last;
        last = line;
        line = lswp;

    }
    ndbm_close(db);
    fclose(fp);
    free(line);
    free(last);

    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int rebuild_oldhistory(char *seqhist)
{
    DBM *db;
    register char *cp;
    register char *line;
    char *name;
    datum key, val;
    FILE *fp;
    FILE *fpt;
    int  addit = (*seqhist == '+');

    /*
     * Before removing the current history try to
     * open the new one.
     */
    if((fp = xfopen(seqhist + addit, "r")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(seqhist));
        return(-1);
    }

    /*
     * Truncate the history file and remove it's index
     */
    if(!addit) {
        strcpy(name = malloc(strlen(cfg.historyfile) + 5), cfg.historyfile);
        cp = strrchr(name, '\0');
        strcpy(cp, ".pag");
        if((fpt = xfopen(name, "w")) == NULL)
            syslog(LOG_ERR, "Truncate %s", _strerror(name));
        else
            fclose(fpt);
        strcpy(cp + 1, "dir");
        if(xremove(name))
            syslog(LOG_ERR, "Delete %s", _strerror(name));
        free(name);
    }

    if((db = dbm_open(cfg.historyfile, O_RDWR | O_CREAT, 0666)) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(cfg.historyfile));
        fclose(fp);
        return(-1);
    }

    line = malloc(MAX_HISTORYLINE);
    while(fgets(line, MAX_HISTORYLINE, fp)) {
        if((cp = strchr(line, '\n')) == NULL) {
            fgets(line, MAX_HISTORYLINE, fp);
            continue;
        }
        *cp = '\0';
        if((cp = strchr(line, ' ')) == NULL)
            continue;
        *cp++ = '\0';
        key.dptr = line;
        key.dsize = strlen(key.dptr) + 1;
        val.dptr = cp;
        val.dsize = strlen(val.dptr) + 1;
        dbm_store(db, key, val, DBM_REPLACE);
    }
    dbm_close(db);
    fclose(fp);
    free(line);

    return(0);
}
