/*
 * This program was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 * This module contains routines to purge article by scanning
 * the history file.
 *
 */

const char testhist_rcsid[] = "$Id: testhist.c,v 1.1 1997/02/13 11:21:53 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#include <chlimits.h>
#include <xfile.h>

#include <ndbm.h>

#include "config.h"
#include "expire.h"

static char line[MAX_HISTORYLINE];
static char *grouplist = NULL;
static long arrived = 0;
static long expires = 0;
static long posted = 0;
static long artsize = 0;

/************************************************************************/
/*                                                                      */
/************************************************************************/
int test_history(void)
{
    int result = 0;
    FILE *fp, *fpnew;
    char pagname[_MAX_PATH];
    char seqname[_MAX_PATH];
    int skipped = 0;
    int ids_retrieved = 0;
    int ids_refused = 0;
    NDBM *db;
    register char *cp;

    strcat(strcpy(pagname, cfg.historyfile), ".pag");
    strcat(strcpy(seqname, cfg.historyfile), ".seq");

    /*
     * Open an existing history file for reading
     */
    if((fp = xfopen(pagname, "r")) == NULL) {
        perror(pagname);
        syslog(LOG_ERR, "%s", _strerror(pagname));
        return(-1);
    }

    /*
     * Open a temporary history file. We'll build
     * it's index later.
     */
    if((fpnew = xfopen(seqname, "w")) == NULL) {
        perror(seqname);
        syslog(LOG_ERR, "%s", _strerror(seqname));
        fclose(fp);
        return(-1);
    }

    /*
     * Loop for all lines in the history file
     */
    if(cfg.verbose)
        fputs("Scanned Retrved Refused Bad\n", stderr);
    while(fgets(line, MAX_HISTORYLINE, fp)) {
        /*
         * Discard broken lines
         */
        if((result = parse_historyline(line, &arrived, &expires, &posted,
                                       &artsize, &grouplist, NULL)) == -1) {
            if(++skipped) {
                fprintf(stderr, "Bad entry '%s'\n", line);
                result = -1;
                break;
            }
            result = 0;
            continue;
        }
        ids_processed++;
        if(cfg.verbose && !(ids_processed % 100))
            fprintf(stderr, "\r%7u %7u %7u %3u ", ids_processed,
                    ids_retrieved, ids_refused, skipped);
        fprintf(fpnew, "%s\n", line);
    }

    if(fclose(fpnew)) {
        perror(seqname);
        syslog(LOG_ERR, "%s", _strerror(seqname));
        result = -1;
    }
    if(fclose(fp)) {
        perror(pagname);
        syslog(LOG_ERR, "%s", _strerror(pagname));
        result = -1;
    }
    if(result)
        return(result);
    if(cfg.verbose)
        fprintf(stderr, "\r%7u %7u %7u %3u ", ids_processed,
                ids_retrieved, ids_refused, skipped);

    /*
     * Test 1: Check if we are able to retrieve all ids.
     */
    if((fp = xfopen(seqname, "r")) == NULL) {
        perror(seqname);
        syslog(LOG_ERR, "%s", _strerror(seqname));
        return(-1);
    }
    if((db = ndbm_open(cfg.historyfile, 0)) == NULL) {
        perror(cfg.historyfile);
        syslog(LOG_ERR, "%s", _strerror(cfg.historyfile));
        fclose(fp);
        return(-1);
    }

    while(fgets(line, MAX_HISTORYLINE, fp)) {
        cp = line;
        while(*cp && *cp != '\n')
            cp++;
        *cp = '\0';
        if(ndbm_fetch(db, line, line)) {
            fprintf(stderr, "Can't fetch '%s'\n", line);
            result = -1;
            break;
        }
        ids_retrieved++;
        if(cfg.verbose && !(ids_retrieved % 100))
            fprintf(stderr, "\r%7u %7u %7u %3u ", ids_processed,
                    ids_retrieved, ids_refused, skipped);
    }
    ndbm_close(db);
    fclose(fp);
    if(result)
        return(result);
    if(cfg.verbose)
        fprintf(stderr, "\r%7u %7u %7u %3u ", ids_processed,
                ids_retrieved, ids_refused, skipped);

    /*
     * Test 2: Check if we are able to add an existing id.
     */
    if((fp = xfopen(seqname, "r")) == NULL) {
        perror(seqname);
        syslog(LOG_ERR, "%s", _strerror(seqname));
        return(-1);
    }
    if((db = ndbm_open(cfg.historyfile, 0)) == NULL) {
        perror(cfg.historyfile);
        syslog(LOG_ERR, "%s", _strerror(cfg.historyfile));
        fclose(fp);
        return(-1);
    }

    while(fgets(line, MAX_HISTORYLINE, fp)) {
        cp = line;
        while(*cp && *cp != '\n')
            cp++;
        *cp = '\0';
        if(ndbm_store(db, line) == 0) {
            fprintf(stderr, "Can duplicate '%s'\n", line);
            result = -1;
            break;
        }
        ids_refused++;
        if(cfg.verbose && !(ids_refused % 100))
            fprintf(stderr, "\r%7u %7u %7u %3u ", ids_processed,
                    ids_retrieved, ids_refused, skipped);
    }
    ndbm_close(db);
    fclose(fp);

    xremove(seqname);

    if(cfg.verbose && !result)
        fprintf(stderr, "\r%7u %7u %7u %3u OK\n", ids_processed,
                ids_retrieved, ids_refused, skipped);

    return(result);
}

/*
 * $Log: testhist.c,v $
 * Revision 1.1  1997/02/13 11:21:53  oliver
 * initial revision
 *
 */
