/* dbm.h
 *
 * simple history database manager for UUPC news
 *
 * Author:  Kai Uwe Rommel <rommel@ars.muc.de>
 * Created: Sun Aug 15 1993
 */

/*
 * $Id$
 */

#ifndef _NDBM_H
#define _NDBM_H

#define NDBM_EXT_DBF     ".pag"
#define NDBM_EXT_IDX     ".idx"

#define IDX_MAXKEY     9   /* 80 */
#define IDX_MINITEM    30  /* 10 */
#define IDX_MAXITEM    (IDX_MINITEM * 2)
#define IDX_MAXSTACK   20

#pragma pack(1)

typedef struct {
    long offset;
    short child;
    short klen;
    char key[IDX_MAXKEY];
} ITEM;

typedef struct {
    short child_0;
    short items;
    ITEM item[IDX_MAXITEM];
} PAGE;

#pragma pack()

typedef struct {
    FILE  *dbfile;      /* pointer to textfile */
    int   fh;           /* handle of index file */
    long  bufrec;       /* current record in buffer */
    char  buffer[MAX_HISTORYLINE]; /* record buffer */
    char  *value;
    short size;         /* number of pages in index */
    PAGE  page;         /* current page */
    short page_number;  /* current page number or -1 */
    int   page_dirty;   /* set if current page is dirty */
    short page_stack[IDX_MAXSTACK];
    int   page_stacksize;
} NDBM;

extern void nidx_debug(int enable); /* 07/16/1996 harald */
extern void ndbm_debug(int enable); /* 07/15/1996 harald */

extern int nidx_init(NDBM *db, char *filename, int read_only);
extern void nidx_exit(NDBM *idx);
extern int nidx_addkey(NDBM *idx, const char *key, const long offset);
extern long nidx_getkey(NDBM *idx, const char *key);

extern NDBM *ndbm_open(const char *name, int read_only);
extern void ndbm_close(NDBM *db);
extern int ndbm_flush(NDBM * db);   /* 07/15/1996 harald */

extern int ndbm_store(NDBM *db, char *val);
extern int __cdecl ndbm_cancel(NDBM *db, const char *key, char *artlist);
extern int ndbm_fetch(NDBM *db, const char *key, char *val);

extern long ndbm_first(NDBM *db, unsigned long tkey, char *line);
extern long ndbm_next(NDBM *db, long offs, char *line);

extern int ndbm_reindex(const char *name);

/*
 * $Log: ndbm.h,v $
 * Revision 1.1  1997/02/13 11:19:57  oliver
 * initial revision
 *
 */

#endif
