/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 *  Arbitary length math routines for Changi
 */

const char arbmath_rcsid[] = "$Id: arbmath.c,v 1.1 1997/02/13 11:09:14 oliver Exp $";

#include <arbmath.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
int add(unsigned char *number,
        const unsigned range,
        const unsigned digits)
{
    unsigned subscript = digits;
    unsigned carry = range;

    while ((carry > 0) && (subscript-- > 0)) {
        unsigned digit = number[subscript] + carry;

        number[subscript] = (unsigned char)(digit % 0x100);
        carry = digit / 0x100;
    }

    return(carry == 0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int adiv(unsigned char *number,
        const unsigned divisor,
        unsigned *remain,
        const unsigned digits)
{
    unsigned subscript;
    int nonzero = 0;

    *remain = 0;

    for (subscript = 0; subscript < digits; subscript++) {
        unsigned digit = *remain * 0x100 + number[subscript];

        nonzero = nonzero || number[subscript];
        *remain = digit % divisor;
        number[subscript] = (unsigned char)(digit / divisor);
    }
    return nonzero;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int mult(unsigned char *number,
         const unsigned range,
         const unsigned digits)
{
    unsigned subscript = digits;
    unsigned carry = 0;

    while (subscript-- > 0) {
        unsigned digit = number[subscript] * range + carry;

        number[subscript] = (unsigned char)(digit % 0x100);
        carry = digit / 0x100;
    }

    return(carry == 0);
}

/*
 * $Log: arbmath.c,v $
 * Revision 1.1  1997/02/13 11:09:14  oliver
 * initial revision
 *
 */
