/*
 * This part was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 */

const char arpatime_rcsid[] = "$Id: arpatime.c,v 1.1 1997/02/13 11:09:14 oliver Exp $";

/* #define STANDALONE */

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include <fcntl.h>

#include <arpatime.h>

static struct tz_table {
    char  *name;
    time_t tdiff;
} tzone[] = {
    { "acsst",    9 * 60 + 90,        }, /* Cent. Australia */
    { "acst",     9 * 60 + 30,        }, /* Cent. Australia */
    { "adt",      -4 * 60 + 60,       }, /* Atlantic (Canada) */
    { "aesst",    10 * 60 + 60,       }, /* E. Australia */
    { "aest",     10 * 60,            }, /* E. Australia */
    { "ast",      -4 * 60,            }, /* Atlantic (Canada) */
    { "awsst",    8 * 60 + 60,        }, /* W. Australia */
    { "awst",     8 * 60,             }, /* W. Australia */
    { "bst",      60,                 }, /* Great Britain summertime */
    { "cdt",      -6 * 60 + 60,       }, /* Central */
    { "cest",     60 + 60,            }, /* Central Europe */
    { "cet",      60,                 }, /* Central Europe */
    { "cetdst",   60 + 60,            }, /* Central Europe */
    { "cst",      -6 * 60,            }, /* Central */
    { "dnt",      60,                 }, /* Denmark */
    { "dst",      60 + 60,            }, /* Denmark */
    { "edt",      -5 * 60 + 60,       }, /* Eastern US */
    { "eest",     2 * 60 + 60,        }, /* Eastern Europe */
    { "eet",      2 * 60,             }, /* Eastern Europe */
    { "eetdst",   2 * 60 + 60,        }, /* Eastern Europe */
    { "est",      -5 * 60,            }, /* Eastern US */
    { "gmt",      0,                  }, /*  */
    { "hdt",      -10 * 60 + 60,      }, /* Hawaii/Alaska */
    { "hst",      -10 * 60,           }, /* Hawaii/Alaska */
    { "ist",      2 * 60,             }, /* Israel */
    { "jst",      9 * 60,             }, /* Japan */
    { "mdt",      -7 * 60 + 60,       }, /* Mountain US */
    { "mest",     60 + 60,            }, /* Central Europe */
    { "met",      60,                 }, /* Central Europe */
    { "metdst",   60 + 60,            }, /* Central Europe */
    { "mst",      -7 * 60,            }, /* Mountain */
    { "ndt",      -3 * 60 -30 + 60,   }, /* Nfld. (Canada) */
    { "nst",      -3 * 60 -30,        }, /* Nfld. (Canada) */
    { "pdt",      -8 * 60 + 60,       }, /* Pacific */
    { "pst",      -8 * 60,            }, /* Pacific */
    { "utc",      0,                  }, /* UTC */
    { "west",     60,                 }, /* Western Europe */
    { "wet",      0,                  }, /* Western Europe */
    { "wetdst",   60,                 }, /* Western Europe */
    { "ydt",      -9 * 60 + 60,       }, /* Yukon */
    { "yst",      -9 * 60,            }  /* Yukon */
};

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int is_month(char *item)
{
    int result = 0;

    if(strlen(item) >= 3) {
        switch(toupper(*item)) {
        case 'J':
            if(strnicmp(item + 1, "an", 2) == 0)
                result = 1;
            else if(strnicmp(item + 1, "un", 2) == 0)
                result = 6;
            else if(strnicmp(item + 1, "ul", 2) == 0)
                result = 7;
            break;
        case 'F':
            if(strnicmp(item + 1, "eb", 2) == 0)
                result = 2;
            break;
        case 'M':
            if(strnicmp(item + 1, "ar", 2) == 0)
                result = 3;
            else if(strnicmp(item + 1, "ay", 2) == 0)
                result = 5;
            break;
        case 'A':
            if(strnicmp(item + 1, "pr", 2) == 0)
                result = 4;
            else if(strnicmp(item + 1, "ug", 2) == 0)
                result = 8;
            break;
        case 'S':
            if(strnicmp(item + 1, "ep", 2) == 0)
                result = 9;
            break;
        case 'O':
            if(strnicmp(item + 1, "ct", 2) == 0)
                result = 10;
            break;
        case 'N':
            if(strnicmp(item + 1, "ov", 2) == 0)
                result = 11;
            break;
        case 'D':
            if(strnicmp(item + 1, "ec", 2) == 0)
                result = 12;
            break;
        }
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int is_wday(char *item)
{
    int result = 0;

    if(strlen(item) >= 3) {
        switch(toupper(*item)) {
        case 'M':
            if(strnicmp(item + 1, "on", 2) == 0)
                result = 1;
            break;
        case 'T':
            if(strnicmp(item + 1, "ue", 2) == 0)
                result = 2;
            else if(strnicmp(item + 1, "hu", 2) == 0)
                result = 4;
            break;
        case 'W':
            if(strnicmp(item + 1, "ed", 2) == 0)
                result = 3;
            break;
        case 'F':
            if(strnicmp(item + 1, "ri", 2) == 0)
                result = 5;
            break;
        case 'S':
            if(strnicmp(item + 1, "at", 2) == 0)
                result = 6;
            else if(strnicmp(item + 1, "un", 2) == 0)
                result = 7;
            break;
        }
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int cmptable(const void *key, const void *tz)
{
    return(stricmp((char *)key, ((struct tz_table *)tz)->name));
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static time_t is_tzone(char *item)
{
    struct tz_table *tz;

    tz = bsearch(item, tzone, sizeof(tzone) / sizeof(tzone[0]),
                 sizeof(tzone[0]), cmptable);
    if(tz)
        return(tz->tdiff);
    return(0);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*  returns GMT                                                         */
/*                                                                      */
/* 06/09/1996 Added code to accept empty date                           */
/*                                                                      */
/************************************************************************/
time_t getadate(char *date)
{
    time_t result = 0;
    register char *cp;
    register int  i;
    int  items = 0;
    char *itm[6];
    int  day = 0, month = 0, year = 0;
    int  hour = 0, minute = 0, second = 0;
    time_t tdiff = 0;
    int  got_wday = 0;
    struct tm tms;
    time_t now;
    struct tm *tmnow;
    char *datebuf;

    if(date == NULL || *date == '\0')
        return(result);

    time(&now);
    tmnow = gmtime(&now);
    cp = datebuf = strdup(date);

    /*
     * Break up into items and search weekday name
     */
    while(*cp) {
        while(*cp == ' ' || *cp == '\t' || *cp == ',')
            *cp++ = '\0';
        if(*cp) {
            if(!got_wday)
                got_wday = is_wday(cp);
            if(items < 6)
                itm[items++] = cp;
            while(*cp && *cp != ' ' && *cp != '\t' && *cp != ',')
                cp++;
        }
    }

    /*
     * Check if any of our items is an alphabetic month.
     */
    for(i = 0; i < items; i++) {
        if((month = is_month(itm[i])) != 0) {
            break;
        }
    }

    if(month) {
        if(i) {
            day = atoi(itm[i - 1]);
            if(items > i + 1)
                year = atoi(itm[i + 1]);
            else if(month >= tmnow->tm_mon)
                year = tmnow->tm_year;
            else
                year = tmnow->tm_year + 1;
        }
        else {
            if(items > i + 1) {
                day = atoi(itm[i + 1]);
                if(items > i + 2)
                    year = atoi(itm[i + 2]);
                else if(month >= tmnow->tm_mon)
                    year = tmnow->tm_year;
                else
                    year = tmnow->tm_year + 1;
            }
            else
                month = 0;
        }
    }
    else {
        for(i = 0; i < items; i++) {
            if(strlen(itm[i]) >= 8) {
                if(itm[i][2] == '/' && itm[i][5] == '/')
                    break;
                if(itm[i][4] == '/' && itm[i][7] == '/')
                    break;
                if(itm[i][2] == '.' && itm[i][5] == '.')
                    break;
            }
        }
        if(i < items) {
            year = atoi(&itm[i][6]);
            if(itm[i][2] == '/') {
                month = atoi(itm[i]);
                if(month <= 12)
                    day = atoi(&itm[i][3]);
                else {
                    day = atoi(itm[i]);
                    month = atoi(&itm[i][3]);
                }
            }
            else if(itm[i][4] == '/') {
                year = atoi(itm[i]);
                month = atoi(&itm[i][5]);
                day  = atoi(&itm[i][8]);
            }
            else {
                month = atoi(&itm[i][3]);
                if(month <= 12)
                    day = atoi(itm[i]);
                else {
                    day = atoi(&itm[i][3]);
                    month = atoi(itm[i]);
                }
            }
        }
        else if(itm[0][0] >= '0' && itm[0][0] <= '9')
            sscanf(itm[0], "%02d%02d%02d%02d%02d%",
                   &year, &month, &day, &hour, &minute);
    }

    if(day && month && year) {
        for(i = 0; i < items; i++) {
            if((cp = strchr(itm[i], ':')) != NULL) {
                *cp++ = '\0';
                hour = atoi(itm[i]);
                minute = atoi(cp);
                if((cp = strchr(cp, ':')) != NULL)
                    second = atoi(cp + 1);
                if(++i < items) {
                    if(itm[i][0] == '+' || itm[i][0] == '-')
                        tdiff = (atoi(itm[i]) / 100) * 60;
                    else
                        tdiff = is_tzone(itm[i]);
                }
                break;
            }
        }
        if(year < 70)
            year += 100;
        else if(year > 1970)
            year -= 1900;

        tms.tm_sec = second;
        tms.tm_min = minute;
        tms.tm_hour = hour;
        tms.tm_mday = day;
        tms.tm_mon = month - 1;
        tms.tm_year = year;
        tms.tm_wday = 0;
        tms.tm_yday = 0;
        tms.tm_isdst = 0;
        result = mktime(&tms) - tdiff * 60;
    }
    free(datebuf);

    return(result);
}

#ifdef STANDALONE

/************************************************************************/
/*                                                                      */
/************************************************************************/
int main(int argc, char *argv[])
{
    int i;
    char line[128];
    time_t t;

    if(argc < 2)
        printf("Usage: %s <arpadate>\n", argv[0]);
    else {
        line[0] = '\0';
        for(i = 1; i < argc; i++) {
            strcat(line, argv[i]);
            strcat(line, " ");
        }
        printf("Input :\t%s\n", line);
        t = getdate(line);
        printf( "Result:\t%s", ctime(&t));
    }
    return(0);
}

#endif

/*
 * $Log: arpatime.c,v $
 * Revision 1.1  1997/02/13 11:09:14  oliver
 * initial revision
 *
 */
