/*
 * This part was written by Harald Kipp
 *
 * Bug reports to
 *
 *  harald@os2point.ping.de
 *
 */

const char artrange_rcsid[] = "$Id: artrange.c,v 1.1 1997/02/13 11:09:15 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <artrange.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
int arngset(ARTRANGE *arng, long *arts, int narts, long lo, long hi)
{
    long *lop, *hip, *artp;
    int i;
    int result;

    arng->a = 0;
    arng->i = 0;
    arng->m = (arts == NULL ? (int)(hi - lo + 1) : narts);
    arng->hwm = hi;

    if(narts == 0) {
        arng->n = 0;
        return(lo <= hi || lo == 0);
    }

    if(arng->x == 0) {
        arng->lo = malloc((arng->x = 16) * sizeof(long));
        arng->hi = malloc(arng->x * sizeof(long));
    }

    if(arts == NULL) {
        arng->n = 1;
        if((result = (lo == 0)) != 0)
            lo = 1;
        arng->lo[0] = lo;
        arng->hi[0] = hi;
        return(result);
    }

    result = (lo != arts[0]);

    lop = arng->lo;
    hip = arng->hi;
    *lop = *hip = *(artp = arts);
    artp++;
    for(i = narts; --i > 0; artp++) {
        if(*artp == ++(*hip) || *artp <= --(*hip))
            continue;
        if(++(arng->n) >= arng->x) {
            arng->x += 16;
            arng->lo = realloc(arng->lo, arng->x * sizeof(long));
            arng->hi = realloc(arng->hi, arng->x * sizeof(long));
            lop = arng->lo + arng->n;
            hip = arng->hi + arng->n;
        }
        else {
            lop++;
            hip++;
        }
        *lop = *hip = *artp;
    }
    if(arng->hwm < arng->hi[arng->n]) {
        arng->hwm = arng->hi[arng->n];
        result = 1;
    }
    arng->n++;

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void arngfree(ARTRANGE *arng)
{
    arng->x = 0;
    free(arng->lo);
    free(arng->hi);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void arngcpy(ARTRANGE *dst, ARTRANGE *src)
{
    if(dst->x < (dst->n = src->n)) {
        dst->x = src->n;
        dst->lo = realloc(dst->lo, dst->x * sizeof(long));
        dst->hi = realloc(dst->hi, dst->x * sizeof(long));
    }
    memcpy(dst->lo, src->lo, src->n * sizeof(long));
    memcpy(dst->hi, src->hi, src->n * sizeof(long));
    dst->a = 0;
    dst->i = 0;
    dst->m = src->m;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long arngfind(ARTRANGE *arng, long artnum)
{
    int i;

    arng->a = 0;
    arng->i = 0;
    for(i = 0; i < arng->n; i++) {
        if(artnum < arng->lo[i]) {
            arng->a = arng->lo[i];
            arng->i = i;
            break;
        }
        if(artnum < arng->hi[i]) {
            arng->a = artnum;
            arng->i = i;
            break;
        }
    }
    return(arng->a);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long arngnext(ARTRANGE *arng)
{
    if(arng->i < 0)
        arng->i = 0;
    if(arng->i >= arng->n) {
        return(0);
    }
    if(arng->a >= arng->hi[arng->i]) {
        arng->i++;
        return(arngnext(arng));
    }
    if(++(arng->a) < arng->lo[arng->i])
        arng->a = arng->lo[arng->i];

    return(arng->a);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long arngprev(ARTRANGE *arng)
{
    if(arng->i >= arng->n)
        arng->i = arng->n - 1;
    if(arng->i < 0)
        return(0);
    if(arng->a <= arng->lo[arng->i]) {
        arng->i--;
        return(arngprev(arng));
    }
    if(--(arng->a) > arng->hi[arng->i])
        arng->a = arng->hi[arng->i];

    return(arng->a);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int arnginfo(ARTRANGE *arng, long *lo, long *hi, long *hwm)
{
    if(arng->n == 0) {
        if(hi)
            *hi = arng->hwm;
        if(lo)
            *lo = arng->hwm + 1;
    }
    else {
        if(lo)
            *lo = arng->lo[0];
        if(hi)
            *hi = arng->hi[arng->n - 1];
    }
    return(arng->m);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long arnginc(ARTRANGE *arng)
{
    arng->m++;
    if(arng->n == 0) {
        if(arng->x == 0) {
            arng->lo = malloc((arng->x = 16) * sizeof(long));
            arng->hi = malloc(arng->x * sizeof(long));
        }
        return(arng->lo[0] = arng->hi[0] = ++(arng->hwm));
    }
    if(arng->hi[arng->n - 1] < arng->hwm) {
        if(++(arng->n) >= arng->x) {
            arng->x += 16;
            arng->lo = realloc(arng->lo, arng->x * sizeof(long));
            arng->hi = realloc(arng->hi, arng->x * sizeof(long));
        }
        return(arng->lo[arng->n - 1] = arng->hi[arng->n - 1] = ++(arng->hwm));
    }
    return(arng->hi[arng->n - 1] = ++(arng->hwm));
}


#if 0

long arts[] = {
    1,2,3,5,6,7,
    11,12,13,15,16,17,20,
    21,22,23,25,26,27,
    31,32,33,35,36,37,
};

int narts = sizeof(arts) / sizeof(arts[0]);

/************************************************************************/
/*                                                                      */
/************************************************************************/
int main(void)
{
    ARTRANGE arng;
    ARTRANGE arng2;
    int i, n;
    long artnum, lo, hi;

    memset(&arng, 0, sizeof(arng));
    memset(&arng2, 0, sizeof(arng2));
    arngset(&arng, arts, narts, 37);
    for(i = 0; i < arng.n; i++)
        printf("%ld-%ld\n", arng.lo[i], arng.hi[i]);

    arngcpy(&arng2, &arng);
    for(i = 0; i < arng2.n; i++)
        printf("%ld-%ld\n", arng2.lo[i], arng2.hi[i]);

    while((artnum = arngnext(&arng2)) != 0)
        printf("%ld,", artnum);
    printf("\n");
    while((artnum = arngprev(&arng2)) != 0)
        printf("%ld,", artnum);
    printf("\n");

    i = arnginfo(&arng, &lo, &hi);
    printf("%d: %ld-%ld\n", i, lo, hi);

    artnum = arngfind(&arng, 24);
    lo = arngfind(&arng, 0);
    hi = arngfind(&arng, 99);
    printf("24->%ld 0->%ld 99->%ld\n", artnum, lo, hi);

    artnum = arngfind(&arng, 24);
    while((artnum = arngnext(&arng)) != 0)
        printf("%ld,", artnum);
    printf("\n");

    for(i = 0; i < 20; i++) {
        artnum = arnginc(&arng);
        n = arnginfo(&arng, &lo, &hi);
        printf("%d: %ld-%ld\n", n, lo, hi);
    }

    return(0);
}

#endif

/*
 * $Log: artrange.c,v $
 * Revision 1.1  1997/02/13 11:09:15  oliver
 * initial revision
 *
 */
