/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char domnpath_rcsid[] = "$Id: domnpath.c,v 1.1 1997/02/13 11:09:17 oliver Exp $";

#include <stdlib.h>
#include <string.h>

#include <domnpath.h>

/************************************************************************/
/*                                                                      */
/* This routine builds a filename from a given hostname. If the host-   */
/* name contains a domain, then the resulting filename will contain     */
/* a corresponding path.                                                */
/*                                                                      */
/* Parameters:                                                          */
/*                                                                      */
/*  path    storage for wanted filename                                 */
/*  remote  name of remote host                                         */
/*  ext     filename extension                                          */
/*                                                                      */
/* Example:                                                             */
/*                                                                      */
/*  If                                                                  */
/*                                                                      */
/*  path   = "c:\changi"                                                */
/*  remote = "nntp.germany.eu.net"                                      */
/*  ext    = ".last"                                                    */
/*                                                                      */
/*  then                                                                */
/*                                                                      */
/*  path = "c:\changi\germany.eu.net\nntp.last"                         */
/*                                                                      */
/************************************************************************/
char *domain_path(char *path, const char *remote, const char *ext)
{
    register char *csp;
    register char *cdp;
    char *host = strdup(remote);

    cdp = path + strlen(path);
    *cdp++ = '\\';
    if((csp = strchr(host, '.')) != NULL) {
        *csp++ = '\0';
        while(*csp)
            *cdp++ = *csp++;
        *cdp++ = '\\';
    }
    csp = host;
    while(*csp)
        *cdp++ = *csp++;
    if(ext)
        strcpy(cdp, ext);
    else
        *cdp = '\0';

    free(host);
    return(path);
}

/*
 * $Log: domnpath.c,v $
 * Revision 1.1  1997/02/13 11:09:17  oliver
 * initial revision
 *
 */
