/*
 * This part was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 */

const char finedir_rcsid[] = "$Id: finedir.c,v 1.1 1997/02/13 11:09:17 oliver Exp $";

#include <stdlib.h>
#include <string.h>

#include <finedir.h>


/************************************************************************/
/*                                                                      */
/* fine_dir                                                             */
/*                                                                      */
/* Setup a fine directory path. If 'path' doesn't contain directory     */
/* information then the contents of 'cwd' will be used to build up      */
/* a full pathname to the file or directory given by 'path'.            */
/*                                                                      */
/* Any slash will be converted to a backslash, as not all OS/2 shells   */
/* will accept slashes.                                                 */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* path         Points to the string that specifies the pathname of a   */
/*              file or directory. If this pointer is NULL or if it     */
/*              points to an empty string, NULL is returned. If not     */
/*              NULL this must point to at least 255 bytes of storage.  */
/*                                                                      */
/* cwd          Points to the string that specifies the name of a       */
/*              directory to be used if 'path' doesn't specify one.     */
/*              This pointer may be NULL in which case no directory     */
/*              is added to 'path'.                                     */
/* ------------ ------------------------------------------------------- */
/* Return value Pointer to 'path' or NULL, if 'path' points to NULL     */
/*              or an empty string.                                     */
/*                                                                      */
/************************************************************************/
char *fine_dir(char *path, char *cwd)
{
    if(path && *path) {
        int  i = (path[1] == ':') ? 2 : 0;
        char *cp = path + i;

        /*
         * All slashes are converted to backslashes
         */
        while(*cp) {
            if(*cp == '/')
                *cp = '\\';
            cp++;
        }

        /*
         * Remove a trailing backslash but restore it
         * if it has been the leading one.
         */
        if(*--cp == '\\') {
            *cp = '\0';
            if(path[i] == '\0')
                strcat(path, "\\");
        }

        /*
         * If 'path' does not but 'cwd' does contain
         * a directory path, then use the one in 'cwd'
         * to create a full path.
         */
        if(cwd && *cwd && strchr(path, '\\') == NULL) {
            cp = strdup(&path[i]);
            if(cwd[1] == ':')
                i = 0;
            path[i] = '\0';
            strcat(path, cwd);
            if(cwd[strlen(cwd) - 1] != '\\')
                strcat(path, "\\");
            strcat(path, cp);
            free(cp);
        }
        return(path);
    }
    return(NULL);
}

/*
 * $Log: finedir.c,v $
 * Revision 1.1  1997/02/13 11:09:17  oliver
 * initial revision
 *
 */
