/*
 * This OS/2 port was hacked by Harald Kipp
 *
 * Bug reports related to THIS modified version should be sent to
 *
 *  harald@os2point.ping.de
 *
 * This module contains routines that deal with a hash table.
 */

const char hash_rcsid[] = "$Id: hash.c,v 1.1 1997/02/13 11:09:19 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#include <hash.h>

#define HASHRANGE   512

typedef struct {
    int nItems;         /* Size of item array       */
    int cItems;         /* Number of items in array */
    HASHITEM **item;    /* Array of item pointers   */
} HASHENTRY;

/************************************************************************/
/* This routine calculates a hash value and uses it as an index into    */
/* an array of HASHENTRY structures.                                    */
/*                                                                      */
/* Retruns a pointer to the selected HASHENTRY.                         */
/*                                                                      */
/************************************************************************/
static HASHENTRY *HashIndex(HASHENTRY *htp, const char *key)
{
    register unsigned int i = 0;

    while(*key)
        i += (i << 5) + *key++;
    return(htp + (i % HASHRANGE));
}

/************************************************************************/
/*                                                                      */
/*  Performs a binary search to retrieve the index of a specific        */
/*  newsgroup in the array of newsgroups with equal hash code.          */
/*                                                                      */
/*  Returns 1 on success or 0 if the given group couldn't be found.     */
/*                                                                      */
/************************************************************************/
static int HashSearch(const char *key, HASHITEM **hipp, int num, int *p)
{
    int c;
    int m = 0;
    int l = 0;
    int h = num - 1;

    while (l <= h) {
        m = (l + h) / 2;
        if ((c = strcmp(key, hipp[m]->key)) < 0)
            h = m - 1;
        else if (c > 0)
            l = m + 1;
        else {
            *p = m;
            return(1);
        }
    }
    *p = l;
    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void *HashInit(void *hash, int items)
{
    HASHENTRY *htp = hash;
    int i;

    if (hash == NULL) {
        htp = malloc(sizeof(HASHENTRY) * HASHRANGE);
        memset(htp, 0, sizeof(HASHENTRY) * HASHRANGE);
    }
    if (htp[0].nItems) {
        for (i = 0; i < HASHRANGE; i++)
            htp[i].cItems = 0;
    }
    else {
        int nItems = items / HASHRANGE;

        if(!nItems)
            nItems = 1;
        for (i = 0; i < HASHRANGE; i++) {
            htp[i].nItems = nItems;
            htp[i].item = malloc(sizeof(HASHITEM *) * (size_t)nItems);
            htp[i].cItems = 0;
        }
    }
    return(htp);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int HashAdd(void *hash, void *item)
{
    char *key = ((HASHITEM *)item)->key;
    HASHENTRY *htp = HashIndex(hash, key);
    register HASHITEM **hipp = htp->item;
    register int j;
    int i;

    if(HashSearch(key, hipp, htp->cItems, &i))
        return(0);

    if (htp->cItems >= htp->nItems) {
        htp->nItems++;
        htp->item = hipp = realloc(hipp, sizeof(HASHITEM *) * (size_t)htp->nItems);
    }
    for(j = htp->cItems++; j > i; j--)
        hipp[j] = hipp[j - 1];
    hipp[i] = item;

    return(1);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void *HashGet(void *hash, const char *key)
{
    HASHENTRY *htp = HashIndex(hash, key);
    int i;

    if(HashSearch(key, htp->item, htp->cItems, &i)) {
        return(htp->item[i]);
    }

    return(NULL);
}

/*
 * $Log: hash.c,v $
 * Revision 1.1  1997/02/13 11:09:19  oliver
 * initial revision
 *
 */
