
const char loadfile_rcsid[] = "$Id: loadfile.c,v 1.1 1997/02/13 11:09:20 oliver Exp $";

#define  INCL_DOSERRORS
#define  INCL_DOSPROCESS
#include <os2.h>

#include <stdlib.h>
#include <stdio.h>

#include <syslog.h>

#include <loadfile.h>

#define ACF_OPNFLG  OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS
#define ACF_OPNMOD  OPEN_FLAGS_SEQUENTIAL | OPEN_SHARE_DENYWRITE |\
                    OPEN_ACCESS_READONLY | OPEN_FLAGS_NOINHERIT

/************************************************************************/
/*                                                                      */
/*  Opens a file in shared mode using a low level system call.          */
/*  If the file could not be opened immediately because of access       */
/*  violations, this functions will retry to open the file after        */
/*  sleeping one second. If the file couldn't be opened after ten       */
/*  retries then this function returns an error.                        */
/*                                                                      */
/*  Returns 0 on success or a DOS errorcode on failures.                */
/*                                                                      */
/************************************************************************/
static APIRET open_file(const char *name, HFILE *phf)
{
    APIRET rc;
    ULONG act;
    int retries = 10;

    while(retries--) {
	rc = DosOpen((char *)name, phf, &act, 0, 0, ACF_OPNFLG, ACF_OPNMOD, NULL);
        if(rc != ERROR_SHARING_VIOLATION)
            break;
        DosSleep(1000);
    }
    return(rc);
}

/************************************************************************/
/*                                                                      */
/*  load_file                                                           */
/*                                                                      */
/*  This procedure reads the complete contents of a file into an        */
/*  allocated memory block. The caller is responsible to free this      */
/*  block if it's no longer needed.                                     */
/*                                                                      */
/*  Returns a pointer to the memory block containing the file contents  */
/*  as a zero terminated string or NULL on errors.                      */
/*                                                                      */
/************************************************************************/
char *load_file(const char *name)
{
    char *result;
    HFILE  hf;
    FILESTATUS3 fsts;
    ULONG cbRead;
    APIRET rc;

    /*
     * Try to open an existing file. Do not report an error to
     * syslog if the file doesn't exist, because the caller
     * may accept this as a normal condition.
     */
    if((rc = open_file(name, &hf)) != 0) {
        syslog(LOG_ERR, "Error %u opening %s", rc, name);
        return(NULL);
    }

    /*
     * Read the file info structure in order to get the
     * file's size.
     */
    if((rc = DosQueryFileInfo(hf, FIL_STANDARD, &fsts, sizeof(fsts))) != 0) {
        syslog(LOG_ERR, "Error %u querying file info of %s", rc, name);
        DosClose(hf);
        return(NULL);
    }

    /*
     * Read the whole file into a single buffer
     */
    result = malloc(fsts.cbFile + 2);
    if((rc = DosRead(hf, result, fsts.cbFile, &cbRead)) != 0) {
        syslog(LOG_ERR, "Error %u reading %ul bytes from %s",
               rc, fsts.cbFile, name);
        DosClose(hf);
        free(result);
        return(NULL);
    }
    DosClose(hf);

    /*
     * Add an extra linefeed at the end to ensure
     * proper termination of the last line.
     */
    *(result + fsts.cbFile) = '\n';
    *(result + fsts.cbFile + 1) = '\0';
    return(result);
}

/*
 * $Log: loadfile.c,v $
 * Revision 1.1  1997/02/13 11:09:20  oliver
 * initial revision
 *
 */
