/*
 * This part was written by Harald Kipp
 *
 * Bug reports should be sent to
 *
 *  harald@os2point.ping.de
 *
 */

const char lpstream_rcsid[] = "$Id: lpstream.c,v 1.1 1997/02/13 11:09:21 oliver Exp $";

#define USE_SYSLOG

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include <chlimits.h>
#include <sostream.h>
#include <lpstream.h>

#ifdef USE_SYSLOG
#include <syslog.h>
#else
#include <lprintf.h>
#endif

static int log_stream = 0;


/************************************************************************/
/*                                                                      */
/************************************************************************/
void LpStreamSetLog(int enable)
{
    log_stream = enable;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int LpStreamBufGets(char *line, size_t n, void *rbv)
{
    int rc = SoStreamBufGets(line, n, rbv);

    if(log_stream && rc > 0)
#ifdef USE_SYSLOG
        syslog(LOG_DEBUG, "%03u> '%s'", *((int *)rbv), line);
#else
        lprintf("%03u> '%s'", *((int *)rbv), line);
#endif
    return(rc);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int LpStreamPuts(int s, const char *line)
{
    if(log_stream) {
        char *buf = strdup(line);
        const char *cps = line;
        while(*cps) {
            char *cpd = buf;
            while(*cps && *cps != '\r' && *cps != '\n')
                *cpd++ = *cps++;
            *cpd = '\0';
#ifdef USE_SYSLOG
            syslog(LOG_DEBUG, "%03u< '%s'", s, buf);
#else
            lprintf("%03u< '%s'", s, buf);
#endif
            while(*cps == '\r' || *cps == '\n')
                cps++;
        }
        free(buf);
    }

    return(SoStreamPuts(s, line));
}


/************************************************************************/
/*                                                                      */
/************************************************************************/
int _System LpStreamPrintf(int s, const char *format,...)
{
    int rc;
    va_list marker;
    char *achbuf = malloc(MAX_BUFSIZ);

    va_start(marker, format);
    vsprintf(achbuf, format, marker);
    rc = LpStreamPuts(s, achbuf);
    va_end(marker);

    free(achbuf);
    return (rc);
}


/************************************************************************/
/*                                                                      */
/************************************************************************/
int LpStreamTransact(void *rbv, const char *question, char *answer)
{
    int result = 0;
    int sock = *((int *)rbv);

    if(LpStreamPuts(sock, question) < 0) {
        result = -1;
    }
    else if(SoStreamPuts(sock, "\r\n") < 0) {
        result = -1;
    }
    else if(LpStreamBufGets(answer, MAX_BUFSIZ, rbv) <= 0) {
        result = -1;
    }

    return(result);
}

/*
 * $Log: lpstream.c,v $
 * Revision 1.1  1997/02/13 11:09:21  oliver
 * initial revision
 *
 */
