/*
 * This part was written by Harald Kipp
 *
 * Bug reports to
 *
 *  harald@os2point.ping.de
 *
 */

const char lrange_rcsid[] = "$Id: lrange.c,v 1.1 1997/02/13 11:09:22 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <lrange.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
void lrange_new(LRANGE *lrp)
{
    lrp->n = 0;
    lrp->lo = malloc((lrp->x = 12) * sizeof(long));
    lrp->hi = malloc(lrp->x * sizeof(long));
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void lrange_free(LRANGE *lrp)
{
    if(lrp->x) {
        lrp->x = 0;
        free(lrp->lo);
        free(lrp->hi);
    }
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void lrange_set(LRANGE *lrp, long lo, long hi)
{
    if(lo > hi) {
        lrp->n = 0;
        return;
    }

    if(lrp->x == 0)
        lrange_new(lrp);

    lrp->lo[0] = lo;
    lrp->hi[0] = hi;
    lrp->n = 1;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void lrange_build(LRANGE *lrp, long *values, int num)
{
    long *vp = values;
    long *lop;
    long *hip;
    int i;

    lrp->n = 0;
    if(num == 0)
        return;

    if(lrp->x == 0)
        lrange_new(lrp);
    lop = lrp->lo;
    hip = lrp->hi;

    *lop = *hip = *vp++;
    for(i = num; --i > 0; vp++) {
        if(*vp == ++(*hip) || *vp <= --(*hip))
            continue;
        if(++(lrp->n) >= lrp->x) {
            lrp->lo = realloc(lrp->lo, (lrp->x += 16) * sizeof(long));
            lrp->hi = realloc(lrp->hi, lrp->x * sizeof(long));
            lop = lrp->lo + lrp->n;
            hip = lrp->hi + lrp->n;
        }
        else {
            lop++;
            hip++;
        }
        *lop = *hip = *vp;
    }
    lrp->n++;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long lrange_next(LRANGE *lrp, long value)
{
    int i = lrp->n / 2;

    if(lrp->n == 0 || value > lrp->hi[lrp->n - 1])
        return(0);
    if(value <= lrp->lo[0])
        return(lrp->lo[0]);

    for(;;) {
        if(value < lrp->lo[i]) {
            if(value <= lrp->hi[--i])
                continue;
            value = lrp->lo[++i];
        }
        else if(value > lrp->hi[i]) {
            if(value > lrp->lo[++i])
                continue;
            value = lrp->lo[i];
        }
        break;
    }
    return(value);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long lrange_prev(LRANGE *lrp, long value)
{
    int i = lrp->n / 2;

    if(lrp->n == 0)
        return(0);
    if(value == -1 || value >= lrp->hi[lrp->n - 1])
        return(lrp->hi[lrp->n - 1]);
    if(value < lrp->lo[0])
        return(0);

    for(;;) {
        if(value < lrp->lo[i]) {
            if(value < lrp->hi[--i])
                continue;
            value = lrp->hi[i];
        }
        else if(value > lrp->hi[i]) {
            if(value >= lrp->lo[++i])
                continue;
            value = lrp->hi[--i];
        }
        break;
    }
    return(value);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void lrange_info(LRANGE *lrp, int *num, long *lo, long *hi)
{
    int i;

    if(lrp->n) {
        if(lo)
            *lo = lrp->lo[0];
        if(hi)
            *hi = lrp->hi[lrp->n - 1];
    }
    else {
        if(lo)
            *lo = 1;
        if(hi)
            *hi = 0;
    }
    if(num) {
        *num = 0;
        for(i = lrp->n; --i >= 0;)
            *num += lrp->hi[i] - lrp->lo[i] + 1;
    }
}

#if 0

long arts[] = {
    1,7,8,
    11,12,13,15,16,17,20
};

int narts = sizeof(arts) / sizeof(arts[0]);

/************************************************************************/
/*                                                                      */
/************************************************************************/
int main(void)
{
    LRANGE lr;
    long artnum;
    int i;

    memset(&lr, 0, sizeof(lr));
    lrange_set(&lr, 2, 5);
    for(i = 0; i < lr.n; i++)
        printf("%ld-%ld\n", lr.lo[i], lr.hi[i]);

    for(artnum = 1; (artnum = lrange_next(&lr, artnum)) != 0; artnum++)
        printf("Next %ld\n", artnum);

    for(artnum = -1; (artnum = lrange_prev(&lr, artnum)) != 0; artnum--)
        printf("Prev %ld\n", artnum);

    lrange_build(&lr, arts, narts);
    for(i = 0; i < lr.n; i++)
        printf("%ld-%ld\n", lr.lo[i], lr.hi[i]);

    for(artnum = 1; (artnum = lrange_next(&lr, artnum)) != 0; artnum++)
        printf("Next %ld\n", artnum);

    for(artnum = -1; (artnum = lrange_prev(&lr, artnum)) != 0; artnum--)
        printf("Prev %ld\n", artnum);

    lrange_free(&lr);
    return(0);
}

#endif

/*
 * $Log: lrange.c,v $
 * Revision 1.1  1997/02/13 11:09:22  oliver
 * initial revision
 *
 */
