/************************************************************************/
/*                                                                      */
/*  Create all subdirectories of a given pathname                       */
/*                                                                      */
/*  Bug reports should be sent to                                       */
/*                                                                      */
/*  harald@os2point.ping.de                                             */
/*                                                                      */
/************************************************************************/

const char mkpath_rcsid[] = "$Id: mkpath.c,v 1.1 1997/02/13 11:09:23 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <errno.h>

/* #include <lprintf.h> */
#include <chanlib.h>

/************************************************************************/
/*  makepath                                                            */
/*                                                                      */
/*  Creates a subdirectory and all its intermediate directories too.    */
/*  The 'pathname' parameter must contain backslashes, not slashes,     */
/*  including a trailing backslash.                                     */
/*                                                                      */
/*  An appended filename is ignored.                                    */
/*                                                                      */
/************************************************************************/
int makepath(char *pathname)
{
    int r;
    char *cp = strrchr(pathname, '\\');

    /*
     * Optimize common case, in which parent directory exists
     */
    if(cp)
        *cp = '\0';
    r = mkdir(pathname);
    if(cp)
        *cp = '\\';
    if(!r)
        return(0);

    /*
     * Set cp on first directory following a drive name
     */
    cp = pathname;
    if(pathname[1] == ':' || (pathname[0] == '\\' && pathname[1] == '\\'))
        cp += 3;

    /*
     * Now loop to create all subdirs
     */
    r = 0;
    while(!r && (cp = strchr(cp, '\\')) != NULL) {
        *cp = '\0';
        if(mkdir(pathname) && errno != EACCES) {
            r = -1;
            perror(pathname);
        }
        *cp++ = '\\';
    }
    return(r);
}


/************************************************************************/
/* grouppath                                                            */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* path         Points to an array of at least _MAX_PATH bytes in size. */
/************************************************************************/
char *grouppath(char *path, char *newsdir, char *group)
{
    char *ngpath = NULL;
    register char *cp;

    if((strlen(newsdir) + strlen(group) + 12) >= _MAX_PATH)
        fprintf(stderr, "File path too long for %s", group);
    else {
        strcpy(path, newsdir);
        strcat(path, "\\");
        strcat(path, group);
        cp = strrchr(path, '\\');
        while((cp = strchr(cp, '.')) != NULL)
            *cp = '\\';
        ngpath = path;
    }
    return(ngpath);
}

/*
 * $Log: mkpath.c,v $
 * Revision 1.1  1997/02/13 11:09:23  oliver
 * initial revision
 *
 */
