/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char nglist_rcsid[] =
"$Id: nglist.c,v 1.2 1997/02/13 17:03:55 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <string.h>
#include <direct.h>
#include <time.h>

#include <xfile.h>
#include <loadtext.h>
#include <domnpath.h>
#include <nglist.h>

/************************************************************************/
/*                                                                      */
/* cmplines                                                             */
/*                                                                      */
/* This local routine is called by the qsort library routine.           */
/*                                                                      */
/************************************************************************/
static int cmplines(const void *ptr1, const void *ptr2)
{
    return(strcmp(*((char **)ptr1), *((char **)ptr2)));
}

/************************************************************************/
/*                                                                      */
/* test_nglist                                                          */
/*                                                                      */
/* Returns TRUE, if a host.newsgroups file exists, FALSE if not.        */
/*                                                                      */
/************************************************************************/
int test_nglist(const char *hostname, int newgroups)
{
    int result = 1;
    char *batchfile = malloc(_MAX_PATH);

    /*
     * For now we stick with the current working directory.
     */
    getcwd(batchfile, _MAX_PATH);
    if(newgroups)
        domain_path(batchfile, hostname, ".newgroups");
    else
        domain_path(batchfile, hostname, ".newsgroups");
    if(access(batchfile, 0))
        result = 0;

    free(batchfile);

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
NGLIST *load_nglist(const char *hostname, int newgroups)
{
    char **lines, **lpp, *lp, *cp;
    int  n;
    NGLIST *nglist = NULL, *nglp;
    char *batchfile = malloc(_MAX_PATH);

    /*
     * For now we stick with the current working directory.
     */
    getcwd(batchfile, _MAX_PATH);
    if(newgroups)
        domain_path(batchfile, hostname, ".newgroups");
    else
        domain_path(batchfile, hostname, ".newsgroups");

    /*
     * Load the newsgroups file using our beloved text loader.
     */
    if((n = load_textfile(batchfile, &lines)) > 0) {
        nglp = nglist = malloc((n + 2) * sizeof(NGLIST));
        nglp->group = *lines;
        nglp++;
        qsort(lines, n, sizeof(char *), cmplines);
        for(lpp = lines; (lp = *lpp++) != NULL; ) {
            nglp->group = lp;
            if(newgroups || (cp = strchr(lp, ' ')) == NULL)
                nglp->description = NULL;
            else {
                *cp++ = '\0';
                nglp->description = cp;
            }
            nglp++;
        }
        nglp->group = NULL;
        nglp->description = NULL;
        free(lines);
    }

    return(nglist);
}

/************************************************************************/
/*                                                                      */
/* free_nglist                                                          */
/*                                                                      */
/* Frees all heap memory occupied by a newsgroups list created by       */
/* 'load_nglist'.                                                       */
/*                                                                      */
/************************************************************************/
int free_nglist(NGLIST *ngl)
{
    free(ngl->group);
    free(ngl);

    return(0);
}

/************************************************************************/
/*                                                                      */
/* This main program is for standalone testing.                         */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#if 0
int main(void)
{
    NGLIST *ngl;

    if((ngl = load_nglist("neutron.ping.de", 1)) != NULL) {
        while(ngl->group) {
            printf("'%s'\n", ngl->group);
            ngl++;
        }
    }

    return(0);
}
#endif

/*
 * $Log: nglist.c,v $
 * Revision 1.2  1997/02/13 17:03:55  oliver
 * main removed
 *
 * Revision 1.1  1997/02/13 11:09:25  oliver
 * initial revision
 *
 */
