const char ntdep_rcsid[] = "$Id: ntdep.c,v 1.1 1997/02/13 11:09:26 oliver Exp $";

#include <windows.h>
#include <stdio.h>

#include <syslog.h>
#include <sysdep.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
void ms_sleep(long ms)
{
    Sleep((DWORD)ms);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void set_priority(int mode)
{
    switch(mode) {
    case -1:
        SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_LOWEST);
        break;
    case 1:
        SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);
        break;
    default:
        SetThreadPriority( GetCurrentThread(), THREAD_PRIORITY_NORMAL);
        break;
    }
}

/************************************************************************/
/*                                                                      */
/*  Opens a file in shared mode using a low level system call.          */
/*  If the file could not be opened immediately because of access       */
/*  violations, this functions will retry to open the file after        */
/*  sleeping one second. If the file couldn't be opened after ten       */
/*  retries then this function returns an error.                        */
/*                                                                      */
/*  Returns 0 on success or a DOS errorcode on failures.                */
/*                                                                      */
/************************************************************************/
static ULONG open_file(char *name, HANDLE *phf)
{
    ULONG rc = 0;
    int retries = 10;

    while(retries--) {
       *phf = CreateFile(name,
                         GENERIC_READ,
                         FILE_SHARE_READ,
                         NULL,
                         OPEN_EXISTING,
                         FILE_ATTRIBUTE_READONLY | FILE_FLAG_SEQUENTIAL_SCAN,
                         NULL);
	if ( (*phf == INVALID_HANDLE_VALUE) &&
             ((rc = GetLastError()) == ERROR_SHARING_VIOLATION) ) {
	    ms_sleep(1000);
	    continue;
	}
	break;
    }
    return(rc);
}

/************************************************************************/
/*                                                                      */
/*  load_file                                                           */
/*                                                                      */
/*  This procedure reads the complete contents of a file into an        */
/*  allocated memory block. The caller is responsible to free this      */
/*  block if it's no longer needed.                                     */
/*                                                                      */
/*  Returns a pointer to the memory block containing the file contents  */
/*  as a zero terminated string or NULL on errors.                      */
/*                                                                      */
/************************************************************************/
char *load_file(char *name)
{
    char *result;
    HANDLE  hf;
    ULONG cbRead;
    ULONG filesize;
    ULONG rc;

    /*
     * Try to open an existing file.
     */
    if((rc = open_file(name, &hf)) != 0) {
        syslog(LOG_ERR, "Error %u opening %s", rc, name);
        return(NULL);
    }

    /*
     * Read the whole file into a single buffer
     */
    filesize = GetFileSize(hf,NULL);
    result = malloc(filesize + 2);

//    if((rc = DosRead(hf, result, fsts.cbFile, &cbRead)) != 0) {
    if ( !ReadFile(hf, result, filesize, &cbRead, NULL)) {
        syslog(LOG_ERR, "Error %u reading %ul bytes from %s",
               GetLastError(), filesize, name);
        CloseHandle(hf);
        free(result);
        return(NULL);
    }
    CloseHandle(hf);

    /*
     * Add an extra linefeed at the end to ensure
     * proper termination of the last line.
     */
    *(result + filesize) = '\n';
    *(result + filesize + 1) = '\0';
    return(result);
}


#if 0

/************************************************************************/
/*                                                                      */
/*  Synchronizes the lowest and highest message number of a single      */
/*  group with what was really found in the groups subdirectory.        */
/*                                                                      */
/************************************************************************/
int sync_group(const char *group, const char *newsdir, long *lomsg, long *himsg)
{
    char *tmplt = malloc(_MAX_PATH);
    char *cp;
    unsigned long cffb = 255;
    HDIR hdir = (HDIR)HDIR_CREATE;
    PFILEFINDBUF3 paffb = malloc(12288);
    PFILEFINDBUF3 pffb;
    APIRET rc;
    int i;
    long artnum;
    int result = 0;

    strcpy(tmplt, newsdir);
    cp = tmplt + strlen(tmplt);
    *cp++ = '\\';
    strcpy(cp, group);
    while(*cp) {
        if(*cp == '.')
            *cp = '\\';
        cp++;
    }
    strcpy(cp, "\\*");
    if((rc = DosFindFirst(tmplt, &hdir, 0, paffb, 12288, &cffb, 1)) == 0) {
        while (!rc && cffb) {
            pffb = paffb;
            for(i = 0; i < cffb; i++) {
                if((artnum = atol(pffb->achName)) != 0) {
                    if(artnum > *himsg) {
                        *himsg = artnum;
                        result = 1;
                    }
                    if(artnum < *lomsg) {
                        *lomsg = artnum;
                        result = 1;
                    }
                }
                pffb = (PFILEFINDBUF3)(((char *)pffb) + pffb->oNextEntryOffset);
            }
            cffb = 255;
            rc = DosFindNext(hdir, paffb, 12288, &cffb);
        }
        DosFindClose(hdir);
    }
    free(paffb);
    free(tmplt);

    return(result);
}

#endif

/************************************************************************/
/*                                                                      */
/************************************************************************/
int file_seq(char *name)
{
    int result = 1;
    char *cp;
    HANDLE hdir;
    WIN32_FIND_DATA ffb;
    BOOL foundnext = TRUE;
    char *tmplt = strcat(strcpy(malloc(_MAX_PATH), name), ".*");

    if( (hdir = FindFirstFile(tmplt, &ffb)) != INVALID_HANDLE_VALUE ) {
        while (foundnext) {
            if((cp = strrchr(ffb.cFileName, '.')) != NULL)
                if(atoi(cp + 1) >= result)
                    result = atoi(cp + 1);
        foundnext = FindNextFile(hdir, &ffb);
        }
    FindClose(hdir);
    }

    free(tmplt);
    return(result);
}

/*
 * $Log: ntdep.c,v $
 * Revision 1.1  1997/02/13 11:09:26  oliver
 * initial revision
 *
 */
