/*
 * This part was written by Harald Kipp
 *
 * Bug reports to
 *
 *  harald@os2point.ping.de
 *
 */

const char ntscandir_rcsid[] = "$Id: ntscandir.c,v 1.1 1997/02/13 11:09:28 oliver Exp $";

#include <windows.h>

#include <stdlib.h>
#include <string.h>

#include <chanlib.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int longcmp(const void *x, const void *y)
{
    if(*(long *)x > *(long *)y)
        return(1);
    else if(*(long *)x < *(long *)y)
        return(-1);
    return(0);
}

/************************************************************************/
/*                                                                      */
/* scan_dir                                                             */
/*                                                                      */
/* Reads article numbers found in a newsgroup directory into an array.  */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* path         Points to the string that specifies the directory to    */
/*              be scanned.                                             */
/*                                                                      */
/* low_msg      Lowest article number to be stored in 'art_array'. If   */
/*              'high_msg' is zero, this paramter will be ignored.      */
/*                                                                      */
/* high_msg     Highest article number to be stored in 'art_array'. If  */
/*              this parameter is set to zero, all article numbers will */
/*              be stored in 'art_array'.                               */
/*                                                                      */
/* art_array    Specifies the array to store the article numbers.       */
/*              This array will be sorted before returning to the       */
/*              caller. This paramter may be set to NULL in which       */
/*              case this routine will only count the number of         */
/*              articles that would have been stored in 'art_array'.    */
/*                                                                      */
/* max_arts     Specifies the number of articles 'art_array' is able    */
/*              to hold. If 'art_array' has been set to NULL, this      */
/*              parameter is ignored.                                   */
/* ------------ ------------------------------------------------------- */
/* Return value Number of articles found.                               */
/*                                                                      */
/************************************************************************/
unsigned int scan_dir(char *path, long low_msg, long high_msg, long **art_array)
{
    long artnum;
    HANDLE hdir;
    WIN32_FIND_DATA ffb;
    BOOL foundnext = TRUE;
    unsigned int num_arts = 0;
    unsigned int num_got = 0;
    char *tmplt = malloc(strlen(path) + 4);

    if(art_array && *art_array) {
        free(*art_array);
        *art_array = NULL;
    }

    strcpy(tmplt, path);
    strcat(tmplt, "\\*.");

    /*
     * Count articles
     */
    if( (hdir = FindFirstFile(tmplt, &ffb)) != INVALID_HANDLE_VALUE ) {
        while (foundnext) {
            if((artnum = atol(ffb.cFileName)) != 0) {
                if(high_msg) {
                    if(artnum >= low_msg && artnum <= high_msg)
                        num_arts++;
                }
                else
                    num_arts++;
            }
            foundnext = FindNextFile(hdir, &ffb);
        }
        FindClose(hdir);
    }

    if(art_array && num_arts) {
        /*
         * Allocate and fill article array
         */
        *art_array = malloc(sizeof(long *) * num_arts);
        foundnext = TRUE;
        if( (hdir = FindFirstFile(tmplt, &ffb)) != INVALID_HANDLE_VALUE ) {
            while (foundnext) {
                if((artnum = atol(ffb.cFileName)) != 0) {
                    if(high_msg) {
                        if(artnum >= low_msg && artnum <= high_msg)
                            (*art_array)[num_got++] = artnum;
                    }
                    else
                        (*art_array)[num_got++] = artnum;
                    if(num_got >= num_arts)
                        break;
                }
                if(num_got >= num_arts)
                    break;
                foundnext = FindNextFile(hdir, &ffb);
            }
            FindClose(hdir);
            if(num_got)
                qsort(*art_array, num_got, sizeof(long), longcmp);
        }
        num_arts = num_got;
    }
    free(tmplt);
    return (num_arts);
}

/************************************************************************/
/*                                                                      */
/* scan_minmax                                                          */
/*                                                                      */
/* Scans a newsgroup directory to count the number of articles as well  */
/* as to find the lowest and highest article number.                    */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* path         Points to the string that specifies the directory to    */
/*              be scanned.                                             */
/*                                                                      */
/* plo          Points to a long to store the lowest article number.    */
/*              This parameter may be set to NULL if the caller is      */
/*              not interested in this value.                           */
/*                                                                      */
/* phi          Points to a long to store the highest article number.   */
/*              This parameter may be set to NULL if the caller is      */
/*              not interested in this value.                           */
/*                                                                      */
/* ------------ ------------------------------------------------------- */
/* Return value Number of articles found.                               */
/*                                                                      */
/************************************************************************/
int scan_minmax(char *path, long *plo, long *phi)
{
    long artnum;
    HANDLE hdir;
    WIN32_FIND_DATA ffb;
    BOOL foundnext = TRUE;
    int num_arts = 0;
    char *tpl = malloc(strlen(path) + 3);

    if(plo)
        *plo = 1;
    if(phi)
        *phi = 0;
    strcat(strcpy(tpl, path), "\\*");
    if( (hdir = FindFirstFile(tpl, &ffb)) != INVALID_HANDLE_VALUE ) {
        while (foundnext) {
            if((artnum = atol(ffb.cFileName)) > 0) {
                if(num_arts++) {
                    if(phi && artnum > *phi)
                        *phi = artnum;
                    else if(plo && artnum < *plo)
                        *plo = artnum;
                }
                else if(plo && phi)
                    *plo = *phi = artnum;
            }
            foundnext = FindNextFile(hdir, &ffb);
        }
        FindClose(hdir);
    }
    free(tpl);

    return (num_arts);
}

/*
 * $Log: ntscandir.c,v $
 * Revision 1.1  1997/02/13 11:09:28  oliver
 * initial revision
 *
 */
