/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char os2dep_rcsid[] = "$Id: os2dep.c,v 1.1 1997/02/13 11:09:29 oliver Exp $";

#define INCL_DOSSEMAPHORES
#define INCL_DOSERRORS
#define INCL_DOSPROCESS
#include <os2.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#include <sysdep.h>

#define ACF_OPNFLG  OPEN_ACTION_FAIL_IF_NEW | OPEN_ACTION_OPEN_IF_EXISTS
#define ACF_OPNMOD  OPEN_FLAGS_SEQUENTIAL | OPEN_SHARE_DENYWRITE |\
                    OPEN_ACCESS_READONLY | OPEN_FLAGS_NOINHERIT

/************************************************************************/
/*                                                                      */
/************************************************************************/
void ms_sleep(long ms)
{
    DosSleep((unsigned long)ms);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void set_priority(int mode)
{
    switch(mode) {
    case -1:
        DosSetPriority(PRTYS_THREAD, PRTYC_IDLETIME, -15, 0);
        break;
    case 1:
        DosSetPriority(PRTYS_THREAD, PRTYC_FOREGROUNDSERVER, 15, 0);
        break;
    default:
        DosSetPriority(PRTYS_THREAD, PRTYC_REGULAR, 0, 0);
        break;
    }
}

/************************************************************************/
/*                                                                      */
/*  Opens a file in shared mode using a low level system call.          */
/*  If the file could not be opened immediately because of access       */
/*  violations, this functions will retry to open the file after        */
/*  sleeping one second. If the file couldn't be opened after ten       */
/*  retries then this function returns an error.                        */
/*                                                                      */
/*  Returns 0 on success or a DOS errorcode on failures.                */
/*                                                                      */
/************************************************************************/
static APIRET open_file(char *name, HFILE *phf)
{
    APIRET rc;
    ULONG act;
    int retries = 10;

    while(retries--) {
        rc = DosOpen(name, phf, &act, 0, 0, ACF_OPNFLG, ACF_OPNMOD, NULL);
        if(rc != ERROR_SHARING_VIOLATION)
            break;
        DosSleep(1000);
    }
    return(rc);
}

/************************************************************************/
/*                                                                      */
/*  load_file                                                           */
/*                                                                      */
/*  This procedure reads the complete contents of a file into an        */
/*  allocated memory block. The caller is responsible to free this      */
/*  block if it's no longer needed.                                     */
/*                                                                      */
/*  Returns a pointer to the memory block containing the file contents  */
/*  as a zero terminated string or NULL on errors.                      */
/*                                                                      */
/************************************************************************/
char *load_file(char *name)
{
    char *result;
    HFILE  hf;
    FILESTATUS3 fsts;
    ULONG cbRead;
    APIRET rc;

    /*
     * Try to open an existing file. Do not report an error to
     * syslog if the file doesn't exist, because the caller
     * may accept this as a normal condition.
     */
    if((rc = open_file(name, &hf)) != 0) {
        syslog(LOG_ERR, "Error %u opening %s", rc, name);
        return(NULL);
    }

    /*
     * Read the file info structure in order to get the
     * file's size.
     */
    if((rc = DosQueryFileInfo(hf, FIL_STANDARD, &fsts, sizeof(fsts))) != 0) {
        syslog(LOG_ERR, "Error %u querying file info of %s", rc, name);
        DosClose(hf);
        return(NULL);
    }

    /*
     * Read the whole file into a single buffer
     */
    result = malloc(fsts.cbFile + 2);
    if((rc = DosRead(hf, result, fsts.cbFile, &cbRead)) != 0) {
        syslog(LOG_ERR, "Error %u reading %ul bytes from %s",
               rc, fsts.cbFile, name);
        DosClose(hf);
        free(result);
        return(NULL);
    }
    DosClose(hf);

    /*
     * Add an extra linefeed at the end to ensure
     * proper termination of the last line.
     */
    *(result + fsts.cbFile) = '\n';
    *(result + fsts.cbFile + 1) = '\0';
    return(result);
}

/************************************************************************/
/*                                                                      */
/*  Synchronizes the lowest and highest message number of a single      */
/*  group with what was really found in the groups subdirectory.        */
/*                                                                      */
/************************************************************************/
int sync_group(const char *group, const char *newsdir, long *lomsg, long *himsg)
{
    char *tmplt = malloc(_MAX_PATH);
    char *cp;
    unsigned long cffb = 255;
    HDIR hdir = (HDIR)HDIR_CREATE;
    PFILEFINDBUF3 paffb = malloc(12288);
    PFILEFINDBUF3 pffb;
    APIRET rc;
    int i;
    long artnum;
    int result = 0;

    strcpy(tmplt, newsdir);
    cp = tmplt + strlen(tmplt);
    *cp++ = '\\';
    strcpy(cp, group);
    while(*cp) {
        if(*cp == '.')
            *cp = '\\';
        cp++;
    }
    strcpy(cp, "\\*");
    if((rc = DosFindFirst(tmplt, &hdir, 0, paffb, 12288, &cffb, 1)) == 0) {
        while (!rc && cffb) {
            pffb = paffb;
            for(i = 0; i < cffb; i++) {
                if((artnum = atol(pffb->achName)) != 0) {
                    if(artnum > *himsg) {
                        *himsg = artnum;
                        result = 1;
                    }
                    if(artnum < *lomsg) {
                        *lomsg = artnum;
                        result = 1;
                    }
                }
                pffb = (PFILEFINDBUF3)(((char *)pffb) + pffb->oNextEntryOffset);
            }
            cffb = 255;
            rc = DosFindNext(hdir, paffb, 12288, &cffb);
        }
        DosFindClose(hdir);
    }
    free(paffb);
    free(tmplt);

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int file_seq(char *name)
{
    int result = 1;
    char *cp;
    unsigned long cffb = 255;
    HDIR hdir = (HDIR)HDIR_CREATE;
    PFILEFINDBUF3 paffb = malloc(12288);
    PFILEFINDBUF3 pffb;
    APIRET rc;
    int i;
    char *tmplt = strcat(strcpy(malloc(_MAX_PATH), name), ".*");

    if((rc = DosFindFirst(tmplt, &hdir, 0, paffb, 12288, &cffb, 1)) == 0) {
        while (!rc && cffb) {
            pffb = paffb;
            for(i = 0; i < cffb; i++) {
                if((cp = strrchr(pffb->achName, '.')) != NULL)
                    if(atoi(cp + 1) >= result)
                        result = atoi(cp + 1);
                pffb = (PFILEFINDBUF3)(((char *)pffb) + pffb->oNextEntryOffset);
            }
            cffb = 255;
            rc = DosFindNext(hdir, paffb, 12288, &cffb);
        }
        DosFindClose(hdir);
    }

    free(tmplt);
    return(result);
}

/*
 * $Log: os2dep.c,v $
 * Revision 1.1  1997/02/13 11:09:29  oliver
 * initial revision
 *
 */
