/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char os2mutex_rcsid[] = "$Id: os2mutex.c,v 1.1 1997/02/13 11:09:31 oliver Exp $";

#define INCL_DOSSEMAPHORES
#include <os2.h>

#include <stdio.h>

#include <syslog.h>

#include <mutex.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
long create_mutex(void)
{
    long mux;
    APIRET rc;

    if((rc = DosCreateMutexSem(0, (unsigned long*)(&mux), 0, 0)) != 0) {
        syslog(LOG_ERR, "Error %u (DosCreateMutexSem)", rc);
        return(-1);
    }
    return(mux);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void destroy_mutex(long mux)
{
    if(mux == -1)
        syslog(LOG_NOTICE, "Bad mutex destroy");
    else
        DosCloseMutexSem(mux);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int request_mutex(long *muxp, long maxwait)
{
    APIRET rc;

    if(*muxp == -1) {
        syslog(LOG_NOTICE, "Bad mutex request %ld", maxwait);
        return(-1);
    }

    if((rc = DosRequestMutexSem(*muxp, maxwait)) != 0) {
        if(maxwait)
            syslog(LOG_ERR, "Error %u (DosRequestMutexSem %ld)", rc, maxwait);
        if(rc == 105) {
            destroy_mutex(*muxp);
            *muxp = create_mutex();
            if((rc = DosRequestMutexSem(*muxp, maxwait)) == 0)
                return(0);
            syslog(LOG_ERR, "Error %u: Failed to recover from 105", rc);
        }
        return(-1);
    }

    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void release_mutex(long mux)
{
    if(mux == -1)
        syslog(LOG_NOTICE, "Bad mutex release");
    else
        DosReleaseMutexSem(mux);
}

/*
 * $Log: os2mutex.c,v $
 * Revision 1.1  1997/02/13 11:09:31  oliver
 * initial revision
 *
 */
