/*
 * This part was written by Harald Kipp
 *
 * Bug reports to
 *
 *  harald@os2point.ping.de
 *
 */

const char os2sgrpd_rcsid[] = "$Id: os2sgrpd.c,v 1.1 1997/02/13 11:09:32 oliver Exp $";

#define INCL_DOSFILEMGR
#include <os2.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <scangrpd.h>

#define FBUFSIZE    5104
#define AINCSIZE    124

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int longcmp(const void *x, const void *y)
{
    if(*(long *)x > *(long *)y)
        return(1);
    else if(*(long *)x < *(long *)y)
        return(-1);
    return(0);
}

/************************************************************************/
/*                                                                      */
/* scan_groupdir                                                        */
/*                                                                      */
/* Scans a given newsgroup directory and returns a sorted array of all  */
/* article numbers found.                                               */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* group        Points to a zero teminated string which specifies the   */
/*              name of the newsgroup to be scanned.                    */
/*                                                                      */
/* newsdir      Points to a zero teminated string which specifies the   */
/*              home directory of all newsgroups.                       */
/*                                                                      */
/* art_array    Points to a pointer of an array of article numbers with */
/*              the size given by siz_array. If the number of articles  */
/*              found does not fit, this routine will reallocate more   */
/*              heap memory.                                            */
/*                                                                      */
/* siz_array    Points to the number of articles which will currently   */
/*              fit into the given array. This value may be changed     */
/*              within this routine if the number of articles found     */
/*              in our newsgroup directory is greater.                  */
/*                                                                      */
/*              Note, that this must point to a zero value if           */
/*              'art_array' points to a NULL pointer.                   */
/* ------------ ------------------------------------------------------- */
/* Return value Number of articles found.                               */
/*                                                                      */
/************************************************************************/
int scan_groupdir(const char *group, const char *newsdir, long **art_array, int *siz_array)
{
    long artnum;
    HDIR hdir;
    PFILEFINDBUF3 pffb;   /* 286 */
    unsigned long cffb;
    APIRET rc;
    int i;
    char *cp;
    int num_arts = 0;
    PFILEFINDBUF3 paffb = malloc(FBUFSIZE);
    int  l = strlen(newsdir);
    char *tmplt = malloc(l + strlen(group) + 5);

    cp = strcpy(tmplt, newsdir) + l;
    *cp++ = '\\';
    for(cp = strcpy(cp, group); *cp; cp++)
        if(*cp == '.')
            *cp = '\\';
    *cp++ = '\\';
    *cp++ = '*';
    *cp++ = '.';
    *cp = '\0';

    cffb = AINCSIZE;
    hdir = (HDIR)HDIR_CREATE;
    if((rc = DosFindFirst(tmplt, &hdir, 0, paffb, FBUFSIZE, &cffb, 1)) == 0) {
        while (!rc && cffb) {
            pffb = paffb;
            for(i = 0; i < cffb; i++) {
                if((artnum = atol(pffb->achName)) != 0) {
                    if(num_arts >= *siz_array) {
                        *siz_array += AINCSIZE;
                        *art_array = realloc(*art_array, *siz_array * sizeof(long));
                    }
                    (*art_array)[num_arts++] = artnum;
                }
                pffb = (PFILEFINDBUF3)(((char *)pffb) + pffb->oNextEntryOffset);
            }
            cffb = AINCSIZE;
            rc = DosFindNext(hdir, paffb, FBUFSIZE, &cffb);
        }
        DosFindClose(hdir);
        if(num_arts)
            qsort(*art_array, num_arts, sizeof(long), longcmp);

    }
    free(paffb);
    free(tmplt);

    return(num_arts);
}

/*
 * $Log: os2sgrpd.c,v $
 * Revision 1.1  1997/02/13 11:09:32  oliver
 * initial revision
 *
 */
