/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char readconf_rcsid[] = "$Id: readconf.c,v 1.1 1997/02/13 11:09:34 oliver Exp $";

#include <stdlib.h>
#include <string.h>
#include <direct.h>
#include <io.h>

#include <readconf.h>

/************************************************************************/
/*                                                                      */
/* Set configuration defaults                                           */
/*                                                                      */
/* 1: Create full pathname of a given filename. Either use the first    */
/*    envtable entry of the current working directory as a prolog.      */
/* 2: If the environment hasn't been defined, use the given default     */
/*    without modification.                                             */
/* 3: Same as 1, but will be set to an empty string if the resulting    */
/*    filename doesn't exist.                                           */
/* 4: Use an alternate environment variable.                            */
/*                                                                      */
/************************************************************************/
void read_conf(ENVTABLE *ep)
{
    register ENVTABLE *ip;
    char *cp;

    if((cp = getenv(ep->ename)) != NULL)
        strcpy(ep->cnfg, cp);
    else
        getcwd(ep->cnfg, _MAX_PATH);

    for(ip = ep + 1; ip->cnfg; ip++) {
        if((cp = getenv(ip->ename)) != NULL) {
            if(strcmp(cp, "NONE"))
                strcpy(ip->cnfg, cp);
            else
                ip->cnfg[0] = '\0';
        }
        else {
            switch(ip->mode) {
            case 0:
                break;
            case 1: /* create full pathname */
            case 3: /* only if file exists  */
                strcpy(ip->cnfg, ep->cnfg);
                cp = ip->cnfg + strlen(ip->cnfg);
                *cp++ = '\\';
                strcpy(cp, ip->fname);
                if(ip->mode == 3 && access(ip->cnfg, 0) != 0)
                    ip->cnfg[0] = '\0';
                break;
            case 2: /* copy default without modification */
                strcpy(ip->cnfg, ip->fname);
                break;
            case 4: /* use alternative environment variable */
                if((cp = getenv(ip->fname)) != NULL)
                    strcpy(ip->cnfg, cp);
                break;
            }
        }
    }
}

/*
 * $Log: readconf.c,v $
 * Revision 1.1  1997/02/13 11:09:34  oliver
 * initial revision
 *
 */
