/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char suckarts_rcsid[] = "$Id: suckarts.c,v 1.2 1997/02/14 13:53:05 harald Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <ctype.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <xfile.h>
#include <domnpath.h>

#include <suckarts.h>
#include <sysdep.h>


static char status[256];
static int busyflag;

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int suck_grouparts(int *rbv, FILE *fpw)
{
    char *buf;

    /*
     * Loop for each group
     */
    buf = malloc(MAX_BUFSIZ);
    while(busyflag && fgets(buf, MAX_GROUPNAME, fpw)) {
        char *cps = buf;

        /*
         * Interpret watermark line.
         */
        while(*cps && *cps != ' ' && *cps != '\t' && *cps != '\n' && *cps != ':')
            *cps++;
        if(*cps != ':')
            continue;
        *cps = '\0';
        strcat(strcpy(status, "Querying "), buf);
        puts(status);
        ms_sleep(888);
    }
    busyflag = 0;
    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int suck_articles(const char *hostname)
{
    int result = -1;
    int sock;
    int *rbv;
    char *buf;
    FILE *fpw;

    busyflag = 1;

    /*
     * Connect to remote
     */
    strcat(strcpy(status, "Connecting "), hostname);
    if((sock = SoStreamConnect((char *)hostname, "nntp", 119)) < 0) {
        syslog(LOG_ERR, "Could not connect to %s", hostname);
        return(-1);
    }

    buf = malloc(MAX_BUFSIZ);
    rbv = SoStreamBufCreate(sock);
    if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, 300) > 0) {
        if(buf[0] == '2') {
            char *watermarklist = malloc(_MAX_PATH);

            getcwd(watermarklist, MAX_BUFSIZ);
            domain_path(watermarklist, hostname, ".watermark");

            /*
             * Open last watermark.
             */
            if((fpw = xfopen(watermarklist, "r")) != NULL) {
                suck_grouparts(rbv, fpw);
                result = 0;
            }
            else
                syslog(LOG_ERR, "%s", _strerror(watermarklist));
        }
        else
            syslog(LOG_ERR, "%s: %s", hostname, buf);
    }
    SoStreamClose(sock);
    SoStreamBufDestroy(rbv);
    free(buf);

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int stat_artsucker(char *statustext)
{
    if(busyflag) {
        strcpy(statustext, status);
        return(0);
    }
    return(-1);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int stop_artsucker(void)
{
    if(busyflag) {
        busyflag = 0;
        return(0);
    }
    return(-1);
}

/************************************************************************/
/*                                                                      */
/* This main program is for standalone testing.                         */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#ifdef STANDALONE
int main(void)
{
    long lastdate = 960101;
    long lasttime = 0;

    /*
     * Initialize socket interface.
     */
    if(SoInitialize()) {
        fprintf(stderr, "FATAL ERROR: TCP/IP initialization failed\n");
        return(128);
    }
    suck_articles("naxos.ping.de");
    return(0);
}
#endif

/*
 * $Log: suckarts.c,v $
 * Revision 1.2  1997/02/14 13:53:05  harald
 * suck_articles now returns 0 on success
 *
 * Revision 1.1  1997/02/13 11:09:37  oliver
 * initial revision
 *
 */
