/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char sucklist_rcsid[] = "$Id: sucklist.c,v 1.1 1997/02/13 11:09:38 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <time.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <xfile.h>
#include <domnpath.h>

#include <sucklist.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int query_list(void *rbv, char *filename, char *query, char *mode, long remotewait)
{
    int items = 0;
    char *buf;
    FILE *fp;

    if((fp = xfopen(filename, mode)) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(filename));
        return(-1);
    }

    buf = malloc(MAX_BUFSIZ);
    for(;;) {
        if(SoStreamTransact(rbv, query, buf, remotewait)) {
            free(buf);
            return(-1);
        }
#if 0
        /*
         * TODO: Authentication
         */
        if(atoi(buf) != 480)
            break;
        if(authenticate(rbv, buf)) {
            free(buf);
            return(-1);
        }
#else
        break;
#endif
    }
    if (buf[0] != '2') {
        syslog(LOG_ERR, "Server error: %s", buf);
        free(buf);
        return(-1);
    }

    for (;;) {
        if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, remotewait) <= 0) {
            items = -1;
            break;
        }

	if (!strcmp(buf, "."))
            break;
        items++;
        fprintf(fp, "%s\n", buf);
    }
    fclose(fp);
    free(buf);

    return(items);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int suck_newsgroups(const char *hostname)
{
    int result = -1;
    int sock;
    int *rbv;
    char *buf;

    /*
     * Connect to remote
     */
    if ((sock = SoStreamConnect((char *)hostname, "nntp", 119)) < 0) {
        syslog(LOG_ERR, "Could not connect to %s", hostname);
        return(-1);
    }

    buf = malloc(MAX_BUFSIZ);
    rbv = SoStreamBufCreate(sock);
    if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, 300) > 0) {
        if(buf[0] == '2') {
            getcwd(buf, MAX_BUFSIZ);
            domain_path(buf, hostname, ".newsgroups");
            if(query_list(rbv, buf, "LIST NEWSGROUPS", "w", 300) > 0)
                result = 0;
        }
        else
            syslog(LOG_ERR, "%s: %s", hostname, buf);
    }
    SoStreamClose(sock);
    SoStreamBufDestroy(rbv);
    free(buf);

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int suck_newgroups(const char *hostname, long *lastdate, long *lasttime)
{
    int result = -1;
    int sock;
    int *rbv;
    char *buf;
    long remotedate = 0;
    long remotetime = 0;

    /*
     * Connect to remote
     */
    if ((sock = SoStreamConnect((char *)hostname, "nntp", 119)) < 0) {
        syslog(LOG_ERR, "Could not connect to %s", hostname);
        return(-1);
    }

    buf = malloc(MAX_BUFSIZ);
    rbv = SoStreamBufCreate(sock);

    /*
     * Get intro
     */
    if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, 300) > 0) {
        if(buf[0] == '2') {
            if(SoStreamTransact(rbv, "DATE", buf, 300) == 0) {
                char *qbuf = malloc(MAX_CMDSIZE);

                if(buf[0] == '1') {
                    remotetime = atol(&buf[12]);
                    buf[12] = '\0';
                    remotedate = atol(&buf[6]);
                }
                getcwd(buf, MAX_BUFSIZ);
                domain_path(buf, hostname, ".newgroups");
                sprintf(qbuf, "NEWGROUPS %06lu %06lu GMT", *lastdate, *lasttime);
                if(query_list(rbv, buf, qbuf, "a", 300) > 0)
                    result = 0;
            }
        }
        else
            syslog(LOG_ERR, "%s: %s", hostname, buf);
    }
    SoStreamClose(sock);
    SoStreamBufDestroy(rbv);
    free(buf);

    if(remotedate) {
        *lastdate = remotedate;
        *lasttime = remotetime;
    }
    else {
        time_t now;
        struct tm *gmt;

        tzset();
        time(&now);
        if((gmt = gmtime(&now)) != NULL) {
            *lastdate = (long)(gmt->tm_year % 100) * 10000L +
                        (long)(gmt->tm_mon + 1) * 100L +
                        (long)gmt->tm_mday;
            *lasttime = (long)gmt->tm_hour * 10000L +
                        (long)gmt->tm_min * 100L +
                        (long)gmt->tm_sec;
        }
    }

    return(result);
}

/************************************************************************/
/*                                                                      */
/* This main program is for standalone testing.                         */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#ifdef STANDALONE
int main(void)
{
    long lastdate = 960101;
    long lasttime = 0;

    /*
     * Initialize socket interface.
     */
    if(SoInitialize()) {
        fprintf(stderr, "FATAL ERROR: TCP/IP initialization failed\n");
        return(128);
    }
    suck_newsgroups("naxos.ping.de");
    suck_newgroups("naxos.ping.de", &lastdate, &lasttime);
    return(0);
}
#endif

/*
 * $Log: sucklist.c,v $
 * Revision 1.1  1997/02/13 11:09:38  oliver
 * initial revision
 *
 */
