/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software has been inspired by several UNIX programs and their   */
/* OS/2 ports:                                                          */
/*                                                                      */
/* slurp - written 1992/93 by Stephen Hebditch and ported to OS/2 by    */
/* Jochen Friedrich.                                                    */
/*                                                                      */
/* nntpxfer - written by Brian Kantor                                   */
/*                                                                      */
/* nntpget - author unknown                                             */
/*                                                                      */
/* nntpclnt - Copyright 1992 by Stan Barber                             */
/*                                                                      */
/* souper - written by Chin Huang                                       */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char suckover_rcsid[] = "$Id: suckover.c,v 1.1 1997/02/13 11:09:39 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>
#include <ctype.h>

#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>
#include <xfile.h>
#include <domnpath.h>

#include <suckover.h>

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int suck_overgroups(int *rbv, FILE *fpw, FILE *fpn, char *overdir)
{
    int broken = 0;
    long nwm;
    long lo, hi;
    int count;
    char *buf;
    char *ngtag;
    long nbytes;
    char *subdir;
    FILE *fpo;

    subdir = overdir + strlen(overdir);
    *subdir++ = '\\';

    /*
     * Loop for each group
     */
    buf = malloc(MAX_BUFSIZ);
    ngtag = malloc(MAX_GROUPNAME);
    while(fgets(buf, MAX_GROUPNAME, fpw)) {
        char *cps = buf;
        char *cpd = ngtag;

        /*
         * If broken, copy all remaining watermark lines.
         */
        if(broken) {
            fputs(buf, fpn);
            continue;
        }

        /*
         * Interpret watermark line.
         */
        while(*cps && *cps != ' ' && *cps != '\t' && *cps != '\n' && *cps != ':')
            *cpd++ = *cps++;
        *cpd = '\0';
        if(*cps != ':' || *ngtag == '\0') {
            fputs(buf, fpn);
            continue;
        }
        if((nwm = atol(cps + 1)) == 0)
            nwm = 1;

        /*
         * Switch remote to this group.
         */
        for(;;) {
            strcpy(buf, "GROUP ");
            strcat(buf, ngtag);
            if(SoStreamTransact(rbv, buf, buf, 300)) {
                broken = 1;
                break;
            }
#if 0
            if(atoi(buf) != 480)
                break;
            if(authenticate(rbv, buf)) {
                broken = 1;
                break;
            }
#else
            break;
#endif
        }
        if(broken) {
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }
        if(buf[0] != '2') {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }
        if(sscanf(buf + 4, "%d %ld %ld", &count, &lo, &hi) != 3) {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        /*
         * Check if new articles arrived.
         */
        if(lo <= nwm)
            lo = nwm + 1;
        if(lo > hi) {
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        /*
         * Send XOVER command.
         */
        for(;;) {
            sprintf(buf, "XOVER %ld-%ld", lo, hi);
            if(SoStreamTransact(rbv, buf, buf, 300)) {
                broken = 1;
                break;
            }
#if 0
            if(atoi(buf) != 480)
                break;
            if(authenticate(rbv, buf)) {
                broken = 1;
                break;
            }
#else
            break;
#endif
        }
        if(broken) {
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }
        if(buf[0] != '2') {
            syslog(LOG_NOTICE, "%s: %s", ngtag, buf);
            fprintf(fpn, "%s: %lu\n", ngtag, nwm);
            continue;
        }

        /*
         * The remote sends overview lines and terminates
         * the transmission by a line containing a single dot.
         */
        for(fpo = NULL;;) {
#if 0
            long n;
#endif

            if((nbytes = SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, 300)) <= 0) {
                broken = 1;
                break;
            }
            if(*buf == '.')
                break;
#if 0
            if(watch_limits(nbytes)) {
                broken = 1;
                break;
            }
            for(cps = buf; ; cps++)
                if(!isdigit(*cps))
                    break;
            if((n = atol(buf)) != 0) {
                /*
                 * Filter unwanted articles
                 */
                if(cfg.usefilterfile && over_drop(ngtag, buf)) {
                    drop_overview(ngtag, buf);
                    acci.num_dropped++;
                }
                else if(cfg.usefilterover && run_filterover(ngtag, buf)) {
                    drop_overview(ngtag, buf);
                    acci.num_dropped++;
                }
                else {
                    if(firstone) {
                        add_artpool(ngtag);
                        firstone = 0;
                    }
                    *cps = '\0';
                    add_artpool(buf);
                }
                if(n > nwm)
                    nwm = n;
            }
#else
            if(fpo == NULL) {
                strcpy(subdir, ngtag);
                if((fpo = xfopen(overdir, "a")) == NULL)
                    syslog(LOG_ERR, "%s", _strerror(overdir));
            }
            if(fpo)
                fprintf(fpo, "%s\n", buf);
#endif
        }
        if(fpo)
            fclose(fpo);
        fprintf(fpn, "%s: %lu\n", ngtag, nwm);
    }
    free(buf);
    free(ngtag);

    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int suck_overview(const char *hostname)
{
    int result = -1;
    int sock;
    int *rbv;
    char *buf;
    FILE *fpw, *fpn;

    /*
     * Connect to remote
     */
    if((sock = SoStreamConnect((char *)hostname, "nntp", 119)) < 0) {
        syslog(LOG_ERR, "Could not connect to %s", hostname);
        return(-1);
    }

    buf = malloc(MAX_BUFSIZ);
    rbv = SoStreamBufCreate(sock);
    if(SoStreamBufRecv(buf, MAX_BUFSIZ, rbv, 300) > 0) {
        if(buf[0] == '2') {
            char *watermarklist = malloc(_MAX_PATH);
            char *watermarktemp = malloc(_MAX_PATH);
            char *overdir = malloc(_MAX_PATH);

            getcwd(watermarklist, MAX_BUFSIZ);
            domain_path(watermarklist, hostname, ".watermark");

            getcwd(watermarktemp, MAX_BUFSIZ);
            domain_path(watermarktemp, hostname, ".watertemp");

            getcwd(overdir, MAX_BUFSIZ);
            domain_path(overdir, hostname, ".overview");

            /*
             * Open last watermark.
             */
            if((fpw = xfopen(watermarklist, "r")) != NULL) {

                /*
                 * Open new watermark.
                 */
                if((fpn = xfopen(watermarktemp, "w")) != NULL)
                    suck_overgroups(rbv, fpw, fpn, overdir);
                else
                    syslog(LOG_ERR, "%s", _strerror(watermarktemp));
            }
            else
                syslog(LOG_ERR, "%s", _strerror(watermarklist));
        }
        else
            syslog(LOG_ERR, "%s: %s", hostname, buf);
    }
    SoStreamClose(sock);
    SoStreamBufDestroy(rbv);
    free(buf);

    return(result);
}


/************************************************************************/
/*                                                                      */
/* This main program is for standalone testing.                         */
/*                                                                      */
/*                                                                      */
/************************************************************************/
#ifdef STANDALONE
int main(void)
{
    suck_overview("naxos.ping.de");
    return(0);
}
#endif

/*
 * $Log: suckover.c,v $
 * Revision 1.1  1997/02/13 11:09:39  oliver
 * initial revision
 *
 */
