/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains routines to extend the standard
 * runtime library file handling.
 *
 */

const char xfile_rcsid[] = "$Id: xfile.c,v 1.1 1997/02/13 11:09:42 oliver Exp $";

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <direct.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <share.h>
#include <io.h>

#include <xfile.h>


#ifdef __OS2__
extern int _System DosSleep(long msec);
extern int _System DosQueryFHState(int hFile, unsigned long *pMode);
extern int _System DosSetFHState(int hFile, unsigned long mode);
#else
extern void _System Sleep(long msec);
#define DosSleep Sleep
#endif

/************************************************************************/
/*                                                                      */
/* xfopen                                                               */
/*                                                                      */
/* Opens a named file in a specified mode. If it is opened for writing, */
/* any subdirectories will be created if they don't exist. If the       */
/* file is blocked by another process, the routine will retry to open   */
/* the file upto 20 times within 10 seconds.                            */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* filename     Points to a zero teminated string which specifies the   */
/*              name of the file to be opened.                          */
/*                                                                      */
/* mode         String pointer which specifies the type of access       */
/*              requested for the file. Valid types are "r" (read),     */
/*              "w" (write), or "a" (append). Any of these may be       */
/*              followed by a "+" to allow both read and write access.  */
/*              The newline translation mode can be disabled by "b"     */
/*              (binary).                                               */
/* ------------ ------------------------------------------------------- */
/* Return value A pointer to the open file, NULL indicates an error.    */
/*                                                                      */
/************************************************************************/
FILE *xfopen(const char *filename, const char *mode)
{
    int fh;
    register int i;
    unsigned int oflg;
    unsigned int shflg = SH_DENYWR;
    int create = 1;

    switch(*mode) {
    case 'r':
        if(*(mode + 1) == '+')
            oflg = O_RDWR;
        else
            oflg = O_RDONLY;
        shflg = SH_DENYNO;
        create = 0;
        break;
    case 'w':
        if(*(mode + 1) == '+')
            oflg = O_RDWR | O_TRUNC | O_CREAT;
        else
            oflg = O_WRONLY | O_TRUNC | O_CREAT;
        break;
    case 'a':
        if(*(mode + 1) == '+')
            oflg = O_RDWR | O_APPEND | O_CREAT;
        else
            oflg = O_WRONLY | O_APPEND | O_CREAT;
        break;
    default:
        errno = EBADMODE;
        return(NULL);
    }
    if(*(mode + 1) == 'b' || *(mode + 2) == 'b')
        oflg = oflg | O_BINARY;
    else
        oflg = oflg | O_TEXT;

    for(i = 1; i < 40; i++) {
        if((fh = _sopen(filename, (int)oflg, (int)shflg, S_IREAD | S_IWRITE)) != -1) {
#ifdef __OS2__
            unsigned long status;

            if(DosQueryFHState(fh, &status) == 0) {
                status &= 0x7f88UL;
                status |= 0x0080UL;
                DosSetFHState(fh, status);
            }
#endif
            break;
        }
        if(errno == ENOENT) {
            if(create == 1) {
                char *pathname = strdup(filename);
                register char *cp;

                for(cp = pathname; *cp; cp++) {
                    if(*cp == '/')
                        *cp = '\\';
                }
                cp = pathname + 1;
                while(*cp == '\\' || *cp == ':')
                    cp++;
                while((cp = strchr(cp, '\\')) != NULL) {
                    *cp = '\0';
                    mkdir(pathname);
                    *cp++ = '\\';
                }
                free(pathname);
                create++;
            }
            else
                break;
        }
        else {
            if(errno != EACCES)
                break;
            DosSleep(500);
        }
    }

    if(fh == -1)
        return(NULL);
    return(fdopen(fh, mode));
}

/************************************************************************/
/*                                                                      */
/* xremove                                                              */
/*                                                                      */
/* Deletes a specified file. If the file is blocked by another process, */
/* the routine will retry to delete it upto 20 times within 10 seconds. */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* filename     Points to a zero teminated string which specifies the   */
/*              name of the file to be deleted.                         */
/* ------------ ------------------------------------------------------- */
/* Return value Returns 0 if the file is successfully deleted,          */
/*              otherwise -1 on errors.                                 */
/*                                                                      */
/************************************************************************/
int xremove(const char *filename)
{
    int result;
    int i;

    for(i = 0; i < 20; i++) {
        if((result = remove(filename)) == 0)
            break;
        if(errno != EISOPEN && errno != EACCESS)
            break;
        DosSleep(500);
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/* xrename                                                              */
/*                                                                      */
/* Renames a file or directory. New names may be given explicitely or   */
/* by a pattern with a leading asterisk. Existing destination files     */
/* will be removed. If any of the files are blocked by another process, */
/* the routine will retry to delete it upto 20 times within 10 seconds. */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* oldname      Points to a zero teminated string which specifies the   */
/*              name of the file to be renamed.                         */
/*                                                                      */
/* newname      Points to a zero teminated string containing the new    */
/*              filename. It may also contain a string with a leading   */
/*              asterisk for specially renaming backup files. (See      */
/*              examples below).                                        */
/* ------------ ------------------------------------------------------- */
/* Return value Returns 0 if the file is successfully deleted,          */
/*              otherwise -1 on errors.                                 */
/*                                                                      */
/* Examples for pattern renaming:                                       */
/*                                                                      */
/*   rename("..\\sample.txt", "*.back") -> "..\\sample.back"            */
/*   rename("sample.txt", "*.   ")      -> "sample"                     */
/*   rename("sample", "*.txt")          -> "sample.txt"                 */
/*   rename("sample.conf", "*.x")       -> "sample.conf.x"              */
/*                                                                      */
/************************************************************************/
int xrename(const char *oldname, const char *newname)
{
    int result;
    int i;
    char *cp;
    char *cvtname = NULL;

    if(*newname == '*') {
        cvtname = malloc(strlen(oldname) + strlen(newname));
        strcpy(cvtname, oldname);
        if((cp = strrchr(cvtname, newname[1])) == NULL)
            strcat(cvtname, newname + 1);
        else if(strlen(cp) < strlen(newname))
            strcpy(cp, newname + 1);
        else
            strcat(cvtname, newname + 1);
        cp = cvtname + strlen(cvtname);
        while(*--cp == ' ' || *cp == '.')
            *cp = '\0';
        newname = cvtname;
    }
    for(i = 0; i < 40; i++) {
        if((result = rename(oldname, newname)) == 0)
            break;
        if(errno == ENOENT) {
            char *pathname = strdup(newname);
            register char *cp;

            for(cp = pathname; *cp; cp++) {
                if(*cp == '/')
                    *cp = '\\';
            }
            cp = pathname + 1;
            while(*cp == '\\' || *cp == ':')
                cp++;
            while((cp = strchr(cp, '\\')) != NULL) {
                *cp = '\0';
                mkdir(pathname);
                *cp++ = '\\';
            }
            free(pathname);
            continue;
        }
        if(errno != EISOPEN && errno != EACCESS)
            break;
        if(errno == EACCESS && xremove(newname))
            break;
        DosSleep(500);
    }
    if(cvtname)
        free(cvtname);

    return(result);
}

/*
 * $Log: xfile.c,v $
 * Revision 1.1  1997/02/13 11:09:42  oliver
 * initial revision
 *
 */
