const char changi_l_rcsid[] =
"$Id: changi_l.cpp,v 1.19 1997/04/05 17:06:14 oliver Exp $";

#include <ibase.hpp>
#include <iapp.hpp>
#include <ifont.hpp>
#include <istring.hpp>
#include <imsgbox.hpp>
#include <icoordsy.hpp>
#include <inotebk.hpp>
#include <icnrolst.hpp>
#include <itrace.hpp>
#include <ihelp.hpp>
#include <iprofile.hpp>
#include <icnrhdr.hpp>

#include "changi_l.hpp"
#include "cnrobj.hpp"
#include "expiresettings.hpp"

#include "changi_l.h" // Resource Definitions
#define APPLICATION_NAME "changi_l"

//***************************************************************************
//   main()
//***************************************************************************

int main()
   {
   IFUNCTRACE_DEVELOP();
   ICoordinateSystem::setApplicationOrientation
      (ICoordinateSystem::originLowerLeft);

#ifdef __WINDOWS__
#ifdef USE_IPF
   IHelpWindow::setDefaultStyle( IHelpWindow::defaultStyle()
                                 | IHelpWindow::ipfCompatible );
#endif
#endif

   try
      {
      ChangiLWindow::getMainWindow()->setFocus();
      ChangiLWindow::getMainWindow()->loadHosts();
      ChangiLWindow::getMainWindow()->show();
      IApplication::current().run();
      }
   catch (IException e) { IMessageBox(0).show(e); }

   return 0;
   }

//***************************************************************************
//   ChangiLWindow
//***************************************************************************

ChangiLWindow :: ChangiLWindow()
   : IFrameWindow(IFrameWindow::defaultStyle() |
                  IFrameWindow::minimizedIcon  ,
                  WND_MAIN)
   , menuBar(WND_MAIN, this)
   , clientArea(WND_CLIENTAREA, this, this)
   , hostCnr(WND_HOSTCONT,&clientArea,this)
   , groupNoteBk(WND_GROUPNOTEBK, &clientArea, this, IRectangle(),
                 INotebook::backPagesTopRight
                 | INotebook::majorTabsTop
                 | INotebook::spiralBinding
                 | INotebook::tabTextCenter
                 | IWindow::visible )
   , allGroupsContainer(WND_ALLGROUPSCONT, this, this)
   , subGroupsContainer(WND_SUBGROUPSCONT, this, this)
   , newGroupsContainer(WND_NEWGROUPSCONT, this, this)
   , infoArea(this)
   , hasHelp(true)
   {
   IFUNCTRACE_DEVELOP();

   setDestroyOnClose(true);
   setAutoDeleteObject(true);
#ifdef __WINDOWS__
   setFont(IFont("Helv",8));
   infoArea.setFont(IFont("Helv",8));
#endif
   setClient(&clientArea);

   hostCnr.setTitle("Host");
   initNotebook();
   setExtensionSize(&infoArea, IFont(&infoArea).maxCharHeight());

   infoArea.setInactiveText(IApplication::current().userResourceLibrary().loadString(STR_IDLE_STATUS));
   infoArea.setBackgroundColor(IColor::paleGray);

   try
      {
      IHelpWindow::Settings helpSettings;
//      helpSettings.setMenuBar(SM_HELP);
      helpSettings.setTitle("Changi Lite Hilfe");
//      helpSettings.setHelpTable(HELP_TABLE);
      helpSettings.setLibraries("changi_l.hlp");
      helpWindow = new IHelpWindow(helpSettings,this);
      }
   catch( ... )
      {
      IMessageBox msgBox(this);
      msgBox.show("Hilfedatei nicht gefunden. Hilfefunktion ist nicht verfgbar.",
                  IMessageBox::okButton
                  | IMessageBox::warningIcon
                  );
      hasHelp = false;
      }

   ICommandHandler::handleEventsFor(this);
   IPageHandler::handleEventsFor(&groupNoteBk);
   IFrameHandler::handleEventsFor(this);
   ICnrHandler::handleEventsFor(&hostCnr);

#ifdef __WINDOWS__
   IProfile profile(APPLICATION_NAME);
#else
   IProfile profile(IString(APPLICATION_NAME)+".ini");
#endif
   try
      {
      long x, y, cx, cy;
      x = profile.integerWithKey("xPosition", "MainWin");
      y = profile.integerWithKey("yPosition", "MainWin");
      cx = profile.integerWithKey("xSize", "MainWin");
      cy = profile.integerWithKey("ySize", "MainWin");
      moveSizeTo(IRectangle(IPoint(x, y), ISize(cx, cy)));
      clientArea.setSplitWindowPercentage(&hostCnr,profile.integerWithKey("splitLeft", "MainWin"));
      clientArea.setSplitWindowPercentage(&groupNoteBk,profile.integerWithKey("splitRight", "MainWin"));
      }
   catch(...)
      {
      clientArea.setSplitWindowPercentage(&hostCnr,15);
      clientArea.setSplitWindowPercentage(&groupNoteBk,85);
      }
   }

ChangiLWindow :: ~ChangiLWindow()
   {
   IFUNCTRACE_DEVELOP();

   mainWindow = NULL;

   if (hasHelp) delete helpWindow;
   ITRACE_DEVELOP("Cnr");
   ICnrHandler::stopHandlingEventsFor(&hostCnr);
   ITRACE_DEVELOP("Frame");
   IFrameHandler::stopHandlingEventsFor(this);
   ITRACE_DEVELOP("Page");
   IPageHandler::stopHandlingEventsFor(&groupNoteBk);
   ITRACE_DEVELOP("Command");
   ICommandHandler::stopHandlingEventsFor(this);
   }

void ChangiLWindow :: initNotebook()
   {
   IFUNCTRACE_DEVELOP();

   IFont nbkFont = groupNoteBk.font();
   IString nbkAll = IApplication::current().userResourceLibrary().loadString(STR_NBK_ALL);
   IString nbkSub = IApplication::current().userResourceLibrary().loadString(STR_NBK_SUB);
   IString nbkNew = IApplication::current().userResourceLibrary().loadString(STR_NBK_NEW);
   long maxsize = nbkFont.textWidth(nbkAll);
   maxsize = max(maxsize, nbkFont.textWidth(nbkSub));
   maxsize = max(maxsize, nbkFont.textWidth(nbkNew));
   groupNoteBk.setMajorTabSize(ISize(maxsize+20,25));
   groupNoteBk.setMinorTabSize(ISize());

   allGroupsPageHandle = groupNoteBk.addLastPage(INotebook::PageSettings(nbkAll,0,
      INotebook::PageSettings::majorTab | INotebook::PageSettings::autoPageSize),
      &allGroupsContainer);
   subGroupsPageHandle = groupNoteBk.addLastPage(INotebook::PageSettings(nbkSub,0,
      INotebook::PageSettings::majorTab | INotebook::PageSettings::autoPageSize),
      &subGroupsContainer);
   newGroupsPageHandle = groupNoteBk.addLastPage(INotebook::PageSettings(nbkNew,0,
      INotebook::PageSettings::majorTab | INotebook::PageSettings::autoPageSize),
      &newGroupsContainer);
   groupNoteBk.turnToPage(subGroupsPageHandle);
   noteBkState_ = SUBPAGE;

   // same as below
   menuBar.enableItem(MI_CHANGEGROUP);
   menuBar.disableItem(MI_ADDGROUP);
   }

Boolean ChangiLWindow::select(IPageSelectEvent& event)
   {
   IFUNCTRACE_DEVELOP();
   ICnrCtl* container;
   if (event.pageHandle() == allGroupsPageHandle)
      {
      noteBkState_ = ALLPAGE;
      container = &allGroupsContainer;
      menuBar.disableItem(MI_CHANGEGROUP);
      menuBar.enableItem(MI_ADDGROUP);
      }
   else if (event.pageHandle() == subGroupsPageHandle)
      {
      noteBkState_ = SUBPAGE;
      container = &subGroupsContainer;
      menuBar.enableItem(MI_CHANGEGROUP);
      menuBar.disableItem(MI_ADDGROUP);
      }
   else if (event.pageHandle() == newGroupsPageHandle)
      {
      noteBkState_ = NEWPAGE;
      container = &newGroupsContainer;
      menuBar.disableItem(MI_CHANGEGROUP);
      menuBar.enableItem(MI_ADDGROUP);
      }

   container->sortByIconText();

   return true;
   }

IBase::Boolean ChangiLWindow :: command(ICommandEvent & cmdEvent)
   {
   IFUNCTRACE_DEVELOP();

   Boolean eventProcessed(true);
   switch (cmdEvent.commandId())
      {
      case MI_ADDHOST:           hostCnr.addHost();                                   break;
      case MI_REMHOST:           hostCnr.removeHost(hostCnr.selectedHost());          break;
      case MI_REMHOST_POP:       hostCnr.removeHost(hostCnr.cursoredHost());          break;
      case MI_SUCKLIST:          hostCnr.suckListForHost(hostCnr.selectedHost());     break;
      case MI_SUCKLIST_POP:      hostCnr.suckListForHost(hostCnr.cursoredHost());     break;
      case MI_SUCKARTS:          hostCnr.suckArtsForHost(hostCnr.selectedHost());     break;
      case MI_SUCKARTS_POP:      hostCnr.suckArtsForHost(hostCnr.cursoredHost());     break;

      case MI_ADDGROUP:          subscribeSelectedGroup(true);                        break;
      case MI_REMGROUP:          subscribeSelectedGroup(false);                       break;
      case MI_CHANGEGROUP:       changeSelectedGroup();                               break;

      case MI_MAIN_HELP_INDEX:   if (hasHelp) helpWindow->show(IHelpWindow::index);   break;
      case MI_MAIN_HELP_GENERAL: if (hasHelp) helpWindow->show(IHelpWindow::general); break;
      case MI_MAIN_HELP_KEYS:    if (hasHelp) helpWindow->show(IHelpWindow::keys);    break;
      case MI_MAIN_HELP_USING:   if (hasHelp) helpWindow->show(IHelpWindow::using);   break;

      case MI_EXIT: close(); break;
      default:      eventProcessed=false;
      }

   return(eventProcessed);
   }

void ChangiLWindow::subscribeSelectedGroup(IBoolean doSubscr)
   {
   IFUNCTRACE_DEVELOP();
   ICnrCtl* container;
   if (noteBkState() == ALLPAGE)
      container = &allGroupsContainer;
   if (noteBkState() == SUBPAGE)
      container = &subGroupsContainer;
   if (noteBkState() == NEWPAGE)
      container = &newGroupsContainer;

   container->setRefreshOff();
   ICnrCtl::ObjectCursor cursor(*container, IContainerObject::selected);
   ISequence<GroupCnrObj*> selectedList;
   forCursor(cursor)
      selectedList.addAsLast((GroupCnrObj*)container->objectAt(cursor));
   ISequence<GroupCnrObj*>::Cursor selectedCursor(selectedList);

   forCursor(selectedCursor)
      if (doSubscr)
         selectedList.elementAt(selectedCursor)->subscribe();
      else
         selectedList.elementAt(selectedCursor)->unsubscribe();
   container->setRefreshOn().refresh();
   }

void ChangiLWindow::changeSelectedGroup()
   {
   IFUNCTRACE_DEVELOP();
   ICnrCtl* container;
   if (noteBkState() == ALLPAGE)
      container = &allGroupsContainer;
   if (noteBkState() == SUBPAGE)
      container = &subGroupsContainer;
   if (noteBkState() == NEWPAGE)
      container = &newGroupsContainer;

   ICnrCtl::ObjectCursor cursor(*container, IContainerObject::selected);
   new ExpireSettingsMany(ChangiLWindow::getMainWindow(), *container, cursor)
      ->showModally();
   }

Boolean ChangiLWindow::closed(IFrameEvent& frameEvent)
   {
   IFUNCTRACE_DEVELOP();

   ITRACE_DEVELOP("creating Profile");
   #ifdef __WINDOWS__
      IProfile profile(APPLICATION_NAME);
   #else
      IProfile profile(IString(APPLICATION_NAME)+".ini");
   #endif
   ITRACE_DEVELOP("gettin' size");
   ISize winsize = size();   ITRACE_DEVELOP("gettin' pos");
   IPoint pos = position();

   IBoolean ret = true;
   if (hostCnr.configDirty())
      {
      IMessageBox::Response answer = IMessageBox(ChangiLWindow::getMainWindow()).show(
         IApplication::current().userResourceLibrary().loadString(STR_SAVE_CONFIG),
         IMessageBox::queryIcon | IMessageBox::yesNoCancelButton);
      if (answer  == IMessageBox::yes)
         {
         hostCnr.saveHostConfig();
         ret = IFrameHandler::closed(frameEvent);
         }
      else if (answer  == IMessageBox::no)
         ret = IFrameHandler::closed(frameEvent);
      else
         ret = true;
      }
   else ret = IFrameHandler::closed(frameEvent);

   ITRACE_DEVELOP("Writing Profile");
   profile.addOrReplaceElementWithKey("xPosition", pos.x(), "MainWin");
   profile.addOrReplaceElementWithKey("yPosition", pos.y(), "MainWin");
   profile.addOrReplaceElementWithKey("xSize", winsize.width(), "MainWin");
   profile.addOrReplaceElementWithKey("ySize", winsize.height(), "MainWin");
   profile.addOrReplaceElementWithKey("splitLeft",
      clientArea.splitWindowPercentage(&hostCnr), "MainWin");
   profile.addOrReplaceElementWithKey("splitRight",
      clientArea.splitWindowPercentage(&groupNoteBk), "MainWin");
   ITRACE_DEVELOP("Done Writing Profile");

   return ret;
   }

ChangiLWindow* ChangiLWindow::mainWindow = NULL;

/*
 * $Log: changi_l.cpp,v $
 * Revision 1.19  1997/04/05 17:06:14  oliver
 * another bugfix (forgot mb Constructor)
 *
 * Revision 1.18  1997/04/05 16:56:58  oliver
 * various bugfixes
 *
 * Revision 1.17  1997/03/18 17:35:22  harald
 * added titleSeparator to groupCnr
 *
 * Revision 1.16  1997/03/18 16:12:39  oliver
 * bugfixes
 * menu changet to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.15  1997/03/15 12:56:42  oliver
 * mainWindow was commented out
 *
 * Revision 1.14  1997/03/15 12:54:02  harald
 * makefile and .def changes to compile under OS/2
 *
 * Revision 1.13  1997/03/10 18:39:04  oliver
 * add/removeGroup Buttons now work again
 * changeSettings now also for multiple Groups
 *
 * Revision 1.12  1997/03/02 14:11:08  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.11  1997/03/01 18:43:42  oliver
 * tried, but threads don't do well
 *
 * Revision 1.10  1997/03/01 11:53:24  oliver
 * added hostcnr
 *
 * Revision 1.9  1997/02/18 16:14:16  oliver
 * added helpwindow
 *
 * Revision 1.8  1997/02/14 17:25:10  oliver
 * added button and menu for expiry settings
 *
 * Revision 1.7  1997/02/14 15:03:57  oliver
 * adding a host is now refued, if the host already exists in the list
 *
 * Revision 1.5  1997/02/14 12:17:08  oliver
 * added new generic OkCancelDialog
 * addHost now refuses to add a host with name
 *
 * Revision 1.3  1997/02/13 19:33:49  oliver
 * Buttons now have noPointerFocus Style;
 * Button commands use cursoredObject() instead of iterating through the
 * whole list
 *
 * Revision 1.1  1997/02/13 13:58:53  oliver
 * initial revision
 *
 */


