#ifndef _CHANGI_L_
#define _CHANGI_L_

/*
 * $Id: changi_l.hpp,v 1.10 1997/04/05 16:56:59 oliver Exp $
 */

// Align classes on four byte boundary.
#pragma pack(4)

#include <iframe.hpp>
#include <iinfoa.hpp>
#include <imenubar.hpp>
#include <icmdhdr.hpp>
#include <icnrctl.hpp>
#include <inotebk.hpp>
#include <imcelcv.hpp>
#include <isplitcv.hpp>
#include <ipushbut.hpp>
#include <ipagehdr.hpp>
#include <iframhdr.hpp>
#include <ihelp.hpp>

#include "hostcnr.hpp"
#include "groupcnr.hpp"

class HostCnrObj;

//**************************************************************************
//   Class ChangiLWindow -- Main Frame Window
//**************************************************************************

class ChangiLWindow : public IFrameWindow
                    , public ICommandHandler
                    , public IPageHandler
                    , public IFrameHandler
                    , protected ICnrHandler
{
  public:
    static ChangiLWindow* getMainWindow()
       {
       if (!mainWindow) mainWindow = new ChangiLWindow();
       return mainWindow;
       };
    static AllGroupCnr& getAllGroupsCnr() { return getMainWindow()->allGroupsContainer; }
    static ICnrCtl& getSubGroupsCnr() { return getMainWindow()->subGroupsContainer; }
    static ICnrCtl& getNewGroupsCnr() { return getMainWindow()->newGroupsContainer; }
    static ICnrCtl& getHostCnr()      { return getMainWindow()->hostCnr; }

    virtual ~ChangiLWindow();

    enum NoteBkState {ALLPAGE, SUBPAGE, NEWPAGE};
    NoteBkState noteBkState() { return noteBkState_; }

    void loadHosts() { hostCnr.loadHosts(); }

    void subscribeSelectedGroup(IBoolean doSubscr = true);
    void changeSelectedGroup();

  protected:
    virtual Boolean command(ICommandEvent& cmdEvent);
    virtual Boolean select(IPageSelectEvent& event);
    virtual Boolean closed(IFrameEvent& frameEvent);
    void initNotebook();

  private:
    ChangiLWindow();

    IMenuBar          menuBar;
    ISplitCanvas      clientArea;
    HostCnr           hostCnr;
    INotebook         groupNoteBk;
    AllGroupCnr       allGroupsContainer;
    IPageHandle       allGroupsPageHandle;
    GroupCnr          subGroupsContainer;
    IPageHandle       subGroupsPageHandle;
    GroupCnr          newGroupsContainer;
    IPageHandle       newGroupsPageHandle;
    IInfoArea         infoArea;
    NoteBkState       noteBkState_;

    IHelpWindow*      helpWindow;
    Boolean           hasHelp;

    ChangiLWindow &operator=(const ChangiLWindow&);
    ChangiLWindow(const ChangiLWindow&);
    static ChangiLWindow*    mainWindow;
};

// Resume compiler default packing.
#pragma pack()

/*
 * $Log: changi_l.hpp,v $
 * Revision 1.10  1997/04/05 16:56:59  oliver
 * various bugfixes
 *
 * Revision 1.9  1997/03/19 14:41:33  oliver
 * added caching to allGroupCnr
 *
 * Revision 1.8  1997/03/18 16:12:40  oliver
 * bugfixes
 * menu changet to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.7  1997/03/10 18:39:05  oliver
 * add/removeGroup Buttons now work again
 * changeSettings now also for multiple Groups
 *
 * Revision 1.6  1997/03/02 14:11:09  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.5  1997/03/01 11:53:24  oliver
 * added hostcnr
 *
 * Revision 1.4  1997/02/18 16:14:18  oliver
 * added helpwindow
 *
 * Revision 1.3  1997/02/14 17:25:11  oliver
 * added button and menu for expiry settings
 *
 * Revision 1.2  1997/02/14 14:44:35  oliver
 * added errormessages to resourcefile
 * now asking for confirmation if config is not saved before sucking articles
 *
 * Revision 1.1  1997/02/13 13:58:55  oliver
 * initial revision
 */

#endif
