const char cnrobj_rcsid[] =
"$Id: cnrobj.cpp,v 1.10 1997/04/05 16:57:00 oliver Exp $";

#include <icnrctl.hpp>
#include <icnrolst.hpp>
#include <imsgbox.hpp>
#include <itrace.hpp>

#include "cnrobj.hpp"
#include "hostcnrobj.hpp"
#include "changi_l.h"
#include "changi_l.hpp"
#include "expiresettings.hpp"

#define Message(x) IMessageBox(0).show(x,IMessageBox::informationIcon | IMessageBox::okButton)

//****************************************************************************
//   GroupCnrObj
//****************************************************************************

GroupCnrObj::GroupCnrObj(const IString& groupName, HostCnrObj* host, NGLIST* group, WMLINK* subscribed)
   : IContainerObject(groupName)
   , description_(group?group->description:"")
   , group_(group)
   , subscribed_(subscribed)
   , host_(host)
   {
   IFUNCTRACE_ALL();
   update();
   }

GroupCnrObj::~GroupCnrObj()
   {
   IFUNCTRACE_ALL();
   }

void GroupCnrObj::update()
   {
   if (isSubscribed())
      setIcon(ICN_SUBSGRP);
   else
      setIcon(ICN_GROUP);
   }

void GroupCnrObj::subscribe()
   {
   if (host_->getSubscription(iconText()) == HostCnrObj::NOT_SUBSCR)
      {
      host_->subGroupsRoot = add_wmlink(host_->subGroupsRoot, iconText(), &subscribed_);
      update();
      // add Object to subGroupsCnr_;
      ChangiLWindow::getSubGroupsCnr().addObject(this);
      }
   host_->markDirty();
   }

void GroupCnrObj::unsubscribe()
   {
   if (host_->getSubscription(iconText()) == HostCnrObj::THIS_HOST)
      {
      host_->subGroupsRoot = del_wmlink(host_->subGroupsRoot, iconText());
      subscribed_ = NULL;
      update();
      // remove Object from subGroupsCnr_:
      ChangiLWindow::getSubGroupsCnr().removeObject(this);
      }
   host_->markDirty();
   }

void GroupCnrObj::handleOpen(IContainerControl* container)
   {
   IFUNCTRACE_DEVELOP();
   if (container->id() == WND_SUBGROUPSCONT)
      {
      if (subscribed_)
         new ExpireSettingsOnce(ChangiLWindow::getMainWindow(), *this)->showModally();
      }
   else // All- oder NewGroupsCnr
      {
      IString otherHost;
      HostCnrObj::Subscription subscr;
      subscr = host_->getSubscription(iconText(), otherHost);

      switch(subscr)
         {
         case HostCnrObj::NOT_SUBSCR:
            {
            ITRACE_DEVELOP("is NOT_SUBSCR");
            subscribe();
            break;
            }
         case HostCnrObj::THIS_HOST:
            {
            ITRACE_DEVELOP("is THIS_HOST");
            unsubscribe();
            break;
            }
         case HostCnrObj::OTHER_HOST:
            {
            ITRACE_DEVELOP("is OTHER_HOST");
            IString msg =
               IApplication::current().userResourceLibrary().loadString(ERR_ALREADY_SUBSCR1)
               + otherHost
               + IApplication::current().userResourceLibrary().loadString(ERR_ALREADY_SUBSCR2);
            IMessageBox(container)
               .show(msg, IMessageBox::errorIcon | IMessageBox::okButton);
            break;
            }
         default:
            ITRACE_RUNTIME("SEVERE ERROR: DEFAULT CALLED ON SWITCH(SUBSCRIBED)!");
            ;
         }
      }
   container->refresh();
   }

Expiration GroupCnrObj::expiration()
   {
   return subscribed_ ? Expiration(subscribed_) : Expiration();
   }

void GroupCnrObj::expiration(const Expiration& exp)
   {
   if (subscribed_)
      {
      subscribed_->days_keep = exp.days;
      subscribed_->arts_keep = exp.arts;
      subscribed_->size_keep = exp.size;
      }
   }

/*
 * $Log: cnrobj.cpp,v $
 * Revision 1.10  1997/04/05 16:57:00  oliver
 * various bugfixes
 *
 * Revision 1.9  1997/03/10 18:39:05  oliver
 * add/removeGroup Buttons now work again
 * changeSettings now also for multiple Groups
 *
 * Revision 1.8  1997/03/02 14:11:09  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.6  1997/03/01 11:53:25  oliver
 * added hostcnr
 *
 * Revision 1.5  1997/02/18 16:14:20  oliver
 * added helpwindow
 *
 * Revision 1.4  1997/02/14 17:01:50  oliver
 * added dialog for expiry options
 *
 * Revision 1.3  1997/02/14 14:44:37  oliver
 * added errormessages to resourcefile
 * now asking for confirmation if config is not saved before sucking articles
 *
 * Revision 1.2  1997/02/13 19:19:26  oliver
 * allGroupsContainer now recognizes subsription status
 *
 * Revision 1.1  1997/02/13 13:58:57  oliver
 * initial revision
 *
 */


