#ifndef GROUPCNROBJ_HPP
#define GROUPCNROBJ_HPP

/*
 * $Id: cnrobj.hpp,v 1.9 1997/04/05 16:57:01 oliver Exp $
 */

// Align classes on four byte boundary.
#pragma pack(4)

#include <icnrobj.hpp>
#include <icnrctl.hpp>
#include <iseq.h>

#include <wmlink.h>
#include <nglist.h>

class HostCnrObj;

class Expiration
{
 public:
   Expiration() : days(0), arts(0), size(0) {};
   Expiration(unsigned long d, unsigned long a, unsigned long s)
      : days(d), arts(a), size(s) {}
   Expiration(WMLINK* wmlink)
      : days(wmlink->days_keep)
      , arts(wmlink->arts_keep)
      , size(wmlink->size_keep)
      {}
   operator==(const Expiration& rhs)
      { return (days==rhs.days && arts==rhs.arts && size==rhs.size); }
   operator!=(const Expiration& rhs)
      { return (!operator==(rhs)); }

   unsigned long days;
   unsigned long arts;
   unsigned long size;
};

//****************************************************************************
//   GroupCnrObj
//****************************************************************************

class GroupCnrObj : public ICnrObj
{
 public:
   GroupCnrObj(const IString& groupName, HostCnrObj* host, NGLIST* group = NULL, WMLINK* subscribed = NULL);
   virtual ~GroupCnrObj();

   virtual void handleOpen(IContainerControl* container);

   Boolean isSubscribed() { return subscribed_ ? true : false; }
   void update();
   void subscribe();
   void unsubscribe();
   Expiration expiration();
   void expiration(const Expiration& exp);

 private:
   const IString description_;
   NGLIST* group_;
   WMLINK* subscribed_;
   HostCnrObj* const host_;

   friend class HandleExpandThread;
   friend class AllGroupCnr;
};

// Resume compiler default packing.
#pragma pack()

/*
 * $Log: cnrobj.hpp,v $
 * Revision 1.9  1997/04/05 16:57:01  oliver
 * various bugfixes
 *
 * Revision 1.8  1997/03/10 18:39:06  oliver
 * add/removeGroup Buttons now work again
 * changeSettings now also for multiple Groups
 *
 * Revision 1.7  1997/03/02 14:11:10  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.6  1997/03/01 18:43:43  oliver
 * tried, but threads don't do well
 *
 * Revision 1.5  1997/03/01 11:53:25  oliver
 * added hostcnr
 *
 * Revision 1.4  1997/02/18 16:14:21  oliver
 * added helpwindow
 *
 * Revision 1.3  1997/02/14 14:44:38  oliver
 * added errormessages to resourcefile
 * now asking for confirmation if config is not saved before sucking articles
 *
 * Revision 1.2  1997/02/13 19:19:26  oliver
 * allGroupsContainer now recognizes subsription status
 *
 * Revision 1.1  1997/02/13 13:58:57  oliver
 * initial revision
 *
 */

#endif
