const char expiresettings_rcsid[] =
"$Id: expiresettings.cpp,v 1.5 1997/04/05 16:57:01 oliver Exp $";

#include <istring.hpp>
#include <itrace.hpp>
#include <ifont.hpp>

#include "expiresettings.hpp"
#include "cnrobj.hpp"
#include "hostcnrobj.hpp"

#include "changi_l.h" // Resource Definitions

//****************************************************************************
//   ExpireSettings
//****************************************************************************

ExpireSettings::ExpireSettings(IWindow* parent, const Expiration& exp)
   : OkCancelDialog(WND_EXPIRE_SETTINGS, parent,
      IApplication::current().userResourceLibrary().loadString(WND_EXPIRE_TITLE))
   , message_(WND_EXPIRE_MSG, &clientCanvas, this, IRectangle(),
      IStaticText::classDefaultStyle | IStaticText::wordBreak )
   , days_(WND_EXPIRE_DAYS, &clientCanvas, this, IRectangle(),
      IEntryField::classDefaultStyle | IControl::group | IControl::tabStop )
   , arts_(WND_EXPIRE_ARTS, &clientCanvas, this, IRectangle(),
      IEntryField::classDefaultStyle | IControl::tabStop )
   , size_(WND_EXPIRE_SIZE, &clientCanvas, this, IRectangle(),
      IEntryField::classDefaultStyle | IControl::tabStop )
   , daystext_(WND_EXPIRE_DAYS_TXT, &clientCanvas, this)
   , artstext_(WND_EXPIRE_ARTS_TXT, &clientCanvas, this)
   , sizetext_(WND_EXPIRE_SIZE_TXT, &clientCanvas, this)
   {
   IFUNCTRACE_DEVELOP();
#ifdef __WINDOWS__
   setFont(IFont("Helv",8));
#endif
   clientCanvas
      .addToCell(&message_,2,2,4,1)
      .addToCell(&daystext_,2,4,1,1)
      .addToCell(&artstext_,2,5,1,1)
      .addToCell(&sizetext_,2,6,1,1)
      .addToCell(&days_,4,4,1,1)
      .addToCell(&arts_,4,5,1,1)
      .addToCell(&size_,4,6,1,1)
      .setColumnWidth(5,0,true)
      .setColumnWidth(6,5,false)
      .setRowHeight(2,0,true)
      .setRowHeight(7,10,false)
      .setBackgroundColor(IColor::paleGray);
   message_
      .setText(IApplication::current().userResourceLibrary().loadString(WND_EXPIRE_MESSAGE))
      .setBackgroundColor(IColor::paleGray)
      .setMinimumSize(ISize(110,40));

   moveSizeToClient(IRectangle(IPoint(),clientCanvas.minimumSize()));
   moveTo(parent->position()+parent->size()/2-size()/2);

   days_.setLimit(4).setText(IString(exp.days));
   arts_.setLimit(6).setText(IString(exp.arts));
   size_.setLimit(6).setText(IString(exp.size));
   daystext_.setText(IApplication::current().userResourceLibrary().loadString(WND_EXPIRE_DAYS_TXT));
   artstext_.setText(IApplication::current().userResourceLibrary().loadString(WND_EXPIRE_ARTS_TXT));
   sizetext_.setText(IApplication::current().userResourceLibrary().loadString(WND_EXPIRE_SIZE_TXT));

   ICommandHandler::handleEventsFor(this);
   days_.setFocus();
   }

ExpireSettings::~ExpireSettings()
   {
   IFUNCTRACE_DEVELOP();
   ICommandHandler::stopHandlingEventsFor(this);
   }

//****************************************************************************
//   ExpireSettingsOnce
//****************************************************************************

ExpireSettingsOnce::ExpireSettingsOnce(IWindow* parent, GroupCnrObj& obj)
   : ExpireSettings(parent, obj.expiration())
   , obj_(obj)
   {}

void ExpireSettingsOnce::doIt()
   {
   Expiration entry(days_.text().asInt(), arts_.text().asInt(), size_.text().asInt());
   if (entry != obj_.expiration())
      {
      HostCnrObj::markDirty();
      obj_.expiration(entry);
      }
   }

//****************************************************************************
//   ExpireSettingsMany
//****************************************************************************

ExpireSettingsMany::ExpireSettingsMany(IWindow* parent, ICnrCtl& container, ICnrCtl::ObjectCursor& cursor)
   : ExpireSettings(parent, getDefault(container, cursor))
   , container_(container)
   , cursor_(cursor)
   {}

void ExpireSettingsMany::doIt()
   {
   HostCnrObj::markDirty();
   Expiration entry(days_.text().asInt(), arts_.text().asInt(), size_.text().asInt());
   forCursor(cursor_)
      ((GroupCnrObj*)container_.objectAt(cursor_))->expiration(entry);
   }

Expiration ExpireSettingsMany::getDefault
   (ICnrCtl& container, ICnrCtl::ObjectCursor& cursor)
   {
   unsigned long objCount = 0;
   unsigned long daysCount = 0;
   unsigned long artsCount = 0;
   unsigned long sizeCount = 0;
   unsigned long daysSum = 0;
   unsigned long artsSum = 0;
   unsigned long sizeSum = 0;

   forCursor(cursor)
      {
      objCount++;
      Expiration actual(((GroupCnrObj*)container.objectAt(cursor))->expiration());
      if (actual.days) { daysCount++; daysSum += actual.days; }
      if (actual.arts) { artsCount++; artsSum += actual.arts; }
      if (actual.size) { sizeCount++; sizeSum += actual.size; }
      }
   return Expiration(
      // > impies >0 for the division.
      daysCount > objCount/2 ? daysSum/daysCount : 0,
      artsCount > objCount/2 ? artsSum/artsCount : 0,
      sizeCount > objCount/2 ? sizeSum/sizeCount : 0
      );
   }

/*
 * $Log: expiresettings.cpp,v $
 * Revision 1.5  1997/04/05 16:57:01  oliver
 * various bugfixes
 *
 * Revision 1.4  1997/03/18 16:12:42  oliver
 * bugfixes
 * menu changet to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.3  1997/03/10 18:39:06  oliver
 * add/removeGroup Buttons now work again
 * changeSettings now also for multiple Groups
 *
 * Revision 1.2  1997/03/02 14:11:10  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.1  1997/02/14 17:01:52  oliver
 * added dialog for expiry options
 *
 */


