const char groupcnr_rcsid[] =
"$Id: groupcnr.cpp,v 1.12 1997/04/05 16:57:03 oliver Exp $";

#include <istparse.hpp>
#include <icnrolst.hpp>
#include <icnrcol.hpp>
#include <imsgbox.hpp>
#include <itrace.hpp>
#include <ireslib.hpp>
#include <ifont.hpp>

#include "GroupCnr.hpp"
#include "CnrObj.hpp"
#include <wmlink.h>

#define CACHE_DELTA 100
#define CACHE_MULTIPLY 5

//****************************************************************************
//   GroupCnr
//****************************************************************************

GroupCnr::GroupCnr(unsigned long id, IWindow* parent, IWindow* owner)
   : IContainerControl(id, parent, owner, IRectangle()
                       , IContainerControl::classDefaultStyle
#ifdef __WINDOWS__
                       | IContainerControl::pmCompatible
#endif
                       , IContainerControl::miniIcons
                       | IContainerControl::orderedTargetEmphasis
                       | IContainerControl::detailsViewTitles
                       | IContainerControl::titleSeparator
//#ifdef __OS2__
//                       | IContainerControl::nameView
//#else
                       | IContainerControl::detailsView
//#endif
                       )
   , popUp(MI_GROUP_POPUP, this)
   {
   IFUNCTRACE_DEVELOP();
   ICnrHandler::handleEventsFor(this);
   ICnrMenuHandler::handleEventsFor(this);
   setFont(IFont("Helv",8));
   setExtendedSelection();
   setLineSpacing(0);

   if (id==WND_SUBGROUPSCONT)
      popUp.disableItem(MI_ADDGROUP);
   else
      popUp.disableItem(MI_CHANGEGROUP);

   groupIconColumn = new IContainerColumn(IContainerColumn::isIcon);
   addColumn(groupIconColumn);

   groupNameColumn = new IContainerColumn(IContainerColumn::isIconViewText);
   groupNameColumn->setDisplayWidth(400);
   groupNameColumn->setHeadingText(IResourceId(STR_NEWSGROUPS));
   addColumn(groupNameColumn);
   }

GroupCnr::~GroupCnr()
   {
   IFUNCTRACE_DEVELOP();
   removeColumn(groupNameColumn);
   removeColumn(groupIconColumn);
   delete groupNameColumn;
   delete groupIconColumn;
   ICnrMenuHandler::stopHandlingEventsFor(this);
   ICnrHandler::stopHandlingEventsFor(this);
   }

Boolean GroupCnr::makePopUpMenu(IMenuEvent& menuEvent)
   {
   IFUNCTRACE_DEVELOP();
   popUp.show(menuEvent.mousePosition());
   return true;
   }

//****************************************************************************
//   AllGroupCnr
//****************************************************************************

AllGroupCnr::AllGroupCnr(unsigned long id, IWindow* parent, IWindow* owner)
   : GroupCnr(id, parent, owner)
   , cache_offset(1)
   {
   IFUNCTRACE_DEVELOP();
   enableCaching(CACHE_DELTA);
   groupDiscrColumn = new IContainerColumn(offsetof(GroupCnrObj, description_));
   groupDiscrColumn->setDisplayWidth(400);
   groupDiscrColumn->setHeadingText(IResourceId(STR_DESCRIPTION));
   addColumn(groupDiscrColumn);
   setDetailsViewSplit(groupNameColumn, 300);
   showSplitBar();

   ICnrHandler::handleEventsFor(this);
   }

AllGroupCnr::~AllGroupCnr()
   {
   IFUNCTRACE_DEVELOP();
   ICnrHandler::stopHandlingEventsFor(this);
   removeColumn(groupDiscrColumn);
   delete groupDiscrColumn;
   }

Boolean AllGroupCnr::deltaReached(ICnrQueryDeltaEvent& event)
   {
   IFUNCTRACE_DEVELOP();
   setRefreshOff();

   if (event.atTopDelta())
      cacheTo(cache_offset>CACHE_DELTA ? cache_offset-CACHE_DELTA : 1);

   else if (event.atBottomDelta())
      //      old index    number in cnr              new to add = last Element behind bounds?
      cacheTo(cache_offset+CACHE_DELTA*CACHE_MULTIPLY+CACHE_DELTA < numberOfElements?
         cache_offset+CACHE_DELTA : numberOfElements-CACHE_DELTA*CACHE_MULTIPLY);

   else if (event.atFirstObject())
      cacheTo(1);

   else if (event.atLastObject())
      cacheTo(numberOfElements-CACHE_DELTA*CACHE_MULTIPLY+1 >1?
              numberOfElements-CACHE_DELTA*CACHE_MULTIPLY+1 : 1);

   setRefreshOn(); refresh();

   if (event.atLastObject())
      {
      ObjectCursor cursor(*this);
      cursor.setToLast();
      ICnrHandler::stopHandlingEventsFor(this);
      scrollToObject(objectAt(cursor));
      ICnrHandler::handleEventsFor(this);
      }

   return true;
   }

void AllGroupCnr::cacheTo(unsigned long new_offset)
   {
   IFUNCTRACE_DEVELOP();
   ITRACE_DEVELOP("cache_offset:"+IString(cache_offset)+" new_offset:"+IString(new_offset));
   if (abs(new_offset-cache_offset) >= CACHE_DELTA*CACHE_MULTIPLY)
      // completely other area
      {
      ITRACE_DEVELOP("completly other area");
      removeAllObjects();
      unsigned long max_obj = new_offset+CACHE_DELTA*CACHE_MULTIPLY-1 < numberOfElements ?
                              new_offset+CACHE_DELTA*CACHE_MULTIPLY-1 : numberOfElements;
      ITRACE_DEVELOP("adding Elements from "+IString(new_offset)+" to "+IString(max_obj));
      for (unsigned long i=new_offset; i<=max_obj; i++)
         addObject(groupList.elementAtPosition(i));
      }
   else
      // parts of new area already in container
      {
      ITRACE_DEVELOP("partially other area");
      ObjectCursor cursor(*this);
      if (new_offset > cache_offset)
         {
         unsigned long new_objs = new_offset-cache_offset;
         ITRACE_DEVELOP("adding Elements from "+IString(new_offset+CACHE_MULTIPLY*CACHE_DELTA-new_objs+1)
                       +" to "+IString(new_offset+CACHE_MULTIPLY*CACHE_DELTA));
         for (unsigned long i=1; i<=new_objs; i++)
            try
               {
               addObject(groupList.elementAtPosition(new_offset+CACHE_MULTIPLY*CACHE_DELTA-new_objs+i));
               cursor.setToFirst();
               removeObject(objectAt(cursor));
               }
            catch (IException&) {}
         }
      else if (new_offset < cache_offset)
         {
         unsigned long new_objs = cache_offset-new_offset;
         ITRACE_DEVELOP("adding Elements from "+IString(cache_offset-1)
                       +" to "+IString(cache_offset-new_objs));
         for (unsigned long i=1; i<=new_objs; i++)
            try
               {
               addObjectAfter(groupList.elementAtPosition(cache_offset-i), 0,0);
               cursor.setToLast();
               removeObject(objectAt(cursor));
               }
            catch (IException&) {}
         }
      }

   cache_offset = new_offset;
   }

void AllGroupCnr::setGroupList(const ISequence<GroupCnrObj*> & newList)
   {
   IFUNCTRACE_DEVELOP();
   groupList.removeAll();
   ITRACE_DEVELOP("removed all Elements");
   groupList.addAllFrom(newList);
   numberOfElements = groupList.numberOfElements();
   ITRACE_DEVELOP("added "+IString(numberOfElements)+" new Elements");
   removeAllObjects();
   unsigned long maxObj = numberOfElements < CACHE_DELTA*CACHE_MULTIPLY ?
      numberOfElements : CACHE_DELTA*CACHE_MULTIPLY;
   for (unsigned long i=1; i<=maxObj; i++)
      addObject(groupList.elementAtPosition(i));
   }

/*
 * $Log: groupcnr.cpp,v $
 * Revision 1.12  1997/04/05 16:57:03  oliver
 * various bugfixes
 *
 * Revision 1.11  1997/03/19 14:41:35  oliver
 * added caching to allGroupCnr
 *
 * Revision 1.10  1997/03/18 17:35:26  harald
 * added titleSeparator to groupCnr
 *
 * Revision 1.9  1997/03/18 16:12:42  oliver
 * bugfixes
 * menu changed to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.8  1997/03/17 14:28:02  harald
 * changes to get os/2-version running
 *
 * Revision 1.7  1997/03/17 12:36:14  oliver
 * added popupmenus
 *
 * Revision 1.6  1997/03/02 14:11:11  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.5  1997/03/01 18:43:44  oliver
 * tried, but threads don't do well
 *
 * Revision 1.4  1997/03/01 11:53:26  oliver
 * added hostcnr
 *
 * Revision 1.3  1997/02/18 16:14:22  oliver
 * added helpwindow
 *
 * Revision 1.2  1997/02/13 19:19:27  oliver
 * allGroupsContainer now recognizes subsription status
 *
 * Revision 1.1  1997/02/13 13:58:59  oliver
 * initial revision
 *
 */


