const char hostcnr_rcsid[] =
"$Id: hostcnr.cpp,v 1.9 1997/04/05 16:57:03 oliver Exp $";

#include <istparse.hpp>
#include <icnrolst.hpp>
#include <imsgbox.hpp>
#include <itrace.hpp>
#include <ipopmenu.hpp>

#include "hostcnr.hpp"
#include "changi_l.h"
#include "hostcnrobj.hpp"
#include "addhostwindow.hpp"
#include "sucklistthread.hpp"
#include "suckartsthread.hpp"

HostCnr::HostCnr(unsigned long id, IWindow* parent, IWindow* owner)
   : IContainerControl(id, parent, owner, IRectangle()
                       , IContainerControl::classDefaultStyle
                       | IContainerControl::autoPosition
//                       | IContainerControl::pmCompatible
                       , IContainerControl::visibleTitle
                       | IContainerControl::titleSeparator
                       | IContainerControl::orderedTargetEmphasis
                       | IContainerControl::readOnlyTitle
                       | IContainerControl::iconView
                      )
   {
   IFUNCTRACE_DEVELOP();

   ICommandHandler::handleEventsFor(this);
   ICnrHandler::handleEventsFor(this);
   ICnrMenuHandler::handleEventsFor(this);
   }

HostCnr::~HostCnr()
   {
   IFUNCTRACE_DEVELOP();
   ICnrMenuHandler::stopHandlingEventsFor(this);
   ICnrHandler::stopHandlingEventsFor(this);
   ICommandHandler::stopHandlingEventsFor(this);
   }

Boolean HostCnr::makePopUpMenu(IMenuEvent& menuEvent)
   {
   IPopUpMenu* pop;
   if (popupMenuObject())
      pop = new IPopUpMenu(MI_HOST_POPUP, this);
   else
      pop = new IPopUpMenu(MI_HOSTCNR_POPUP, this);

   pop->setAutoDeleteObject();
   pop->show(menuEvent.mousePosition());

   return true;
   }

void HostCnr::loadHosts()
   {
   IFUNCTRACE_DEVELOP();
   hostsRoot = load_rmtlink();
   for(RMTLINK* iter = hostsRoot; iter; iter = iter->next)
      new HostCnrObj(iter->name);
   sortByIconText();
   }

void HostCnr::addHost()
   {
   IFUNCTRACE_DEVELOP();
   new AddHostWindow(this)->show();
   }

void HostCnr::addHost(const IString& hostName)
   {
   IFUNCTRACE_DEVELOP();

   if (hostName.length())
      {
      // check if hostname is already there
      for (RMTLINK* iter = hostsRoot; iter; iter = iter->next)
         if (hostName == IString(iter->name))
            {
            IMessageBox(this).show(
               IApplication::current().userResourceLibrary().loadString(ERR_HOST_EXISTS),
               IMessageBox::errorIcon | IMessageBox::okButton );
            return;
            }

      hostsRoot = add_rmtlink(hostsRoot, hostName);
      new HostCnrObj(hostName);
      HostCnrObj::markDirty();
      sortByIconText();
      }
   }

void HostCnr::removeHost(HostCnrObj* host)
   {
   IFUNCTRACE_DEVELOP();
   if (host)
      {
      if (IMessageBox(this).show(
             IApplication::current().userResourceLibrary().loadString(STR_REMHOST_MESSAGE)
             + host->iconText(),
             IMessageBox::queryIcon
             | IMessageBox::yesNoButton)
          == IMessageBox::yes )
         {
         hostsRoot = del_rmtlink(hostsRoot, host->iconText());
         HostCnrObj::deleteHostCnrObj(host);
         HostCnrObj::markDirty();
         sortByIconText();
         }
      }
   else ITRACE_DEVELOP("parameter host==NULL");
   }

void HostCnr::suckListForHost(HostCnrObj* host)
   {
   if (host) IThread suck( new SuckListThread(host), true);
   else ITRACE_DEVELOP("parameter host==NULL");
   }

void HostCnr::suckArtsForHost(HostCnrObj* host)
   {
   if (host) SuckArticlesThread::startThread(this, host);
   else ITRACE_DEVELOP("parameter host==NULL");
   }

HostCnrObj* HostCnr::selectedHost()
   {
   IFUNCTRACE_DEVELOP();
   ICnrCtl::ObjectCursor cursor(*this, ICnrObj::selected);
   cursor.setToFirst();
   if (cursor.isValid())
      return (HostCnrObj*)objectAt(cursor);
   return NULL;
   }

HostCnrObj* HostCnr::cursoredHost()
   {
   IFUNCTRACE_DEVELOP();
   ICnrCtl::ObjectCursor cursor(*this, ICnrObj::cursored);
   cursor.setToFirst();
   if (cursor.isValid())
      return (HostCnrObj*)objectAt(cursor);
   return NULL;
   }

void HostCnr::saveHostConfig()
   {
   HostCnrObj::saveConfigs();
   save_rmtlink(hostsRoot);
   }

Boolean HostCnr::configDirty()
   { return HostCnrObj::isDirty(); }

/*
 * $Log: hostcnr.cpp,v $
 * Revision 1.9  1997/04/05 16:57:03  oliver
 * various bugfixes
 *
 * Revision 1.8  1997/03/19 14:41:36  oliver
 * added caching to allGroupCnr
 *
 * Revision 1.7  1997/03/18 17:35:27  harald
 * added titleSeparator to groupCnr
 *
 * Revision 1.6  1997/03/18 16:12:43  oliver
 * bugfixes
 * menu changet to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.5  1997/03/17 14:28:04  harald
 * changes to get os/2-version running
 *
 * Revision 1.4  1997/03/17 12:36:15  oliver
 * added popupmenus
 *
 * Revision 1.3  1997/03/02 14:11:11  Oliver
 * removed references to containers from HostCnrObj
 * fixed getSelectedHost to search hostCnr
 *
 * Revision 1.2  1997/03/01 18:43:45  oliver
 * tried, but threads don't do well
 *
 * Revision 1.1  1997/03/01 11:53:26  oliver
 * added hostcnr
 *
 */


