const char hostcnrobj_rcsid[] =
"$Id: hostcnrobj.cpp,v 1.7 1997/04/05 16:57:04 oliver Exp $";

#include <icnrctl.hpp>
#include <imsgbox.hpp>
#include <itrace.hpp>

#include "cnrobj.hpp"
#include "changi_l.h"
#include "changi_l.hpp"
#include "suckwindow.hpp"
#include "expiresettings.hpp"
#include "messagewnd.hpp"
#include "..\include\suckarts.h"
#include "..\include\sysdep.h"

#include "hostcnrobj.hpp"

#define Message(x) IMessageBox(0).show(x,IMessageBox::informationIcon | IMessageBox::okButton)

//****************************************************************************
//   HostCnrObj
//****************************************************************************

HostCnrObj::HostCnrObj(const IString& hostName)
   : IContainerObject(hostName, ICN_HOST)
   , completelyRead(false)
   , allGroupsRoot(NULL)
   , subGroupsRoot(NULL)
   , newGroupsRoot(NULL)
   {
   IFUNCTRACE_DEVELOP();
   ChangiLWindow::getHostCnr().addObject(this);
   hostList.add(this);
   }

HostCnrObj::~HostCnrObj()
   {
   IFUNCTRACE_DEVELOP();

   if (allGroupsRoot)
      free_nglist(allGroupsRoot);
   if (subGroupsRoot)
      free_wmlink(subGroupsRoot);
   if (newGroupsRoot)
      free_nglist(newGroupsRoot);

   ChangiLWindow::getHostCnr().removeObject(this, true);

   ISequence<HostCnrObj*>::Cursor cursor(hostList);
   forCursor(cursor)
      if (hostList.elementAt(cursor) == this)
         {
         hostList.removeAt(cursor);
         break;
         }
   }

void HostCnrObj::saveConfigs()
   {
   IFUNCTRACE_DEVELOP();
   ISequence<HostCnrObj*>::Cursor cursor(hostList);
   forCursor(cursor)
      if (hostList.elementAt(cursor)->dirty)
         {
         save_wmlink(hostList.elementAt(cursor)->iconText(),
                     hostList.elementAt(cursor)->subGroupsRoot);
         }
   dirty = false;
   }

void HostCnrObj::deleteHostCnrObj(HostCnrObj* host, IBoolean save)
   {
   IFUNCTRACE_DEVELOP();
   if (save)
      save_wmlink(host->iconText(), host->subGroupsRoot);
   delete host;
   }

HostCnrObj::Subscription HostCnrObj::getSubscription(const IString& groupName, IString& otherHost)
   {
   for(WMLINK* iter = subGroupsRoot; iter; iter=iter->next)
      if (groupName == iter->group)
         return THIS_HOST;

   ISequence<HostCnrObj*>::Cursor cursor(hostList);
   forCursor(cursor)
      {
      HostCnrObj* host = hostList.elementAt(cursor);
      if (host == this) continue;
      if (!host->subGroupsRoot)
         host->subGroupsRoot = load_wmlink(host->iconText());

      for(WMLINK* iter = host->subGroupsRoot; iter; iter=iter->next)
         if (groupName == iter->group)
            {
            otherHost = host->iconText();
            return OTHER_HOST;
            }
      }

   return NOT_SUBSCR;
   }

ISequence<HostCnrObj*> HostCnrObj::hostList;
IString HostCnrObj::dummy;
IBoolean HostCnrObj::dirty = false;

void HostCnrObj::handleSelectedChange(IContainerControl*, Boolean acquired)
   {
   IFUNCTRACE_DEVELOP();

   // ### Warum kommt diese Benachrichtigung immer doppelt?
   static Boolean first = false;
   first = !first;

   if (first)
      if (acquired)
         {
         MessageWnd* message = new MessageWnd(ChangiLWindow::getMainWindow(),
            IApplication::current().userResourceLibrary().loadString(STR_LOADGROUPS));
         HandleExpandThread::stopThread();
         HandleExpandThread::startThread(*this, *message);
//       message->show();
         }
   }

//****************************************************************************
//   HandleExpandThread
//****************************************************************************

HandleExpandThread::HandleExpandThread(HostCnrObj& host, IFrameWindow& message)
   : IThreadFn()
   , host_(host)
   , message_(message)
   {
   IFUNCTRACE_DEVELOP();
   }

void HandleExpandThread::run()
   {
   IFUNCTRACE_DEVELOP();
   ChangiLWindow::getAllGroupsCnr().setRefreshOff().removeAllObjects();
   ChangiLWindow::getSubGroupsCnr().setRefreshOff().removeAllObjects();
   ChangiLWindow::getNewGroupsCnr().setRefreshOff().removeAllObjects();

   if (!host_.completelyRead)
      {
      // wenn schonmal ein Teil gelesen wurde, dann diesen wieder zerstoeren.
      if (!host_.allGroupList.isEmpty())
         host_.allGroupList.removeAll();

      if (!host_.subGroupsRoot)
         host_.subGroupsRoot = load_wmlink(host_.iconText());

      WMLINK* wmlink = host_.subGroupsRoot;

      if (!host_.allGroupsRoot) host_.allGroupsRoot = load_nglist(host_.iconText(), false);
      if (host_.allGroupsRoot)
         {
         for(int i=1; host_.allGroupsRoot[i].group; i++)
            {
            WMLINK* found = NULL;
            while (wmlink && IString(wmlink->group) < IString(host_.allGroupsRoot[i].group))
               wmlink = wmlink->next;
            if (wmlink && IString(wmlink->group) == IString(host_.allGroupsRoot[i].group))
               found = wmlink;

            GroupCnrObj* newObj = new GroupCnrObj(host_.allGroupsRoot[i].group, &host_,
                                                  &host_.allGroupsRoot[i], found);
            host_.allGroupList.add(newObj);
//            ChangiLWindow::getAllGroupsCnr().addObject(newObj);
            if (found)
               ChangiLWindow::getSubGroupsCnr().addObject(newObj);
            }
         }

      if (!host_.newGroupsRoot) host_.newGroupsRoot = load_nglist(host_.iconText(), true);
      if (host_.newGroupsRoot)
         {
         for(int i=1; host_.newGroupsRoot[i].group; i++)
            {
            WMLINK* found = NULL;
            while (wmlink && IString(wmlink->group) < IString(host_.newGroupsRoot[i].group))
               wmlink = wmlink->next;
            if (wmlink && IString(wmlink->group) == IString(host_.newGroupsRoot[i].group))
               found = wmlink;

            GroupCnrObj* newObj = new GroupCnrObj(host_.newGroupsRoot[i].group, &host_,
                                                  &host_.newGroupsRoot[i], found);
            host_.newGroupList.add(newObj);
            ChangiLWindow::getNewGroupsCnr().addObject(newObj);
            if (found)
               ChangiLWindow::getSubGroupsCnr().addObject(newObj);
            }
         }

      host_.completelyRead = true;
      }
   else
      {
      ISequence<GroupCnrObj*>::Cursor cursor(host_.allGroupList);
      GroupCnrObj* obj;
      forCursor(cursor)
         {
         obj = host_.allGroupList.elementAt(cursor);
         if (obj->subscribed_)
            ChangiLWindow::getSubGroupsCnr().addObject(obj);
         }
      }

   ChangiLWindow::getAllGroupsCnr().setGroupList(host_.allGroupList);

   ChangiLWindow::getSubGroupsCnr().sortByIconText();
   ChangiLWindow::getAllGroupsCnr().setRefreshOn().refresh();
   ChangiLWindow::getSubGroupsCnr().setRefreshOn().refresh();
   ChangiLWindow::getNewGroupsCnr().setRefreshOn().refresh();
// message_.dismiss();
   message_.hide();
   }

void HandleExpandThread::startThread(HostCnrObj& host, IFrameWindow& message)
   {
   IFUNCTRACE_DEVELOP();
   IThread thread(new HandleExpandThread(host, message), true);
   }

void HandleExpandThread::stopThread()
   {
   IFUNCTRACE_DEVELOP();
   }


/*
 * $Log: hostcnrobj.cpp,v $
 * Revision 1.7  1997/04/05 16:57:04  oliver
 * various bugfixes
 *
 * Revision 1.6  1997/03/19 14:41:36  oliver
 * added caching to allGroupCnr
 *
 * Revision 1.5  1997/03/18 16:12:43  oliver
 * bugfixes
 * menu changet to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.4  1997/03/17 12:36:15  oliver
 * added popupmenus
 *
 * Revision 1.3  1997/03/10 18:39:07  oliver
 * add/removeGroup Buttons now work again
 * changeSettings now also for multiple Groups
 *
 * Revision 1.2  1997/03/02 16:08:56  Oliver
 * please
 *
 * Revision 1.1  1997/03/02 14:14:15  Oliver
 * made separate file for hostCnrObj
 *
 */

