#ifndef HOSTCNROBJ_HPP
#define HOSTCNROBJ_HPP

/*
 * $Id: hostcnrobj.hpp,v 1.4 1997/04/05 16:57:05 oliver Exp $
 */

// Align classes on four byte boundary.
#pragma pack(4)

#include <icnrobj.hpp>
#include <icnrctl.hpp>
#include <ithread.hpp>
#include <itimer.hpp>
#include <iseq.h>

class IFrameWindow;

#include <wmlink.h>
#include <nglist.h>

//****************************************************************************
//   HostCnrObj
//****************************************************************************

class HostCnrObj : public ICnrObj
{
 public:
   HostCnrObj(const IString& hostName);
   virtual ~HostCnrObj();

   virtual void handleSelectedChange(IContainerControl* container, Boolean acquired);

   NGLIST* allGroupsRoot;
   NGLIST* newGroupsRoot;
   WMLINK* subGroupsRoot;

   static void markDirty() { dirty = true; }
   static Boolean isDirty() { return dirty; }

   static void deleteHostCnrObj(HostCnrObj* host, IBoolean save = false);
   static void saveConfigs();
   enum Subscription { NOT_SUBSCR, THIS_HOST, OTHER_HOST };
   Subscription getSubscription(const IString& hostName, IString& otherHost = dummy);

   friend class GroupCnrObj;

 private:
   static IBoolean dirty;

   Boolean completelyRead;

   ISequence<GroupCnrObj*> allGroupList;
   ISequence<GroupCnrObj*> newGroupList;

   static ISequence<HostCnrObj*> hostList;
   static IString dummy;

   friend class HandleExpandThread;
};


//****************************************************************************
//   HandleExpandThread
//****************************************************************************

class HandleExpandThread : public IThreadFn
{
 public:
   static void startThread(HostCnrObj& host, IFrameWindow& message);
   static void stopThread();
   virtual void run();
 private:
   HandleExpandThread(HostCnrObj& host, IFrameWindow& message);
   HostCnrObj& host_;
   IFrameWindow& message_;
};

// Resume compiler default packing.
#pragma pack()

/*
 * $Log: hostcnrobj.hpp,v $
 * Revision 1.4  1997/04/05 16:57:05  oliver
 * various bugfixes
 *
 * Revision 1.3  1997/03/18 16:12:44  oliver
 * bugfixes
 * menu changet to resource
 * help added
 * profile for Window-position added
 *
 * Revision 1.2  1997/03/17 12:36:16  oliver
 * added popupmenus
 *
 * Revision 1.1  1997/03/02 14:14:15  Oliver
 * made separate file for hostCnrObj
 *
 */

#endif
