const char art_read_rcsid[] = "$Id$";

/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <chlimits.h>

#include "article.h"

static KEYWORD key[] = {
    { "Approved"    , ID_Approved     },
    { "Control"     , ID_Control      },
    { "Date"        , ID_Date         },
    { "Distribution", ID_Distribution },
    { "Expires"     , ID_Expires      },
    { "Followup-To" , ID_Followup_To  },
    { "From"        , ID_From         },
    { "Keywords"    , ID_Keywords     },
    { "Lines"       , ID_Lines        },
    { "Message-ID"  , ID_Message_ID   },
    { "Newsgroups"  , ID_Newsgroups   },
    { "Organization", ID_Organization },
    { "Path"        , ID_Path         },
    { "References"  , ID_References   },
    { "Reply-To"    , ID_Reply_To     },
    { "Sender"      , ID_Sender       },
    { "Subject"     , ID_Subject      },
    { "Summary"     , ID_Summary      },
    { "X-Newsgroups", ID_X_Newsgroups },
    { "Xref"        , ID_Xref         }
};

static int nkeys = (sizeof(key) / sizeof(*key));

static int cmpkey(const void *line, const void *pkey);

/************************************************************************/
/*                                                                      */
/*  read_header                                                         */
/*                                                                      */
/*  Reads an article header from file fp into buf with size bufsiz.     */
/*  Note that buf may contain a previously read string.                 */
/*                                                                      */
/*  Returns number of bytes read or zero on any error.                  */
/*                                                                      */
/************************************************************************/
size_t _System ChanReadHeader(FILE *fp, char *buf, size_t *bufsiz, HEADER *header)
{
    size_t result = 0;
    size_t buflen = *bufsiz;
    size_t boff = strlen(buf);    /* Any previously read characters */
    char *line = buf;
    int  first = 1;
    int  cont;
    char *cp, *cp1, *cp2;
    int  nxheader = 0;
    int  i;

    *bufsiz = 0;

    /*
     * Initialize header storage
     */
    for(i = 0; i < nkeys; i++)
        header[key[i].idx].key = key[i].name;

    /*
     * Read all lines upto the first empty line and
     * interpret them as header lines.
     */
    while(buflen > boff) {
        if(fgets(line + boff, (int)(buflen - boff) + 1, fp) == NULL)
            break;

        /*
         * If linefeed is missing, either the input file
         * is broken or we ran out of space.
         *
         * 07/07/97 harald
         * Count <CR> in header lines.
         */
        if((cp = strchr(line, '\r')) == NULL)
            cp = strchr(line, '\n');
        else
            result++;
        if(cp == NULL) {
            result = 0;
            break;
        }

        /*
         * Chop off the line feed and check if this line is empty.
         */
        *cp = '\0';
        result += strlen(line) + 1 + boff;
        boff = 0;
        if(*line == '\0')
            break;

        /*
         * Continuation lines start with space or tab. Ignore
         * them until we found our first real header entry.
         */
        if((cont = (*line == ' ' || *line == '\t')) != 0 && first)
            continue;

        /*
         * Ignore 'From ' and 'Received: ' lines which may have been
         * left over by some mailers.
         */
        if(!strnicmp(line, "From ", 5) || !strnicmp(line, ">From ", 6))
            continue;
        if(!strnicmp(line, "Received: ", 10))
            continue;

        /*
         * Pack the line by removing extra spaces and control characters.
         */
        cp1 = line - 1;
        cp2 = line;
        while(*cp2) {
#if 0
            if(*cp2 == ' ' || *cp2 == '\t') {
                if(*cp1 && *cp1 != ' ' && *cp1 != '\t')
                    *++cp1 = ' ';
            }
            else if(!iscntrl(*cp2))
                *++cp1 = *cp2;
            cp2++;
#else
            *++cp1 = *cp2++;
#endif
        }
        *(cp1 + 1) = '\0';
#if 0
        while(*cp1 == ' ')
            *cp1-- = '\0';
#endif

        /*
         * Get length here because x-headers will insert
         * a string delimiter
         */
        first = (int)strlen(line) + 1;

        /*
         * On continuation lines we will change the string delimiter
         * of our previous line to a linefeed.
         */
        if(cont) {
            *(line - 1) = '\n';
            cont = 0;
        }
        else {
            if((cp = strchr(line, ':')) != NULL) {
                KEYWORD *pkey;
                char *info = cp + 1;

                *cp = '\0';
                pkey = bsearch(line, key, (size_t)nkeys, sizeof(KEYWORD), cmpkey);
                while(*info == ' ' || *info == '\t')
                    info++;
                if(pkey)
                    header[pkey->idx].info = info;
                else if(nxheader < MAX_XHEADER) {
                    header[ID_LASTENTRY + nxheader].key = line;
                    header[ID_LASTENTRY + nxheader++].info = info;
                }
            }
        }

        buflen -= first;
        *bufsiz += first;
        first = 0;
        line = cp1 + 2;
    }
    header[ID_LASTENTRY + nxheader].key = NULL;

    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int _System ChanReadBody(FILE *fp, BODY **body, size_t nbytes)
{
    int  nlines = 0;
    size_t cc;
    char *line = malloc(MAX_ARTLINE);
    char *cp;
    BODY root = { NULL, NULL };
    BODY *bp = &root;

    while(fgets(line, MAX_ARTLINE, fp)) {
        cc = strlen(line);
        cp = line + cc;

        /*
         * 12/17/96 harald
         * Bug corrected: If count <CR><LF> as a single character.
         */
        if(*(--cp) == '\n' || *cp == '\r') {
            cc--;
            *cp = '\0';
            while(*(--cp) == '\n' || *cp == '\r')
                *cp = '\0';
        }
        nlines++;
        bp->next = malloc(sizeof(BODY));
        bp = bp->next;
        bp->next = NULL;
        bp->line = strdup(line);
        if(nbytes) {
            if(nbytes <= ++cc)
                break;
            nbytes -= cc;
        }
    }
    *body = root.next;
    free(line);

    return(nlines);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int cmpkey(const void *line, const void *pkey)
{
    /*
     * Changed from strcmp to stricmp. A problem occurs on
     * some messages imported from maus-net, which contain
     * a lower case 'd' in the Message-ID header. Rnews
     * added a new id, while Changi, case-insensitive,
     * rejected the article because of two Message-ID
     * headers.
     * Now, Rnews and Changi are both case-insensitive.
     */
    return(stricmp((char *)line, ((KEYWORD *)pkey)->name));
}
