const char article_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#ifdef __OS2__
#include <pipe.h>
#endif

#include <chlimits.h>
#include <xfile.h>

#include "config.h"
#include "article.h"
#include "rnews.h"

static int compressproc(void *rbv, FILE *fp, char *cmd, char *buf,
                         size_t buflen, size_t cc);
static int batchproc(void *rbv, FILE *fp, char *buf, size_t buflen, size_t art_size);
static int singleproc(void *rbv, FILE *fp, char *buf, size_t buflen, size_t art_size);

/************************************************************************/
/*                                                                      */
/* artfileproc                                                          */
/*                                                                      */
/* Processes an article file.                                           */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Filepointer to the article file.                        */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
int artfileproc(void *rbv, FILE *fp)
{
    int result = -1;
    size_t buflen = PROCBUFSIZE;
    char *buf = malloc(buflen);

    /*
     * Read the first two byte to determine the type of this article file
     */
    if(fread(buf, 1, 2, fp) == 2) {
        if(DOLOG(LOG_READ))
            syslog(LOG_DEBUG, "Read 2 bytes from article: x'%02x x'%02x",
                    (unsigned char)buf[0], (unsigned char)buf[1]);

        /*
         * If the first two bytes are "#!" then we found a batch
         */
        if(*buf == '#' && *(buf + 1) == '!') {
            /*
             * Read the rest of the first line to distinguish between
             * compressed and non-compressed batches.
             */
            if(fgets(buf, (int)buflen, fp) == NULL)
                syslog(LOG_ERR, "Failed to read rest of #! line");

            else if(strnicmp(buf + 2, "unbatch", 7) == 0) {
                /*
                 * We found a compressed batch. At this point
                 * the first line of the file was read and the
                 * rest contains the pure compressed batch. With
                 * this we recursively call this procedure again.
                 */
                if(DOLOG(LOG_READ))
                    syslog(LOG_DEBUG, "This is a compressed batch");
                result = artfileproc(rbv, fp);
            }

            else if(strnicmp(buf + 1, "rnews", 5) == 0) {
                /*
                 * We found a non-compressed batch if the first
                 * line is "#! rnews <size>", where <size> is the
                 * size of the next article in bytes. More articles
                 * might follow.
                 */
                if(DOLOG(LOG_READ))
                    syslog(LOG_DEBUG, "This is a non-compressed batch");
                result = batchproc(rbv, fp, buf, buflen, (size_t)atol(buf + 6));
            }
            else
                syslog(LOG_ERR, "Article file has unknown format");
        }

        /*
         * The two bytes \x1F\x9D indicate a compressed, \x1F\x8B
         * a zipped archive.
         */
        else if(*buf == '\x1F') {
            switch(*(buf + 1)) {
            case '\x9D':
                if(DOLOG(LOG_READ))
                    syslog(LOG_DEBUG, "Batch is compressed");
                result = compressproc(rbv, fp, cfg.uncompresscall, buf, buflen, 2);
                break;
            case '\x8B':
                if(DOLOG(LOG_READ))
                    syslog(LOG_DEBUG, "Batch is gzipped");
                result = compressproc(rbv, fp, cfg.gunzipcall, buf, buflen, 2);
                break;
            default:
                syslog(LOG_ERR, "Compression method of article file is unknown");
                break;
            }
        }

        /*
         * In any other case we asume this being a single article
         */
        else {
            *(buf + 2) = '\0';
            result = singleproc(rbv, fp, buf, buflen, 0);
        }
    }
    else
        syslog(LOG_DEBUG, "Failed reading article file header");
    free(buf);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* compressproc                                                         */
/*                                                                      */
/* Processes a compressed article file.                                 */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Filepointer to the batch file.                          */
/*                                                                      */
/* cmd          Points to a string that specifies command line to un-   */
/*              compress the file.                                      */
/*                                                                      */
/* buf          This buffer will be used by this procedure to read      */
/*              the compressed file. It may already contain the         */
/*              first part of the file (see parameter cc).              */
/*                                                                      */
/* buflen       Size of buf.                                            */
/*                                                                      */
/* cc           Number of valid characters in buf. These characters     */
/*              may have been read from the file by the caller into     */
/*              buf.                                                    */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
static int compressproc(void *rbv, FILE *fp, char *cmd, char *buf,
                        size_t buflen, size_t cc)
{
    int result = -1;
    char *tname = tempnam(cfg.workdir, "ART");
#ifndef __OS2__
    char *t2name;
#endif
    char *cmdline = malloc(_MAX_PATH);
    FILE *ft;
    int rc;

    if((ft = xfopen(tname, "wb")) != NULL) {
        do {
            fwrite(buf, 1, cc, ft);
            cc = fread(buf, 1, buflen, fp);
            if(cc && DOLOG(LOG_READ))
                syslog(LOG_DEBUG, "Read another %u bytes from compressed batch", cc);
        } while(cc);
        if(fclose(ft))
            syslog(LOG_ERR, "%s %m", tname);
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "%s %m", tname);

    if(result == 0) {
#ifdef __OS2__
        strcat(strcat(strcpy(cmdline, cmd), " "), tname);
        if(DOLOG(LOG_READ))
            syslog(LOG_DEBUG, "Calling '%s'", cmdline);

        if((ft = popen(cmdline, "rb")) != NULL) {
            result = artfileproc(rbv, ft);
            if((rc = pclose(ft)) != 0)
                syslog(LOG_ERR, "%s returned %u", cmd, (unsigned int)rc >> 8);
            else {
                if(unlink(tname))
                    syslog(LOG_ERR, "%s %m", tname);
            }
        }
        else
            syslog(LOG_ERR, "Failed to run %s", cmd);
#else
        t2name = tempnam(cfg.workdir, "UNZ");
        sprintf(cmdline, "%s %s > %s", cmd, tname, t2name);
        if((rc = system(cmdline)) != 0)
            syslog(LOG_ERR, "'%s' returned error %d", cmdline, rc);
        else if((ft = xfopen(t2name, "rb")) == NULL)
            syslog(LOG_ERR, "%s", _strerror(t2name));
        else {
            result = artfileproc(rbv, ft);
            if(fclose(ft))
                syslog(LOG_ERR, "%s", _strerror(tname));
        }
        if(unlink(tname))
            syslog(LOG_ERR, "%s %m", tname);
        if(unlink(t2name))
            syslog(LOG_ERR, "%s %m", t2name);
        free(t2name);
#endif
    }

    free(tname);
    free(cmdline);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* batchproc                                                            */
/*                                                                      */
/* Processes a batched article file.                                    */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Filepointer to the article file.                        */
/*                                                                      */
/* buf          This buffer will be used by this procedure to read      */
/*              the compressed file.                                    */
/*                                                                      */
/* buflen       Size of buf.                                            */
/*                                                                      */
/* art_size     Size of the first article in this batch.                */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
static int batchproc(void *rbv, FILE *fp, char *buf, size_t buflen, size_t art_size)
{
    int result = -1;
    int skipped;

    while(art_size) {
        skipped = 0;
        *buf = '\0';
        if(DOLOG(LOG_READ))
            syslog(LOG_DEBUG, "Processing batched article with %ld bytes", art_size);
        if((result = singleproc(rbv, fp, buf, buflen, art_size)) != 0)
            break;

        /*
         * Search the next article.
         */
        art_size = 0;
        while(!art_size) {
            if(fgets(buf, (int)buflen, fp) == NULL)
                break;
            if(strnicmp(buf + 3, "rnews", 5) == 0)
                art_size = (size_t)atol(buf + 8);
            else {
                skipped++;
                if(DOLOG(LOG_READ))
                    syslog(LOG_DEBUG, "Line skipped: %s", buf);
            }
        }
        if(skipped)
            syslog(LOG_ERR, "%u lines skipped in batch", skipped);
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/* singleproc                                                           */
/*                                                                      */
/* Processes a single article.                                          */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           File pointer to the article file.                       */
/*                                                                      */
/* buf          This buffer will be used by this procedure to read      */
/*              the compressed file. It may already contain the         */
/*              first part of the file.                                 */
/*                                                                      */
/* buflen       Size of buf.                                            */
/*                                                                      */
/* max_bytes    Size of the article or zero if we should read the       */
/*              whole file in which case an EOF will mark the end       */
/*              of this article.                                        */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
static int singleproc(void *rbv, FILE *fp, char *buf, size_t buflen, size_t max_bytes)
{
    int result = 0;
    size_t cc;
    size_t hdr_bytes;
    size_t hdr_size = sizeof(HEADER) * (ID_LASTENTRY + MAX_XHEADER);
    HEADER *header = memset(malloc(hdr_size), 0, hdr_size);
    BODY   *body = NULL;
    BODY   *bp;

    n_proc++;
    if(max_bytes && max_bytes < buflen)
        cc = max_bytes;
    else
        cc = buflen;
    /*
     * ChanReadHeader reads the article header into buf and
     * set the header pointer array. It will not read more than
     * the number of bytes given by cc and returns the number of
     * bytes read from the input file or zero on errors. After
     * return cc will contain the number of bytes of buf being
     * occupied as this may differ from the number of bytes
     * read.
     */
    if((hdr_bytes = ChanReadHeader(fp, buf, &cc, header)) != 0) {
        if(DOLOG(LOG_READ))
            syslog(LOG_DEBUG, "Read %ld bytes of header lines", hdr_bytes);
        if(max_bytes == 0 || max_bytes > hdr_bytes) {
            if(DOLOG(LOG_READ))
                syslog(LOG_DEBUG, "Reading body with %ld bytes", max_bytes ? max_bytes - hdr_bytes : 0);
            ChanReadBody(fp, &body, max_bytes ? max_bytes - hdr_bytes : 0);
        }
        if(validate_header(buf + cc, buflen - cc, header))
            if((result = offer_article(rbv, header, body)) < 0)
                syslog(LOG_ERR, "Connection died");
    }
    else
        syslog(LOG_ERR, "Failed to read header");

    while(body) {
        free(body->line);
        bp = body;
        body = body->next;
        free(bp);
    }
    free(header);

    return(result);
}
