/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char config_rcsid[] = "$Id: config.c,v 1.1 1997/02/13 14:00:56 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <version.h>
#include <syslog.h>
#include <chlimits.h>
#include <readconf.h>
#include <sostream.h>

#include "config.h"

static ENVTABLE et[] = {
    { cfg.workdir          , "CHANGIWORKDIR"          , "", 0 },
    { cfg.seqf             , "CHANGISEQF"             , "SEQF", 1 },
    { cfg.uncompresscall   , "CHANGIUNCOMPRESS"       , "gzip -dc", 2 },
    { cfg.gunzipcall       , "CHANGIUNZIP"            , "gzip -dc", 2 },
    { cfg.distribution     , "CHANGIDISTRIBUTION"     , "", 0 },
    { cfg.mydomain         , "CHANGIHOSTNAME"         , "HOSTNAME", 4 },
    { cfg.remoteaddr       , "CHANGILOCALHOST"        , "127.0.0.1", 2 },
    { cfg.remoteport       , "CHANGINEWSPORT"         , "nntp", 2 },
    { cfg.logname          , "CHANGILOGNAME"          , "USER", 4 },
    { cfg.realname         , "CHANGIREALNAME"         , "REALNAME", 4 },
    { NULL                 , NULL                     , "", 0 }
};

CONFIG cfg;

static int dump_cfg(void);

/************************************************************************/
/*                                                                      */
/*  Setup initial configuration.                                        */
/************************************************************************/
void init_cfg(char icp)
{
    cfg.sockwait = 60;
    read_conf(et);
    if(icp != 'r') {
        cfg.usepost = 1;
    }
}

/************************************************************************/
/*                                                                      */
/*  Validate configuration                                              */
/*                                                                      */
/*  Returns 0 on success, -1 otherwise.                                 */
/*                                                                      */
/************************************************************************/
int validate_cfg(void)
{
    int result = 0;
    long myaddr;
    char *cp;

    if(cfg.mydomain[0] == '\0' || strchr(cfg.mydomain, '.') == NULL)
        SoGetLocalHost(cfg.mydomain, &myaddr);
    if(cfg.mydomain[0] == '\0' || strchr(cfg.mydomain, '.') == NULL) {
        syslog(LOG_ERR, "Fatal error, bad [CHANGI]HOSTNAME");
        fprintf(stderr, "Set [CHANGI]HOSTNAME to FQDN\n");
        result = -1;
    }
    if(cfg.authentication[0]) {
        if((cp = strchr(cfg.authentication, ':')) == NULL) {
            syslog(LOG_ERR, "Fatal error, missing password");
            result = -1;
        }
        else {
            *cp = '\0';
            strcpy(cfg.auth_user, cfg.authentication);
            strcpy(cfg.auth_pass, cp + 1);
            *cp = ':';
        }
    }

    if(DOLOG(LOG_CONFIG))
        dump_cfg();

    return(result);
}

/************************************************************************/
/*                                                                      */
/* Dumps configuration                                                  */
/*                                                                      */
/************************************************************************/
static int dump_cfg(void)
{
    int result = -1;
    FILE *fp;
    ENVTABLE *ep;
    time_t t;

    if((fp = fopen("rnews.log", "a")) != NULL) {
        time(&t);
        fprintf(fp, "REM %s", ctime(&t));
        fprintf(fp, "REM Changi Rnews Version %s\n", version);
        fprintf(fp, "REM Compiled %s\n", __DATE__);
        for(ep = et; ep->cnfg; ep++)
            fprintf(fp, "SET %s=%s&\n", ep->ename, ep->cnfg);
        fputc('\n', fp);
        if(fclose(fp))
            syslog(LOG_ERR, "Config dump: %m");
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "Config dump: %m");

    return(result);
}

/*
 * $Log: config.c,v $
 * Revision 1.1  1997/02/13 14:00:56  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:07:45  oliver
 * initial revision
 *
 */
