#ifndef CONFIG_H
#define CONFIG_H

/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Id: config.h,v 1.1 1997/02/13 14:00:57 oliver Exp $
 */

#define PROCBUFSIZE     16384   /* size of article processing buffer */

/*----------------------------------------------------------------------*/
/*  Configuration                                                       */
/*                                                                      */
/*----------------------------------------------------------------------*/
typedef struct {
    char workdir[_MAX_PATH];
    char mydomain[MAX_DOMAINNAME];  /* local node name including domain */
    char remoteaddr[MAX_DOMAINNAME];
    char remoteport[128];
    char newsfile[_MAX_PATH];       /* article file if not stdin */
    char uncompresscall[_MAX_PATH]; /* call to uncompress */
    char gunzipcall[_MAX_PATH];     /* call to gunzip */
    char approved[MAX_CLPSIZE];     /* overwrite approved line */
    char distribution[MAX_CLPSIZE]; /* overwrite distribution line */
    char expires[MAX_CLPSIZE];      /* overwrite expire line */
    char followupto[MAX_CLPSIZE];   /* overwrite followup-to line */
    char newsgroups[MAX_CLPSIZE];   /* overwrite newsgroups line */
    char organization[MAX_CLPSIZE]; /* overwrite organization line */
    char replyto[MAX_CLPSIZE];      /* overwrite replyto line */
    char signfile[MAX_CLPSIZE];     /* append signature file */
    char subject[MAX_CLPSIZE];      /* overwrite subject line */
    char sender[MAX_CLPSIZE];       /* overwrite from line */
    char seqf[_MAX_PATH];           /* seqf file pathname */
    unsigned long logflg;           /* debugging flags */
    long sockwait;                  /* timeout for offer reply */
    int  del_input;
    int  usepost;
    int  useihave;                  /* 08/28/97 added */
    int  printid;                   /* message ids printed if 1 */
    char authentication[MAX_CLPSIZE]; /* user and password */
    char auth_user[MAX_CLPSIZE];    /* user id */
    char auth_pass[MAX_CLPSIZE];    /* password */
    char logname[MAX_CLPSIZE];      /* added 08/07/1996 harald */
    char realname[MAX_CLPSIZE];     /* added 08/07/1996 harald */
} CONFIG;

extern CONFIG cfg;

extern void init_cfg(char icp);
extern int validate_cfg(void);

/*----------------------------------------------------------------------*/
/*  Logging flags                                                       */
/*----------------------------------------------------------------------*/
#define LOG_READ    0x00000002L
#define LOG_WRITE   0x00000004L
#define LOG_HISTORY 0x00000008L
#define LOG_HEAP    0x00000010L
#define LOG_TEST    0x00000020L
#define LOG_JUNK    0x00000040L
#define LOG_DUPES   0x00000080L
#define LOG_CONFIG  0x00000100L

#define DOLOG(x)    ((cfg.logflg & (x)) != 0)

/*
 * $Log: config.h,v $
 * Revision 1.1  1997/02/13 14:00:57  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:07:45  oliver
 * initial revision
 *
 */

#endif
