/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains routines to send articles to a server
 * using the IHAVE protocol.
 */

const char offer_rcsid[] = "$Id: offer.c,v 1.1 1997/02/13 14:00:57 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <chlimits.h>
#include <sostream.h>

#include <syslog.h>

#include "config.h"
#include "article.h"
#include "rnews.h"

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
int offer_article(void *rbv, HEADER *header, BODY *body)
{
    int  result = 0;
    char *buf = malloc(MAX_BUFSIZ);
    int i;
    int sock = *((int *)rbv);
    BODY *bp;

    /*
     * Send IHAVE and get response
     */
    for(;;) {
        if(cfg.usepost)
            strcpy(buf, "POST");
        else {
            strcpy(buf, "IHAVE ");
            strcat(buf, header[ID_Message_ID].info);
        }
        if(SoStreamTransact(rbv, buf, buf, cfg.sockwait) < 0) {
            free(buf);
            return(-1);
        }
        if(atoi(buf) == 480) {
            if(cfg.auth_user[0] == '\0' || cfg.auth_pass[0] == '\0') {
                syslog(LOG_ERR, "%s", buf);
                free(buf);
                return(-1);
            }
            sprintf(buf, "AUTHINFO USER %s", cfg.auth_user);
            if(SoStreamTransact(rbv, buf, buf, cfg.sockwait) < 0) {
                free(buf);
                return(-1);
            }
            if(atoi(buf) != 381) {
                syslog(LOG_ERR, "%s", buf);
                free(buf);
                return(-1);
            }
            sprintf(buf, "AUTHINFO PASS %s", cfg.auth_pass);
            if(SoStreamTransact(rbv, buf, buf, cfg.sockwait) < 0) {
                free(buf);
                return(-1);
            }
            if(atoi(buf) != 281) {
                syslog(LOG_ERR, "%s", buf);
                free(buf);
                return(-1);
            }
        }
        else
            break;
    }

    if (buf[0] != '3') {
        syslog(LOG_ERR, "%s", buf);
        free(buf);
        return(0);
    }

    for(i = 0; header[i].key; i++) {
        if(header[i].info) {
            sprintf(buf, "%s: %s\r\n", header[i].key, header[i].info);
            if(SoStreamPuts(sock, buf) < 0) {
                result = -1;
                break;
            }
        }
    }
    if(result == 0 && SoStreamPuts(sock, "\r\n") < 0)
        result = -1;
    if(result == 0) {
        for(bp = body; bp; bp = bp->next) {
            if(*(bp->line) == '.') {
                if(SoStreamPuts(sock, ".") < 0) {
                    result = -1;
                    break;
                }
            }
            if(SoStreamPuts(sock, bp->line) < 0) {
                result = -1;
                break;
            }
            if(SoStreamPuts(sock, "\r\n") < 0) {
                result = -1;
                break;
            }
        }
    }

    if(result == 0) {
        if(SoStreamTransact(rbv, ".", buf, cfg.sockwait) < 0)
            result = -1;
        else if(buf[0] != '2')
            syslog(LOG_ERR, "%s", buf);
    }

    free(buf);

    return(result);
}

/*
 * $Log: offer.c,v $
 * Revision 1.1  1997/02/13 14:00:57  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:07:46  oliver
 * initial revision
 *
 */
