/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char rheader_rcsid[] = "$Id: rheader.c,v 1.1 1997/02/13 14:00:58 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include <syslog.h>

#include <version.h>
#include <chlimits.h>
#include <jobid.h>

#include "config.h"
#include "article.h"
#include "rnews.h"

extern char *radixpack(unsigned long val, char *pck);

/************************************************************************/
/*  validate_header                                                     */
/*                                                                      */
/*  Returns true if header is valid or false otherwise.                 */
/*                                                                      */
/************************************************************************/
int validate_header(char *buf, size_t bufsiz, HEADER *header)
{
    int result = 1;
    char *item = malloc(MAX_ITEMSIZE);
    int i;
    size_t len;

    /*
     * Now check each required header entry
     */
    *item = '\0';
    for(i = 0; i < ID_LASTENTRY; i++) {
        switch(i) {

        case ID_Message_ID:
            if(header[i].info == NULL) {
                char pck[MAX_PCKSTRING];
                sprintf(item, "<c%s.%s$%s@%s>", version,
                        radixpack((unsigned long)time(NULL), pck),
                        job_id(cfg.seqf, 0), cfg.mydomain);
                if(cfg.printid) {
                    fputs(item, stdout);
                    fputc('\n', stdout);
                }
            }
            else if(cfg.printid) {
                fputs(header[i].info, stdout);
                fputc('\n', stdout);
            }
            break;

        case ID_Approved:
            if(cfg.approved[0])
                header[i].info = cfg.approved;
            break;

        case ID_Distribution:
            if(cfg.distribution[0])
                header[i].info = cfg.distribution;
            break;

        case ID_Expires:
            if(cfg.expires[0])
                header[i].info = cfg.expires;
            break;

        case ID_Followup_To:
            if(cfg.followupto[0])
                header[i].info = cfg.followupto;
            break;

        case ID_Organization:
            if(cfg.organization[0])
                header[i].info = cfg.organization;
            break;

        case ID_Reply_To:
            if(cfg.replyto[0])
                header[i].info = cfg.replyto;
            break;

        /*
         * In any case we need the newsgroups line
         */
        case ID_Newsgroups:
            if(cfg.newsgroups[0])
                header[i].info = cfg.newsgroups;
            break;

        /*
         * If one of the following lines is missing we'll add one
         */
        case ID_From:
            if(cfg.sender[0])
                header[i].info = cfg.sender;
            else if(header[i].info == NULL) {
                if(cfg.logname[0])
                    strcpy(item, cfg.logname);
                else
                    strcpy(item, "unknown");
                strcat(item, "@");
                strcat(item, cfg.mydomain);
                strcat(item, " (");
                if(cfg.realname[0])
                    strcat(item, cfg.realname);
                else
                    strcat(item, "Unknown User");
                strcat(item, ")");
                syslog(LOG_NOTICE, "Created missing from line");
            }
            break;

        case ID_Subject:
            if(cfg.subject[0])
                header[i].info = cfg.subject;
            break;

        /*
         * Remove any xref leftovers
         */
        case ID_Xref:
            header[i].info = NULL;
            break;
        }

        if(*item) {
            if((len = strlen(item) + 1) > bufsiz) {
                result = 0;
                syslog(LOG_ERR, "Buffer overflow while updating header");
                break;
            }
            else {
                strcpy(buf, item);
                header[i].info = buf;
                buf += len;
                bufsiz -= len;
                *item = '\0';
            }
        }
    }

    free(item);
    return(result);
}

/*
 * $Log: rheader.c,v $
 * Revision 1.1  1997/02/13 14:00:58  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:07:47  oliver
 * initial revision
 *
 */
