/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char rmews_rcsid[] = "$Id: rnews.c,v 1.1 1997/02/13 14:00:59 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>

#include <getopt.h>

#include <version.h>
#include <chlimits.h>
#include <sostream.h>
#include <syslog.h>

#include "config.h"
#include "article.h"
#include "rnews.h"

int n_proc = 0;     /* number of articles processed */

static int syslog_opt = LOG_CONS | LOG_NDELAY | LOG_NOWAIT;

static void parsedbgopt(char *opt);
static void usage(void);
static void cleanup(int sig);

static char icp;

/************************************************************************/
/*                                                                      */
/*  RNEWS                                                               */
/*                                                                      */
/*  This program receives incoming news articles into a news directory. */
/*  It accepts batched and compressed news directly. The inews program  */
/*  uses this one to post news locally.                                 */
/*                                                                      */
/************************************************************************/
int main(int argc, char **argv)
{
    int result = 0;
    FILE *fp;
    int option;
    time_t t_proc;
    void *rbv;
    int sock;
    char *cp;

    icp = tolower(argv[0][0]);
    for(cp = argv[0]; *cp; cp++)
        if(*cp == '\\' || *cp == '/' || *cp == ':')
            icp = tolower(*(cp + 1));

    tzset();
    t_proc = time(NULL);

    /*
     * Initialize socket interface.
     */
    if(SoInitialize()) {
        fprintf(stderr, "FATAL ERROR: TCP/IP initialization failed\n");
        return(128);
    }


    /*
     * Set up initial configuration
     */
    init_cfg(icp);

    /*
     * Init interupt handling
     */
    signal(SIGABRT, cleanup);
    signal(SIGBREAK, cleanup);
    signal(SIGINT, cleanup);
    signal(SIGSEGV, cleanup);
    signal(SIGTERM, cleanup);

    /*
     * Process command line options
     */
    while((option = getopt(argc, argv,
            "?a:A:C:d:D:e:F:f:H:hiIN:o:Pp:r:S:s:t:UW:x:")) != EOF) {
        switch(option) {
        case 'a':
            strcpy(cfg.authentication, optarg);
            break;
        case 'A':
            strcpy(cfg.approved, optarg);
            break;
        case 'C':
            strcpy(cfg.followupto, optarg);
            break;
        case 'd':
            parsedbgopt(optarg);
            break;
        case 'D':
            strcpy(cfg.distribution, optarg);
            break;
        case 'e':
            strcpy(cfg.expires, optarg);
            break;
        case 'F':
            cfg.del_input = 1;
            strcpy(cfg.newsfile, optarg);
            break;
        case 'f':
            if(icp != 'r')
                strcpy(cfg.sender, optarg);
            else
                strcpy(cfg.newsfile, optarg);
            break;
        case 'H':
            strcpy(cfg.remoteaddr, optarg);
            break;
        case 'h':
            /* accecpt headers, ignored for compatibility */
            break;
        case 'i':
            break;
        case 'I':
            cfg.useihave++;
            break;
        case 'N':
            strcpy(cfg.newsgroups, optarg);
            break;
        case 'o':
            strcpy(cfg.organization, optarg);
            break;
        case 'p':
            strcpy(cfg.remoteport, optarg);
            break;
        case 'P':
            cfg.usepost++;
            break;
        case 'r':
            strcpy(cfg.replyto, optarg);
            break;
        case 's':
            break;
        case 'S':
            strcpy(cfg.signfile, optarg);
            break;
        case 't':
            strcpy(cfg.subject, optarg);
            break;
        case 'U':
            break;
        case 'v':
            cfg.printid++;
            break;
        case 'W':
            cfg.sockwait = atol(optarg);
            break;
        case 'x':   /* ignore for UUPC compatibility */
            break;
        default:
            syslog(LOG_INFO, "Unknown option -%c", option);
            break;
        case '?':
            usage();
            return(1);
        }
    }

    openlog(icp != 'r' ? "inews" : "rnews", syslog_opt, LOG_NEWS);
    syslog(LOG_INFO, "Version %s-%s up and running...", version, __DATE__);

     /*
     * Ignore additional command line parameters, which
     * are sent by bugy UUPC.
     */
    argc -= optind;
    argv += optind;
    if(argc) {
        if(icp != 'r') {
            cfg.del_input = 1;
            strcpy(cfg.newsfile, argv[0]);
        }
        else
            syslog(LOG_NOTICE, "Additional parameter %s ignored", argv[0]);
    }

    if((fp = freopen(cfg.newsfile, "rb", stdin)) == NULL) {
        syslog(LOG_ERR, "Input %s: %m", cfg.newsfile);
        result = 3;
    }

    if(fp) {
        if(validate_cfg() == 0) {
            char *line;

            if ((sock = SoStreamConnect(cfg.remoteaddr, cfg.remoteport, 119)) < 0)
                return(3);

            rbv = SoStreamBufCreate(sock);
            line = malloc(MAX_BUFSIZ);
            if(SoStreamBufRecv(line, MAX_BUFSIZ, rbv, cfg.sockwait) <= 0) {
                syslog(LOG_ERR, "Failed to get server intro");
                free(line);
                SoStreamBufDestroy(rbv);
                SoStreamClose(sock);
                return(3);
            }

            if (line[0] != '2') {
                syslog(LOG_ERR, "Error: %s", line);
                free(line);
                SoStreamBufDestroy(rbv);
                SoStreamClose(sock);
                return(3);
            }
            if(cfg.useihave == 0 && cfg.usepost == 0) {
                long l = filelength(fileno(fp));

                if(l < 0) {
                    syslog(LOG_ERR, "Can't determine file size");
                    free(line);
                    SoStreamBufDestroy(rbv);
                    SoStreamClose(sock);
                    return(3);
                }
                /*
                 * Send IHAVE and get response
                 */
                for(;;) {
                    sprintf(line, "XBATCH %ld", l);
                    if(SoStreamTransact(rbv, line, line, cfg.sockwait) < 0) {
                        syslog(LOG_ERR, "XBATCH failed");
                        free(line);
                        SoStreamBufDestroy(rbv);
                        SoStreamClose(sock);
                        return(3);
                    }
                    if(atoi(line) == 480) {
                        if(cfg.auth_user[0] == '\0' || cfg.auth_pass[0] == '\0') {
                            syslog(LOG_ERR, "%s", line);
                            free(line);
                            SoStreamBufDestroy(rbv);
                            SoStreamClose(sock);
                            return(3);
                        }
                        sprintf(line, "AUTHINFO USER %s", cfg.auth_user);
                        if(SoStreamTransact(rbv, line, line, cfg.sockwait) < 0) {
                            syslog(LOG_ERR, "XBATCH failed");
                            free(line);
                            SoStreamBufDestroy(rbv);
                            SoStreamClose(sock);
                            return(3);
                        }
                        if(atoi(line) != 381) {
                            syslog(LOG_ERR, "%s", line);
                            free(line);
                            SoStreamBufDestroy(rbv);
                            SoStreamClose(sock);
                            return(3);
                        }
                        sprintf(line, "AUTHINFO PASS %s", cfg.auth_pass);
                        if(SoStreamTransact(rbv, line, line, cfg.sockwait) < 0) {
                            syslog(LOG_ERR, "XBATCH failed");
                            free(line);
                            SoStreamBufDestroy(rbv);
                            SoStreamClose(sock);
                            return(3);
                        }
                        if(atoi(line) != 281) {
                            syslog(LOG_ERR, "%s", line);
                            free(line);
                            SoStreamBufDestroy(rbv);
                            SoStreamClose(sock);
                            return(3);
                        }
                    }
                    else
                        break;
                }
                if(atoi(line) != 339) {
                    syslog(LOG_ERR, "%s", line);
                    free(line);
                    SoStreamBufDestroy(rbv);
                    SoStreamClose(sock);
                    return(3);
                }

                while((l = fread(line, 1, MAX_BUFSIZ, fp)) > 0) {
                    if(SoStreamBinSend(line, l, sock) < 0) {
                        free(line);
                        SoStreamBufDestroy(rbv);
                        SoStreamClose(sock);
                        return(3);
                    }
                }

                if(SoStreamBufRecv(line, MAX_BUFSIZ, rbv, cfg.sockwait) <= 0) {
                    syslog(LOG_ERR, "Failed to get server respones");
                    free(line);
                    SoStreamBufDestroy(rbv);
                    SoStreamClose(sock);
                    return(3);
                }
                if(atoi(line) != 239) {
                    syslog(LOG_ERR, "%s", line);
                    free(line);
                    SoStreamBufDestroy(rbv);
                    SoStreamClose(sock);
                    return(3);
                }
            }
            else if(artfileproc(rbv, fp))
                result = 2;
            free(line);
            SoStreamBufDestroy(rbv);
            SoStreamClose(sock);
        }
        else
            result = 3;
        fclose(fp);
    }

    if(result == 0 && cfg.del_input && unlink(cfg.newsfile)) {
        syslog(LOG_ERR, "%s %m", cfg.newsfile);
        result = 5;
    }
    if(result)
        syslog(LOG_ERR, "Return with error %d", result);
    else {
        if((t_proc = time(NULL) - t_proc) == 0)
            t_proc = 1;
        if(cfg.useihave || cfg.usepost)
            syslog(LOG_NOTICE, "Processed %d article%s in %lu second%s",
                               n_proc, n_proc == 1 ? "" : "s",
                               t_proc, t_proc == 1 ? "" : "s");
        else
            syslog(LOG_NOTICE, "Transmitted batch in %lu second%s",
                               t_proc, t_proc == 1 ? "" : "s");
    }

    if(DOLOG(LOG_HEAP))
        _dump_allocated(0);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* Parse debug options                                                  */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsedbgopt(char *opt)
{
    while(*opt) {
        switch(*opt) {
        case 'C':
            cfg.logflg |= LOG_CONFIG;
            break;
        case 'd':
            cfg.logflg |= LOG_DUPES;
            break;
        case 'h':
            cfg.logflg |= LOG_HISTORY;
            break;
        case 'j':
            cfg.logflg |= LOG_JUNK;
            break;
        case 'm':
            cfg.logflg |= LOG_HEAP;
            break;
        /*
         * 08/30/1996
         * Added option -do and -dp
         */
        case 'o':
            syslog_opt |= LOG_PERROR;
            break;
        case 'p':
            syslog_opt |= LOG_PID;
            break;
        case 'r':
            cfg.logflg |= LOG_READ;
            break;
        case 't':
            cfg.logflg |= LOG_TEST;
            break;
        case 'w':
            SoStreamDebug(1);
            break;
        default:
            syslog(LOG_NOTICE, "Unknown option -d%c ignored", *opt);
            break;
        }
        opt++;
    }
}

/************************************************************************/
/*                                                                      */
/* Usage display                                                        */
/*                                                                      */
/************************************************************************/
static void usage(void)
{
    fprintf(stderr, "\n%cnews %s - %s %s\n\n", icp, version, __DATE__, __TIME__);
    fprintf(stderr, "usage: %cnews [options]\n", icp);
    fputs("options:\n"
         "  -a<user:passwd>  authentication  -I               use IHAVE cmd\n"
         "  -A<approved>     approved by     -N<grouplist>    to newsgroups\n"
         "  -C<filename>     follow-up       -o<organization> organization\n"
         "  -d<log-flags>    logfile flags   -P               use POST cmd\n"
         "  -D<distribution> distribution    -p<port>         server port\n"
         "  -e<date>         expire date     -r<address>      reply to\n"
         "  -f<filename>     input file      -S<filename>     append signature\n"
         "  -F<filename>     -f with delete  -t<title>        subject\n"
         "  -H<remote>       remote host     -W<time>         socket timeout\n", stderr);
}

/************************************************************************/
/*                                                                      */
/* Signal handler                                                       */
/*                                                                      */
/************************************************************************/
static void cleanup(int sig)
{
    char *msg = "Unknown signal";
    int level = LOG_INFO;

    signal(sig, SIG_IGN);
    switch(sig) {
    case SIGSEGV:
        level = LOG_ERR;
        msg = "internal error";
        break;
    case SIGTERM:
        msg = "killed";
        break;
    case SIGABRT:
        level = LOG_ERR;
        msg = "aborted";
        break;
    case SIGBREAK:
    case SIGINT:
        msg = "interrupted";
        break;
    }
    syslog(level, msg);
    fprintf(stderr, "\n%cnews %s\n", icp, msg);
    if(DOLOG(LOG_HEAP))
        _dump_allocated(0);
    signal(SIGABRT, SIG_DFL);
    abort();
}

/*
 * $Log: rnews.c,v $
 * Revision 1.1  1997/02/13 14:00:59  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:07:47  oliver
 * initial revision
 *
 */
