const char access_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 *
 * This module is used to set access permissions.
 *
 */
/*
 * $Log$
 */



#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <io.h>
#include <ctype.h>
#include <time.h>

#include <sys/stat.h>

#include <syslog.h>
#include <sostream.h>
#include <match.h>
#include <loadtext.h>

#include "config.h"
#include "access.h"

typedef struct {
    char **name;
    char **line;
    int  n;
    int  x;
} ALIAS_BUFF;

typedef struct {
    char *host;
    char *groups;
    char *nopost;
    int  canpost;
    int  canread;
    int  canxfer;
    int  needreadauth;
    int  needpostauth;
} HOST_PERM;

static char **host_lines = NULL;
static time_t host_mtime = 0;
static int nhost_perms = 0;
static HOST_PERM *host_perm = NULL;

typedef struct {
    char *user;
    char *pass;
    char *groups;
    char *nopost;
} USER_PERM;

static char **user_lines = NULL;
static time_t user_mtime = 0;
static int nuser_perms = 0;
static USER_PERM *user_perm = NULL;

typedef struct {
    char *host;
    char *user;
    char *perm;
} CTRL_PERM;

static char **ctrl_lines = NULL;
static time_t ctrl_mtime = 0;
static int nctrl_perms = 0;
static CTRL_PERM *ctrl_perm = NULL;

typedef struct {
    char *groups;
    char *hosts[2];
} TEAM_PERM;

static char **team_lines = NULL;
static time_t team_mtime = 0;
static int nteam_perms = 0;
static TEAM_PERM *team_perm = NULL;
static ALIAS_BUFF team_alias = { 0, 0, 0, 0 };

/************************************************************************/
/*                                                                      */
/* anti_alias                                                           */
/*                                                                      */
/************************************************************************/
static char *anti_alias(char *list, ALIAS_BUFF *pab)
{
    register char *cp;
    char *pl = list;
    int xitems = 0;
    int nitems = 0;
    char **items = NULL;
    int i, j;
    int len;

    /*
     * Break comma separated lists into items.
     */
    while(pl && *pl) {
        if((cp = strchr(pl, ',')) != NULL)
            *cp++ = 0;
        if(nitems == xitems) {
            xitems += 16;
            items = realloc(items, xitems * sizeof(char *));
        }
        items[nitems++] = strdup(pl);
        pl = cp;
    }
    if(nitems == 0)
        return strdup(list);

    /*
     * Convert any item representing an alias.
     */
    for(i = 0; i < nitems; i++) {
        if(items[i][0] != '$')
            continue;
        cp = items[i] + 1;
        for(j = 0; j < pab->n; j++) {
            if(strcmp(cp, pab->name[j]) == 0)
                break;
        }
        if(j < pab->n) {
            free(items[i]);
            items[i] = anti_alias(pl = strdup(pab->line[j]), pab);
        }
        else
            items[i][0] = 0;
    }

    /*
     * Concat items to a new list.
     */
    for(len = i = 0; i < nitems; i++)
        if(items[i][0])
            len += strlen(items[i]) + 1;
    if(len) {
        pl = malloc(len + 1);
        *pl = 0;
        for(i = 0; i < nitems; i++) {
            if(items[i][0]) {
                if(*pl)
                    strcat(pl, ",");
                strcat(pl, items[i]);
            }
        }
    }

    for(i = 0; i < nitems; i++)
        free(items[i]);
    free(items);

    return pl;
}

/************************************************************************/
/*                                                                      */
/* update_hostperms                                                     */
/*                                                                      */
/************************************************************************/
static void update_hostperms(char *accessfile)
{
    struct stat st;
    int rc;
    int n_lines;
    char **lpp;
    char *lp;
    char *cp;

    /*
     * If host permission file exists and the modification
     * time didn't change, no update is needed.
     */
    if((rc = stat(accessfile, &st)) == 0 && st.st_mtime == host_mtime) {
        if(dolog(LOG_PERM))
            syslog(LOG_DEBUG, "No change in %s", accessfile);
        return;
    }

    /*
     * Free previously allocated space.
     */
    if(nhost_perms) {
        free(host_lines[0]);
        free(host_lines);
        free(host_perm);
        host_perm = NULL;
        nhost_perms = 0;
        host_mtime = 0;
    }

    /*
     * Load the access permission file and process each line,
     * if it exists.
     */
    if(rc == 0 && (n_lines = load_textfile(accessfile, &host_lines)) > 0) {
        if(dolog(LOG_PERM))
            syslog(LOG_DEBUG, "(Re)Loading %s, %d lines", accessfile, n_lines);
        host_mtime = st.st_mtime;
        host_perm = malloc(n_lines * sizeof(HOST_PERM));
        for(lpp = host_lines; (lp = *lpp++) != NULL; ) {
            memset(&host_perm[nhost_perms], 0, sizeof(HOST_PERM));

            /*
             * First field is the host name
             */
            cp = host_perm[nhost_perms].host = lp;
            while(*cp && *cp != ' ')
                cp++;

            /*
             * We need a second field.
             */
            if(*cp != ' ') {
                syslog(LOG_INFO, "Bad line in %s: '%s'", accessfile, lp);
                continue;
            }
            *cp++ = '\0';
            strlwr(host_perm[nhost_perms].host);

            /*
             * Accept 'default' for backward compatibility.
             */
            if(strcmp(host_perm[nhost_perms].host, "default") == 0)
                strcpy(host_perm[nhost_perms].host, "*");

            /*
             * Second field specifies read/xfer permission. We
             * use the first character only.
             */
            switch(*cp) {
            case 'B':
                host_perm[nhost_perms].needreadauth = 1;
            case 'b':
                host_perm[nhost_perms].canread = 1;
                host_perm[nhost_perms].canxfer = 1;
                break;
            case 'R':
                host_perm[nhost_perms].needreadauth = 1;
            case 'r':
                host_perm[nhost_perms].canread = 1;
                break;
            case 'X':
                host_perm[nhost_perms].needreadauth = 1;
            case 'x':
                host_perm[nhost_perms].canxfer = 1;
                break;
            }
            while(*cp && *cp != ' ')
                cp++;

            /*
             * We need a third field.
             */
            if(*cp != ' ') {
                syslog(LOG_INFO, "Bad line in %s: '%s'", accessfile, lp);
                continue;
            }

            /*
             * Third field specifies posting permission. We
             * use the first character only.
             */
            switch(*++cp) {
            case 'P':
                host_perm[nhost_perms].needpostauth = 1;
            case 'p':
                host_perm[nhost_perms].canpost = 1;
                break;
            }

            /*
             * The third field might contain a slash separated
             * subfield containing groups to be excluded from
             * posting.
             */
            while(*cp && *cp != ' ' && *cp != '/')
                cp++;
            if(*cp == '/') {
                *cp++ = '\0';
                host_perm[nhost_perms].nopost = cp;
                while(*cp && *cp != ' ')
                    cp++;
            }

            /*
             * The optional fourth field contains a list of
             * newsgroup patterns.
             */
            if(*cp == ' ') {
                *cp++ = '\0';
                host_perm[nhost_perms].groups = cp;
            }
            nhost_perms++;
        }
    }
    else if(dolog(LOG_PERM) && accessfile && *accessfile)
        syslog(LOG_DEBUG, "%s not found", accessfile);
    return;
}

/************************************************************************/
/*                                                                      */
/* update_userperms                                                     */
/*                                                                      */
/************************************************************************/
static void update_userperms(char *accessfile)
{
    struct stat st;
    int rc;
    int n_lines;
    char **lpp;
    char *lp;
    char *cp;

    /*
     * If user permission file exists and the modification
     * time didn't change, no update is needed.
     */
    if((rc = stat(accessfile, &st)) == 0 && st.st_mtime == user_mtime)
        return;

    /*
     * Free previously allocated space.
     */
    if(nuser_perms) {
        free(user_lines[0]);
        free(user_lines);
        free(user_perm);
        user_perm = NULL;
        nuser_perms = 0;
        user_mtime = 0;
    }

    /*
     * Load the access permission file and process each line,
     * if it exists.
     */
    if(rc == 0 && (n_lines = load_textfile(accessfile, &user_lines)) > 0) {
        user_mtime = st.st_mtime;
        user_perm = malloc(n_lines * sizeof(USER_PERM));
        for(lpp = user_lines; (lp = *lpp++) != NULL; ) {
            memset(&user_perm[nuser_perms], 0, sizeof(USER_PERM));

            /*
             * First field is the user id
             */
            cp = user_perm[nuser_perms].user = lp;
            while(*cp && *cp != ' ')
                cp++;

            /*
             * We need a second field.
             */
            if(*cp != ' ') {
                syslog(LOG_INFO, "Bad line in %s: '%s'", accessfile, lp);
                continue;
            }
            *cp++ = '\0';

            /*
             * Second field is the unencrypted password.
             */
            user_perm[nuser_perms].pass = cp;
            while(*cp && *cp != ' ')
                cp++;

            /*
             * The third field is optional and may contain a comma
             * separated list of newsgroup patterns, optionally
             * followed by a comma separated list of newsgroup
             * patterns to be excluded from posting.
             */
            if(*cp) {
                *cp++ = '\0';
                user_perm[nuser_perms].groups = cp;
                while(*cp && *cp != '/')
                    cp++;
                if(*cp == '/') {
                    *cp++ = '\0';
                    user_perm[nuser_perms].nopost = cp;
                }
            }
            nuser_perms++;
        }
    }

    return;
}

/************************************************************************/
/*                                                                      */
/* update_ctrlperms                                                     */
/*                                                                      */
/************************************************************************/
static void update_ctrlperms(char *accessfile)
{
    struct stat st;
    int rc;
    int n_lines;
    char **lpp;
    char *lp;
    char *cp;

    /*
     * If ctrl permission file exists and the modification
     * time didn't change, no update is needed.
     */
    if((rc = stat(accessfile, &st)) == 0 && st.st_mtime == ctrl_mtime)
        return;

    /*
     * Free previously allocated space.
     */
    if(nctrl_perms) {
        free(ctrl_lines[0]);
        free(ctrl_lines);
        free(ctrl_perm);
        ctrl_perm = NULL;
        nctrl_perms = 0;
        ctrl_mtime = 0;
    }

    /*
     * Load the access permission file and process each line,
     * if it exists.
     */
    if(rc == 0 && (n_lines = load_textfile(accessfile, &ctrl_lines)) > 0) {
        ctrl_mtime = st.st_mtime;
        ctrl_perm = malloc(n_lines * sizeof(CTRL_PERM));
        for(lpp = ctrl_lines; (lp = *lpp++) != NULL; ) {
            memset(&ctrl_perm[nctrl_perms], 0, sizeof(CTRL_PERM));

            /*
             * First field is the host name
             */
            cp = ctrl_perm[nctrl_perms].host = lp;
            while(*cp && *cp != ' ')
                cp++;

            /*
             * We need a second field.
             */
            if(*cp != ' ') {
                syslog(LOG_INFO, "Bad line in %s: '%s'", accessfile, lp);
                continue;
            }
            *cp++ = '\0';

            /*
             * Second field is the user-id.
             */
            ctrl_perm[nctrl_perms].user = cp;
            while(*cp && *cp != ' ')
                cp++;

            /*
             * We need a third field.
             */
            if(*cp != ' ') {
                syslog(LOG_INFO, "Bad line in %s: '%s'", accessfile, lp);
                continue;
            }
            *cp++ = '\0';
            ctrl_perm[nctrl_perms++].perm = cp;
        }
    }
    return;
}

/************************************************************************/
/*                                                                      */
/* update_teamperms                                                     */
/*                                                                      */
/************************************************************************/
static void update_teamperms(char *accessfile)
{
    struct stat st;
    int rc;
    int n_lines;
    char **lpp;
    char *lp;
    char *cp;
    int i;
    int n;
    char *fld[3];

    /*
     * If team permission file exists and the modification
     * time didn't change, no update is needed.
     */
    if((rc = stat(accessfile, &st)) == 0 && st.st_mtime == team_mtime) {
        if(dolog(LOG_PERM))
            syslog(LOG_DEBUG, "No change in %s", accessfile);
        return;
    }

    /*
     * Free previously allocated space.
     */
    if(nteam_perms) {
        free(team_lines[0]);
        free(team_lines);
        for(i = 0; i < nteam_perms; i++) {
            free(team_perm[i].groups);
            free(team_perm[i].hosts[0]);
            if(team_perm[i].hosts[1]) {
                free(team_perm[i].hosts[1]);
            }
        }
        free(team_perm);
        team_perm = NULL;
        nteam_perms = 0;
        team_mtime = 0;
    }
    if(team_alias.n) {
        for(i = 0; i < team_alias.n; i++)
            free(team_alias.line[i]);
        free(team_alias.line);
        free(team_alias.name);
        memset(&team_alias, 0, sizeof(team_alias));
    }

    /*
     * Load the access permission file and process each line,
     * if it exists.
     */
    if(rc == 0 && (n_lines = load_textfile(accessfile, &team_lines)) > 0) {
        if(dolog(LOG_PERM))
            syslog(LOG_DEBUG, "(Re)Loading %s, %d lines", accessfile, n_lines);
        team_mtime = st.st_mtime;
        team_perm = malloc(n_lines * sizeof(TEAM_PERM));
        for(lpp = team_lines; (lp = *lpp++) != NULL; ) {
            memset(&team_perm[nteam_perms], 0, sizeof(TEAM_PERM));

            /*
             * Check, if line is an alias definition.
             */
            if(isupper(*lp) && (cp = strchr(lp, '=')) != NULL) {
                if(team_alias.n == team_alias.x) {
                    team_alias.x += 16;
                    team_alias.line = realloc(team_alias.line, team_alias.x * sizeof(char *));
                    team_alias.name = realloc(team_alias.name, team_alias.x * sizeof(char *));
                }
                *cp++ = 0;
                team_alias.name[team_alias.n] = lp;
                team_alias.line[team_alias.n++] = anti_alias(cp, &team_alias);
                continue;
            }

            /*
             * Break line into fields.
             */
            for(fld[0] = cp = lp, n = 1; *cp; cp++) {
                if(*cp == ' ') {
                    *cp++ = 0;
                    fld[n] = cp;
                    if(++n == 3)
                        break;
                }
            }

            /*
             * We need at least two fields.
             */
            if(n < 2) {
                syslog(LOG_NOTICE, "Bad line in %s: '%s'", accessfile, lp);
                continue;
            }

            /*
             * First field is a list of groups
             */
            team_perm[nteam_perms].groups = anti_alias(fld[0], &team_alias);

            /*
             * Remaining fields are lists of hosts.
             */
            team_perm[nteam_perms].hosts[0] = anti_alias(fld[1], &team_alias);
            if(n == 3)
                team_perm[nteam_perms].hosts[1] = anti_alias(fld[2], &team_alias);
            else
                team_perm[nteam_perms].hosts[1] = NULL;

            nteam_perms++;
        }
    }
    else if(dolog(LOG_PERM) && accessfile && *accessfile)
        syslog(LOG_DEBUG, "%s not found", accessfile);

    return;
}

/************************************************************************/
/*                                                                      */
/* host_access                                                          */
/*                                                                      */
/************************************************************************/
int host_access(char *accessfile, char *remotehost,
                long remoteaddr, ACCESSPERMS *perms)
{
    int i;
    int result = -1;
    char remote_ip[32];
    char *remote_nm;
    memset(perms, 0, sizeof(ACCESSPERMS));

    /*
     * Normalize remote name to lower case and convert ip address
     * to a string.
     */
    remote_nm = strlwr(strdup(remotehost));
    sprintf(remote_ip, "%lu.%lu.%lu.%lu",
            ((unsigned long)remoteaddr      ) & 0xff,
            ((unsigned long)remoteaddr >> 8 ) & 0xff,
            ((unsigned long)remoteaddr >> 16) & 0xff,
            ((unsigned long)remoteaddr >> 24) & 0xff);

    if(dolog(LOG_PERM))
        syslog(LOG_DEBUG, "Checking host permissions for %s (%s)",
               remote_nm, remote_ip);

    /*
     * If accessfile is unspecified, allow local hosts only.
     */
    if(accessfile == NULL || *accessfile == '\0' || access(accessfile, 0)) {
        if(strcmp(remote_ip, "127.0.0.1") == 0 ||
           strcmp(remote_nm, "localhost") == 0 ||
           stricmp(remote_nm, cfg.mydomain) == 0) {
            perms->canpost = perms->canread = perms->canxfer = 1;
            result = 0;
            if(dolog(LOG_PERM))
                syslog(LOG_DEBUG, "Local access granted");
        }
        else
            syslog(LOG_WARNING, "Remote access of %s denied", remote_nm);
        free(remote_nm);
        return(result);
    }

    /*
     * See if our internal buffers need an update.
     */
    update_hostperms(accessfile);

    /*
     * Locate the table entry, which fits with the given hostname
     * or ip address. Start searching from the end of the table,
     * because we want the last item to be the most significant
     * and we want to break our search loop upon the first entry
     * found.
     */
    for(i = nhost_perms; --i >= 0; ) {
        if(wildmat(remote_nm, host_perm[i].host) ||
           wildmat(remote_ip, host_perm[i].host)) {

            /*
             * Reject connects from host, which neither can
             * read nor xfer.
             */
            if(!host_perm[i].canread && !host_perm[i].canxfer) {
                syslog(LOG_WARNING, "Access of %s denied", remote_nm);
                break;
            }

            /*
             * Set the permission structure.
             */
            perms->canread = host_perm[i].canread;
            perms->canxfer = host_perm[i].canxfer;
            perms->canpost = host_perm[i].canpost;
            perms->needreadauth = host_perm[i].needreadauth;
            perms->needpostauth = host_perm[i].needpostauth;
            if(host_perm[i].groups)
                perms->ngaccess = strdup(host_perm[i].groups);
            if(host_perm[i].nopost)
                perms->ngreadonly = strdup(host_perm[i].nopost);
            if(dolog(LOG_PERM)) {
                syslog(LOG_DEBUG, "%s %s %s%s%s %s", remotehost,
                       perms->canread ? (perms->canxfer ? "both": "read") :
                                        (perms->canxfer ? "xfer" : "no"),
                       perms->canpost ? "post" : "no",
                       perms->ngreadonly ? "/" : "",
                       perms->ngreadonly ? perms->ngreadonly : "",
                       perms->ngaccess ? perms->ngaccess : "");
                if(perms->needreadauth || perms->needpostauth)
                    syslog(LOG_DEBUG, "Authentication required %s",
                           perms->needreadauth ? "": "for posting");
            }
            result = 0;
            break;
        }
        else if(dolog(LOG_PERM))
            syslog(LOG_DEBUG, "%s (%s) doesn't match %s",
                   remote_nm, remote_ip, host_perm[i].host);
    }
    free(remote_nm);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* user_access                                                          */
/*                                                                      */
/* Queries access permissions for a specified user.                     */
/*                                                                      */
/* Returns -1, if the user is not allowed to login, 0 on success. We    */
/* have three cases, where this may fail:                               */
/*                                                                      */
/*    1. No user_access file.                                           */
/*    2. The specified user has no entry in user_access.                */
/*    3. Incorrect password.                                            */
/*                                                                      */
/************************************************************************/
int user_access(char *accessfile, char *username,
                char *password, ACCESSPERMS *perms)
{
    int i;
    int result = -1;

    /*
     * If user_access doesn't exist, we reject any client access.
     */
    if(accessfile == NULL || accessfile[0] == '\0' || access(accessfile, 0)) {
        syslog(LOG_ERR, "%s not found, access denied", accessfile);
        return(result);
    }

    if(dolog(LOG_PERM))
        syslog(LOG_DEBUG, "Checking permissions for %s", username);

    /*
     * See if our internal buffers need an update.
     */
    update_userperms(accessfile);

    /*
     * Locate the table entry, which fits with the given user-id
     * Start searching from the end of the table, so the last items
     * are the most significant. We stop the search loop upon the
     * first entry found and check the password. If the password
     * is wrong, we won't continue the search but return an error
     * to the caller.
     */
    for(i = nuser_perms; --i >= 0; ) {
        if(wildmat(username, user_perm[i].user)) {
            if(wildmat(password, user_perm[i].pass)) {

                /*
                 * Found a valid entry, update the permission structure.
                 */
                perms->needreadauth = 0;
                perms->needpostauth = 0;
                if(user_perm[i].groups) {
                    if(perms->ngaccess)
                        free(perms->ngaccess);
                    perms->ngaccess = strdup(user_perm[i].groups);
                }
                if(user_perm[i].nopost) {
                    if(perms->ngreadonly)
                        free(perms->ngreadonly);
                    perms->ngreadonly = strdup(user_perm[i].nopost);
                }
                result = 0;
                break;
            }
            else {
                syslog(LOG_NOTICE, "Bad password from %s", username);
                break;
            }
        }
    }

    return(result);
}

/************************************************************************/
/*                                                                      */
/* ctrl_perms                                                           */
/*                                                                      */
/************************************************************************/
int ctrl_perms(char *accessfile, void *from, char *username,
               char *permissions)
{
    int i;
    char remote_ip[32];
    char remotename[128];
    long remoteaddr;
    int result = -1;

    SoGetRemoteHost(from, remotename, &remoteaddr);

    /*
     * Normalize remote name to lower case and convert ip address
     * to a string.
     */
    strlwr(remotename);
    sprintf(remote_ip, "%lu.%lu.%lu.%lu",
            ((unsigned long)remoteaddr      ) & 0xff,
            ((unsigned long)remoteaddr >> 8 ) & 0xff,
            ((unsigned long)remoteaddr >> 16) & 0xff,
            ((unsigned long)remoteaddr >> 24) & 0xff);

    if(dolog(LOG_PERM))
        syslog(LOG_DEBUG, "Checking control permissions for %s (%s)",
               remotename, remote_ip);

    /*
     * If accessfile is unspecified, allow local hosts only and
     * give them global access permission.
     */
    if(accessfile == NULL || *accessfile == '\0' || access(accessfile, 0)) {
        if(strcmp(remote_ip, "127.0.0.1") == 0 ||
           strcmp(remotename, "localhost") == 0 ||
           stricmp(remotename, cfg.mydomain) == 0) {
            strcpy(permissions, "*");
            if(dolog(LOG_PERM))
                syslog(LOG_DEBUG, "Local control access granted for user %s",
                       username);
            result = 0;
        }
        else
            syslog(LOG_NOTICE, "Remote control access of %s denied", remotename);
        return(result);
    }

    /*
     * See if our internal buffers need an update.
     */
    update_ctrlperms(accessfile);

    /*
     * Locate the table entry, which fits with the given hostname
     * or ip address. Start searching from the end of the table,
     * because we want the last item to be the most significant.
     */
    for(i = nctrl_perms; --i >= 0; ) {
        if(wildmat(remotename, ctrl_perm[i].host) ||
           wildmat(remote_ip, ctrl_perm[i].host)) {
            if(wildmat(username, ctrl_perm[i].user)) {
                strcpy(permissions, ctrl_perm[i].perm);
                result = 0;
                if(dolog(LOG_PERM))
                    syslog(LOG_DEBUG, "Remote control access granted for %s@%s",
                           username, remotename);
                break;
            }
        }
    }

    return(result);
}

/************************************************************************/
/*                                                                      */
/* team_access                                                          */
/*                                                                      */
/************************************************************************/
int team_access(char *accessfile, char *remotehost,
                long remoteaddr, char *group, int posting)
{
    int i;
    int result = -1;
    char remote_ip[32];
    char *remote_nm;

    /*
     * If accessfile is unspecified, deny access.
     */
    if(accessfile == NULL || *accessfile == '\0' || access(accessfile, 0))
        return(-1);

    /*
     * Normalize remote name to lower case and convert ip address
     * to a string.
     */
    remote_nm = strlwr(strdup(remotehost));
    sprintf(remote_ip, "%lu.%lu.%lu.%lu",
            ((unsigned long)remoteaddr      ) & 0xff,
            ((unsigned long)remoteaddr >> 8 ) & 0xff,
            ((unsigned long)remoteaddr >> 16) & 0xff,
            ((unsigned long)remoteaddr >> 24) & 0xff);

    if(dolog(LOG_PERM))
        syslog(LOG_DEBUG, "Checking team permissions for %s (%s)",
               remote_nm, remote_ip);

    /*
     * See if our internal buffers need an update.
     */
    update_teamperms(accessfile);

    /*
     * Locate the table entry, which fits with the given groupname.
     * Start searching from the end of the table, because we want
     * the last item to be the most significant and we want to break
     * our search loop upon the first entry found.
     */
    for(i = nteam_perms; --i >= 0; ) {
        if(ngmatch(group, team_perm[i].groups)) {
            if(team_perm[i].hosts[posting] &&
               (ngmatch(remote_nm, team_perm[i].hosts[posting]) ||
                ngmatch(remote_ip, team_perm[i].hosts[posting]))) {
                if(dolog(LOG_PERM))
                    syslog(LOG_WARNING, "Team access of %s granted", remote_nm);
                result = 0;
            }
            else {
                if(dolog(LOG_PERM))
                    syslog(LOG_WARNING, "Team access of %s denied", remote_nm);
                result = -1;
            }
            break;
        }
        else if(dolog(LOG_PERM))
            syslog(LOG_DEBUG, "%s doesn't match %s",
                   group, team_perm[i].groups);
    }
    free(remote_nm);
    return(result);
}
