const char artbatch_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the RNEWS program written by   */
/* Mike Lipsie.                                                         */
/*                                                                      */
/* Parts are copyright by Kendra Electronic Wonderworks.                */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#ifdef __OS2__
#include <pipe.h>
#endif

#include <chlimits.h>
#include <xfile.h>

#include "config.h"
#include "nntp.h"
#include "article.h"
#include "serve.h"
#include "artqueue.h"
#include "client.h"
#include "moderate.h"
#include "control.h"
#include "newsfeeds.h"
#include "distrib.h"
#include "artbatch.h"

static int compressproc(PNEWSCLIENT pnc, FILE *fp, char *cmd, char *buf, size_t buflen, size_t cc);
static int batchproc(PNEWSCLIENT pnc, FILE *fp, char *buf, size_t buflen, size_t art_size);
static int singleproc(PNEWSCLIENT pnc, FILE *fp, char *buf, size_t buflen, size_t art_size);
static size_t read_header(FILE *fp, char *buf, size_t max_bytes, LINBUF *headlines);
static int read_body(FILE *fp, size_t nbytes, LINBUF *bodylines);

extern int dirty_cache;

/************************************************************************/
/*                                                                      */
/* artfileproc                                                          */
/*                                                                      */
/* Processes an article file.                                           */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Filepointer to the article file.                        */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
int artfileproc(PNEWSCLIENT pnc, FILE *fp)
{
    int result = -1;
    size_t buflen = MAX_ARTCHUNK;
    char *buf = malloc(buflen);

    /*
     * Read the first two byte to determine the type of this article file
     */
    if(fread(buf, 1, 2, fp) == 2) {
        if(dolog(LOG_BATCH))
            syslog(LOG_DEBUG, "Read 2 bytes from article: x'%02x x'%02x",
                    (unsigned char)buf[0], (unsigned char)buf[1]);

        /*
         * If the first two bytes are "#!" then we found a batch
         */
        if(*buf == '#' && *(buf + 1) == '!') {
            /*
             * Read the rest of the first line to distinguish between
             * compressed and non-compressed batches.
             */
            if(fgets(buf, (int)buflen, fp) == NULL)
                syslog(LOG_ERR, "Failed to read rest of #! line");

            else if(strnicmp(buf + 2, "unbatch", 7) == 0) {
                /*
                 * We found a compressed batch. At this point
                 * the first line of the file was read and the
                 * rest contains the pure compressed batch. With
                 * this we recursively call this procedure again.
                 */
                if(dolog(LOG_BATCH))
                    syslog(LOG_DEBUG, "This is a compressed batch");
                result = artfileproc(pnc, fp);
            }

            else if(strnicmp(buf + 1, "rnews", 5) == 0) {
                /*
                 * We found a non-compressed batch if the first
                 * line is "#! rnews <size>", where <size> is the
                 * size of the next article in bytes. More articles
                 * might follow.
                 */
                if(dolog(LOG_BATCH))
                    syslog(LOG_DEBUG, "This is a non-compressed batch");
                result = batchproc(pnc, fp, buf, buflen, (size_t)atol(buf + 6));
            }
            else
                syslog(LOG_ERR, "Batch file has unknown format");
        }

        /*
         * The two bytes \x1F\x9D indicate a compressed, \x1F\x8B
         * a zipped archive.
         */
        else if(*buf == '\x1F') {
            switch(*(buf + 1)) {
            case '\x9D':
                if(dolog(LOG_BATCH))
                    syslog(LOG_DEBUG, "Batch is compressed");
                result = compressproc(pnc, fp, cfg.uncompresscall, buf, buflen, 2);
                break;
            case '\x8B':
                if(dolog(LOG_BATCH))
                    syslog(LOG_DEBUG, "Batch is gzipped");
                result = compressproc(pnc, fp, cfg.gunzipcall, buf, buflen, 2);
                break;
            default:
                syslog(LOG_ERR, "Compression method of article file is unknown");
                break;
            }
        }

        /*
         * In any other case we asume this being a single article
         */
        else {
            *(buf + 2) = '\0';
            result = singleproc(pnc, fp, buf, buflen, 0);
        }
    }
    else
        syslog(LOG_DEBUG, "Failed reading article file header");
    free(buf);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* compressproc                                                         */
/*                                                                      */
/* Processes a compressed article file.                                 */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Filepointer to the batch file.                          */
/*                                                                      */
/* cmd          Points to a string that specifies command line to un-   */
/*              compress the file.                                      */
/*                                                                      */
/* buf          This buffer will be used by this procedure to read      */
/*              the compressed file. It may already contain the         */
/*              first part of the file (see parameter cc).              */
/*                                                                      */
/* buflen       Size of buf.                                            */
/*                                                                      */
/* cc           Number of valid characters in buf. These characters     */
/*              may have been read from the file by the caller into     */
/*              buf.                                                    */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
static int compressproc(PNEWSCLIENT pnc, FILE *fp, char *cmd, char *buf, size_t buflen, size_t cc)
{
    int result = -1;
    char *tname = tempnam(cfg.workdir, "ART");
#ifndef __OS2__
    char *t2name;
#endif
    char *cmdline = malloc(_MAX_PATH);
    FILE *ft;
    int rc;

    if((ft = xfopen(tname, "wb")) != NULL) {
        do {
            fwrite(buf, 1, cc, ft);
            cc = fread(buf, 1, buflen, fp);
            if(cc && dolog(LOG_BATCH))
                syslog(LOG_DEBUG, "Read another %u bytes from compressed batch", cc);
        } while(cc);
        if(fclose(ft) == EOF)
            syslog(LOG_ERR, "%s", _strerror(tname));
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "%s", _strerror(tname));

    if(result == 0) {
#ifdef __OS2__
        strcat(strcat(strcpy(cmdline, cmd), " "), tname);
        if(dolog(LOG_BATCH))
            syslog(LOG_DEBUG, "Calling '%s'", cmdline);

        if((ft = popen(cmdline, "rb")) != NULL) {
            result = artfileproc(pnc, ft);
            if((rc = pclose(ft)) != 0)
                syslog(LOG_ERR, "%s returned %u", cmd, (unsigned int)rc >> 8);
            else {
                if(xremove(tname))
                    syslog(LOG_ERR, "%s", _strerror(tname));
            }
        }
        else
            syslog(LOG_ERR, "Failed to run %s", cmdline);
#else
        t2name = tempnam(cfg.workdir, "UNZ");
        sprintf(cmdline, "%s %s > %s", cmd, tname, t2name);
        if((rc = system(cmdline)) != 0)
            syslog(LOG_ERR, "'%s' returned error %d", cmdline, rc);
        else if((ft = xfopen(t2name, "rb")) == NULL)
            syslog(LOG_ERR, "%s", _strerror(t2name));
        else {
            result = artfileproc(pnc, ft);
            if(fclose(ft))
                syslog(LOG_ERR, "%s", _strerror(tname));
        }
        if(xremove(tname))
            syslog(LOG_ERR, "%s", _strerror(tname));
        if(xremove(t2name))
            syslog(LOG_ERR, "%s", _strerror(t2name));
        free(t2name);
#endif
    }

    free(tname);
    free(cmdline);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* batchproc                                                            */
/*                                                                      */
/* Processes a batched article file.                                    */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Filepointer to the article file.                        */
/*                                                                      */
/* buf          This buffer will be used by this procedure to read      */
/*              the compressed file.                                    */
/*                                                                      */
/* buflen       Size of buf.                                            */
/*                                                                      */
/* art_size     Size of the first article in this batch.                */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
static int batchproc(PNEWSCLIENT pnc, FILE *fp, char *buf, size_t buflen, size_t art_size)
{
    int result = -1;
    int skipped;

    while(art_size) {
        skipped = 0;
        *buf = '\0';
        if(dolog(LOG_BATCH))
            syslog(LOG_DEBUG, "Processing batched article with %ld bytes", art_size);
        if(singleproc(pnc, fp, buf, buflen, art_size) != 0)
            result = -1;

        /*
         * Search the next article.
         */
        art_size = 0;
        while(!art_size) {
            if(fgets(buf, (int)buflen, fp) == NULL)
                break;
            if(strnicmp(buf + 3, "rnews", 5) == 0)
                art_size = (size_t)atol(buf + 8);
            else {
                skipped++;
                if(dolog(LOG_BATCH))
                    syslog(LOG_DEBUG, "Line skipped: %s", buf);
            }
        }
        if(skipped)
            syslog(LOG_ERR, "%u lines skipped in batch", skipped);
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/* singleproc                                                           */
/*                                                                      */
/* Processes a single article.                                          */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           File pointer to the article file.                       */
/*                                                                      */
/* buf          This buffer will be used by this procedure to read      */
/*              the compressed file. It may already contain the         */
/*              first part of the file.                                 */
/*                                                                      */
/* buflen       Size of buf.                                            */
/*                                                                      */
/* max_bytes    Size of the article or zero if we should read the       */
/*              whole file in which case an EOF will mark the end       */
/*              of this article.                                        */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if OK, -1 on errors                                   */
/*                                                                      */
/************************************************************************/
static int singleproc(PNEWSCLIENT pnc, FILE *fp, char *buf, size_t buflen, size_t max_bytes)
{
    int result = 0;
    size_t cc;
    size_t hdr_bytes;

#if 0
    /*
     * Number of processed articles. Copied from rnews, but
     * not defined within Changi.
     */
    n_proc++;
#endif

    if(max_bytes && max_bytes < buflen)
        cc = max_bytes;
    else
        cc = buflen;

    if((hdr_bytes = read_header(fp, buf, cc, pnc->art.headbuf)) == 0) {
        return(-1);
    }
    if(dolog(LOG_BATCH))
        syslog(LOG_DEBUG, "Read %ld bytes of header lines", hdr_bytes);
    if(max_bytes == 0 || max_bytes > hdr_bytes) {
        if(dolog(LOG_BATCH))
            syslog(LOG_DEBUG, "Reading body with %ld bytes", max_bytes ? max_bytes - hdr_bytes : 0);
        read_body(fp, max_bytes ? max_bytes - hdr_bytes : 0, pnc->art.bodybuf);
    }
    else if(dolog(LOG_BATCH))
        syslog(LOG_DEBUG, "No room for body, hdr=%d, max=%d", hdr_bytes, max_bytes);

    /*
     * Parse the header and set up the header line array of the
     * article structure. Headers containing duplicate lines are
     * rejected.
     */
    if(!parse_header(&(pnc->art))) {
        pnc->rejected++;
        return(-1);
    }

    if(servermode != SMOD_RUNNING) {
        if(que_article(pnc->remotehost, &(pnc->art))) {
            syslog(LOG_ERR, "Failed to queue article.");
            pnc->rejected++;
            result = -1;
        }
        else {
            syslog(LOG_INFO, "Queued.");
            pnc->received++;
        }
        return(result);
    }

    /*
     * Inspect header lines.
     */
    if(inspect_article(&(pnc->art), NULL, mysite.feedexcl, mysite.feeddist) == 0) {
        syslog(LOG_INFO, "Illegal header, xfer failed.");
        pnc->rejected++;
        return(-1);
    }

    /*
     * Process control messages.
     */
    process_control(&(pnc->art));

    /*
     * Unapproved articles to moderated groups will be mailed to
     * the moderator. We store all other articles in our local
     * newsbase and/or distribute them to external feeds.
     */
    if(mail_moderator(&(pnc->art), cfg.smailcall)) {
        syslog(LOG_INFO, "Can't send article to moderator.");
        pnc->rejected++;
        return(-1);
    }
    switch(file_article(&(pnc->art))) {
    default:
        if(addhistart(&(pnc->art)) == 0)
            break;
    case -1:
        pnc->rejected++;
        return(-1);
    case 0:
        break;
    }
    if(distrib_art(&(pnc->art), 0)) {
        pnc->rejected++;
        return(-1);
    }
    dirty_cache = atoi(cfg.autoflush);
    pnc->received++;

    return(result);
}

/************************************************************************/
/*                                                                      */
/*  read_header                                                         */
/*                                                                      */
/*  Reads an article header from file fp into buf with size bufsiz.     */
/*  Note that buf may contain a previously read string.                 */
/*                                                                      */
/*  Returns number of bytes read or zero on any error.                  */
/*                                                                      */
/************************************************************************/
static size_t read_header(FILE *fp, char *buf, size_t max_bytes, LINBUF *headlines)
{
    int nlines = 0;
    size_t result = 0;
    size_t boff = strlen(buf);    /* Any previously read characters */
    char *cp;

    /*
     * Initialize header storage
     */
    linbuf_clear(headlines);

    /*
     * Read all lines upto the first empty line and
     * interpret them as header lines.
     */
    max_bytes -= boff;
    while(max_bytes) {

        if(fgets(buf + boff, max_bytes, fp) == NULL)
            break;

        /*
         * If linefeed is missing, either the input file
         * is broken or we ran out of space.
         */
        if((cp = strchr(buf, '\r')) == NULL) {
            if((cp = strchr(buf, '\n')) == NULL) {
                result = 0;
                break;
            }
        }

        /*
         * Count <CR> in header lines.
         */
        else
            result++;

        /*
         * Chop off the line feed and check if this line is empty.
         */
        *cp = '\0';
        result += strlen(buf) + 1;
        max_bytes -= strlen(buf) + 1;
        if(*buf == '\0')
            break;
        boff = 0;

        /*
         * Continuation lines start with space or tab. Ignore
         * them until we found our first real header line.
         */
        if(*buf == ' ' || *buf == '\t') {
            /*
             * Ignore continuation lines until we found our first
             * real header line.
             */
            if(nlines)
                linbuf_append(headlines, buf, '\n');
        }

        /*
         * Ignore 'From ' and 'Received: ' lines which may have been
         * left over by some mailers.
         */
        else if(!strnicmp(buf, "From ", 5) ||
                !strnicmp(buf, ">From ", 6) ||
                !strnicmp(buf, "Received: ", 10))
            continue;

        /*
         * Add this line to our buffer.
         */
        else {
            linbuf_add(headlines, buf);
            nlines++;
        }
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int read_body(FILE *fp, size_t nbytes, LINBUF *bodylines)
{
    int  nlines = 0;
    size_t cc;
    char *line = malloc(MAX_BUFSIZ);
    char *cp;

    linbuf_clear(bodylines);
    while(fgets(line, MAX_BUFSIZ, fp)) {
        cc = strlen(line);
        cp = line + cc;

        /*
         * Count <CR><LF> as a single character.
         */
        if(*(--cp) == '\n' || *cp == '\r') {
            cc--;
            *cp = '\0';
            while(*(--cp) == '\n' || *cp == '\r')
                *cp = '\0';
        }
        linbuf_add(bodylines, line);
        nlines++;
        if(nbytes) {
            if(nbytes <= ++cc)
                break;
            nbytes -= cc;
        }
    }

    free(line);

    return(nlines);
}
