/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module deals with article files.
 */

const char artfile_rcsid[] = "$Id: artfile.c,v 1.1 1997/02/13 14:03:28 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <syslog.h>
#include <xfile.h>

#include <chlimits.h>
#include <sostream.h>
#include <nhistory.h>

#include "globals.h"
#include "config.h"
#include "chartran.h"
#include "artfile.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
FILE *openartlink(const char *path)
{
    FILE *fp, *fplink;

    if((fp = xfopen((char *)path, "r")) != NULL) {
        char *buf = malloc(MAX_BUFSIZ);

        if(fgets(buf, MAX_BUFSIZ, fp)) {
            if(*buf == 'X' && strncmp(buf, "X-Linked:", 9) == 0) {
                char *cp, *ip;
                char *link = malloc(_MAX_PATH);

                cp = strchr(strcpy(link, cfg.newsdir), '\0');
                *cp++ = '\\';
                for(ip = buf + 9; *ip == ' '; ip++)
                    continue;
                for(; *ip && *ip != ' ' && *ip != '\n'; ip++)
                    if(*ip == '.' || *ip == ':')
                        *cp++ = '\\';
                    else
                        *cp++ = *ip;
                *cp = '\0';
                if((fplink = xfopen(link, "r")) != NULL) {
                    fclose(fp);
                    fp = fplink;
                }
                else
                    syslog(LOG_NOTICE, "Missing %s", buf);
                free(link);
            }
            else
                rewind(fp);
        }
        free(buf);
    }

    return(fp);
}

/************************************************************************/
/*                                                                      */
/* openartbyid                                                          */
/*                                                                      */
/* Open an article by message-id.                                       */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* msg_id       Points to the string that specifies the message-id of   */
/*              the article to open.                                    */
/* ------------ ------------------------------------------------------- */
/* Return value File pointer to opened article, or NULL if the article  */
/*              was not in the history file or could not be opened.     */
/*                                                                      */
/************************************************************************/
FILE *openartbyid(char *msg_id)
{
    FILE *fp = NULL;
    char *name = strcpy(malloc(_MAX_PATH), cfg.newsdir);

    if(gethistart(msg_id, name))
        fp = openartlink(name);

    free(name);
    return(fp);
}

/************************************************************************/
/*                                                                      */
/* openartbynum                                                         */
/*                                                                      */
/* Open an article by article number.                                   */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* grpdir       Points to the string that specifies the full pathname   */
/*              of the newsgroup's directory.                           */
/*                                                                      */
/* art_num      The number of the article to be opened.                 */
/*                                                                      */
/* msg_id       Points to a character buffer in which the message-id    */
/*              of the opened article will be stored.                   */
/* ------------ ------------------------------------------------------- */
/* Return value File pointer to opened article, or NULL if the article  */
/*              was not found int the group or could not be opened.     */
/*                                                                      */
/************************************************************************/
FILE *openartbynum(char *grpdir, long art_num, char *msg_id)
{
    FILE *fp;
    register char *cp;
    register char *ip;
    char *buf = malloc(MAX_BUFSIZ);

    sprintf(buf, "%s\\%ld", grpdir, art_num);
    if((fp = openartlink(buf)) != NULL) {
        strcpy(msg_id, "<0>");
        while (fgets(buf, MAX_BUFSIZ, fp)) {
            if(*buf == '\n')
                break;

            if(*buf != 'M')
                continue;
            if(strnicmp(buf, "Message-ID:", 11))
                continue;
            cp = buf + 11;
            while(*cp && *cp != '<')
                cp++;
            if(*cp++ != '<')
                continue;
            ip = msg_id + 1;
            while(*cp && *cp != '\n' && *cp != '>') {
                if(*cp != ' ' && *cp != '\t')
                    *ip++ = *cp;
                else
                    *ip++ = '_';
                cp++;
            }
            *ip++ = '>';
            *ip = '\0';
            break;
        }
        rewind(fp);
    }
    free(buf);

    return (fp);
}

/************************************************************************/
/*                                                                      */
/* get_headeritem                                                       */
/*                                                                      */
/* Retrieves a specified header line from the open article file.        */
/*                                                                      */
/************************************************************************/
int get_headeritem(FILE *fp, char *item, char *buf)
{
    register char *cp;
    register char *cw;
    char lc_item = tolower(*item);
    int result = -1;
    char *buf2;

    while(fgets(buf, MAX_BUFSIZ, fp)) {
        if(*buf == '\n')
            break;
        if(tolower(*buf) != lc_item)
            continue;
        for(cp = buf; *cp; cp++)
            if(*cp == ':')
                break;
        if(*cp != ':')
            continue;
        *cp++ = '\0';
        if(stricmp(buf, item) != 0)
            continue;
        while(*cp == ' ' || *cp == '\t')
            cp++;
        for(cw = buf; *cp && *cp != '\n'; )
            *cw++ = *cp++;
        *cw = '\0';
        result = 0;
        break;
    }

    /*
     * 07/07/97 harald
     * Add followup line.
     */
    if(result)
        return(result);

    /*
     * 07/19/97 harald
     * Using buf again destroyed our buffer contents. Hacked in
     * a new buffer.
     */
    buf2 = malloc(MAX_BUFSIZ);
    while(fgets(buf2, MAX_BUFSIZ, fp)) {
        if(*buf2 == '\n' || (*buf2 != ' ' && *buf2 != '\t'))
            break;
        for(cp = buf2; *cp; cp++)
            if(*cp != ' ' && *cp != '\t')
                break;
        for(*cw++ = ' '; *cp && *cp != '\n'; )
            *cw++ = *cp++;
        *cw = '\0';
    }
    free(buf2);
    return(0);
}

/************************************************************************/
/*                                                                      */
/* spew_artfile                                                         */
/*                                                                      */
/* Spews out the contents of an article file to a socket stream.        */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* s            Socket handle to send our output to.                    */
/*                                                                      */
/* fp           File pointer of the article to be transmitted.          */
/*                                                                      */
/* how          This parameter tells what part of the article we want   */
/*              spewed:                                                 */
/*                  WANT_ARTICLE The entire thing.                      */
/*                  WANT_HEAD    Just the first part.                   */
/*                  WANT_BODY    Just the second part.                  */
/* ------------ ------------------------------------------------------- */
/* Return value None.                                                   */
/*                                                                      */
/************************************************************************/
void spew_artfile(int s, FILE * fp, int how, void (*ctran)(char *))
{
    char *xline;
    register char *line;
    register char *cp;

    if(how == WANT_STAT)
        return;

    xline = malloc(MAX_BUFSIZ);
    *xline = '.';
    line = xline + 1;
    while (fgets(line, MAX_BUFSIZ - 6, fp) != NULL && *line != '\n') {
        if(how == WANT_BODY)
            continue;
#if 0
        if(cfg.char_translate && strncmp(line, "Subject:", 8) == 0)
            tran2rdr(line + 9);
#endif
        if(ctran != NULL && strncmp(line, "Subject:", 8) == 0)
            (*ctran)(line + 9);
        if((cp = strchr(line, '\n')) != NULL) {
            *cp++ = '\r';
            *cp++ = '\n';
            *cp = '\0';
        }
        else
            strcat(line, "\r\n");
        if (*line == '.') {
            if (SoStreamPuts(s, xline) == -1) {
                free(xline);
                return;
            }
        }
        else if (SoStreamPuts(s, line) == -1) {
            free(xline);
            return;
        }
    }

    if(how == WANT_HEAD) {
        SoStreamPuts(s, ".\r\n");
        free(xline);
	return;
    }
    if(how == WANT_ARTICLE) {
        if(SoStreamPuts(s, "\r\n") == -1) {
            free(xline);
            return;
        }
    }

    while (fgets(line, MAX_BUFSIZ - 6, fp) != NULL) {
#if 0
        if(cfg.char_translate)
            tran2rdr(line);
#endif
        if(ctran != NULL)
            (*ctran)(line);
        if((cp = strchr(line, '\n')) != NULL) {
            *cp++ = '\r';
            *cp++ = '\n';
            *cp = '\0';
        }
        else
            strcat(line, "\r\n");
        if (*line == '.') {
            if(SoStreamPuts(s, xline) == -1)
                break;
        }
        else if(SoStreamPuts(s, line) == -1)
            break;
    }

    free(xline);
    SoStreamPuts(s, ".\r\n");
}

/*
 * $Log: artfile.c,v $
 * Revision 1.1  1997/02/13 14:03:28  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:32  oliver
 * initial revision
 *
 */
