const char article_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * This module contains routines to add new articles
 */
/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include <direct.h>
#include <errno.h>

#include <version.h>
#include <arpatime.h>
#include <sostream.h>
#include <jobid.h>

#include <syslog.h>
#include <xfile.h>

#include <chlimits.h>
#include <split.h>
#include <radixpck.h>
#include <nactive.h>
#include <nhistory.h>
#include <overview.h>

#include "config.h"
#include "misc.h"
#include "grpdist.h"
#include "article.h"

KEYWORD hdkey[] = {
    { "Approved"         , ID_Approved         , HO_IGNORE   , HO_IGNORE   , 0 },
    { "Bytes"            , ID_Bytes            , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Content-Transfer-Encoding", ID_Content_Transfer_Encoding, HO_IGNORE , HO_IGNORE , 1 },
    { "Content-Type"     , ID_Content_Type     , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Control"          , ID_Control          , HO_IGNORE   , HO_IGNORE   , 0 },
    { "Date"             , ID_Date             , HO_MUSTHAVE , HO_IGNORE   , 1 },
    { "Date-Received"    , ID_Date_Received    , HO_REMOVE   , HO_REMOVE   , 1 },
    { "Distribution"     , ID_Distribution     , HO_IGNORE   , HO_IGNORE   , 0 },
    { "Expires"          , ID_Expires          , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Followup-To"      , ID_Followup_To      , HO_IGNORE   , HO_IGNORE   , 0 },
    { "From"             , ID_From             , HO_MUSTHAVE , HO_MUSTHAVE , 0 },
    { "Keywords"         , ID_Keywords         , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Lines"            , ID_Lines            , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Message-ID"       , ID_Message_ID       , HO_MUSTHAVE , HO_IGNORE   , 0 },
    { "Mime-Version"     , ID_Mime_Version     , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Newsgroups"       , ID_Newsgroups       , HO_MUSTHAVE , HO_MUSTHAVE , 0 },
    { "NNTP-Posting-Host", ID_NNTP_Posting_Host, HO_IGNORE   , HO_REMOVE   , 0 },
    { "Organization"     , ID_Organization     , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Path"             , ID_Path             , HO_MUSTHAVE , HO_IGNORE   , 0 },
    { "Posted"           , ID_Posted           , HO_REMOVE   , HO_REMOVE   , 1 },
    { "Posting-Version"  , ID_Posting_Version  , HO_REMOVE   , HO_REMOVE   , 1 },
    { "Received"         , ID_Received         , HO_REMOVE   , HO_REMOVE   , 1 },
    { "References"       , ID_References       , HO_IGNORE   , HO_IGNORE   , 0 },
    { "Relay-Version"    , ID_Relay_Version    , HO_REMOVE   , HO_REMOVE   , 1 },
    { "Reply-To"         , ID_Reply_To         , HO_IGNORE   , HO_IGNORE   , 0 },
    { "Sender"           , ID_Sender           , HO_IGNORE   , HO_IGNORE   , 0 },
    { "Subject"          , ID_Subject          , HO_MUSTHAVE , HO_MUSTHAVE , 0 },
    { "Summary"          , ID_Summary          , HO_IGNORE   , HO_IGNORE   , 1 },
    { "Supersedes"       , ID_Supersedes       , HO_IGNORE   , HO_IGNORE   , 0 },
    { "X-Newsgroups"     , ID_X_Newsgroups     , HO_REMOVE   , HO_REMOVE   , 1 },
    { "Xref"             , ID_Xref             , HO_REMOVE   , HO_REMOVE   , 1 }
};

int nhdkeys = (sizeof(hdkey) / sizeof(*hdkey));

static int cmpkey(const void *line, const void *pkey);

static KEYIDX link_header[] = {
    ID_Xref      ,
    ID_Newsgroups,
    ID_Message_ID,
    ID_References,
    ID_Subject   ,
    ID_Date      ,
    ID_From      ,
    ID_Lines
};

static int nlinkheads = (sizeof(link_header) / sizeof(*link_header));

/************************************************************************/
/*                                                                      */
/************************************************************************/
static int cmpkey(const void *line, const void *pkey)
{
    return(stricmp((char *)line, ((KEYWORD *)pkey)->name));
}

/************************************************************************/
/*                                                                      */
/* check_control                                                        */
/*                                                                      */
/* See if a control message is valid.                                   */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* ctrl         Points to a string containing the control message.      */
/*              This is usually pointing to the Control: header line.   */
/*                                                                      */
/* cmd          Pointer to a character buffer of 'maxlen' + 1 bytes     */
/*              in size, where this routine will store the lowercase    */
/*              control word.                                           */
/*                                                                      */
/* maxlen       Maximum size of a legal control word. Note that 'cmd'   */
/*              must point to a buffer with one byte more in size for   */
/*              the string delimiter.                                   */
/* ------------ ------------------------------------------------------- */
/* Return value 1 if the control message is valid, 0 if it's unknown    */
/*              and -1 on errors (missing id in cancel)                 */
/************************************************************************/
static int check_control(char *ctrl, char *cmd, size_t maxlen)
{
    char *cp;

    /*
     * Get first word
     */
    while(*ctrl == ' ' || *ctrl == '\t')
        ctrl++;

    for(cp = cmd; *ctrl && *ctrl != ' ' && *ctrl != '\t'; ctrl++) {
        if(maxlen-- == 0) {
            syslog(LOG_INFO, "Control command too long");
            return(-1);
        }
        *cp++ = tolower(*ctrl);
    }
    *cp = '\0';

    /*
     * Check message-id in cancel
     */
    if(strcmp(cmd, "cancel") == 0) {
        while(*ctrl == ' ' || *ctrl == '\t')
            ctrl++;
        if(*ctrl != '<') {
            syslog(LOG_INFO, "Missing or invalid message-id in cancel");
            return(-1);
        }
        return(1);
    }

    /*
     * See if it's a valid control message
     */
    else if(strcmp(cmd, "sendsys") == 0 ||
            strcmp(cmd, "senduuname") == 0 ||
            strcmp(cmd, "version") == 0 ||
            strcmp(cmd, "checkgroups") == 0 ||
            strcmp(cmd, "ihave") == 0 ||
            strcmp(cmd, "sendme") == 0 ||
            strcmp(cmd, "newgroup") == 0 ||
            strcmp(cmd, "rmgroup") == 0) {
        return(1);
    }

    syslog(LOG_ERR, "Unknown control message '%s'", cmd);
    return(0);
}

/************************************************************************/
/* parse_header                                                         */
/*                                                                      */
/* Parses an article header. Sets up the header line array in the       */
/* article structure. From line leftovers are removed.                  */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* artp         Points to a structure containing the article.           */
/* ------------ ------------------------------------------------------- */
/* Return value 1 if header is valid, 0 if rejected (due to duplicate   */
/*              header lines).                                          */
/*                                                                      */
/************************************************************************/
int parse_header(ARTICLE *artp)
{
    int  i;
    int  nxheader = 0;
    char *cp;
    char *line;

    /*
     * Initialize header storage
     */
    memset(artp->header, 0, sizeof(artp->header));
    for(i = nhdkeys; --i >= 0; )
        artp->header[hdkey[i].idx].key = hdkey[i].name;

    /*
     * Parse all lines
     */
    for(i = 0; i < artp->headbuf->nlines; i++) {
        line = LBPTR(artp->headbuf, i);

        if((cp = strchr(line, ':')) != NULL) {
            KEYWORD *pkey;
            char *info = cp + 1;

            *cp = '\0';
            pkey = bsearch(line, hdkey, (size_t)nhdkeys, sizeof(KEYWORD), cmpkey);

            /*
             * Skip leading spaces and do not store empty lines
             */
            while(*info == ' ' || *info == '\t')
                info++;
            if(*info) {
                if(pkey) {
                    /*
                     * Do not accept articles with duplicate headers
                     * 07/07/97 harald
                     * Headers are rejected only, if they are
                     * important (new 'acpt_dupes' in KEYWORD)
                     * and if they contain different infos.
                     */
                    if(artp->header[pkey->idx].info) {
                        if(stricmp(artp->header[pkey->idx].info, info)) {
                            syslog(pkey->acpt_dupes ? LOG_INFO : LOG_NOTICE,
                                   "Duplicate '%s' header %s\n", pkey->name,
                                   pkey->acpt_dupes ? "removed" : "rejected");
                            syslog(LOG_INFO, "First is '%s'\n", artp->header[pkey->idx].info);
                            syslog(LOG_INFO, "Second is '%s'\n", info);
                            if(pkey->acpt_dupes == 0)
                                return(0);
                        }
                        else
                            syslog(LOG_INFO, "Duplicate '%s' header removed\n", pkey->name);
                    }
                    else
                        artp->header[pkey->idx].info = info;
                }
                else if(nxheader < MAX_XHEADER) {
                    artp->header[ID_LASTENTRY + nxheader].key = line;
                    artp->header[ID_LASTENTRY + nxheader++].info = info;
                }
            }
        }
    }
    artp->header[ID_LASTENTRY + nxheader].key = NULL;

    return(1);
}

/************************************************************************/
/*                                                                      */
/* Trim trailing spaces, return pointer to first non-space char.        */
/*                                                                      */
/************************************************************************/
static char *trim_spaces(char *s)
{
    register char *cp;
    char *first;
    char *last = NULL;

    for(first = s; *first == ' ' || *first == '\t';)
        first++;

    for(cp = first; *cp; cp++)
        if(*cp != ' ' && *cp != '\t')
            last = cp;

    if(last)
        *++last = '\0';

    return(first);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int want_distrib(char **list, char *dist)
{
    int negated = 0;
    char *cp;

    for(; *list; list++) {
        if(*(cp = *list) == '!') {
            negated = 1;
            if(stricmp(cp + 1, dist) == 0)
                return(0);
        }
        else if(stricmp(cp, dist) == 0)
            return(1);
    }

    return(negated);
}

/************************************************************************/
/*                                                                      */
/* ------------ ------------------------------------------------------- */
/* Return value 1 if successfully added, 0 if group not found or        */
/*              -1 if group already in list.                            */
/************************************************************************/
static int add_xref(ARTICLE *artp, const char *group)
{
    int i;
    long artnum;

    /*
     * Do not accept duplicates
     */
    for(i = artp->nxrefs; --i >= 0;)
        if(*(artp->xrefs[i]) == *group && strcmp(artp->xrefs[i], group) == 0) {
            return(-1);
        }

    /*
     * Get a new article number for the current group. If the
     * newsgroup hasn't been defined in the active file, then
     * 'inc_active' returns zero.
     */
    if((artnum = inc_active(group)) == 0)
        return(0);

    /*
     * If there isn't enough room left in our xref array, then
     * let's allocate ten more pointers.
     */
    if((artp->nxrefs + 1) >= artp->xxrefs) {
        artp->xxrefs += 10;
        artp->xrefs = realloc(artp->xrefs, sizeof(char *) * artp->xxrefs);
        artp->xrefnums = realloc(artp->xrefnums, sizeof(long) * artp->xxrefs);
    }

    /*
     * Add this article reference to the xref array.
     */
    artp->xrefnums[artp->nxrefs] = artnum;
    artp->xrefs[artp->nxrefs++] = (char *)group;
    artp->xrefs[artp->nxrefs] = NULL;

    return(1);
}

/************************************************************************/
/*                                                                      */
/* addhistart                                                           */
/*                                                                      */
/* Adds a new line to the history database.                             */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* artp         Points to the article structure.                        */
/* ------------ ------------------------------------------------------- */
/* Return value 0 on success, -1 on errors.                             */
/*                                                                      */
/************************************************************************/
int addhistart(ARTICLE *artp)
{
    char *xp;
    int  i;
    int  result;
    char *artlist = malloc(MAX_HISTORYLINE);

    /*
     * Set the date fields
     */
    time(&artp->arrived);
    if(artp->header[ID_Expires].info)
        artp->expires = getadate(artp->header[ID_Expires].info);
    if(artp->header[ID_Date].info)
        artp->posted = getadate(artp->header[ID_Date].info);

    /*
     * Build a group list with format 'group:num[,group:num...]'
     */
    xp = artlist;
    if(artp->nxrefs) {
        for(i = 0; i < artp->nxrefs; i++) {
            if(artp->xrefnums[i]) {
                strcpy(xp, artp->xrefs[i]);
                xp += strlen(xp);
                *xp++ = ':';
                ltoa(artp->xrefnums[i], xp, 10);
                xp += strlen(xp);
                *xp++ = ',';
            }
        }
        *--xp = '\0';
    }
    else
        *xp = '\0';

    /*
     * Add the message-ID to the history database.
     */
    if(artp->posted)
        result = addhistentry(artp->header[ID_Message_ID].info,
                              artp->arrived, artp->expires, artp->posted,
                              artp->artsize, artlist);
    else
        result = addhistentry(artp->header[ID_Message_ID].info,
                              artp->arrived, artp->expires, artp->arrived,
                              artp->artsize, artlist);
    free(artlist);
    return(result);
}

/************************************************************************/
/* inspect_article                                                      */
/*                                                                      */
/* Validates an incoming article.                                       */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* artp         Points to a structure containing the article.           */
/*                                                                      */
/* poster       Points to a string that specifies the name of the       */
/*              posting host. This name will be used to build the       */
/*              sender line for the header. Set this pointer to NULL    */
/*              if the article wasn't received by a POST command.       */
/*                                                                      */
/* exclusions   An array of strings specifying additional hostnames     */
/*              of our site. If any of these names appears in the       */
/*              message path line, the article is rejected. The last    */
/*              item must contain a NULL pointer. You may use a NULL    */
/*              pointer for the array itself to declare an empty        */
/*              array.                                                  */
/*                                                                      */
/* distribs     This string array specifies a list of distributions     */
/*              to be matched against the articles distribution header. */
/*              If it doesn't match, then the article is rejected.      */
/*              The last item must contain a NULL pointer. If the       */
/*              array is a NULL pointer itself, any distribution is     */
/*              accepted.                                               */
/* ------------ ------------------------------------------------------- */
/* Return value 1 if header is valid, 0 if rejected                     */
/*                                                                      */
/************************************************************************/
int inspect_article(ARTICLE *artp, char *poster, char **exclusions, char **distribs)
{
    char *item = malloc(MAX_ITEMSIZE);
    time_t now;
    char *cp;
    HEADER *hp;
    int i;
    HOPTION hopt;
    int valid = 1;
    char **ip;

    /*
     * Clear leftovers from last article.
     */
    time(&now);
    artp->artsize = 0;
    artp->posted = 0;
    artp->arrived = now;
    artp->expires = 0;
    artp->x_control[0] = artp->x_control[8] = '\0';
    artp->nxrefs = 0;
    artp->hopcount = 0;
    artp->n_followups = 0;

    if(artp->distribinfo) {
        free(artp->distribinfo);
        artp->distribinfo = NULL;
        if(artp->distriblist) {
            free(artp->distriblist);
            artp->distriblist = NULL;
        }
    }

    if(artp->groupsinfo) {
        free(artp->groupsinfo);
        artp->groupsinfo = NULL;
        if(artp->groupslist) {
            free(artp->groupslist);
            artp->groupslist = NULL;
        }
    }

    if(artp->pathinfo) {
        free(artp->pathinfo);
        artp->pathinfo = NULL;
        if(artp->pathlist) {
            free(artp->pathlist);
            artp->pathlist = NULL;
        }
    }

    /*
     * Check if all header lines are acceptable or if
     * any are missing.
     */
    *item = '\0';
    for(i = ID_LASTENTRY; valid && --i >= 0; ) {
        if(poster)
            hopt = hdkey[i].in_post;
        else
            hopt = hdkey[i].in_xfer;

        hp = &(artp->header[hdkey[i].idx]);
        switch(hopt) {

        /*
         * Silently remove this header
         */
        case HO_REMOVE:
            if(hp->info) {
#if 0
                syslog(LOG_INFO, "Removed %s: %s", hp->key, hp->info);
#endif
                hp->info = NULL;
            }
            break;

        /*
         * Drop articles without this header
         */
        case HO_MUSTHAVE:
            if(hp->info)
                hp->info = trim_spaces(hp->info);
            else {
                syslog(LOG_INFO, "Header '%s' missing in post", hp->key);
                valid = 0;
            }
            break;

        /*
         * Drop articles with this header
         */
        case HO_DONTWANT:
            if(hp->info) {
                syslog(LOG_INFO, "Header '%s' not accepted in post\n", hp->key);
                valid = 0;
            }
            break;

        case HO_IGNORE :
            if(hp->info)
                hp->info = trim_spaces(hp->info);
            break;

        default:
            syslog(LOG_ERR, "Internal error inspecting post");
            valid = 0;
        }
    }

    /*
     * Create a newsgroup list first because we need it with
     * the distribution header.
     *
     * 09/05/1996 harald
     * Create for valid articles only, otherwise we may die.
     */
    if(valid) {
        artp->groupsinfo = strlwr(strdup(artp->header[ID_Newsgroups].info));
        artp->groupslist = split_list(artp->groupsinfo, ',');
        for(artp->n_groups = 0, ip = artp->groupslist; *ip; ip++) {
            if((cp = strrchr(*ip, '.')) != NULL && strcmp(cp, ".ctl") == 0)
                *cp = '\0';
            artp->n_groups++;
        }
    }

    /*
     * Now loop through all headers again for some
     * more specific processing.
     */
    for(i = 0; valid && i < ID_LASTENTRY; i++) {
        hp = &(artp->header[i]);
        switch(i) {
        /*
         * If there isn't a date or if we can't interpret
         * it or if the article has been posted in the future,
         * then we fill this item. RFC 1036 recommends that
         * the date should be transmitted in GMT.
         */
        case ID_Date:
            if(hp->info) {
                if((artp->posted = getadate(hp->info)) == 0)
                    syslog(LOG_INFO, "Can't parse header date '%s'", hp->info);
                else if(artp->posted > now + (time_t)86400) {
                    syslog(LOG_INFO, "Article posted in future '%s'", hp->info);
                    artp->posted = 0;
                }
            }
            if(artp->posted == 0) {
                artp->posted = now;
                if(poster)
                    strftime(item, MAX_ITEMSIZE, "%a, %d %b %Y %H:%M:%S GMT", gmtime(&now));
            }
            break;

        /*
         * If the message id is missing, then we add a new one.
         */
        case ID_Message_ID:
            if(hp->info == NULL) {
                char pck[MAX_PCKSTRING];
                sprintf(item, "<c%s.%s$%s@%s>", version,
                        radixpack((unsigned long)time(NULL), pck),
                        job_id(cfg.seqf, 0), cfg.mydomain);
            }
            else {
                cp = hp->info;
                if(*cp != '<') {
                    valid = 0;
                    break;
                }
                else {
                    for( ; *cp; cp++)
                        if(*cp == ' ' || *cp == '\t') {
                            valid = 0;
                            break;
                        }
                    if(*--cp != '>')
                        valid = 0;
                }
                if(valid == 0)
                    syslog(LOG_INFO, "Invalid message id %s", hp->info);

                /*
                 * 08/21/97 harald
                 * When processing IHAVE, we don't need to check
                 * for dupes again. Added 'if(poster)'.
                 */
                if(poster && gethistart(hp->info, NULL)) {
                    valid = 0;
                    syslog(LOG_INFO, "Duplicate article rejected: '%s'", hp->info);
                }
            }
            break;

        case ID_Sender:
            if(poster) {
                char *addr, *user;
                char *sender = strdup(artp->header[ID_From].info);
                if(get_originator(sender, &addr, &user)) {
                    syslog(LOG_INFO, "Bad From line: %s", sender);
                    valid = 0;
                }
                else {
                    if(*poster != '\0') {
                        strcpy(item, poster);
                        strcat(item, "@");
                        strcat(item, cfg.pathhost);
                    }
                    else {
                        if((cp = strchr(addr, '@')) != NULL)
                            *cp = '\0';
                        strcpy(item, addr);
                        strcat(item, "@");
                        strcat(item, artp->fromhost);
                    }
                }
                free(sender);
            }
            break;

        /*
         * If this is a valid control message then we create
         * the subject line. Articles containing invalid
         * control messages are rejected.
         */
        case ID_Subject:
            if(artp->header[ID_Control].info) {
                if(check_control(artp->header[ID_Control].info,
                                 &(artp->x_control[8]),
                                 sizeof(artp->x_control) - 9) == -1) {
                    valid = 0;
                }

                /*
                 * 07/08/97 harald
                 * Do not modify control messages
                 */
#if 0
                else {
                    strcpy(item, "cmsg ");
                    strcat(item, artp->header[ID_Control].info);
                }
#endif
            }
            break;

        /*
         * If the first word in the subject line is 'cmsg' then
         * we create a missing control header.
         */
        case ID_Control:
            if(hp->info == NULL &&
               strnicmp(artp->header[ID_Subject].info, "cmsg ", 5) == 0) {
                cp = artp->header[ID_Subject].info + 5;
                while(*cp == ' ' || *cp == '\t')
                    cp++;
                if(check_control(cp, &(artp->x_control[8]),
                                 sizeof(artp->x_control) - 9) == -1) {
                    valid = 0;
                }
                else
                    hp->info = cp;
            }
            break;

        /*
         * If the number of lines isn't supplied, we use our count
         * to create this one.
         */
        case ID_Lines:
            if(hp->info == NULL)
                ltoa(artp->bodybuf->nlines, item, 10);
            break;

        /*
         * We remove expire dates we couldn't parse.
         */
        case ID_Expires:
            if(hp->info != NULL) {
                if((artp->expires = getadate(hp->info)) == 0) {
                    syslog(LOG_INFO, "Can't parse expire date '%s'", hp->info);
                    if(poster)
                        hp->info = NULL;
                }
            }
            break;

        /*
         * Reject articles with invalid distributions.
         * 09/01/1996 harald
         * Added some code to allow posting to local groups.
         */
        case ID_Distribution:
            if(poster) {
                if(cfg.distribution[0])
                    strcpy(item, cfg.distribution);
                else if((cp = get_groupdistrib(artp->groupslist)) != NULL)
                    strcpy(item, cp);
                if(*item) {
                    hp->info = linbuf_add(artp->headbuf, item);
                    *item = '\0';
                }
            }
            if(hp->info != NULL)
                artp->distribinfo = strdup(hp->info);
            else
                break;
            artp->distriblist = split_list(artp->distribinfo, ',');
            if(distribs) {
                valid = 0;
                for(ip = artp->distriblist; *ip; ip++) {
                    if(want_distrib(distribs, *ip)) {
                        valid = 1;
                        break;
                    }
                }
            }
            if(!valid)
                syslog(LOG_INFO, "Unwanted distribution \"%s\"", hp->info);
            break;

        /*
         * Make a list of newsgroups.
         */
        case ID_Newsgroups:
            if(artp->n_groups == 0) {
                valid = 0;
                syslog(LOG_INFO, "Empty newsgroups header");
                break;
            }

            /*
             * In the following part we set three values:
             *
             * 'groupsmode' is an array of characters keeping one
             * character per group. Groups marked 'y' will accept
             * this article for local storage. Local posts to groups
             * marked 'm' will be mailed to the moderator.
             *
             * 'validgroups' counts the number of groups we accepted.
             * In fact this is the number of 'y' characters in
             * 'groupsmode'.
             *
             * 'moderated' counts the number of moderated groups,
             * if article has been approved. This number is included
             * in 'validgroups'.
             *
             * 09/05/1996 harald
             * Some bugs found here:
             * We should accept 'n' groups received by xfer.
             * 'moderated' wasn't cleared.
             * Accepted moderated groups for distribution.
             */
            artp->groupsmode = realloc(artp->groupsmode, artp->n_groups);
            artp->validgroups = artp->moderated = 0;
            for(cp = artp->groupsmode, ip = artp->groupslist; *ip; cp++, ip++) {
                if((*cp = test_active(*ip)) == 'm') {
                    artp->moderated++;
                    if(artp->header[ID_Approved].info != NULL)
                        *cp = 'y';
                }
                else if(*cp == 'n' && poster == NULL)
                    *cp = 'y';
                if(*cp == 'y')
                    artp->validgroups++;
            }
            if(poster && artp->validgroups == 0 && artp->moderated == 0) {
                valid = 0;
                syslog(LOG_INFO, "No local posts allowed in group(s)");
            }
            break;

        case ID_Followup_To:
            if(hp->info != NULL && poster != NULL) {
                if(stricmp(hp->info, "poster") == 0)
                    artp->n_followups = 1;
                else {
                    char ch;
                    char *groupsinfo = strlwr(strdup(hp->info));
                    char **groupslist = split_list(groupsinfo, ',');
                    for(ip = groupslist; *ip; ip++) {
                        if((ch = test_active(*ip)) == 'm' || ch == 'y')
                            artp->n_followups++;
                    }
                    if(artp->n_followups == 0) {
                        valid = 0;
                        syslog(LOG_INFO, "Invalid followup group(s)");
                    }
                    free(groupsinfo);
                    free(groupslist);
                }
            }
            break;

        /*
         * If configured to do so, we'll add our organization
         * unless there isn't defined one already.
         */
        case ID_Organization:
            if(poster && hp->info == NULL && cfg.organization[0])
                hp->info = cfg.organization;
            break;

        /*
         * Check for unwanted sites in the path. Add ourself to
         * the path or create a new one, if none exists already.
         */
        case ID_Path:
            if(hp->info) {
                artp->pathinfo = strlwr(strdup(hp->info));
                artp->pathlist = split_pathline(artp->pathinfo);
                for(ip = artp->pathlist; *ip != NULL && valid != 0; ip++) {
                    artp->hopcount++;
                    if(stricmp(*ip, cfg.pathhost) == 0) {
                        syslog(LOG_INFO, "Found %s in path", cfg.pathhost);
                        valid = 0;
                    }
                    else if(exclusions != NULL) {
                        if(in_list(*ip, exclusions)) {
                            syslog(LOG_INFO, "Found %s in path", *ip);
                            valid = 0;
                            break;
                        }
                    }
                }
                if(valid) {
                    strcpy(item, cfg.pathhost);
                    strcat(item, "!");
                    strcat(item, hp->info);
                }
            }
            else {
                strcpy(item, cfg.pathhost);
                strcat(item, "!");
                strcat(item, cfg.pathmaster);
                artp->hopcount = 1;
            }
            break;

        /*
         * Add mime information if configured to do so.
         */
        case ID_Mime_Version:
            if(poster && hp->info == NULL && cfg.mimeversion[0]) {
                hp->info = cfg.mimeversion;
                if(artp->header[ID_Content_Type].info == NULL && cfg.mimetype[0])
                    artp->header[ID_Content_Type].info = cfg.mimetype;
                if(artp->header[ID_Content_Transfer_Encoding].info == NULL && cfg.mimeencoding[0])
                    artp->header[ID_Content_Transfer_Encoding].info = cfg.mimeencoding;
            }
            break;

        /*
         * If this is a post, we only trust ourself knowing
         * the one who posted this articles.
         */
        case ID_NNTP_Posting_Host:
            if(poster) {
                hp->info = NULL;
                strcpy(item, artp->fromhost);
            }
            break;
        }

        /*
         * Add new items to the header.
         */
        if(valid && *item) {
            hp->info = linbuf_add(artp->headbuf, item);
            *item = '\0';
        }
    }

    /*
     * Return now for invalid articles.
     */
    if(!valid) {
        free(item);
        return(0);
    }

    /*
     * Create xref list.
     */
    if(artp->x_control[8]) {
        memcpy(artp->x_control, "control.", 8);
        if(add_xref(artp, artp->x_control) == 0) {
            artp->x_control[7] = '\0';
            add_xref(artp, artp->x_control);
        }
    }
    else {
        for(cp = artp->groupsmode, ip = artp->groupslist; *ip; ip++, cp++) {
            if(*cp == 'y') {
                if(strncmp(*ip, "to.", 3) == 0)
                    add_xref(artp, "to");
                else
                    add_xref(artp, *ip);
            }
        }

        if(artp->nxrefs == 0) {
            if(dolog(LOG_JUNK))
                syslog(LOG_DEBUG, "Junked %s, no valid groups", artp->header[ID_Message_ID].info);
            strcpy(artp->x_control, cfg.junkgroup);
            add_xref(artp, artp->x_control);
        }

        /*
         * Add all replies to us to our local reply group.
         *
         * 09/05/1996 harald
         * Need a copy to split the list in order to preserve the
         * original references.
         */
        if(cfg.local_replygroup && artp->header[ID_References].info != NULL) {
            char *refinfo = strdup(artp->header[ID_References].info);
            char **reflist = split_list(refinfo, ' ');

            for(ip = reflist; *ip; ip++) {
                if((cp = strrchr(*ip, '@')) != NULL &&
                   strnicmp(cfg.mydomain, cp + 1, strlen(cfg.mydomain)) == 0) {
                    add_xref(artp, "local.reference");
                    break;
                }
            }
            free(reflist);
            free(refinfo);
        }
    }

    /*
     * Create xref line.
     */
    if(artp->nxrefs) {
        cp = strcpy(item, cfg.mynode);
        for(i = 0; i < artp->nxrefs; i++) {
            cp += strlen(cp);
            sprintf(cp, " %s:%lu", artp->xrefs[i], artp->xrefnums[i]);
        }
        artp->header[ID_Xref].info = linbuf_add(artp->headbuf, item);
    }
    free(item);
    return(valid);
}

/************************************************************************/
/*                                                                      */
/* init_article                                                         */
/*                                                                      */
/************************************************************************/
void init_article(ARTICLE *artp, char *remotehost)
{
    artp->fromhost = remotehost;
    artp->headbuf = linbuf_create(NULL);
    artp->bodybuf = linbuf_create(NULL);
}

/************************************************************************/
/*                                                                      */
/* free_article                                                         */
/*                                                                      */
/************************************************************************/
void free_article(ARTICLE *artp)
{
    linbuf_destroy(artp->headbuf);
    linbuf_destroy(artp->bodybuf);
    if(artp->pathinfo) {
        if(artp->pathlist)
            free(artp->pathlist);
        free(artp->pathinfo);
        artp->hopcount = 0;
    }
    if(artp->distribinfo) {
        if(artp->distriblist)
            free(artp->distriblist);
        free(artp->distribinfo);
    }
    if(artp->groupsinfo) {
        if(artp->groupslist)
            free(artp->groupslist);
        free(artp->groupsinfo);
    }
    if(artp->groupsmode)
        free(artp->groupsmode);
    if(artp->xxrefs) {
        free(artp->xrefs);
        free(artp->xrefnums);
    }
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long article_size(ARTICLE *artp, int with_xref)
{
    long result = 0;
    int i;

    if(with_xref && artp->header[ID_Xref].info != NULL)
        result += strlen(artp->header[ID_Xref].info) + 7;
    for(i = 0; artp->header[i].key; i++)
        if(i != ID_Xref && artp->header[i].info != NULL)
                result += strlen(artp->header[i].key) +
                                 strlen(artp->header[i].info) + 3;
    result++;
    for(i = 0; i < artp->bodybuf->nlines; i++)
        result += strlen(LBPTR(artp->bodybuf, i)) + 1;

    return(result);
}

/************************************************************************/
/*                                                                      */
/* spew_article                                                         */
/*                                                                      */
/* Spews out the contents of an article structure file to a file stream */
/* optionally inserting a batch record in front.                        */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           File pointer of the open stream.                        */
/*                                                                      */
/* artp         Points to the article structure. All headers should     */
/*              have been properly set before calling this routine.     */
/*                                                                      */
/* batched      Points to a string buffer containing the header name    */
/*              for the batch. Should be set to NULL to avoid writing   */
/*              a batch record.                                         */
/*                                                                      */
/* linked       If this is not zero, a special link header without the  */
/*              articles body will be written.                          */
/* ------------ ------------------------------------------------------- */
/* Return value Number of bytes written to the stream.                  */
/*                                                                      */
/************************************************************************/
long spew_article(FILE *fp, ARTICLE *artp, char *batched, int linked, int with_xref)
{
    long result = 0;
    register int i;

    /*
     * Writing batch command line. We must recalculate the article size.
     */
    if(batched)
        fprintf(fp, "#! %s %ld\n", batched, article_size(artp, with_xref));

    /*
     * Writing header
     */
    if(linked) {
        result += fprintf(fp, "X-Linked: %s:%lu\n",
                          artp->xrefs[artp->firstgroup],
                          artp->xrefnums[artp->firstgroup]);
        for(i = 0; i < nlinkheads; i++) {
            int lh = link_header[i];

            if(artp->header[lh].info)
                result += fprintf(fp, "%s: %s\n", artp->header[lh].key,
                                                  artp->header[lh].info);
        }
    }
    else {
        if(with_xref)
            result += fprintf(fp, "Xref: %s\n", artp->header[ID_Xref].info);
        for(i = 0; artp->header[i].key; i++) {
            if(i != ID_Xref && artp->header[i].info)
                result += fprintf(fp, "%s: %s\n", artp->header[i].key,
                                                  artp->header[i].info);
        }
    }
    fputc('\n', fp);
    result++;

    /*
     * Writing body
     */
    if(linked == 0) {
        for(i = 0; i < artp->bodybuf->nlines; i++)
            result += fprintf(fp, "%s\n", LBPTR(artp->bodybuf, i));
    }

    return(result);
}

/************************************************************************/
/*                                                                      */
/* file_article                                                         */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* artp         Points to the article structure. All headers should     */
/*              have been properly set before calling this routine.     */
/* ------------ ------------------------------------------------------- */
/* Return value Number of stored article files or -1 on errors.         */
/*                                                                      */
/************************************************************************/
int file_article(ARTICLE *artp)
{
    int  result = 0;
    FILE *fp;
    char *xp;
    char *cp;
    char *artpath = strcpy(malloc(_MAX_PATH), cfg.newsdir);
    char *subpath = artpath + strlen(artpath);
    int  i;
    int  linked = 0;
    int  written;

    /*
     * Add a trailing backslash to the newsgroup's homedir
     * and let 'subpath' point to the variable part of the
     * article's pathname.
     */
    *subpath++ = '\\';

    /*
     * Loop through all xrefs.
     */
    artp->firstgroup = -1;
    for(i = 0; i < artp->nxrefs; i++) {
        cp = artp->xrefs[i];
        xp = subpath;
        while(*cp) {
            if(*cp == '.')
                *xp++ = '\\';
            else
                *xp++ = *cp;
            cp++;
        }
        *xp++ = '\\';
        itoa(artp->xrefnums[i], xp, 10);

        if((fp = xfopen(artpath, "wb")) == NULL) {
            syslog(LOG_ERR, "%s (file_article open)", _strerror(artpath));
            result = -1;
            break;
        }
        *xp = '\0';

        written = spew_article(fp, artp, NULL, linked, artp->nxrefs > 1);

        if(fclose(fp)) {
            syslog(LOG_ERR, "%s (file_article close)", _strerror(artpath));
            result = -1;
            break;
        }

        if(artp->firstgroup == -1) {
            artp->artsize = written;
            artp->firstgroup = i;
            linked = cfg.emulate_links;
        }
        result++;
    }

    free(artpath);

    return(result);
}
