/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char artqueue_rcsid[] = "$Id: artqueue.c,v 1.1 1997/02/13 14:03:30 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <io.h>

#include <syslog.h>
#include <sysdep.h>
#include <mutex.h>
#include <xfile.h>
#include <scangrpd.h>

#include "config.h"
#include "serve.h"
#include "artqueue.h"

static long mutex_wait = 40000L;
#define MUTEXWAIT   (mutex_wait + __LINE__)

static long aq_mutex = -1;
static char quepath[_MAX_PATH];
static char *subpath;
static char scanpath[_MAX_PATH];
static char spoolname[_MAX_PATH];
static char spoolcmd[_MAX_PATH];

static int go_on;
static int busy = 0;

/************************************************************************/
/*                                                                      */
/* artqueue_maxwait                                                     */
/*                                                                      */
/* Set max wait for mutex semaphores.                                   */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* maxwait      Time to wait for mutex access in milliseconds. -1 will  */
/*              wait forever.                                           */
/* ------------ ------------------------------------------------------- */
/* Return value None.                                                   */
/*                                                                      */
/************************************************************************/
void artqueue_maxwait(long maxwait)
{
    mutex_wait = maxwait;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static void _Optlink dequeue_thread(void *pv)
{
    int rc;

    busy = 1;
    while(go_on) {
        for(rc = 0; go_on && rc < 30; rc++)
            ms_sleep(1000);
        if(!go_on)
            break;
        if(servermode != SMOD_RUNNING || !cfg.auto_dequeue)
            continue;
        if(access(spoolname, 0) == 0 || grab_quefile(scanpath, spoolname) == 0) {
            syslog(LOG_INFO, "Respooling %s", spoolname);
            if((rc = system(spoolcmd)) != 0)
                syslog(LOG_ERR, "Error %d: %s", rc, spoolcmd);
            else if(xremove(spoolname))
                syslog(LOG_ERR, "%s", _strerror(spoolname));
        }
    }
    busy = 0;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int init_artque(char *spooldir, char *rnewscall)
{
    if((aq_mutex = create_mutex()) == -1)
        return(-1);

    strcpy(quepath, spooldir);
    strcat(quepath, "\\incoming\\");
    subpath = quepath + strlen(quepath);
    strcpy(scanpath, spooldir);
    strcat(scanpath, "\\incoming");
    strcpy(spoolname, spooldir);
    strcat(spoolname, "\\respool");
    strcpy(spoolcmd, rnewscall);
    strcat(spoolcmd, " < ");
    strcat(spoolcmd, spoolname);

    go_on = 1;
    _beginthread(dequeue_thread, NULL, 32767, NULL);

    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void exit_artque(void)
{
    int retries = 10;

    if(aq_mutex != -1) {
        go_on = 0;
        while(busy && retries--)
            ms_sleep(200);
        destroy_mutex(aq_mutex);
    }
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int que_article(char *remote, ARTICLE *artp)
{
    int result = -1;
    FILE *fp;

    /*
     * Check if we are initialized.
     */
    if(aq_mutex == -1) {
        syslog(LOG_ERR, "Article queue not initialized");
        return(-1);
    }
    if(request_mutex(&aq_mutex, MUTEXWAIT))
        return(-1);

    strcpy(subpath, remote);
    if((fp = xfopen(quepath, "ab")) != NULL) {
        artp->artsize = 0;
        spew_article(fp, artp, "rnews", 0, 0);
        if(fclose(fp))
            syslog(LOG_ERR, "%s", _strerror(quepath));
        else
            result = 0;
    }

    release_mutex(aq_mutex);
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int que_batch(char *remote, char *binbuf, size_t binsiz)
{
    static long batchcount = 0;
    int result = -1;
    FILE *fp;

    /*
     * Check if we are initialized.
     */
    if(aq_mutex == -1) {
        syslog(LOG_ERR, "Article queue not initialized");
        return(-1);
    }
    if(request_mutex(&aq_mutex, MUTEXWAIT))
        return(-1);

    sprintf(subpath, "%s.%lx%lx", remote, time(NULL), ++batchcount);
    if((fp = xfopen(quepath, "wb")) != NULL) {
        fwrite(binbuf, binsiz, 1, fp);
        if(fclose(fp))
            syslog(LOG_ERR, "%s", _strerror(quepath));
        else
            result = 0;
    }

    release_mutex(aq_mutex);
    return(result);
}

/*
 * $Log: artqueue.c,v $
 * Revision 1.1  1997/02/13 14:03:30  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:34  oliver
 * initial revision
 *
 */
