/*----------------------------------------------------------------------*/
/* Copyright (c) 1996-97 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/
/*
 * $Log$
 */
const char changi_rcsid[] = "$Id$";

#ifndef __OS2__
#include <windows.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <direct.h>
#include <process.h>
#include <time.h>
#include <signal.h>
#include <io.h>

#include <version.h>
#include <getopt.h>
#include <sostream.h>

#include <syslog.h>

#include <chlimits.h>
#include <sysdep.h>
#include <dircache.h>
#include <nactive.h>
#include <nhistory.h>
#include <overview.h>

#include "config.h"
#include "globals.h"
#include "newsfeeds.h"
#include "serve.h"
#include "client.h"
#include "artqueue.h"
#include "moderate.h"
#include "grpdist.h"
#include "oversync.h"
#include "changi.h"
#include "parsit.h"

int num_groups;

#ifndef __OS2__
static void service_mode(void);
#else
static void usage(void);
#endif

static void cleanup(int sig);
static void parsedbgopt(char *opt);
static void parselimopt(char *opt);
static void parsexclopt(char *opt);
static int optbew(int argc,char **argv);
static FILE *fppid = NULL;
static int syslog_opt = LOG_CONS | LOG_NDELAY | LOG_NOWAIT;

int dirty_cache = 0;

#ifdef __OS2__
int Changimain(int argc, char **argv);
#else
void _Optlink Changimain(void);
#endif

/************************************************************************/
/*                                                                      */
/*                       CHANGI NNTP SERVER                             */
/*                                                                      */
/************************************************************************/
int main(int argc, char **argv)
{

    /*
     * Initialize socket interface.
     */

    if(SoInitialize()) {
        fprintf(stderr, "FATAL ERROR: TCP/IP initialization failed\n");
        return(128);
    }

#ifndef __OS2__

    /*
     * We get one parameter, our full pathname. Use it
     * to change to the directory we are coming from.
     */

    if(argc == 1) {
        char *cp;
        char *mydir = strdup(argv[0]);
        while((cp = strchr(mydir, '/')) != NULL)
            *cp = '\\';
        if((cp = strrchr(mydir, '\\')) != NULL) {
            *cp = 0;
            chdir(mydir);
        }
        service_mode();
        abort();
    }



    if (optbew(argc,argv)==2){
       Changimain();
       return(0);
    }


#else

    return(Changimain(argc,argv));

#endif
    return(0);
}

/************************************************************************/
/*                                                                      */
/* Changimain                                                           */
/*                                                                      */
/* This function is called from main under OS/2 or started as a thread  */
/* from ServiceMain under Windows NT.                                   */
/*                                                                      */
/************************************************************************/
#ifdef __OS2__
int Changimain(int argc, char **argv)
#else
void _Optlink Changimain(void)
#endif

{
    int s;      /* socket for accepting connections */
    int result = 2;
    time_t t_proc;
    int pidrc = -1;

    tzset();
    t_proc = time(NULL);

    fprintf(stderr, "\nChangi NNTP Server Version %s\n", version);
    fprintf(stderr, "Copyright 1996-98 by egnite Software GmbH\n");
    fprintf(stderr, "Report Bugs to harald@egnite.com\n\n");

#ifdef __OS2__
    /*
     * We don't process the abort signal on NT, because
     * abort() seems to be the only way to exit the program
     * and force all threads to die.
     */
    signal(SIGABRT, cleanup);
#endif

    signal(SIGBREAK, cleanup);
    signal(SIGINT, cleanup);
    signal(SIGSEGV, cleanup);
    signal(SIGTERM, cleanup);

    /*
     * Set up initial configuration
     */
    init_cfg();

    /*
     * Create pid file and keep it open as a lock
     * against duplicate instances.
     */
    if(cfg.pidfile[0] != '\0') {
        pidrc = access(cfg.pidfile, 0);

        if((fppid = fopen(cfg.pidfile, "w")) == NULL) {
            perror(cfg.pidfile);
            return(8);
        }
        fprintf(fppid, "%u\n", _getpid());
        fflush(fppid);
        if(pidrc == 0)
            fprintf(stderr, "\aWARNING!\a Previous run didn't shut down properly\n\a         Run 'expire -rv'\n");
    }

#ifdef __OS2__

    if(optbew(argc,argv) == 1)
        return(1);
#endif

    openlog("nntpd", syslog_opt, LOG_NEWS);
    if(pidrc == 0)
        syslog(LOG_WARNING, "Previous run didn't shut down properly");
    if(validate_cfg())
        result = 3;

    else if((s = SoStreamSetServer(cfg.newsport, 119)) != -1) {
        init_artque(cfg.spooldir, cfg.rnewscall);
        read_mailpaths(cfg.mailpathsfile);
        /*
         * 09/01/1996 harald
         * Added group.distrib file
         */
        read_groupdistrib(cfg.groupdistribfile);

        if(init_dircache(cfg.newsdir, 32) == 0) {
            if(init_overview(cfg.overviewfmtfile, cfg.newsdir, cfg.overdir, cfg.headerdummy, 0) == 0) {
                if(init_active(cfg.newsdir) == 0) {
                    if(init_history(cfg.historyfile, 0) == 0) {
                        if(load_active(cfg.activefile) > 0) {
                            read_newsfeeds(cfg.newsfeedsfile);
                            start_commander();
                            if(init_clientab() == 0) {
                                syslog(LOG_INFO, "Version %s-%s up and running...", version, __DATE__);
                                if((syslog_opt & LOG_PERROR) == 0)
                                    fputs("Up and running...\n", stderr);
                                _beginthread(ClientThread, NULL, 49136, &s);
                                while(!downshut) {
                                    if(dirty_cache && --dirty_cache == 0) {
                                        syslog(LOG_INFO, "Flushing active...");
                                        save_active(cfg.activefile, 0);
                                        syslog(LOG_INFO, "Flushing overview...");
                                        flush_overview();
                                        flush_dircache();
                                    }
                                    ms_sleep(1000);
                                }
                                stop_commmander();
                                ms_sleep(2000);
                                if(dolog(LOG_HEAP))
                                    _dump_allocated(0);

                                result = 0;
                                exit_clientab();
                            }
                            else {
                                syslog(LOG_ERR, "Client table initialization failed");
                                fprintf(stderr, "Client table initialization failed\n");
                            }
                            flush_site(NULL);
                            exit_oversync();
                            syslog(LOG_INFO, "Flushing active...");
                            save_active(cfg.activefile, 0);
                        }
                        else {
                            syslog(LOG_ERR, "Loading %s failed", cfg.activefile);
                            fprintf(stderr, "Loading %s failed\n", cfg.activefile);
                        }
                        syslog(LOG_INFO, "Flushing history...");
                        exit_history();
                    }
                    else {
                        syslog(LOG_ERR, "Init history failed");
                        fprintf(stderr, "Init history failed\n");
                    }
                    exit_active();
                }
                else {
                    syslog(LOG_ERR, "Init active failed");
                    fprintf(stderr, "Init active failed\n");
                }
                syslog(LOG_INFO, "Flushing overview...");
                exit_overview();
            }
            else {
                syslog(LOG_ERR, "Init overview failed");
                fprintf(stderr, "Init overview failed\n");
            }
            exit_dircache();
        }
        else {
            syslog(LOG_ERR, "Init dircache failed");
            fprintf(stderr, "Init dircache failed\n");
        }
        exit_artque();
        /*
         * 03/27/98 harald
         * Changi hangs here with tcp stack 4.02r, most probably
         * because our service thread sits in an accept. So we
         * don't close the server socket, hopefully tcp will
         * take over the cleanup.
         */
        /* SoStreamClose(s); */
    }

    t_proc = time(NULL) - t_proc;
    if(result)
        syslog(LOG_ERR, "Changi stopped after %lu seconds with error %d", t_proc, result);
    else
        syslog(LOG_ERR, "Changi stopped after %lu seconds", t_proc);

    if(fppid) {
        fclose(fppid);
        unlink(cfg.pidfile);
    }
#ifdef __OS2__
    return(result);
#else
    abort();
#endif
}

/************************************************************************/
/*                                                                      */
/* Parse debug options                                                  */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsedbgopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
          case 'a':
              debug_active(1);
              break;
          case 'b':
              debug_clientab(1);
              break;
          /*
           * added 08/30/97 harald
           */
          case 'B':
              cfg.logflg |= LOG_BATCH;
              break;
          case 'c':
              cfg.logflg |= LOG_CMDFAIL;
              break;
          case 'C':
              cfg.logflg |= LOG_CONFIG;
              break;
          case 'd':
              debug_dircache(1);
              break;

          /*
           * 09/01/1996 harald
           * Added to log new distribution features.
           */
          case 'D':
              cfg.logflg |= LOG_DISTRIB;
              break;
          /*
           * -df changed with version 0.9p
           * previous was log file flushing
           * which isn't used with our new
           * syslog interface. Now used to
           * debug feeding threads.
           */
          case 'f':
              cfg.logflg |= LOG_FEED;
              break;
          case 'F':
              cfg.logflg |= LOG_FEEDINI;
              break;
          case 'h':
              debug_history(1);
              break;
          case 'j':
              cfg.logflg |= LOG_JUNK;
              break;
          case 'm':
              cfg.logflg |= LOG_HEAP;
              break;
          case 'n':
              cfg.logflg |= LOG_ACCOUNT;
              break;
          case 'o':
              syslog_opt |= LOG_PERROR;
              break;
          /*
           * Added -dP to log access permission checks
           * 07/06/96 harald
           */
          case 'P':
              cfg.logflg |= LOG_PERM;
              break;
          /*
           * Was partitioning logfiles, now used to log pid with syslog
           * 08/07/96
           */
          case 'p':
              syslog_opt |= LOG_PID;
              break;
          /*
           * Log received telegrams
           */
          case 'r':
              cfg.logflg |= LOG_RECV;
              SoStreamDebug(1);
              break;
          /*
           * Log sent telegrams
           */
          case 's':
              cfg.logflg |= LOG_SEND;
              SoStreamDebug(1);
              break;
          /*
           * Log TCP failures
           */
          case 't':
              cfg.logflg |= LOG_TCPFAIL;
              SoStreamDebug(1);
              break;
          /*
           * Log overview
           */
          case 'v':
              debug_overview(1);
              debug_oversync(1);
              break;
          default:
              fprintf(stderr, "Unknown option -d%c ignored", *opt);
              break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Parse limit options                                                  */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parselimopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
          case 'c':
              opt++;
              if(isdigit(*opt)) {
                  cfg.max_clients = atoi(opt);
                  while(isdigit(*opt))
                      opt++;
              }
              break;
          case 'f':
              opt++;
              if(isdigit(*opt)) {
                  cfg.min_free = atol(opt);
                  while(isdigit(*opt))
                      opt++;
              }
              break;
          case 'm':
              opt++;
              if(isdigit(*opt)) {
                  cfg.min_core = atol(opt);
                  while(isdigit(*opt))
                      opt++;
              }
              break;
          case 's':
              opt++;
              if(isdigit(*opt)) {
                  cfg.max_load = atol(opt);
                  while(isdigit(*opt))
                      opt++;
              }
              break;
          case 't':
              opt++;
              if(isdigit(*opt)) {
                  cfg.max_threads = atoi(opt);
                  while(isdigit(*opt))
                      opt++;
              }
              break;
          default:
              fprintf(stderr, "Unknown option -l%c ignored", *opt);
              opt++;
              break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Parse exclude options                                                */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsexclopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
          case 'l':
              cfg.emulate_links = 0;
              break;
          case 'q':
              cfg.queue_incoming = 0;
              break;
          case 'Q':
              cfg.auto_dequeue = 0;
              break;
          case 'r':
              cfg.local_replygroup = 0;
              break;
          default:
              fprintf(stderr, "Unknown option -x%c ignored", *opt);
              break;
          }
   opt++;
   }
}

#ifdef __OS2__
/************************************************************************/
/*                                                                      */
/*  Usage display                                                       */
/*                                                                      */
/************************************************************************/
static void usage(void)
{
    fputs("usage: changi [options]\n\n"
          "options:\n"
          "  -a<filename>      host permisson file        (nntp_access)\n"
          "  -A<filename>      user permisson file        (user_access)\n"
          "  -c<filename>      control permisson file     (ctrl_access)\n"
          "  -D<distribution>  distribution overwrite\n"
          "  -d<flags>         debugging flags\n"
          "  -F<number>        force followup\n"
          "  -l<limit>         system load limits\n"
          "  -n<directory>     news home directory        (news)\n"
          "  -O<organization>  organization overwrite\n"
          "  -p<portnumber>    tcp port                   (nntp)\n"
          "  -Q<quotelimit>    posting quotation limit\n"
          "  -S<filename>      signature file\n"
          "  -T<filename>      character translation file\n"
          "  -t<directory>     temporary directory\n"
          "  -u<user-id>       id of user running changi\n"
          "  -x<flags>         exclusion flags\n\n"
          "debugging flags:\n"
          "  a  active file processing     j  junked articles\n"
          "  c  client command failures    m  heap dump\n"
          "  C  configuration dump         n  accounting\n"
          "  d  directory cache            o  log to console\n"
          "  D  distribution header        P  access permissions\n"
          "  f  site feeding               p  log with pid\n"
          "  F  site feeding init          t  client communication\n"
          "  h  history file processing    v  overview processing\n\n"
          "system load limits:\n"
          "  c<number>  max. number of connected hosts\n"
          "  f<MBytes>  min. disk space\n"
          "  m<MBytes>  min. core space\n"
          "  s<number>  max. system load\n"
          "  t<number>  max. number of connections\n\n"
          "exclusion flags:\n"
          "  l  disable link emulation\n"
          "  Q  disable automatic queue processing\n"
          "  q  disable queueing while flushed\n"
          "  r  disable local reply group\n", stderr);
}

#endif

/************************************************************************/
/*                                                                      */
/* Signal handler                                                       */
/*                                                                      */
/************************************************************************/
static void cleanup(int sig)
{
    char *msg = "Unknown signal";
    int level = LOG_INFO;

    signal(sig, SIG_IGN);
    switch(sig) {
    case SIGSEGV:
        level = LOG_ERR;
        msg = "Internal error";
        break;
    case SIGTERM:
        msg = "Killed";
        break;
    case SIGABRT:
        level = LOG_ERR;
        msg = "Aborted";
        break;
    case SIGBREAK:
    case SIGINT:
        msg = "Interrupted";
        break;
    }
    syslog(level, msg);
    fputc('\n', stderr);
    fputs(msg, stderr);
    fputc('\n', stderr);

    /*
     * 07/07/97 harald
     * Do not do any file i/o in the signal routine anymore.
     */
    downshut = 1;
}

static int optbew(int argc,char **argv)
{
    int option,i=0;

    while((option = getopt(argc,argv,"?a:A:c:D:d:F:i:l:n:O:p:Q:r:S:T:t:u:W:x:K")) != EOF) {
       switch(option) {
       case 'a':
           strcpy(cfg.hostaccessfile, optarg);
           break;
       case 'A':
           strcpy(cfg.useraccessfile, optarg);
           break;
       case 'c':
           strcpy(cfg.ctrlaccessfile, optarg);
           break;
       case 'D':
           strcpy(cfg.distribution, optarg);
           break;
       case 'd':
           parsedbgopt(optarg);
           break;
       case 'F':
           cfg.force_followup = atoi(optarg);
           break;
       case 'i':
           break;
       case 'l':
           parselimopt(optarg);
           break;
       case 'n':
           strcpy(cfg.newsdir, optarg);
           break;
       case 'O':
           strcpy(cfg.organization, optarg);
           break;
       case 'p':
           strcpy(cfg.newsport, optarg);
           break;
       case 'Q':
           cfg.max_quotelines = atoi(optarg);
           break;
       case 'r':
           strcpy(cfg.rnewscall, optarg);
           break;
       case 'S':
           strcpy(cfg.signfile, optarg);
           break;
       case 'T':   /* character translation */
           strcpy(cfg.chartranfile, optarg);
           break;
       case 't':       /* Define temporary directory */
           strcpy(cfg.tempdir, optarg);
           break;
       case 'u':
           strcpy(cfg.logname, optarg);
           break;
       case 'W':
           cfg.sockwait = atol(optarg);
           break;
       case 'x':
           parsexclopt(optarg);
           break;
       case 'K':
           i=2;
           break;
       default:
#ifdef __OS2__
           usage();
#endif
           if(fppid) {
               fclose(fppid);
               unlink(cfg.pidfile);
           }

#ifdef __OS2__
           return(1);
#endif
       }
    }
    return(i);
}



#ifndef __OS2__

/************************************************************************
*
*                       C h a n g i   S E R V I C E
*
************************************************************************/

#define SERVICE_NAME    "Changi"
#define CLO_KEY "SOFTWARE\\HARALD_KIPP_INTERNET_SYSTEME\\CHANGI"
#define MAIN_KEY HKEY_LOCAL_MACHINE

static int s_running = 0;
static int s_paused = 0;

static SERVICE_STATUS_HANDLE svcst_hdl = 0;




static int regopt(void)
{
    HKEY hcloKey;
    DWORD dwType,cbData;
    char  *cpData,**argv;
    int argc;
    if (RegOpenKeyEx(MAIN_KEY,
                     CLO_KEY,
                     0,
                     KEY_ALL_ACCESS,
                     &hcloKey)== ERROR_SUCCESS){

        if (RegQueryValueEx( hcloKey,
                            "PARAMETER",
                             NULL,
                             &dwType,
                             NULL,
                             &cbData)==ERROR_SUCCESS){

            cpData = malloc (cbData + 1);
            if (RegQueryValueEx( hcloKey,
                                "PARAMETER",
                                NULL,
                                &dwType,
                                cpData,
                                &cbData) != ERROR_SUCCESS)
                fprintf(stderr,"Error open Value");


         }

    }
    RegCloseKey(hcloKey);
    argv = (char **) NULL;
    argc = parsit(cpData, &argv);
    optbew(argc,argv);
    return(0);
}


/***********************************************************************
*                                                                      *
* Reports service status to SCM.                                       *
*                                                                      *
************************************************************************/

int ReportSvcStatus(DWORD state, DWORD wxit, DWORD sxit, DWORD chkp, DWORD hint)
{
    SERVICE_STATUS ss;

    ss.dwServiceType = SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS;
    ss.dwCurrentState = state;
    if(state == SERVICE_START_PENDING)
        ss.dwControlsAccepted = 0;
    else
        ss.dwControlsAccepted = SERVICE_ACCEPT_STOP |
                                SERVICE_ACCEPT_SHUTDOWN;

    /*
     * Either set our error code, or use a Win32 error.
     */

    if((ss.dwServiceSpecificExitCode = sxit) != 0)
        ss.dwWin32ExitCode = ERROR_SERVICE_SPECIFIC_ERROR;
    else
        ss.dwWin32ExitCode = wxit;
    ss.dwCheckPoint = chkp;
    ss.dwWaitHint = hint;
    if(!SetServiceStatus(svcst_hdl, &ss)) {
        downshut=1;
        return(-1);
    }
    return(0);
}
/************************************************************************
*
* Handle control requests from Service Control Manager.
*
************************************************************************/

void _System ServiceCtrl(DWORD dwCtrlCode)
{
    switch(dwCtrlCode) {

    /*
     * Pause service.
     */
#if 0
    case SERVICE_CONTROL_PAUSE:
        if(s_running && !s_paused) {
            ReportSvcStatus(SERVICE_PAUSE_PENDING, 0, 0, 1, 5000);
            s_paused = 1;
        }

        break;
#endif

#if 0
    /*
     * Resume after pause.
     */
    case SERVICE_CONTROL_CONTINUE:
        if(s_running && s_paused) {
            ReportSvcStatus(SERVICE_CONTINUE_PENDING, 0, 0, 1, 5000);
            s_paused = 0;
        }
        break;
#endif

    /*
     * System shutdown is pending.
     */
    case SERVICE_CONTROL_SHUTDOWN:

        downshut=1;
        break;
    /*
     * Stop service.
     */
    case SERVICE_CONTROL_STOP:
        s_running = 0;
        ReportSvcStatus(SERVICE_STOP_PENDING, 0, 0, 1, 5000);
        downshut=1;
        closelog();
        break;

    /*
     * Requests current status.
     */

    case SERVICE_CONTROL_INTERROGATE:
        syslog(LOG_DEBUG, "ServiceCtrl: Interrogate");
        break;

    /*
     * Service specific messages (128..255).
     */
    default:
        syslog(LOG_DEBUG, "ServiceCtrl: Something else");
        return;
    }

    if(s_running) {
        if(s_paused) {
            syslog(LOG_DEBUG, "ServiceCtrl: Report paused");
            ReportSvcStatus(SERVICE_PAUSED, 0, 0, 0, 0);
        }
        else {
            syslog(LOG_DEBUG, "ServiceCtrl: Report running");
            ReportSvcStatus(SERVICE_RUNNING, 0, 0, 0, 0);
        }
    }

    return;
}


/************************************************************************
*
* Called by Windows NT Service Control Manager.
*
************************************************************************/
void _System ServiceMain(int argc, char **argv)
{
    /*
     * Register our service handler.
     */
    if((svcst_hdl = RegisterServiceCtrlHandler(SERVICE_NAME,
                    (LPHANDLER_FUNCTION) ServiceCtrl)) == 0) {
        Beep(1000, 1000);
        return;
    }

    /*
     * If we didn't receive any parameters on service
     * start, the we try to take them from the registry.
     */
    if(argc == 1)
        regopt();
    else
        optbew(argc, argv);

    if(ReportSvcStatus(SERVICE_START_PENDING, 0, 0, 1, 5000)) {
        Beep(1000, 1000);
        return;
    }


    /*
     * Start Changi service thread.
     */
    if(_beginthread(Changimain, NULL,48000,NULL) == -1) {
        ReportSvcStatus(SERVICE_STOPPED, 0, 17, 0, 0);
        return;
    }

    /*
     * Call control interface service.
     */
    if(ReportSvcStatus(SERVICE_RUNNING, 0, 0, 0, 0))
        return;

    s_running = 0;

    if(downshut == 1){
        ReportSvcStatus(SERVICE_STOPPED, 0, 0, 0, 0);
        return;
    }

    return;
}

/************************************************************************
*
* Initial entry.
*
************************************************************************/
void service_mode(void)
{
    SERVICE_TABLE_ENTRY dispatch[] = {
         { SERVICE_NAME, (LPSERVICE_MAIN_FUNCTION)ServiceMain },
         { NULL, NULL }
    };

    StartServiceCtrlDispatcher(dispatch);
}

#endif

