/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 *
 * This module contains routines to handle character translation.
 *
 */

const char chartran_rcsid[] = "$Id: chartran.c,v 1.1 1997/02/13 14:03:33 oliver Exp $";

#include <stdlib.h>
#include <ctype.h>

#include <syslog.h>

#include <loadline.h>

#include "chartran.h"

static char bse2rdr[256];
static char rdr2bse[256];

static unsigned int char_code(register unsigned char *cp);

/************************************************************************/
/*                                                                      */
/************************************************************************/
int read_chartrans(char *name)
{
    int result = -1;
    register unsigned char *cp;
    char **lines;
    char **lp;
    int i;
    unsigned int x, y;

    if(load_textfile(name, &lines) > 0) {
        result = 0;
        for(i = 256; i >= 0; i--)
            bse2rdr[i] = rdr2bse[i] = (char)i;

        for(lp = lines; (cp = *lp++) != NULL; ) {
            if((x = char_code(cp)) < '@') {
                syslog(LOG_WARNING, "Will not translate %u", x);
                continue;
            }
            while(*cp && *cp != ' ' && *cp != '\t')
                cp++;
            while(*cp == ' ' || *cp == '\t')
                cp++;
            if(*cp == '\0') {
                syslog(LOG_ERR, "Error in translation table, need two values");
                result = -1;
                break;
            }
            if((y = char_code(cp)) < '@') {
                syslog(LOG_WARNING, "Will not translate %u to %u", x, y);
                continue;
            }
            bse2rdr[x] = y;
            rdr2bse[y] = x;
        }
        free(*lines);
        free(lines);
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void tran2rdr(register char *s)
{
    while(*s)
        *s = bse2rdr[(unsigned char)*s++];
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void tran2bse(register char *s)
{
    while(*s)
        *s = rdr2bse[(unsigned char)*s++];
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static unsigned int char_code(register unsigned char *cp)
{
    unsigned int result = 0;

    if(isdigit(*cp)) {
        do {
            result += *cp - '0';
        } while(*++cp >= '0' && *cp <= '9');
    }
    else if(*cp == '\\') {
        switch(toupper(*cp)) {
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
            do {
                result <<= 3;
                result += *cp - '0';
            } while(*++cp >= '0' && *cp <= '7');
            break;
        case 'X':
            for(cp++; isxdigit(*cp); cp++) {
                result <<= 4;
                result += toupper(*cp) - ((*cp > '9') ? 'A' - 10 : '0');
            }
            break;
        }
    }
    else {
        result = *cp;
    }
    return(result & 255);
}

/*
 * $Log: chartran.c,v $
 * Revision 1.1  1997/02/13 14:03:33  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:37  oliver
 * initial revision
 *
 */
