#ifndef CLIENT_H
#define CLIENT_H

/*
 * $Id: client.h,v 1.1 1997/02/13 14:03:34 oliver Exp $
 */

#include "access.h"
#include "article.h"

/*----------------------------------------------------------------------*/
/*  Structure with info about connect clients.                          */
/*                                                                      */
/*----------------------------------------------------------------------*/
typedef struct _NEWSCLIENT {
                            /* Key                                      */
                            /* ---------------------------------------- */
    unsigned int tid;       /* id of this thread                        */
    long remoteaddr;        /* ip address of remote host                */

                            /* Socket connection                        */
                            /* ---------------------------------------- */
    int  s;                 /* socket of client connection              */
    void *rb;               /* socket receive buffer                    */

                            /* Client identity and permissions          */
                            /* ---------------------------------------- */
    char *remotehost;       /* name of connected host                   */
    char username[9];       /* username set by authentication           */
    ACCESSPERMS perms;      /* permissions of the client's host         */
    int  kickout;           /* client will be kicked                    */

                            /* Current group info                       */
                            /* ---------------------------------------- */
    int    ingroup;         /* set if client has selected a newsgroup   */
    char   group_name[256]; /* name of selected newsgroup               */
    char   mydir[256];      /* directory of selected newsgroup          */
    char   myovr[256];      /* overview file name of selected newsgroup */
    long   art_num;         /* article number currently selected        */
    int    num_arts;        /* number of articles in current group      */

                            /* Current article                          */
                            /* ---------------------------------------- */
    char **nglist;          /* temporary list of newsgroups */
    ARTICLE art;

                            /* Statistics                               */
                            /* ---------------------------------------- */
    int  art_count;         /* number of articles read                  */
    int  grp_count;         /* number of groups selected                */
    int  post_count;        /* number of successful posts               */
    int  rejct_count;       /* number of rejected posts                 */
    unsigned long usr_clock; /* active clock cycles                     */
    int  received;          /* number of valid articles received        */
    int  refused;           /* number of duplicate ids rejected         */
    int  rejected;          /* number of bad articles rejected          */
} NEWSCLIENT, *PNEWSCLIENT;

/*----------------------------------------------------------------------*/
/*  Functions for client table administration                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

extern void debug_clientab(int enable);
extern void clientab_maxwait(long maxwait);

extern int init_clientab(void);
extern int exit_clientab(void);
extern NEWSCLIENT *create_client(int sock, char *remotehost, long remoteaddr);
extern int destroy_client(NEWSCLIENT *pnc);
extern NEWSCLIENT *find_client(int tid);
extern int get_hostentry(int index, char *info);
extern int get_userentry(int index, char *info);
extern void add_userentry(long remoteaddr, char *remotename, char *username);
extern int kick_entries(char *name);

/*----------------------------------------------------------------------*/
/*  Functions performing main nntp commands                             */
/*                                                                      */
/*----------------------------------------------------------------------*/
extern void ahbs(PNEWSCLIENT, int, char **);
extern void dodate(PNEWSCLIENT, int, char **);
extern void group(PNEWSCLIENT, int, char **);
extern void help(PNEWSCLIENT, int, char **);
extern void ihave(PNEWSCLIENT, int, char **);
extern void uwant(PNEWSCLIENT, int, char **);
extern void ignore(PNEWSCLIENT, int, char **);
extern void list(PNEWSCLIENT, int, char **);
extern void newgroups(PNEWSCLIENT, int, char **);
extern void newnews(PNEWSCLIENT, int, char **);
extern void nextlast(PNEWSCLIENT, int, char **);
extern void post(PNEWSCLIENT, int, char **);
extern void slave(PNEWSCLIENT, int, char **);
extern void listgroup(PNEWSCLIENT, int, char **);
extern void xhdr(PNEWSCLIENT, int, char **);
extern void xpat(PNEWSCLIENT, int, char **);
extern void xpath(PNEWSCLIENT, int, char **);
extern void doxover(PNEWSCLIENT, int, char **);
extern void xrover(PNEWSCLIENT, int, char **);
extern void xgtitle(PNEWSCLIENT, int, char **);
extern void domode(PNEWSCLIENT, int, char **);
extern void doxauthinfo(PNEWSCLIENT, int, char **);
extern void xbatch(PNEWSCLIENT, int, char **);

/*
 * $Log: client.h,v $
 * Revision 1.1  1997/02/13 14:03:34  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:38  oliver
 * initial revision
 *
 */

#endif
