/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/


/*
 * $Log$
 */

const char config_rcsid[] = "$Id$";

#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <string.h>
#include <direct.h>
#include <time.h>

#include <version.h>

#include <syslog.h>
#include <readconf.h>
#include <sostream.h>

#include <overview.h>

#include "config.h"
#include "chartran.h"

extern void active_maxwait(long maxwait);
extern void artqueue_maxwait(long maxwait);
extern void clientab_maxwait(long maxwait);
extern void dircache_maxwait(long maxwait);
extern void distrib_maxwait(long maxwait);
extern void history_maxwait(long maxwait);
extern void newsfeeds_maxwait(long maxwait);
extern void overview_maxwait(long maxwait);


CONFIG cfg;

static ENVTABLE et[] = {
    { cfg.workdir          , "CHANGIWORKDIR"          , "", 0 },

    { cfg.signfile         , "CHANGISIGNFILE"         , "", 0 },
    { cfg.chartranfile     , "CHANGICHARTRANFILE"     , "", 0 },

    { cfg.pidfile          , "CHANGIPIDFILE"          , "changi.pid", 1 },
    { cfg.newsdir          , "CHANGINEWSDIR"          , "news", 1 },
    { cfg.overdir          , "CHANGIOVERDIR"          , "news", 1 },
    { cfg.activefile       , "CHANGIACTIVEFILE"       , "active", 1 },
    { cfg.active_times     , "CHANGIACTIVETIMES"      , "active.times", 1 },
    { cfg.distributionsfile, "CHANGIDISTRIBUTIONSFILE", "distributions", 1 },
    { cfg.newsgroupsfile   , "CHANGINEWSGROUPSFILE"   , "newsgroups", 1 },
    { cfg.subscriptionsfile, "CHANGISUBSCRIPTIONSFILE", "subscriptions", 1 },
    { cfg.newsfeedsfile    , "CHANGINEWSFEEDSFILE"    , "newsfeeds", 1 },
    { cfg.historyfile      , "CHANGIHISTORY"          , "history", 1 },
    { cfg.seqf             , "CHANGISEQF"             , "SEQF", 1 },
    { cfg.spooldir         , "CHANGISPOOLDIR"         , "spool", 1 },
    { cfg.tempdir          , "CHANGITEMPDIR"          , "TMP", 4 },

    { cfg.smailcall        , "CHANGISMAILCALL"        , "sendmail -t", 2 },
    { cfg.grepcall         , "CHANGIGREPCALL"         , "grep -sli", 2 },
    { cfg.rnewscall        , "CHANGIRNEWSCALL"        , "rnews", 2 },
    { cfg.compresscall     , "CHANGICOMPRESSCALL"     , "gzip -c", 2 },
    { cfg.compressintro    , "CHANGICOMPRESSINTRO"    , "", 0 },
    { cfg.uncompresscall   , "CHANGIUNCOMPRESSCALL"   , "gzip -dc", 2 },
    { cfg.gunzipcall       , "CHANGIUNZIPCALL"        , "gzip -dc", 2 },

    { cfg.junkgroup        , "CHANGIJUNKGROUP"        , "junk", 2 },
    { cfg.dupegroup        , "CHANGIDUPEGROUP"        , "duplicates", 2 },
    { cfg.controlgroup     , "CHANGICONTROLGROUP"     , "control", 2 },

    { cfg.hostaccessfile   , "CHANGIHOSTACCESSFILE"   , "nntp_access", 3 },
    { cfg.useraccessfile   , "CHANGIUSERACCESSFILE"   , "user_access", 3 },
    { cfg.ctrlaccessfile   , "CHANGICTRLACCESSFILE"   , "ctrl_access", 3 },
    { cfg.teamaccessfile   , "CHANGITEAMACCESSFILE"   , "team_access", 3 },
    { cfg.groupdistribfile , "CHANGIGROUPDISTRIBFILE" , "group.distrib", 3 },
    { cfg.mailpathsfile    , "CHANGIMAILPATHSFILE"    , "mailpaths", 3 },
    { cfg.overviewfmtfile  , "CHANGIOVERVIEWFMTFILE"  , "overview.fmt", 3 },
    { cfg.controlfile      , "CHANGICONTROLFILE"      , "control.ctl", 3 },

    { cfg.mynode           , "CHANGINODENAME"         , "HOSTNAME", 4 },
    { cfg.mydomain         , "CHANGIHOSTNAME"         , "HOSTNAME", 4 },
    { cfg.pathhost         , "CHANGIPATHHOST"         , "HOSTNAME", 4 },
    { cfg.newsmaster       , "CHANGINEWSMASTER"       , "", 0 },
    { cfg.pathmaster       , "CHANGIPATHMASTER"       , "not-for-mail", 2 },
    { cfg.newsserver       , "NNTPNEIGHBOR"           , "NNTPSERVER", 4 },

    { cfg.organization     , "CHANGIORGANIZATION"     , "", 0 },
    { cfg.mimeversion      , "CHANGIMIMEVERSION"      , "", 0 },
    { cfg.mimetype         , "CHANGIMIMETYPE"         , "", 0 },
    { cfg.mimeencoding     , "CHANGIMIMEENCODING"     , "", 0 },
    { cfg.distribution     , "CHANGIDISTRIBUTION"     , "", 0 },

    { cfg.logname          , "CHANGILOGNAME"          , "USER", 4 },
    { cfg.newsadmin        , "CHANGINEWSADMIN"        , "NEWSADMIN", 4 },
    { cfg.newsport         , "CHANGINEWSPORT"         , "nntp", 2 },
    { cfg.controlport      , "CHANGICONTROLPORT"      , "119", 2 },
    { cfg.headerdummy      , "CHANGIHEADERDUMMY"      , "", 0 },

    { cfg.autoflush        , "CHANGIAUTOFLUSH"        , "30", 2 },
    { cfg.controlctldelimit, "CHANGICONTROLCTLDELIMIT", "44", 2 },

    { cfg.welcome          , "CHANGIWELCOME"          , "Changi NNTP server ready", 2 },

    { NULL                 , NULL                     , "", 0 }
};

/************************************************************************/
/*                                                                      */
/* Set configuration defaults                                           */
/*                                                                      */
/************************************************************************/
void init_cfg(void)
{
    char *cp;

    cfg.sockwait = 3600;
    read_conf(et);

    cfg.emulate_links = 1;
    cfg.local_replygroup = 1;
    cfg.queue_incoming = 1;
    cfg.auto_dequeue = 1;

    if((cp = getenv("CHANGIMUXWAIT")) != NULL) {
        long maxwait = atol(cp);
        active_maxwait(maxwait);
        artqueue_maxwait(maxwait);
        clientab_maxwait(maxwait);
        dircache_maxwait(maxwait);
        distrib_maxwait(maxwait);
        history_maxwait(maxwait);
        newsfeeds_maxwait(maxwait);
        overview_maxwait(maxwait);
    }

    if((cp = getenv("CHANGIARTQUEUEMUXWAIT")) != NULL)
        artqueue_maxwait(atol(cp));
    if((cp = getenv("CHANGIACTIVEMUXWAIT")) != NULL)
        active_maxwait(atol(cp));
    if((cp = getenv("CHANGICLIENTABMUXWAIT")) != NULL)
        clientab_maxwait(atol(cp));
    if((cp = getenv("CHANGIDIRCACHEMUXWAIT")) != NULL)
        dircache_maxwait(atol(cp));
    if((cp = getenv("CHANGIDISTRIBMUXWAIT")) != NULL)
        distrib_maxwait(atol(cp));
    if((cp = getenv("CHANGIHISTORYMUXWAIT")) != NULL)
        history_maxwait(atol(cp));
    if((cp = getenv("CHANGINEWSFEEDSMUXWAIT")) != NULL)
        newsfeeds_maxwait(atol(cp));
    if((cp = getenv("CHANGIOVERVIEWMUXWAIT")) != NULL)
        overview_maxwait(atol(cp));
}

/************************************************************************/
/*                                                                      */
/*  Validates configuration                                             */
/*                                                                      */
/************************************************************************/
int validate_cfg(void)
{
    int result = 0;
    char *cp;
    long myaddr;

    if(cfg.mydomain[0] == '\0' || strchr(cfg.mydomain, '.') == NULL)
        SoGetLocalHost(cfg.mydomain, &myaddr);
    if(cfg.mydomain[0] == '\0' || strchr(cfg.mydomain, '.') == NULL) {
        syslog(LOG_ERR, "Fatal error, bad [CHANGI]HOSTNAME");
        fprintf(stderr, "Set [CHANGI]HOSTNAME to FQDN\n");
        result = -1;
    }
    else {
        if(cfg.mynode[0] == '\0')
            strcpy(cfg.mynode, cfg.mydomain);
        if(cfg.pathhost[0] == '\0')
            strcpy(cfg.pathhost, cfg.mydomain);
        if(cfg.newsmaster[0] == '\0') {
            strcpy(cfg.newsmaster, "usenet@");
            strcat(cfg.newsmaster, cfg.mydomain);
        }
    }
    if((cp = strchr(cfg.mynode, '.')) != NULL)
        *cp = '\0';

    /*
     * 07/20/97 harald
     * Set newsadmin, if empty.
     */
    if(cfg.newsadmin[0] == 0) {
        strcpy(cfg.newsadmin, "newsadmin@");
        strcat(cfg.newsadmin, cfg.mydomain);
    }

    /*
     * 07/06/97 harald
     * Use CHANGITEMPDIR. Now really use it!
     */
    if(cfg.tempdir[0] == '\0')
        strcpy(cfg.tempdir, cfg.workdir);
    else
        putenv("TMP=");

    if(dolog(LOG_PERM)) {
        syslog(LOG_DEBUG, "%sostaccess permission file %s",
               cfg.hostaccessfile[0] ? "H" : "No h",
               cfg.hostaccessfile[0] ? cfg.hostaccessfile : "");
        syslog(LOG_DEBUG, "%sseraccess permission file %s",
               cfg.useraccessfile[0] ? "U" : "No u",
               cfg.useraccessfile[0] ? cfg.useraccessfile : "");
        syslog(LOG_DEBUG, "%sontrolaccess permission file %s",
               cfg.ctrlaccessfile[0] ? "C" : "No c",
               cfg.ctrlaccessfile[0] ? cfg.ctrlaccessfile : "");
        syslog(LOG_DEBUG, "%seamaccess permission file %s",
               cfg.teamaccessfile[0] ? "T" : "No t",
               cfg.teamaccessfile[0] ? cfg.teamaccessfile : "");
    }

    if(cfg.chartranfile[0] != '\0' && read_chartrans(cfg.chartranfile) != -1)
        cfg.char_translate = 1;

    if(dolog(LOG_CONFIG))
        dump_cfg();

    return(result);
}

/************************************************************************/
/*                                                                      */
/* Dump configuration                                                   */
/*                                                                      */
/************************************************************************/
int dump_cfg(void)
{
    int result = -1;
    FILE *fp;
    ENVTABLE *ep;
    time_t t;

    if((fp = fopen("changi.log", "a")) != NULL) {
        time(&t);
        fprintf(fp, "REM %s", ctime(&t));
        fprintf(fp, "REM Changi NNTP Server Version %s\n", version);
        fprintf(fp, "REM Compiled %s\n", __DATE__);
        for(ep = et; ep->cnfg; ep++)
            fprintf(fp, "SET %s=%s&\n", ep->ename, ep->cnfg);
        fputc('\n', fp);
        if(fclose(fp))
            syslog(LOG_ERR, "%s", _strerror("Config dump"));
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "%s", _strerror("Config dump"));

    return(result);
}
