const char control_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* <sys\types.h> */
#include <sys\stat.h>

#include <syslog.h>
#include <pipe.h>
#include <xfile.h>

#include <chlimits.h>
#include <split.h>
#include <nhistory.h>
#include <nactive.h>
#include <overview.h>
#include <loadtext.h>
#include <match.h>

#include "globals.h"
#include "config.h"
#include "misc.h"
#include "article.h"
#include "pgpvrify.h"
#include "control.h"

static int store_rmgroup(char *cmdfile, char *cmdinfo, char *user);
static int store_newgroup(char *cmdfile, char *cmdinfo, char *user);

typedef struct {
    char *rquest;
    char *sender;
    char *grppat;
    char **grplst;
    char *action;
    char *actlog;
} CTLCFG;

int n_cc = 0;
int x_cc = 0;
CTLCFG *ctlcfg;

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static int load_control(char *ctl_file)
{
    struct stat st;
    int rc;
    int i;
    char **fields;
    int n_lines;
    char **ctl_lines = NULL;
    char **lpp;
    char *lp;
    char *cp;
    static time_t ctl_mtime = 0;
    char grpdlm = (char)atoi(cfg.controlctldelimit);

    /*
     * If control file exists and the modification
     * time didn't change, no update is needed.
     */
    if((rc = stat(ctl_file, &st)) == 0 && st.st_mtime == ctl_mtime)
        return(0);
    ctl_mtime = st.st_mtime;

    /*
     * Free previously allocated space.
     */
    for(i = 0; i < n_cc; i++) {
        free(ctlcfg[i].rquest);
        free(ctlcfg[i].sender);
        free(ctlcfg[i].grppat);
        free(ctlcfg[i].grplst);
        free(ctlcfg[i].action);
        if(ctlcfg[i].actlog)
            free(ctlcfg[i].actlog);
    }
    n_cc = 0;

    /*
     * Load the control file and process each line,
     * if it exists.
     */
    if(rc == 0 && (n_lines = load_textfile(ctl_file, &ctl_lines)) > 0) {
        for(lpp = ctl_lines; (lp = *lpp++) != NULL; ) {
            fields = split_list(lp, ':');
            for(i = 0; fields[i]; i++) {
                if(stricmp(fields[i], "all") == 0)
                    strcpy(fields[i], "*");
            }
            if(i != 4)
                syslog(LOG_ERR, "Bad entry in %s", ctl_file);
            else {
                if(n_cc == x_cc) {
                    x_cc += 32;
                    ctlcfg = realloc(ctlcfg, x_cc * sizeof(CTLCFG));
                }
                ctlcfg[n_cc].rquest = strdup(fields[0]);
                ctlcfg[n_cc].sender = strdup(fields[1]);
                ctlcfg[n_cc].grppat = strdup(fields[2]);
                /*
                 * 08/03/97 harald
                 * Configurable delimiter. We use comma by default, but
                 * INN uses a pipe symbol.
                 */
                ctlcfg[n_cc].grplst = split_list(ctlcfg[n_cc].grppat, grpdlm);
                if((cp = strchr(fields[3], '=')) == NULL) {
                    ctlcfg[n_cc].action = strdup(fields[3]);
                    ctlcfg[n_cc++].actlog = NULL;
                }
                else {
                    *cp++ = 0;
                    ctlcfg[n_cc].action = strdup(fields[3]);
                    ctlcfg[n_cc++].actlog = strdup(cp);
                }
            }
            free(fields);
        }
        free(*ctl_lines);
        free(ctl_lines);
    }
    return(0);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static CTLCFG *find_ctlcfg(char *group, char *sender, char *rquest)
{
    int i, j;

    if(cfg.controlfile[0] == 0)
        return(NULL);
    if(load_control(cfg.controlfile))
        return(NULL);
    for(i = n_cc; --i >= 0; ) {
        if(wildmat(rquest, ctlcfg[i].rquest) &&
           wildmat(sender, ctlcfg[i].sender)) {
            for(j = 0; ctlcfg[i].grplst[j]; j++) {
                if(wildmat(group, ctlcfg[i].grplst[j]))
                    return(&ctlcfg[i]);
            }
        }
    }
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static void mail_newgroup(ARTICLE *artp, char *group, char mode)
{
    FILE *fpp;
    int  cnt;
    int  rc;
    char *cmd = malloc(_MAX_PATH);
#ifndef __OS2__
    char *tname;
#endif

    if((cnt = strings_in_format(cfg.smailcall)) == 1)
        sprintf(cmd, cfg.smailcall, cfg.newsadmin);
    else
        strcpy(cmd, cfg.smailcall);

#ifdef __OS2__
    if((fpp = popen(cmd, "wb")) != NULL) {
#else
    if((tname = tempnam(cfg.tempdir, "CTL")) == NULL) {
        syslog(LOG_ERR, "Can't create tempnam for %s", cfg.smailcall);
        free(cmd);
        return;
    }
    if((fpp = xfopen(tname, "r")) != NULL) {
        strcat(cmd, " < ");
        strcat(cmd, tname);
#endif
        if(cnt == 0)
            fprintf(fpp, "To: %s\n", cfg.newsadmin);
        fprintf(fpp, "From: \"Changi Newsserver\" <Changi@%s>\n", cfg.mydomain);
        fprintf(fpp, "Subject: %s\n\n", artp->header[ID_Subject].info);

        fprintf(fpp, "%s requested that %s be %s.\n", artp->header[ID_From].info,
                                                      group, "created");
        fprintf(fpp, "To do this, type the following:\n\n");
        fprintf(fpp, "  %s\\chanco newgroup %s %c %s\n\n", cfg.workdir,
                                                           group,
                                                           mode,
                                                           artp->header[ID_From].info);
        fprintf(fpp, "The original article was:\n\n");
        spew_article(fpp, artp, NULL, 0, 0);

#ifdef __OS2__
        if((rc = pclose(fpp)) != 0)
            syslog(LOG_ERR, "%s returned %d", cmd, rc >> 8);
#else
        if(fclose(fpp))
            syslog(LOG_ERR, "Close %s", _strerror(tname));
        else if((rc = system(cmd)) != 0)
            syslog(LOG_ERR, "'%s' returned error %d", cmd, rc);
        if(xremove(tname))
            syslog(LOG_WARNING, "Delete %s", _strerror(tname));
#endif
    }
    else
#ifdef __OS2__
        syslog(LOG_ERR, "Failed to run %s", cmd);
#else
        syslog(LOG_ERR, "%s", _strerror(tname));
    free(tname);
#endif

    free(cmd);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static void mail_rmgroup(ARTICLE *artp, char *group)
{
    FILE *fpp;
    int  cnt;
    int  rc;
    char *cmd = malloc(_MAX_PATH);
#ifndef __OS2__
    char *tname;
#endif

    if((cnt = strings_in_format(cfg.smailcall)) == 1)
        sprintf(cmd, cfg.smailcall, cfg.newsadmin);
    else
        strcpy(cmd, cfg.smailcall);

#ifdef __OS2__
    if((fpp = popen(cmd, "wb")) != NULL) {
#else
    if((tname = tempnam(cfg.tempdir, "CTL")) == NULL) {
        syslog(LOG_ERR, "Can't create tempnam for %s", cfg.smailcall);
        free(cmd);
        return;
    }
    if((fpp = xfopen(tname, "r")) != NULL) {
        strcat(cmd, " < ");
        strcat(cmd, tname);
#endif
        if(cnt == 0)
            fprintf(fpp, "To: %s\n", cfg.newsadmin);
        fprintf(fpp, "From: \"Changi Newsserver\" <Changi@%s>\n", cfg.mydomain);
        fprintf(fpp, "Subject: %s\n\n", artp->header[ID_Subject].info);

        fprintf(fpp, "%s requested that %s be %s.\n", artp->header[ID_From].info,
                                                      group, "removed");
        fprintf(fpp, "To do this, type the following:\n\n");
        fprintf(fpp, "  %s\\chanco rmgroup %s %s\n\n", cfg.workdir,
                                                          group,
                                                          artp->header[ID_From].info);
        fprintf(fpp, "The original article was:\n\n");
        spew_article(fpp, artp, NULL, 0, 0);

#ifdef __OS2__
        if((rc = pclose(fpp)) != 0)
            syslog(LOG_ERR, "%s returned %d", cmd, rc >> 8);
#else
        if(fclose(fpp))
            syslog(LOG_ERR, "Close %s", _strerror(tname));
        else if((rc = system(cmd)) != 0)
            syslog(LOG_ERR, "'%s' returned error %d", cmd, rc);
        if(xremove(tname))
            syslog(LOG_WARNING, "Delete %s", _strerror(tname));
#endif
    }
    else
#ifdef __OS2__
        syslog(LOG_ERR, "Failed to run %s", cmd);
#else
        syslog(LOG_ERR, "%s", _strerror(tname));
    free(tname);
#endif
    free(cmd);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static void add_newsgroups(ARTICLE *artp, char *group)
{
    int i;
    char *keyline = "For your newsgroups file:";

    for(i = 0; (i + 1) < artp->bodybuf->nlines; i++) {
        if(strnicmp(keyline, LBPTR(artp->bodybuf, i), strlen(keyline)) == 0) {
            FILE *fps, *fpd;
            char *tmp_file = malloc(_MAX_PATH);
            char *line = malloc(MAX_BUFSIZ);

            strcpy(tmp_file, cfg.newsgroupsfile);
            strcat(tmp_file, ".tmp");
            if((fpd = xfopen(tmp_file, "w")) != NULL) {
                if((fps = xfopen(cfg.newsgroupsfile, "r")) != NULL) {
                    while(fgets(line, MAX_BUFSIZ, fps)) {
                        if(strnicmp(line, group, strlen(group)))
                            fputs(line, fpd);
                    }
                    fclose(fps);
                }
                else
                    syslog(LOG_ERR, "%s", _strerror(cfg.newsgroupsfile));
                fputs(LBPTR(artp->bodybuf, i + 1), fpd);
                fputc('\n', fpd);
                fclose(fpd);
                xrename(tmp_file, cfg.newsgroupsfile);
            }
            else
                syslog(LOG_ERR, "%s", _strerror(tmp_file));
            free(tmp_file);
            free(line);
            break;
        }
    }
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static void perform_newgroup(ARTICLE *artp)
{
    char *from;
    char *ctrl;
    char *addr;
    char *group;
    char mode = 'y';
    char *cp;
    CTLCFG *ccp;

    /*
     * Split control info.
     */
    cp = ctrl = strdup(artp->header[ID_Control].info);
    while(*cp && *cp != ' ' && *cp != '\t')         /* skip 'newgroup' */
        cp++;
    while(*cp == ' ' || *cp == '\t')                /* skip spaces */
        cp++;
    group = cp;
    while(*cp && *cp != ' ' && *cp != '\t')         /* skip group */
        cp++;
    if(*cp == ' ' || *cp == '\t')
        *cp++ = 0;
    while(*cp == ' ' || *cp == '\t')                /* skip spaces */
        cp++;
    if(*cp)
        mode = *cp;

    /*
     * Split 'from' sender.
     */
    from = strdup(artp->header[ID_From].info);
    get_originator(from, &addr, NULL);
    if((ccp = find_ctlcfg(group, addr, "newgroup")) == NULL) {
        store_newgroup("newgroup.cmd", artp->header[ID_Control].info, addr);
        free(ctrl);
        free(from);
        return;
    }

    if(stricmp(ccp->action, "doit") == 0) {
        add_active(group, 1, 0, mode, addr, cfg.active_times);
        add_newsgroups(artp, group);
    }
    else if(stricmp(ccp->action, "drop") == 0) {
        syslog(LOG_INFO, "%s from %s dropped", artp->header[ID_Control].info, addr);
    }
    else if(stricmp(ccp->action, "log") == 0) {
        store_newgroup("newgroup.cmd", artp->header[ID_Control].info, addr);
    }
    else if(stricmp(ccp->action, "mail") == 0) {
        mail_newgroup(artp, group, mode);
    }
    else if(strnicmp(ccp->action, "verify-", 7) == 0) {
        syslog(LOG_INFO, "verify newgroup");
        if(pgp_verify(artp, ccp->action + 7))
            syslog(LOG_NOTICE, "PGP Verification failed");
        else {
            add_active(group, 1, 0, mode, addr, cfg.active_times);
            add_newsgroups(artp, group);
        }
    }
    free(from);
    free(ctrl);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static void perform_rmgroup(ARTICLE *artp)
{
    char *from;
    char *ctrl;
    char *addr;
    char *group;
    char *cp;
    CTLCFG *ccp;

    /*
     * Split control info.
     */
    cp = ctrl = strdup(artp->header[ID_Control].info);
    while(*cp && *cp != ' ' && *cp != '\t')         /* skip 'newgroup' */
        cp++;
    while(*cp == ' ' || *cp == '\t')                /* skip spaces */
        cp++;
    group = cp;
    while(*cp && *cp != ' ' && *cp != '\t')         /* skip group */
        cp++;
    *cp = 0;

    /*
     * Split 'from' sender.
     */
    from = strdup(artp->header[ID_From].info);
    get_originator(from, &addr, NULL);
    if((ccp = find_ctlcfg(group, addr, "newgroup")) == NULL) {
        store_rmgroup("rmgroup.cmd", artp->header[ID_Control].info, addr);
        free(ctrl);
        free(from);
        return;
    }

    if(stricmp(ccp->action, "doit") == 0) {
        del_active(group, addr);
    }
    else if(stricmp(ccp->action, "drop") == 0) {
        syslog(LOG_INFO, "%s from %s dropped", artp->header[ID_Control].info, addr);
    }
    else if(stricmp(ccp->action, "log") == 0) {
        store_rmgroup("rmgroup.cmd", artp->header[ID_Control].info, addr);
    }
    else if(stricmp(ccp->action, "mail") == 0) {
        mail_rmgroup(artp, group);
    }
    else if(strnicmp(ccp->action, "verify-", 7) == 0) {
        if(pgp_verify(artp, ccp->action + 7))
            syslog(LOG_NOTICE, "PGP Verification failed");
        else
            del_active(group, addr);
    }
    free(from);
    free(ctrl);
}

/************************************************************************/
/*                                                                      */
/* process_control                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
int process_control(ARTICLE *artp)
{
    int result = -1;
    char *cp;

    switch(artp->x_control[8]) {
    case '\0':
        if(artp->header[ID_Supersedes].info)
            cancel_art(artp->header[ID_Supersedes].info);
        return(0);
    case 'c':
        if(strcmp(artp->x_control + 8, "cancel") == 0) {
            cp = artp->header[ID_Control].info + 7;
            while(*cp == ' ' || *cp == '\t')
                cp++;
            result = cancel_art(cp);
        }
        else {  /* checkgroups */
            syslog(LOG_DEBUG, "Got checkgroups control");
        }
        break;
    case 'i':   /* ihave */
        syslog(LOG_DEBUG, "Got ihave control");
        break;
    case 'n':   /* newgroup */
        perform_newgroup(artp);
        break;
    case 'r':   /* rmgroup */
        perform_rmgroup(artp);
        break;
    case 's':
        if(strcmp(artp->x_control + 8, "sendme") == 0) {
            syslog(LOG_DEBUG, "Got sendme control");
        }
        else if(strcmp(artp->x_control + 8, "sendsys") == 0) {
            syslog(LOG_DEBUG, "Got sendsys control");
        }
        else {  /* senduuname */
            syslog(LOG_DEBUG, "Got senduuname control");
        }
        break;
    case 'v':   /* version */
        syslog(LOG_DEBUG, "Got version control");
        break;
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
int cancel_art(char *msg_id)
{
    int result = -1;
    char *artlist = malloc(MAX_HISTORYLINE);

    if(cancelhistart(msg_id, artlist) == 0) {
        char *newspath = strcpy(malloc(_MAX_PATH), cfg.newsdir);
        char *subpath = newspath + strlen(newspath);
        char **arts = split_list(artlist, ',');
        int i;

        *subpath++ = '\\';
        for(i = 0; arts[i]; i++) {
            strcpy(subpath, arts[i]);
            xremove(newspath);
        }
        free(arts);
        free(newspath);
        result = 0;
    }

    free(artlist);
    return(result);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static int store_newgroup(char *cmdfile, char *cmdinfo, char *user)
{
    FILE *fp;
    char *cp = cmdinfo;

    while(*cp && *cp != ' ' && *cp != '\t')
        cp++;
    while(*cp == ' ' || *cp == '\t')
        cp++;
    while(*cp && *cp != ' ' && *cp != '\t')
        cp++;
    while(*cp == ' ' || *cp == '\t')
        cp++;

    if((fp = xfopen(cmdfile, "a")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(cmdfile));
        return(-1);
    }
    fprintf(fp, "chanco -u%s %s%s %s\n", user, cmdinfo,
            (*cp == '\0') ? " y" : "", user);
    if(fclose(fp)) {
        syslog(LOG_ERR, "%s", _strerror(cmdfile));
        return(-1);
    }
    return(0);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static int store_rmgroup(char *cmdfile, char *cmdinfo, char *user)
{
    FILE *fp;

    if((fp = xfopen(cmdfile, "a")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(cmdfile));
        return(-1);
    }
    fprintf(fp, "chanco -u%s %s\n", user, cmdinfo);
    if(fclose(fp)) {
        syslog(LOG_ERR, "%s", _strerror(cmdfile));
        return(-1);
    }
    return(0);
}
