/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char grepnews_rcsid[] = "$Id: grepnews.c,v 1.1 1997/02/13 14:03:39 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#ifdef __OS2__
#include <pipe.h>
#endif
#include <match.h>
#include <split.h>
#include <xfile.h>
#include <nactive.h>

#include <chlimits.h>

#include "config.h"
#include "misc.h"
#include "changi.h"


typedef struct {
    char *mvgroup;  /* newsgroup where matching articles are moved */
    char *pattern;  /* pattern to search in articles */
    char *gplist;   /* pattern list of groups to be searched */
} SCANINFO;

/************************************************************************/
/*                                                                      */
/************************************************************************/
static void _Optlink grep_thread(void *pv)
{
    char *cp;
    SCANINFO *scip = pv;
    FILE *fp;
    FILE *fpp;
    long artnum;
    char *dpath = malloc(_MAX_PATH);
    char *dname;
    FILE *fpw;
    FILE *fpr;
#ifndef __OS2__
    char *tname;
#endif

    dname = strchr(strcpy(dpath, cfg.newsdir), '\0');
    *dname++ = '\\';
    strcpy(dname, scip->mvgroup);
    while(*dname) {
        if(*dname == '.')
            *dname = '\\';
        dname++;
    }
    *dname++ = '\\';
    *dname = '\0';

    if((fp = xfopen(cfg.activefile, "r")) != NULL) {
        char *cmd = malloc(_MAX_PATH);
        char *line = malloc(MAX_BUFSIZ);

        while(fgets(line, MAX_BUFSIZ, fp)) {
            for(cp = line; *cp && *cp != ' ' && *cp != '\t';)
                cp++;
            *cp = '\0';
            if(ngmatch(line, scip->gplist) == 0)
                continue;
            for(cp = line; *cp; cp++)
                if(*cp == '.')
                    *cp = '\\';
#ifdef __OS2__
            sprintf(cmd, "%s %s %s\\%s\\*", cfg.grepcall, scip->pattern, cfg.newsdir, line);
            if((fpp = popen(cmd, "r")) == NULL) {
                syslog(LOG_INFO, "Failed to run grep");
                continue;
            }
#else
            tname = tempnam(cfg.tempdir, "GRP");
            sprintf(cmd, "%s %s %s\\%s\\* > %s",
                    cfg.grepcall, scip->pattern, cfg.newsdir, line, tname);
            if(system(cmd)) {
                syslog(LOG_ERR, "'%s' returned error", cmd);
                free(tname);
                continue;
            }
            if((fpp = xfopen(tname, "r")) == NULL) {
                syslog(LOG_ERR, "%s", _strerror(tname));
                free(tname);
                continue;
            }
#endif
            while(fgets(line, MAX_BUFSIZ, fpp)) {
                for(cp = line; *cp; cp++)
                    if(*cp == ' ' || *cp == '\t' || *cp == '\n') {
                        *cp = '\0';
                        break;
                    }
                if(*line == '\0')
                    continue;
                if((artnum = inc_active(scip->mvgroup)) == 0) {
                    syslog(LOG_DEBUG, "No new num");
                    continue;
                }
                ltoa(artnum, dname, 10);
                if((fpr = xfopen(line, "rb")) != NULL) {
                    if((fpw = xfopen(dpath, "wb")) != NULL) {
                        while(fgets(line, MAX_BUFSIZ, fpr))
                            fputs(line, fpw);
                        fclose(fpw);
                    }
                    else
                        syslog(LOG_INFO, "%s", _strerror(dpath));
                    fclose(fpr);
                }
                else
                    syslog(LOG_INFO, "%s", _strerror(line));
            }
#ifdef __OS2__
            pclose(fpp);
#else
            if(fclose(fpp))
                syslog(LOG_ERR, "%s", _strerror(tname));
            if(xremove(tname))
                syslog(LOG_WARNING, "%s", _strerror(tname));
            free(tname);
#endif
        }
        fclose(fp);
        free(line);
        free(cmd);
    }
    else
        syslog(LOG_ERR, "%s", _strerror(cfg.activefile));

    free(dpath);
    free(scip->mvgroup);
    free(scip->pattern);
    free(scip->gplist);
    free(scip);
}



/************************************************************************/
/*                                                                      */
/************************************************************************/
int start_grep(char *what, char *poster)
{
    char *cp;
    char *gpinv;
    char **gplist;
    char **ip;
    char *orig;
    SCANINFO *scip;
    size_t sz;
    int n;
    int i;

    /*
     * Do some prechecks.
     */
    if(*poster == '\0' || (cp = strchr(what, ':')) == NULL)
        return(-1);
    if(get_originator(poster, &orig, NULL))
        return(-1);

    /*
     * Allocate scan info structure and store patterns.
     */
    *cp = '\0';
    scip = malloc(sizeof(SCANINFO));
    scip->pattern = strdup(cp + 1);

    /*
     * Store the list of group patterns to scan.
     */
    scip->gplist = strlwr(strdup(what));

    /*
     * Create the name of the destination group and create the
     * group itself if it doesn't exist.
     */
    cp = gpinv = strlwr(strdup(orig));
    while((cp = strchr(cp, '@')) != NULL)
        *cp++ = '.';
    gplist = split_list(gpinv, '.');
    for(n = 0, sz = 11, ip = gplist; *ip; ip++, n++)
        if(**ip != '\0')
            sz += strlen(*ip) + 1;

    scip->mvgroup = malloc(sz);
    strcpy(scip->mvgroup, "local.grep");
    for(i = n; --i >= 0; )
        if(*gplist[i] != '\0') {
            strcat(scip->mvgroup, ".");
            strcat(scip->mvgroup, gplist[i]);
        }

    free(gpinv);
    add_active(scip->mvgroup, 1, 0, 'y', NULL, NULL);

    _beginthread(grep_thread, NULL, 32767, scip);

    return(0);
}

/*
 * $Log: grepnews.c,v $
 * Revision 1.1  1997/02/13 14:03:39  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:42  oliver
 * initial revision
 *
 */
