const char group_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 *
 * This module processes the GROUP command.
 *
 */

/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>

#include <sostream.h>

#include <syslog.h>
#include <nactive.h>
#include <dircache.h>
#include <match.h>

#include "config.h"
#include "globals.h"
#include "nntp.h"
#include "misc.h"
#include "ngmatch.h"
#include "client.h"

/************************************************************************/
/*                                                                      */
/* GROUP newsgroup                                                      */
/*                                                                      */
/* Change the current group to the specified newsgroup. If the          */
/* newsgroup specified is invalid, the old newsgroup remains selected.  */
/*                                                                      */
/************************************************************************/
void group(PNEWSCLIENT pnc, int argc, char *argv[])
{
    long high_msg, low_msg;
    int  num_arts;

    /*
     * 07/19/97 harald
     * Following RFC 977, the group name is not case sensitive.
     */
    strlwr(argv[1]);

    /*
     * Do not allow groupnames containing slash/backslash.
     * We log this unconditionally because it might be an
     * attack.
     */
    if(!valid_newsgroup(argv[1])) {
        SoStreamPrintf(pnc->s, "%d Invalid group name (bad format).\r\n",
                       ERR_NOGROUP);
        syslog(LOG_INFO, "%s: Tried to access %s", pnc->remotehost, argv[1]);
	return;
    }

    /*
     * Check for group access permission.
     */
    if (pnc->perms.ngaccess && ngmatch(argv[1], pnc->perms.ngaccess) == 0) {
        if(team_access(cfg.teamaccessfile, pnc->remotehost, pnc->remoteaddr, argv[1], 0)) {
            SoStreamPrintf(pnc->s, "%d You're not allowed to read %s, sorry.\r\n",
                           ERR_ACCESS, argv[1]);
            syslog(LOG_INFO, "%s: Tried to access %s", pnc->remotehost, argv[1]);
            return;
        }
    }

    /*
     * See if this one is in our active file.
     */
    if(query_active(argv[1], &num_arts, &low_msg, &high_msg) == '\0') {
        SoStreamPrintf(pnc->s, "%d Invalid group name (not in active).\r\n",
		  ERR_NOGROUP);
        if(dolog(LOG_CMDFAIL))
            syslog(LOG_INFO, "%s: Requests unknown group %s", pnc->remotehost, argv[1]);
        return;
    }

    /*
     * Build dirpaths to articles and overview. We can
     * rely on successful name creation because the path
     * lengths had been checked when the groups were
     * created.
     */
    group2path(pnc->mydir, cfg.newsdir, argv[1]);
    group2path(pnc->myovr, cfg.overdir, argv[1]);
    strcat(pnc->myovr, "\\.overview");
    strcpy(pnc->group_name, argv[1]);
    pnc->ingroup = 1;
    /*
     * 07/08/97 harald
     * To follow RFC 977, the current article must be set
     * to the lowest article number. Previous versions
     * set num_arts to 0.
     */
    pnc->art_num = low_msg;
    pnc->num_arts = -1;
    pnc->grp_count++;

    /*
     * Return the number of articles found and the
     * lowest as well as the highest article number.
     */
    SoStreamPrintf(pnc->s, "%d %d %ld %ld %s\r\n", OK_GROUP,
                   num_arts, low_msg, high_msg, argv[1]);
}

/************************************************************************/
/*                                                                      */
/* LISTGROUP [group]                                                    */
/*                                                                      */
/* Lists all article numbers (filenames) in the given group. Used by    */
/* newsreaders such as nn and trn for fast validation of a database.    */
/* If a group name is given it becomes the current group.               */
/*                                                                      */
/* This command is an extension, and not included in RFC 977.           */
/*                                                                      */
/************************************************************************/
void listgroup(PNEWSCLIENT pnc, int argc, char *argv[])
{
    int num_arts = 0;
    int i;
    long *art_array = NULL;

    /*
     * If caller supplies the name of a newsgroup, then we first
     * execute the normal group command. Remember that routine
     * 'group' will output a success or failure message. So we
     * sync the group first to get the proper values.
     */
    if(argc == 2) {
        pnc->ingroup = 0;
#if 0
        num_arts = sync_active(argv[1], &(pnc->art_array), &(pnc->max_arts), 0);
#endif
	group(pnc, argc, argv);
        if(!pnc->ingroup)
	    return;
        num_arts = query_groupdir(argv[1], &art_array);
    }

    /*
     * If the caller didn't supply a group name, then we must be in
     * a newsgroup already. We check the existence again because the
     * group might have disappeared in the meantime.
     */
    else {
        if(pnc->ingroup) {
            num_arts = query_groupdir(pnc->group_name, &art_array);
#if 0
            num_arts = sync_active(pnc->group_name, &(pnc->art_array),
                                   &(pnc->max_arts), 0);
#endif
            if(num_arts < 0)
                pnc->ingroup = 0;
        }
        if(!pnc->ingroup) {
            SoStreamPrintf(pnc->s, "%d You are not currently in a newsgroup.\r\n",
                           ERR_NCING);
            if(dolog(LOG_CMDFAIL))
                syslog(LOG_INFO, "%s: No newsgroup selected", pnc->remotehost);
            return;
        }
    }
    pnc->num_arts = num_arts;

    /*
     * 07/10/97 harald
     * Do only output this line, if no group argument was given,
     * because otherwise 'group' called above has done it for us.
     */
    if(argc == 1)
        SoStreamPrintf(pnc->s, "%d %d %ld %ld %s\r\n",
                       OK_GROUP,
                       num_arts,
                       (num_arts > 0 ? art_array[0] : 0L),
                       (num_arts > 0 ? art_array[num_arts - 1] : 0L),
                       pnc->group_name);

    /*
     * Dump article numbers.
     */
    for(i = 0; i < num_arts; i++)
        if(SoStreamPrintf(pnc->s, "%ld\r\n", art_array[i]) == -1)
            break;
    SoStreamPuts(pnc->s, ".\r\n");
    if(art_array)
        free(art_array);
}
