/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 *
 * This module contains routines to fix the distribution
 * header of certain groups.
 *
 */

const char grpdist_rcsid[] = "$Id: grpdist.c,v 1.1 1997/02/13 14:03:40 oliver Exp $";

#include <stdlib.h>
#include <string.h>

#include <syslog.h>
#include <match.h>
#include <loadline.h>

#include "config.h"
#include "grpdist.h"

static int n_grouppat = 0;     /* size of 'grouppat' array             */
static char **grouppat = NULL; /* array of lines in group.distrib file */
static char **distrib = NULL;  /* pointers to 2nd field                */

static char localdistrib[] = "local";

/************************************************************************/
/*                                                                      */
/************************************************************************/
int read_groupdistrib(char *name)
{
    register char *cp;
    int i;

    if(name == NULL || *name == '\0') {
        if(dolog(LOG_DISTRIB))
            syslog(LOG_DEBUG, "No group.distrib file found");
        return(0);
    }

    if((n_grouppat = load_textfile(name, &grouppat)) > 0) {
        distrib = malloc(n_grouppat * sizeof(char *));
        for(i = 0; i < n_grouppat; i++) {
            if((cp = strchr(grouppat[i], ' ')) == NULL) {
                syslog(LOG_ERR, "Bad line in %s: %s", name, grouppat[i]);
                grouppat[i] = NULL;
                distrib[i] = NULL;
            }
            else {
                *cp++ = '\0';
                while(*cp == ' ' || *cp == '\t')
                    cp++;
                distrib[i] = cp;
            }
        }
    }
    return(n_grouppat);
}

/************************************************************************/
/*                                                                      */
/* get_groupdistrib                                                     */
/*                                                                      */
/*                                                                      */
/************************************************************************/
char *get_groupdistrib(char **groups)
{
    int level = -1;
    register int i;
    char **ip;

    for(ip = groups; *ip; ip++) {
        if(n_grouppat == 0) {
            if(strncmp(*ip, "local.", 6) == 0) {
                if(dolog(LOG_DISTRIB))
                    syslog(LOG_DEBUG, "%s is local by default", *ip);
                return(localdistrib);
            }
        }
        else {
            for(i = n_grouppat; --i >= 0; ) {
                if(grouppat[i] == NULL)
                    continue;
                if(wildmat(*ip, grouppat[i]) && level < i) {
                    level = i;
                    if(dolog(LOG_DISTRIB))
                        syslog(LOG_DEBUG, "Distribute %s to %s", *ip, distrib[level]);
                    break;
                }
                else if(dolog(LOG_DISTRIB))
                    syslog(LOG_DEBUG, "%s doesn't match %s", *ip, grouppat[i]);
            }
        }
    }
    if(level >= 0)
        return(distrib[level]);

    return(NULL);
}

/*
 * $Log: grpdist.c,v $
 * Revision 1.1  1997/02/13 14:03:40  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:43  oliver
 * initial revision
 *
 */
