const char ihave_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/
/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <direct.h>

#include <syslog.h>

#include <chlimits.h>
#include <sostream.h>
#include "article.h"
#include <nhistory.h>

#include "config.h"
#include "globals.h"
#include "nntp.h"
#include "recvart.h"
#include "moderate.h"
#include "control.h"
#include "newsfeeds.h"
#include "distrib.h"
#include "serve.h"
#include "artqueue.h"
#include "client.h"

extern int dirty_cache;

/************************************************************************/
/*                                                                      */
/* IHAVE <messageid>                                                    */
/*                                                                      */
/* Accept an article for transferral if we haven't seen it before.      */
/*                                                                      */
/************************************************************************/
void ihave(PNEWSCLIENT pnc, int argc, char *argv[])
{
    if(gethistart(argv[1], NULL)) {
        SoStreamPrintf(pnc->s, "%d Got it.\r\n", ERR_GOTIT);
        if(dolog(LOG_CMDFAIL))
            syslog(LOG_INFO, "%s: Got %s", pnc->remotehost, argv[1]);
        pnc->refused++;
	return;
    }
    SoStreamPrintf(pnc->s, "%d Ok\r\n", CONT_XFER);

    /*
     * Receive article's header into an array of line pointers.
     * This will concat continuation lines.
     */
    if(receive_header(pnc->rb, pnc->art.headbuf) <= 0) {
        SoStreamPrintf(pnc->s, "%d Xfer failed receiving header.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    }

    /*
     * Receive article's body lines into an array of string pointers.
     * At this point we accept empty bodies because it may be a control
     * message.
     */
    if(receive_block(pnc->rb, pnc->art.bodybuf, NULL) < 0) {
        SoStreamPrintf(pnc->s, "%d Xfer failed receiving body.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    }

    /*
     * Parse the header and set up the header line array of the
     * article structure. Headers containing duplicate lines are
     * rejected.
     */
    if(!parse_header(&(pnc->art))) {
        SoStreamPrintf(pnc->s, "%d Bad header, xfer failed.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    }

    if(servermode != SMOD_RUNNING) {
        if(!cfg.queue_incoming) {
            SoStreamPrintf(pnc->s, "%d No articles currently accepted. Try later.\r\n", ERR_XFERFAIL);
            syslog(LOG_NOTICE, "%d Article rejected, server flushed.");
            pnc->rejected++;
        }
        else if(que_article(pnc->remotehost, &(pnc->art))) {
            SoStreamPrintf(pnc->s, "%d Failed to queue article.\r\n", ERR_XFERFAIL);
            pnc->rejected++;
        }
        else {
            SoStreamPrintf(pnc->s, "%d Queued.\r\n", OK_XFERED);
            pnc->received++;
        }
        return;
    }

    /*
     * Inspect header lines.
     */
    if(inspect_article(&(pnc->art), NULL, mysite.feedexcl, mysite.feeddist) == 0) {
        SoStreamPrintf(pnc->s, "%d Illegal header, xfer failed.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    }

    /*
     * Process control messages.
     */
    process_control(&(pnc->art));

    /*
     * Unapproved articles to moderated groups will be mailed to
     * the moderator. We store all other articles in our local
     * newsbase and/or distribute them to external feeds.
     */
    if(mail_moderator(&(pnc->art), cfg.smailcall)) {
        SoStreamPrintf(pnc->s, "%d Can't send article to moderator.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    }
    switch(file_article(&(pnc->art))) {
    default:
        if(addhistart(&(pnc->art)) == 0)
            break;
    case -1:
        SoStreamPrintf(pnc->s, "%d Failed to store article.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    case 0:
        break;
    }
    if(distrib_art(&(pnc->art), 0)) {
        SoStreamPrintf(pnc->s, "%d Failed distributing article.\r\n", ERR_XFERFAIL);
        pnc->rejected++;
        return;
    }
    dirty_cache = atoi(cfg.autoflush);
    SoStreamPrintf(pnc->s, "%d Thanks.\r\n", OK_XFERED);
    pnc->received++;
}
