const char list_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <chanpath.h>
#include <sostream.h>

#include <syslog.h>
#include <xfile.h>
#include <nactive.h>
#include <match.h>

#include "config.h"
#include "globals.h"
#include "nntp.h"
#include "ngmatch.h"
#include "client.h"

/************************************************************************/
/*                                                                      */
/* LIST                                                                 */
/*                                                                      */
/* List active newsgroups, newsgroup descriptions, distributions,       */
/* active_times, and subscriptions.                                     */
/*                                                                      */
/************************************************************************/
void list(PNEWSCLIENT pnc, int argc, char *argv[])
{
    char *line;
    char *filename;
    char *items;
    char *format;
    char *ngaccess = pnc->perms.ngaccess;
    register char *cp;
    FILE *fp;

    /*
     * Convert argument to uppercase to be easier compared.
     */
    if(argc > 1)
        strupr(argv[1]);

    /*
     * Setup list specific parameters
     */
    if(argc == 1 || strcmp(argv[1], "ACTIVE") == 0) {
        /*
         * 08/29/1996 harald
         * Removed for the time being, since trn becomes
         * to slow.
         */
#if 0
        if(save_active(NULL, 0)) {
            SoStreamPrintf(pnc->s, "%d Group update failed. Try later.\r\n",
                           ERR_FAULT);
            return;
        }
#endif
        filename = cfg.activefile;
        items = "active newsgroups";
        format = "Newsgroups in form \"group high low y/n/m\".";
    }
    else if(strcmp(argv[1], "NEWSGROUPS") == 0) {
        filename = cfg.newsgroupsfile;
	items = "newsgroup descriptions";
	format = "Descriptions in form \"group description\".";
    }
    else if(strcmp(argv[1], "DISTRIBUTIONS") == 0) {
        filename = cfg.distributionsfile;
	items = "newsgroup distributions";
        format = "Distributions in form \"area description\".";
        ngaccess = NULL;
    }
    else if(strcmp(argv[1], "SUBSCRIPTIONS") == 0) {
        filename = cfg.subscriptionsfile;
        items = "group subscriptions";
	format = "New-user subscription list in form \"group\".";
    }
    else if(stricmp(argv[1], "ACTIVE.TIMES") == 0) {
        filename = cfg.active_times;
        items = "active times";
        format = "Active times in form \"group time email name\".";
    }
    else if(argc == 2 && strcmp(argv[1], "OVERVIEW.FMT") == 0) {
        filename = cfg.overviewfmtfile;
        items = "overview format";
        format = "Order of fields in overview database.";
        ngaccess = NULL;
    }
    else {
        SoStreamPrintf(pnc->s, "%d Usage: "
                  "LIST [ACTIVE|NEWSGROUPS|DISTRIBUTIONS|SUBSCRIPTIONS|OVERVIEW.FMT|ACTIVE.TIMES] [pattern]\r\n",
                  ERR_CMDSYN);
        if(dolog(LOG_CMDFAIL))
            syslog(LOG_INFO, "%s: Syntax error in list command", pnc->remotehost);
	return;
    }

    if((fp = xfopen(filename, "r")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(filename));
        SoStreamPrintf(pnc->s, "%d No list of %s available.\r\n",
		  ERR_FAULT, items);
	return;
    }

    SoStreamPrintf(pnc->s, "%d %s\r\n", OK_GROUPS, format);
    line = malloc(MAX_BUFSIZ);
    while (fgets(line, MAX_BUFSIZ, fp)) {
        if ((cp = strchr(line, '\n')) == NULL) {
            syslog(LOG_INFO, "%s: Long line skipped", filename);
            if(fgets(line, MAX_BUFSIZ, fp))
                continue;
            break;
        }
        *cp = '\0';

        /*
         * Limit line at the first whitespace.
         * 07/09/1996 harald
         */
        for(cp = line; *cp && *cp != ' ' && *cp != '\t'; )
            cp++;
        if(*cp)
            *cp = '\0';
        else
            cp = NULL;

        if(ngaccess && ngmatch(line, ngaccess) == 0) {
            syslog(LOG_DEBUG, "%s doesn't match %s", line, ngaccess);
            continue;
        }

        if(argc < 3 || ngmatch(line, argv[2])) {
            if(cp)
                *cp = ' ';
            if(SoStreamPrintf(pnc->s, "%s\r\n", line) == -1)
                break;
        }
    }
    fclose(fp);
    free(line);

    SoStreamPuts(pnc->s, ".\r\n");
}

/************************************************************************/
/*                                                                      */
/* XGTITLE                                                              */
/*                                                                      */
/* List all active newsgroups matching a specified pattern.             */
/*                                                                      */
/* Added 07/09/1996 harald                                              */
/*                                                                      */
/************************************************************************/
void xgtitle(PNEWSCLIENT pnc, int argc, char *argv[])
{
    char *v[3];

    /*
     * 07/10/97 harald
     * Windows NT crashed, 'coz v[1] will be converted to upper case
     * in routine list and "newsgroups" string lives in code segment.
     */
    if(argc > 1)
        v[2] = strdup(argv[1]);
    else {
        /*
         * 07/19/97 harald
         * XGTITLE without argument lists the current group.
         */
        if(!pnc->ingroup) {
            SoStreamPrintf(pnc->s, "%d You are not currently in a newsgroup.\r\n",
                      ERR_NCING);
            return;
        }
        v[2] = strdup(pnc->group_name);
    }
    v[0] = strdup("list");
    v[1] = strdup("NEWSGROUPS");
    list(pnc, 3, v);
    free(v[0]);
    free(v[1]);
    free(v[2]);
}
