/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char misc_rcsid[] = "$Id: misc.c,v 1.1 1997/02/13 14:03:45 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <direct.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <share.h>
#include <io.h>

#include <sostream.h>
#include <xfile.h>
#include <syslog.h>
#include <dircache.h>
#include <nactive.h>
#include <match.h>

#include "nntp.h"
#include "config.h"
#include "globals.h"
#include "ngmatch.h"
#include "client.h"
#include "parsit.h"
#include "misc.h"

/************************************************************************/
/*                                                                      */
/* check_ngperm                                                         */
/*                                                                      */
/* Check to see if they're allowed to see this article by matching      */
/* Newsgroups: and Distribution: line.                                  */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           File pointer of the article to be checked.              */
/*                                                                      */
/* ngaccess     Points to a comma separated list of accessable groups.  */
/* ------------ ------------------------------------------------------- */
/* Return value 0 if they're not allowed to see it, 1 if they are.      */
/*                                                                      */
/************************************************************************/
int check_ngperm(FILE *fp, char *ngaccess, char *remotehost, long remoteaddr)
{
    int result = 1;
    register char *cp;
    register int i;
    char **ngarray = NULL;
    int ngcount = 0;
    char *buf;

    if(ngaccess == NULL)
        return(1);

    buf = malloc(MAX_BUFSIZ);
    while(fgets(buf, MAX_BUFSIZ, fp) != NULL) {
        if (buf[0] == '\n')
            /* End of header */
            break;
        if (buf[0] != 'N' && buf[0] != 'n')
            continue;
        if((cp = strchr(buf, '\n')) == NULL) {
            syslog(LOG_ERR, "Long header line skipped");
            continue;
        }
        else
            *cp = '\0';
        if((cp = strchr(buf, ':')) == NULL)
            continue;
        *cp = '\0';
        if (strcmp(buf, "Newsgroups") == 0) {
            if(dolog(LOG_PERM))
                syslog(LOG_DEBUG, "Permcheck %s", cp + 2);
            ngcount = get_nglist(&ngarray, cp + 2);
            break;
        }
    }
    free(buf);

    rewind(fp);

    for(i = 0; i < ngcount; i++) {
        if((result = ngmatch(ngarray[i], ngaccess)) != 0)
            break;
        if(team_access(cfg.teamaccessfile, remotehost, remoteaddr, ngarray[i], 0) == 0) {
            result = 1;
            break;
        }
    }

    if(ngarray) {
        for(i = 0; ngarray[i]; i++)
            free(ngarray[i]);
        free(ngarray);
    }
    return(result);
}


/************************************************************************/
/*                                                                      */
/* in_list                                                              */
/*                                                                      */
/* Checks if a string is found in an unsorted array. String comparision */
/* is case sensitive.                                                   */
/*                                                                      */
/************************************************************************/
int in_list(char *key, char **list)
{
    register char *q;
    register char c;

    if(list == NULL)
        return(0);
    for (c = *key; (q = *list) != NULL; list++)
        if (c == *q && strcmp(key, q) == 0)
            return(1);
    return(0);
}


/************************************************************************/
/*                                                                      */
/* arg_list                                                             */
/*                                                                      */
/* Converts a blank separated list into an array of arguments. The      */
/* array will be allocated within this routine. If the routine is       */
/* entered with 'array' pointing to a non-NULL pointer, the array will  */
/* be freed before allocating a new one.                                */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* list         String pointer to a blank separated list.               */
/*                                                                      */
/* array        Points to an array pointer of string pointers for the   */
/*              result.                                                 */
/* ------------ ------------------------------------------------------- */
/* Return value Retuns then number of arguments found in the list.      */
/*                                                                      */
/************************************************************************/
int arg_list(char *list, char ***array)
{
    register char *cp;
    register char **ap;
    register int  ac = 2;   /* argument counter */

    /*
     * Free previous allocation
     */
    if (*array) {
        free(*array[0]);
        free(*array);
        *array = NULL;
    }

    if(list == NULL)
        return(0);

    /*
     * Make a copy to leave original list untouched
     */
    while(*list == ' ' || *list == '\t')
        list++;
    if(*list == '\0')
        return(0);
    cp = strdup(list);

    /*
     * Count number of arguments
     */
    while(*list) {
        if(*list == ' ' || *list == '\t') {
            ac++;
            while(*list == ' ' || *list == '\t')
                list++;
        }
        else
            list++;
    }

    /*
     * Allocate argument pointer space
     */
    *array = ap = malloc((size_t)ac * sizeof(char *));

    /*
     * Collect arguments
     */
    ac = 0;
    while(*cp) {
        /*
         * Add argument
         */
        *ap++ = cp;
        ac++;

        /*
         * Find end of argument
         */
        while(*cp && *cp != ' ' && *cp != '\t')
            cp++;
        if(*cp == '\0')
	    break;
        *cp++ = '\0';

        /*
         * Skip spaces
         */
        while(*cp == ' ' || *cp == '\t')
            cp++;
    }
    *ap = NULL;

    return(ac);
}


/************************************************************************/
/*                                                                      */
/* get_distlist                                                         */
/*                                                                      */
/* Returns a nicely set up array of distribution groups along with a    */
/* count, when given an NNTP-spec distribution list in the form         */
/* <dist1,dist2,...,distn>.                                             */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* array        Points to an array pointer of string pointers for the   */
/*              result.                                                 */
/*                                                                      */
/* list         String pointer to the NNTP distribution list.           */
/* ------------ ------------------------------------------------------- */
/* Return value Retuns then number of arguments found in the list.      */
/*                                                                      */
/************************************************************************/
int get_distlist(char ***array, char *list)
{
    char *cp;

    if (*list != '<' || (cp = strchr(list + 1, '>')) == NULL)
	return (-1);

    *cp = '\0';
    for (cp = list + 1; *cp != '\0'; ++cp)
	if (*cp == ',')
            *cp = ' ';
    return(arg_list(list + 1, array));
}


/************************************************************************/
/*                                                                      */
/* file_glue                                                            */
/*                                                                      */
/* Glue a string, a char, and a string together. Useful for making      */
/* filenames.                                                           */
/*                                                                      */
/************************************************************************/
char *file_glue(char *p, char *n1, char c, char *n2)
{
    char *result = p;

    p += strlen(strcpy(p, n1));
    *p++ = c;
    strcpy(p, n2);

    return(result);
}

/************************************************************************/
/*                                                                      */
/* group2path                                                           */
/*                                                                      */
/************************************************************************/
char *group2path(char *path, const char *home, const char *group)
{
    register char *cp;
    char *result = NULL;
    size_t l_home = strlen(home);

    if(l_home + strlen(group) + 12 <= _MAX_PATH) {
        result = memcpy(path, home, l_home);
        cp = path + l_home;
        *cp++ = '\\';
        for(; *group; group++)
            if(*group == '.')
                *cp++ = '\\';
            else
                *cp++ = *group;
        *cp = '\0';
    }

    return(result);
}

/************************************************************************/
/* getrange                                                             */
/*                                                                      */
/* This routine parses a range of article numbers. It will also check,  */
/* if we are currently in a group, if any article is selected, and if   */
/* the range is acceptable. In case of errors it will send a proper     */
/* response to the client.                                              */
/*                                                                      */
/* On success the routine will store a low and a high index into the    */
/* 'art_array' of the given NEWSCLIENT structure.                       */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* pnc          Points to a NEWSCLIENT structure containing all local   */
/*              information of the current client connection.           */
/*                                                                      */
/* range        Points to a zero terminated string to be parsed. This   */
/*              pointer may be NULL to specify the currently selected   */
/*              article. The following range formats are recognized:    */
/*                                                                      */
/*                  5589-6325   specifies article numbers 5589 to 6325  */
/*                  5589-       specifies article numbers 5589 and up   */
/*                  5589        specifies article number 5589 only      */
/*                  -6325       specifies all article numbers upto 6325 */
/*                                                                      */
/* lo           Pointer to an integer where the routine will store      */
/*              the low index of the range.                             */
/*                                                                      */
/* hi           Pointer to an integer where the routine will store      */
/*              the high index of the range.                            */
/* ------------ ------------------------------------------------------- */
/* Return value Retuns 0 on success, -1 on failures.                    */
/*                                                                      */
/************************************************************************/
int getrange(PNEWSCLIENT pnc, char *range, long *lo, long *hi, long **art_array)
{
    *art_array = NULL;

    /*
     * Retrieving by number requires a selected group.
     */
    if(pnc->ingroup) {
        if((pnc->num_arts = query_groupdir(pnc->group_name, art_array)) < 0)
            pnc->ingroup = 0;
    }
    if(!pnc->ingroup) {
        SoStreamPrintf(pnc->s, "%d You are not currently in a newsgroup.\r\n",
                       ERR_NCING);
        if(dolog(LOG_CMDFAIL))
            syslog(LOG_INFO, "%s: Not in a newsgroup", pnc->remotehost);
        return(-1);
    }

    /*
     * Check if there are any articles in this group.
     */
    if(pnc->num_arts == 0) {
        SoStreamPrintf(pnc->s, "%d No articles in group.\r\n", ERR_NOARTIG);
        if(dolog(LOG_CMDFAIL))
            syslog(LOG_INFO, "%s: No articles in group", pnc->remotehost);
        return(-1);
    }
    *lo = (*art_array)[0];
    *hi = (*art_array)[pnc->num_arts - 1];

    /*
     * No argument, use current article
     */
    if(range == NULL) {
        if(pnc->art_num == 0) {
            SoStreamPrintf(pnc->s, "%d No article is currently selected.\r\n",
                           ERR_NOCRNT);
            if(dolog(LOG_CMDFAIL))
                syslog(LOG_INFO, "%s: No article selected", pnc->remotehost);
            return(-1);
        }
        *lo = pnc->art_num;
        *hi = pnc->art_num;
    }

    /*
     * Parse argument
     */
    else {
        char *cp = strchr(range, '-');

        /*
         * Argument is a single number
         */
        if(cp == NULL) {
            if((pnc->art_num = atol(range)) == 0 ) {
                SoStreamPrintf(pnc->s, "%d Invalid article number %s.\r\n",
                               ERR_NOARTIG, range);
                if(dolog(LOG_CMDFAIL))
                    syslog(LOG_INFO, "%s: Illegal article number %s",
                           pnc->remotehost, range);
                return(-1);
            }
            *lo = pnc->art_num;
            *hi = pnc->art_num;
        }

        /*
         * Argument is a range
         */
        else {
            int lo_num, hi_num;

            *cp = '\0';
            lo_num = atol(range);
            hi_num = atol(cp + 1);
            *cp = '-';
            if(lo_num)
                *lo = lo_num;
            if(hi_num)
                *hi = hi_num;
	}
    }
    return(0);
}

/************************************************************************/
/* get_originator                                                       */
/*                                                                      */
/* Retrieves an internet address and/or user name from a any mailing    */
/* address format specified in RFC 1036. Valid formats are:             */
/*                                                                      */
/*      address                                                         */
/*      address (user)                                                  */
/*      user <address>                                                  */
/*                                                                      */
/************************************************************************/
int get_originator(char *sender, char **addr, char **user)
{
    int result = 0;
    char *sp = sender;
    char *angle = NULL;
    char *paran = NULL;
    char *clsap = NULL;

    if(*sp == ' ' || *sp == '\t' || *sp == '<' || *sp == '(')
        return(-1);

    for( ; *sp && result == 0; sp++) {
        if(clsap != NULL) {
            result = -1;
            break;
        }
        switch(*sp) {
        case '(':
            if(angle != NULL || paran != NULL)
                result = -1;
            else
                paran = sp;
            break;
        case ')':
            if(angle != NULL || paran == NULL)
                result = -1;
            else
                clsap = sp;
            break;
        case '<':
            if(angle != NULL || paran != NULL)
                result = -1;
            else
                angle = sp;
            break;
        case '>':
            if(angle == NULL || paran != NULL)
                result = -1;
            else
                clsap = sp;
            break;
        }
    }
    if(result == 0) {
        if(angle != NULL) {
            if(clsap == NULL)
                result = -1;
            else {
                if(addr != NULL) {
                    *addr = angle + 1;
                    *clsap = '\0';
                }
                if(user != NULL) {

                    while(*--angle == ' ' || *angle == '\t')
                        continue;
                    if(*angle == '"' && *sender == '"') {
                        *user = sender + 1;
                        *angle = '\0';
                    }
                    else {
                        *user = sender;
                        *(angle + 1) = '\0';
                    }
                }
            }
        }
        else if(paran != NULL) {
            if(clsap == NULL)
                result = -1;
            else {
                if(user != NULL) {
                    *user = paran + 1;
                    *clsap = '\0';
                }
                if(addr != NULL) {
                    *addr = sender;
                    while(*--paran == ' ' || *paran == '\t')
                        continue;
                    *(paran + 1) = '\0';
                }
            }
        }
        else {
            if(addr != NULL)
                *addr = sender;
            if(user != NULL)
                *user = sp;
        }
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int strings_in_format(register char *format)
{
    int result = 0;
    int got_pcent = 0;

    for(; *format; format++) {
        if(got_pcent) {
            if(*format == 's')
                result++;
            else if(*format != '%')
                return(-1);
            got_pcent = 0;
        }
        else
            got_pcent = (*format == '%');
    }
    return(result);
}

/*
 * get_nglist -- return a nicely set up array of newsgroups
 * along with a count, when given an NNTP-spec newsgroup list
 * in the form ng1,ng2,ng...
 *
 *      Parameters:             "array" is storage for our array,
 *                              set to point at some static data.
 *                              "list" is the NNTP newsgroup list.
 *
 *      Returns:                Number of group specs found.
 *
 */

int get_nglist(char ***array, char *list)
{
    char *cp;
    int ngcount;

    for (cp = list; *cp != '\0'; ++cp)
	if (*cp == ',')
	    *cp = ' ';

    ngcount = parsit(list, array);

    return (ngcount);
}

/*
 * $Log: misc.c,v $
 * Revision 1.1  1997/02/13 14:03:45  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:46  oliver
 * initial revision
 *
 */
