const char moderate_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 *
 * This module contains routines to mail articles for moderated
 * groups to the moderator.
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#include <pipe.h>
#include <match.h>
#include <loadline.h>

#include <chlimits.h>
#include <xfile.h>

#include "globals.h"
#include "config.h"
#include "misc.h"
#include "moderate.h"

static int n_mailpaths = 0;     /* size of 'mailpaths' array        */
static char **mailpaths = NULL; /* array of lines in mailpaths file */
static char **mailfmts = NULL;  /* pointers to 2nd field            */
static int  *inc_group = NULL;  /* set if format contain %s         */

/************************************************************************/
/*                                                                      */
/************************************************************************/
int read_mailpaths(char *name)
{
    register char *cp;
    int i;

    if(name == NULL || *name == '\0')
        return(0);

    if((n_mailpaths = load_textfile(name, &mailpaths)) > 0) {
        mailfmts = malloc(n_mailpaths * sizeof(char *));
        inc_group = malloc(n_mailpaths * sizeof(int));
        memset(inc_group, 0, n_mailpaths * sizeof(int));
        for(i = 0; i < n_mailpaths; i++) {
            if((cp = strchr(mailpaths[i], ' ')) == NULL) {
                syslog(LOG_ERR, "Bad line in %s: %s", name, mailpaths[i]);
                *(mailpaths[i]) = '\0';
                mailfmts[i] = mailpaths[i];
            }
            else {
                *cp++ = '\0';
                while(*cp == ' ' || *cp == '\t')
                    cp++;
                mailfmts[i] = cp;

                /*
                 * Make sure we have not more than one string specifier.
                 */
                if((inc_group[i] = strings_in_format(cp)) > 1) {
                    syslog(LOG_ERR, "Bad format in %s: %s", name, mailfmts[i]);
                    *(mailpaths[i]) = '\0';
                }
            }
        }
    }
    return(n_mailpaths);
}

/************************************************************************/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
static int get_moderator(char *group, char *moderator)
{
    register int i;
    register char *cp;
    char *user;

    for(i = n_mailpaths; --i >= 0; ) {
        if(mailpaths[i][0] == '\0')
            continue;
        if(wildmat(group, mailpaths[i])) {
            if(inc_group[i] != 0) {
                for(cp = user = strdup(group); *cp; cp++)
                    if(*cp == '.')
                        *cp = '-';
                sprintf(moderator, mailfmts[i], user);
                free(user);
            }
            else
                strcpy(moderator, mailfmts[i]);
            return(0);
        }
    }
    return(-1);
}

/************************************************************************/
/*                                                                      */
/* mail_moderator                                                       */
/*                                                                      */
/*                                                                      */
/* What about mailing crossposts via carbon copy?                       */
/*                                                                      */
/************************************************************************/
int mail_moderator(ARTICLE *artp, char *mailcall)
{
    int  result = 0;
    unsigned int rc;
    FILE *fpp;
    int  cnt;
    char *cmd = malloc(_MAX_PATH);
    char *moderator = malloc(MAX_DOMAINNAME);
    char *cp;
    char **ngp;
#ifndef __OS2__
    char *tname;
#endif

    for(cp = artp->groupsmode, ngp = artp->groupslist; *ngp; ngp++, cp++) {
        /*
         * Process all groups marked with 'm'. Approved articles
         * for moderated groups had been marked 'y' by 'inspect_article()'.
         */
        if(*cp != 'm')
            continue;
        if(get_moderator(*ngp, moderator) == 0) {
#ifdef __OS2__
            if((cnt = strings_in_format(mailcall)) == 1)
                sprintf(cmd, mailcall, moderator);
            else
                strcpy(cmd, mailcall);
            if((fpp = popen(cmd, "wb")) != NULL) {
                if(cnt == 0)
                    fprintf(fpp, "To: %s\n", moderator);
                spew_article(fpp, artp, NULL, 0, 0);
                if((rc = pclose(fpp)) != 0) {
                    syslog(LOG_ERR, "%s returned %d", cmd, rc >> 8);
                    result = -1;
                    break;
                }
            }
            else {
                syslog(LOG_ERR, "Failed to run %s", cmd);
                result = -1;
                break;
            }
#else
            tname = tempnam(cfg.tempdir, "MOD");
            if((fpp = xfopen(tname, "wb")) == NULL) {
                syslog(LOG_ERR, "%s", _strerror(tname));
                result = -1;
                free(tname);
                break;
            }

            if((cnt = strings_in_format(mailcall)) == 1)
                sprintf(cmd, mailcall, moderator);
            else
                strcpy(cmd, mailcall);
            strcat(cmd, "< ");
            strcat(cmd, tname);

            if(cnt == 0)
                fprintf(fpp, "To: %s\n", moderator);
            spew_article(fpp, artp, NULL, 0, 0);

            if(fclose(fpp)) {
                syslog(LOG_ERR, "%s", _strerror(tname));
                result = -1;
                free(tname);
                break;
            }

            if((rc = system(cmd)) != 0) {
                syslog(LOG_ERR, "'%s' returned error %d", cmd, rc);
                result = -1;
                free(tname);
                break;
            }
            if(xremove(tname))
                syslog(LOG_WARNING, "%s", _strerror(tname));
            free(tname);
#endif
        }
        else {
            syslog(LOG_INFO, "No mailing address for %s", *ngp);
            result = -1;

            if((fpp = xfopen("to.mod", "ab")) != NULL) {
                fprintf(fpp, "-------------------------------------------\n");
                spew_article(fpp, artp, NULL, 0, 0);
                fclose(fpp);
            }

            break;
        }
    }

    free(moderator);
    free(cmd);
    return(result);
}

/*
 * $Log: moderate.c,v $
 * Revision 1.1  1997/02/13 14:03:46  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:47  oliver
 * initial revision
 *
 */
