/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char newnews_rcsid[] = "$Id: newnews.c,v 1.1 1997/02/13 14:03:47 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <io.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <syslog.h>
#include <xfile.h>

#include <chlimits.h>
#include <datetime.h>
#include <sostream.h>
#include <nhistory.h>

#include "config.h"
#include "nntp.h"
#include "globals.h"
#include "misc.h"
#include "artfile.h"
#include "ngmatch.h"
#include "client.h"

int seekuntil(FILE * fp, char *akey, char *line, int linesize);
int get_histlist(char *array[], char *list);
int distmatch(char *distlist[], int distcount, char *grouplist[], int groupcount);
int getword(FILE * fp, char *w, char *line, int linesize);
int compare(char *s, char *t);

/************************************************************************/
/*                                                                      */
/* NEWNEWS newsgroups date time ["GMT"] [<distributions>]               */
/*                                                                      */
/* Return the message-id's of any news articles past a certain date and */
/* time, within the specified distributions.                            */
/*                                                                      */
/************************************************************************/
void newnews(PNEWSCLIENT pnc, int argc, char *argv[])
{
    char *cp, *ngp;
    char *cpo;
    char datebuf[32];
    char *line;
    char *histlist[20];
    int  ngcount = 0;
    int  histcount;
    long date;
    int  distcount = 0;
    char **dist_list = NULL;
    long offs;
    int  all = (argv[1][0] == '*' && argv[1][1] == '\0');

    if (!all) {
        ngcount = get_nglist(&pnc->nglist, argv[1]);
	if (ngcount == 0) {
            SoStreamPrintf(pnc->s, "%d Bogus newsgroup specifier: %s\r\n",
                      ERR_CMDSYN, argv[1]);
            syslog(LOG_INFO, "%s: Bad newsgroup in newnews command", pnc->remotehost);
	    return;
	}
    }

    if (strlen(argv[2]) != 6 || strlen(argv[3]) != 6) {
        SoStreamPrintf(pnc->s, "%d Date/time must be in form YYMMDD HHMMSS.\r\n",
                  ERR_CMDSYN);
        syslog(LOG_INFO, "%s: Bad date in newnews command", pnc->remotehost);
	return;
    }

    strcpy(datebuf, argv[2]);
    strcat(datebuf, argv[3]);

    argc -= 4;
    argv += 4;

    date = dtol(datebuf);

    /*
     * mipri@albnet.de found out, that Changi
     * crashes on the 1.1.70 between 00:00 and 00:59.
     * He lives 1 hour east of GMT, which results in a
     * negative number of seconds. We'll now demand
     * the second day of 1970.
     */
    if (date < 86400) {
        SoStreamPrintf(pnc->s, "%d Invalid date specification.\r\n", ERR_CMDSYN);
        syslog(LOG_INFO, "%s: Bad date in newnews command", pnc->remotehost);
	return;
    }

    if (argc > 0) {
        if (!stricmp(*argv, "GMT")) {
	    date = gmt_to_local(date);
	    ++argv;
	    --argc;
	}
    }

    /*
     * Convert from local to GMT since this is what we expect in the
     * history file.
     */
    date = local_to_gmt(date);
    ltod(date, datebuf);
    if (argc > 0) {
        distcount = get_distlist(&dist_list, *argv);
        if (distcount < 0) {
            SoStreamPrintf(pnc->s, "%d Bad distribution list: %s\r\n", ERR_CMDSYN,
                      *argv);
            syslog(LOG_INFO, "%s: Bad distribution list in newnews command", pnc->remotehost);
	    return;
	}
    }

    SoStreamPrintf(pnc->s, "%d New news by message id follows\r\n", OK_NEWNEWS);

    line = malloc(MAX_HISTORYLINE);
    offs = firsthist((unsigned long)date, line);
    while(offs != -1) {

        /*
         * Each line must begin with a message-id.
         */
        if(*line != '<') {
            offs = nexthist(offs, line);
	    continue;
        }

        /*
         * Let 'cp' point to the first space, which is a
         * delimiter between message-id and arrival time.
         */
        if ((cp = strchr(line, ' ')) == NULL) {
            offs = nexthist(offs, line);
	    continue;
        }

        /*
         * Let 'cpo' point to the size field.
         */
        if ((cpo = strchr(cp + 1, ' ')) == NULL) {
            offs = nexthist(offs, line);
            continue;
        }

        if ((ngp = strchr(cpo + 1, ' ')) == NULL) {
            offs = nexthist(offs, line);
            continue;
        }
        ++ngp;
        if (*ngp == '\r' || *ngp == '\n' || *ngp == '\0') {
            offs = nexthist(offs, line);
	    continue;
        }

        /*
         * Set an array of pointers to newsgroups
         */
        histcount = get_histlist(histlist, ngp);
        if (histcount == 0) {
            offs = nexthist(offs, line);
	    continue;
        }

	/*
         * For each newsgroup on this line in the history file,
         * check it against the newsgroup names we're given.
         * If it matches, then see if we're hacking distributions.
         * If so, open the file and match the distribution line.
	 */

	if (!all)
            if (!nglistmatch(restreql, 0, pnc->nglist, ngcount, histlist, histcount)) {
                offs = nexthist(offs, line);
		continue;
            }

        if (distcount)
            if (!distmatch(dist_list, distcount, histlist, histcount)) {
                offs = nexthist(offs, line);
		continue;
            }

        *cp = '\0';
        if(SoStreamPuts(pnc->s, line) == -1)
            break;
        if(SoStreamPuts(pnc->s, "\r\n") == -1)
            break;
        offs = nexthist(offs, line);

    }
    SoStreamPuts(pnc->s, ".\r\n");
    arg_list(NULL, &dist_list);
    free(line);
}

/*
 * distmatch -- see if a file matches a set of distributions.
 * We have to do this by (yech!) opening the file, finding
 * the Distribution: line, if it has one, and seeing if the
 * things match.
 *
 *      Parameters:     "distlist" is the distribution list
 *                      we want.
 *                      "distcount" is the count of distributions in it.
 *                      "grouplist" is the list of groups (articles)
 *                      for this line of the history file.  Note that
 *                      this isn't quite a filename.
 *                      "groupcount" is the count of groups in it.
 *
 *      Returns:        1 if the article is in the given distribution.
 *                      0 otherwise.
 */

int distmatch(char *distlist[], int distcount, char *grouplist[], int groupcount)
{
    char c;
    char *cp;
    FILE *fp;
    int i, j;
    char *buf = malloc(MAX_BUFSIZ);

    file_glue(buf, cfg.newsdir, '\\', grouplist[0]);

    for (cp = buf; *cp; cp++)
	if (*cp == '.')
            *cp = '\\';

    fp = openartlink(buf);
    if (fp == NULL) {
        syslog(LOG_ERR, "%s", _strerror(buf));
        free(buf);
	return (0);
    }

    while (fgets(buf, sizeof(buf), fp) != NULL) {
	if ((c = buf[0]) == '\n')	/* End of header */
	    break;
	if (c != 'd' && c != 'D')
	    continue;
	cp = strchr(cp + 1, '\n');
	if (cp)
	    *cp = '\0';
	cp = strchr(buf, ':');
	if (cp == NULL)
	    continue;
	*cp = '\0';
	if (!stricmp(buf, "distribution")) {
	    for (i = 0; i < distcount; ++i) {
		if (!stricmp(cp + 2, distlist[i])) {
                    fclose(fp);
                    free(buf);
		    return (1);
		}
	    }
            fclose(fp);
            free(buf);
	    return (0);
	}
    }
    free(buf);
    fclose(fp);

    /*
     * We've finished the header with no distribution field.
     * So we'll assume that the distribution is the characters
     * up to the first dot in the newsgroup name.
     */

    for (i = 0; i < groupcount; i++) {
	cp = strchr(grouplist[i], '.');
	if (cp)
	    *cp = '\0';
	for (j = 0; j < distcount; j++)
	    if (!stricmp(grouplist[i], distlist[j]))
		return (1);
    }

    return (0);
}


/*
 * get_histlist -- return a nicely set up array of newsgroups
 * (actually, net.foo.bar/article_num) along with a count.
 *
 *      Parameters:             "array" is storage for our array,
 *                              set to point at some static data.
 *                              "list" is the history file newsgroup list.
 *
 *      Returns:                Number of group specs found.
 *
 *      Side effects:           Changes static data area.
 *                              Also puts null bytes in "list"
 *
 */

int get_histlist(char *hist_list[], char *list)
{
    int histcount = 0;

    for (;;) {
        while(*list == ' ' || *list == '\t')
            list++;

	if (*list == '\0' || *list == '\n')
	    break;

	if (histcount < 20)
	    hist_list[histcount++] = list;

        while(*list && *list != ',' && *list != '\n')
            list++;

	if (*list)
	    *(list++) = '\0';
    }

    hist_list[histcount] = (char *)NULL;

    return (histcount);
}

/*
 * $Log: newnews.c,v $
 * Revision 1.1  1997/02/13 14:03:47  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:48  oliver
 * initial revision
 *
 */
