#ifndef NEWSFEEDS_H
#define NEWSFEEDS_H

/*
 * $Id: newsfeeds.h,v 1.1 1997/02/13 14:03:49 oliver Exp $
 */

typedef enum {
    CST_NONE,
    CST_FILE,
    CST_PROC
} CSTYP;

/*----------------------------------------------------------------------*/
/*                                                                      */
/* One SITE structure is created for each entry in the newsfeeds file.  */
/*                                                                      */
/*----------------------------------------------------------------------*/
typedef struct {
    char  *sitename;        /* Name of this site                         */
    char  **feedexcl;       /* Site exclusions                           */
    char  **feeddist;       /* Distributions                             */
    char  **feedpatt;       /* Subscription patterns                     */
    long  max_artsize;      /* Only send articles below this size        */
                            /*   Flag <                                  */
    int   need_distrib;     /* Only send articles with distrib. header   */
                            /*   Flag Ad                                 */
    int   ignore_path;      /* Ignore pathline when sending              */
                            /*   Flag Ap                                 */
    int   max_hops;         /* Only send with this or less hosts in path */
                            /*   Flag H                                  */
    int   local_only;       /* Only local postings are sent              */
                            /*   Flag L                                  */
    int   max_xposts;       /* Only send articles with max. crossposts   */
                            /*   Flag G                                  */
    char  *feedparm;        /* Optional parameter field                  */
                            /*   Specifies spool file or program to feed */
    char  writeflags[16];   /* Send file info, not complete article      */
                            /*   Flag W                                  */
    int   just_moderated;   /* Send moderated groups only                */
                            /*   Flag m and Nm                           */
    int   just_unmoderated; /* Send unmoderated groups only              */
                            /*   Flag u and Nu                           */
    long  flush_size;       /* Flush after specified number of bytes     */
                            /*   Flag S                                  */
    char  queuefile[_MAX_PATH]; /* Name of the spool queue file          */
                                /* %SPOOL%/queue/sitename                */
    long  queuesize;
    int   uupc_mangle;      /* Mangle spool file names for UUPC/extended */
                            /*   Flag TU                                 */
    char  uupc_grade;       /* UUPC/extended job grade                   */
    int   dropped;          /* Set if site is dropped from feeding       */
    int   need_artfile;     /* Site needs a filed article                */
                            /*   Flag Wn, Wf, Wg and WG                  */
    FILE  *fp;              /* Output handle, pipe or file               */
    long  mutex;            /* Mutex access semaphore to this structure  */
    int   batch;            /* Send batchfiles                           */
                            /*   Flag Tb                                 */
    int   compress;         /* Send compressed batchfiles                */
                            /*   Flag Tc                                 */
    int   send_info;        /* Do not send article but infoline only     */
                            /*   Flag I, n, Ti, Wb, Wm, Ws, Wt, WD, WH,  */
                            /*        WN and WO                          */
                            /*   Last field specifies file or program    */
    CSTYP feed_type;        /* Type of this feed                         */
                            /*   Flag Tf, Tp and TU                      */
    int   s_inparm;         /* Number of %s in feedparm                  */
    char  *batchname;       /* Name of program feed's batch file         */
#ifndef __OS2__
    char  *tname;           /* Temporary file to overcome NT pipe probs  */
#endif
} SITE;

extern SITE mysite;
extern SITE *sites;
extern int n_sites;

extern void newsfeeds_maxwait(long maxwait);

extern int read_newsfeeds(char *name);
extern SITE *find_site(char *name);
extern int flush_a_site(SITE *sp);
extern int flush_site(SITE *sp);
extern int drop_site(SITE *sp);

/*
 * $Log: newsfeeds.h,v $
 * Revision 1.1  1997/02/13 14:03:49  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:49  oliver
 * initial revision
 *
 */

#endif
