/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char nextlast_rcsid[] = "$Id: nextlast.c,v 1.1 1997/02/13 14:03:49 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <sostream.h>

#include <syslog.h>
#include <dircache.h>

#include "nntp.h"
#include "globals.h"
#include "config.h"
#include "misc.h"
#include "chartran.h"
#include "artfile.h"
#include "client.h"

/************************************************************************/
/*                                                                      */
/* NEXT                                                                 */
/* LAST                                                                 */
/*                                                                      */
/* Retrieve the message-id of the next or last article in the           */
/* newsgroup. Position the current article pointer to this article.     */
/*                                                                      */
/************************************************************************/
void nextlast(PNEWSCLIENT pnc, int argc, char *argv[])
{
    char art_id[MAX_MSGID];
    long art_num;
    FILE *fp;
    int  next = (argv[0][0] == 'N');

    /*
     * 07/05/97 harald
     * Check for group first, then check for previous article to
     * conform to RFC 977.
     */
    if(!pnc->ingroup) {
        SoStreamPrintf(pnc->s, "%d You are not currently in a newsgroup.\r\n",
		  ERR_NCING);
	return;
    }

    if(pnc->art_num == 0) {
        SoStreamPrintf(pnc->s, "%d No current article selected.\r\n",
		  ERR_NOCRNT);
	return;
    }

    /*
     * 07/19/97 harald
     * We must keep our current article number intact.
     */
    if(next)
        art_num = query_nextart(pnc->group_name, pnc->art_num);
     else
        art_num = query_prevart(pnc->group_name, pnc->art_num);
    if(art_num == 0 ||
       (fp = openartbynum(pnc->mydir, art_num, art_id)) == NULL) {
        SoStreamPrintf(pnc->s, "%d No %s article to retrieve.\r\n",
		next ? ERR_NONEXT : ERR_NOPREV, next ? "next" : "previous");
	return;
    }
    fclose(fp);
    pnc->art_num = art_num;
    SoStreamPrintf(pnc->s, "%d %ld %s Article retrieved; request text separately.\r\n",
              OK_NOTEXT, pnc->art_num, art_id);
}

/*
 * $Log: nextlast.c,v $
 * Revision 1.1  1997/02/13 14:03:49  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:50  oliver
 * initial revision
 *
 */
