const char oldover_rcsid[] = "$Id: oldover.c,v 1.1 1997/02/13 14:03:52 oliver Exp $";

extern char *fake_overview(FILE *fp, long artnum, char *ovrline, char *name);
extern char *overviewline(char *line, long artnum, ARTICLE *artp);
extern int add_overview(char *ovrname, long artnum, ARTICLE *artp);
extern int del_overview(char *ovrname, long artnum);
extern long next_overview(FILE *fp, long artnum, char *ovrline, int item);

/************************************************************************/
/*                                                                      */
/* fake_overview                                                        */
/*                                                                      */
/* Returns a single header line or a complete overview line from an     */
/* opened article file. We don't make attempts of error recovery,       */
/* handling continuation or very long lines, but prefer to leave items  */
/* blank in order to be as fast as possible.                            */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           Stream pointer of the open article file.                */
/*                                                                      */
/* artnum       Article number, which will be put in front of the       */
/*              overview line.                                          */
/*                                                                      */
/* ovrline      Pointer to a buffer where this routine will store       */
/*              the header or overview line. Note, that this buffer     */
/*              must be able to hold at least MAX_OVERLINE characters.  */
/*              If this pointer is NULL, then the buffer will be        */
/*              allocated by this routine and must be freed by the      */
/*              caller.                                                 */
/*                                                                      */
/* name         Specifies the name of the header line to be returned.   */
/*              If this parameter is NULL, then the routine will return */
/*              a complete overview line in 'ovrline'.                  */
/* ------------ ------------------------------------------------------- */
/* Return value Pointer to the header or overview line.                 */
/*                                                                      */
/************************************************************************/
char *fake_overview(FILE *fp, long artnum, char *ovrline, char *name)
{
    register char *cp;
    char *wp;
    int  i;
    char **iarray;

    if(name == NULL) {
        int  sz = novfmt * sizeof(char *);
        iarray = memset(malloc(sz), 0, sz);
    }

    if(ovrline == NULL)
        ovrline = malloc(MAX_OVERLINE);

    while(fgets(ovrline, MAX_OVERLINE, fp)) {
        if((cp = strchr(ovrline, '\n')) == NULL)
            break;
        *cp = '\0';
        if((cp = strchr(ovrline, ':')) == NULL)
            break;
        *cp++ = '\0';
        while(*cp == ' ' || *cp == '\t')
            cp++;
        if(name != NULL) {
            if(stricmp(ovrline, name) == 0) {
                wp = ovrline;
                while(*cp)
                    *wp++ = *cp++;
                *wp = '\0';
                return(ovrline);
            }
        }
        else {
            for(i = 0; i < novfmt; i++) {
                if(stricmp(ovrline, ovfmt[i].name) == 0) {
                    if(iarray[i] == NULL)
                        iarray[i] = strdup(cp);
                    break;
                }
            }
        }
    }
    if(name != NULL) {
        strcpy(ovrline, cfg.headerdummy);
        return(ovrline);
    }

    ltoa(artnum, ovrline, 10);
    cp = ovrline + strlen(ovrline);
    for(i = 0; i < novfmt; i++) {
        *cp++ = '\t';
        if(ovfmt[i].full) {
            strcpy(cp, ovfmt[i].name);
            cp += strlen(cp);
            *cp++ = ':';
            *cp++ = ' ';
        }
        if(iarray[i] == NULL)
            strcpy(cp, cfg.headerdummy);
        else {
            strcpy(cp, iarray[i]);
            free(iarray[i]);
        }
        cp += strlen(cp);
    }
    free(iarray);

    return(ovrline);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static char *add_overviewitem(char *linep, int id, ARTICLE *artp, int full)
{
    if(full) {
        strcpy(linep, artp->header[id].key);
        linep += strlen(linep);
        *linep++ = ':';
    }

    if(id == ID_Bytes)
        ltoa(artp->artsize, linep, 10);
    else if(artp->header[id].info == NULL)
        strcpy(linep, cfg.headerdummy);
    else
        strcpy(linep, artp->header[id].info);

    return(linep + strlen(linep));
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
char *overviewline(char *line, long artnum, ARTICLE *artp)
{
    char *lp = line;
    int i;

    ltoa(artnum, line, 10);
    lp = line + strlen(line);
    for(i = 0; i < novfmt; i++) {
        *lp++ = '\t';
        lp = add_overviewitem(lp, ovfmt[i].idx, artp, ovfmt[i].full);
    }
    return(line);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int add_overview(char *ovrname, long artnum, ARTICLE *artp)
{
    int result = -1;
    FILE *fp;
    char *line = malloc(MAX_OVERLINE);

    overviewline(line, artnum, artp);
    if((fp = xfopen(ovrname, "a")) != NULL) {
        fputs(line, fp);
        fputc('\n', fp);
        if(fclose(fp))
            syslog(LOG_ERR, "%s", _strerror(ovrname));
        else
            result = 0;
    }
    else
        syslog(LOG_ERR, "%s", _strerror(ovrname));

    free(line);
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int del_overview(char *ovrname, long artnum)
{
    int result = -1;
    int left = 0;
    FILE *fp;
    FILE *np;
    char *newname = malloc(strlen(ovrname) + 5);
    char *line = malloc(MAX_OVERLINE);

    strcat(strcpy(newname, ovrname), ".new");
    if((fp = xfopen(ovrname, "r")) != NULL) {
        if((np = xfopen(newname, "w")) != NULL) {
            while(fgets(line, MAX_OVERLINE, fp)) {
                if(atol(line) != artnum) {
                    fputs(line, np);
                    left++;
                }
                else
                    result = 0;
            }
            fclose(np);
        }
        else
            syslog(LOG_ERR, "%s", _strerror("Open overview"));
        fclose(fp);
    }
    else
        syslog(LOG_ERR, "%s", _strerror("Open overview"));

    if(result == 0) {
        if(xremove(ovrname)) {
            syslog(LOG_ERR, "%s", _strerror(ovrname));
            xremove(newname);
        }
        else if(left) {
            if(xrename(newname, ovrname))
                syslog(LOG_ERR, "%s->%s", newname, _strerror(ovrname));
        }
        else
            xremove(newname);
    }

    free(newname);
    free(line);
    return(result);
}

/************************************************************************/
/*                                                                      */
/* next_overview                                                        */
/*                                                                      */
/* Returns a specified item or a complete overview line of a given      */
/* article number. If this article does not exist, the info of the next */
/* available article is returned.                                       */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* fp           File pointer of the overview file.                      */
/*                                                                      */
/* artnum       Number of the wanted article.                           */
/*                                                                      */
/* ovrline      Points to a buffer where this routine will store the    */
/*              requested item of the overview line. Needs to be at     */
/*              least MAX_OVERLINE bytes in size.                       */
/*                                                                      */
/* item         Specifies the index of the item to be returned. This    */
/*              parameter must be set to 0, if the complete overview    */
/*              line should be returned in 'ovrline'.                   */
/* ------------ ------------------------------------------------------- */
/* Return value Article number of the overview line found. This number  */
/*              is greater or equal than 'artnum'.                      */
/*                                                                      */
/************************************************************************/
long next_overview(FILE *fp, long artnum, char *ovrline, int item)
{
    long n;
    register char *cp;
    register char *wp;
    int full;

    while(fgets(ovrline, MAX_OVERLINE, fp)) {
        /*
         * Skip lines which are too long
         */
        if((cp = strchr(ovrline, '\n')) == NULL) {
            syslog(LOG_INFO, "Overview: Long line skipped");
            while(fgets(ovrline, MAX_OVERLINE, fp))
                if(strchr(ovrline, '\n') == NULL)
                    break;
            continue;
        }

        /*
         * See if we found an article number equal or above the
         * requested one.
         */
        if((n = atol(ovrline)) < artnum)
            continue;

        /*
         * We found a valid line, chop off linefeed. If our caller
         * requested a specific item, move it to the line begin.
         */
        *cp = '\0';
        if(item > 0 && item <= novfmt) {
            wp = cp = ovrline;
            full = ovfmt[item - 1].full;
            while(item--) {
                while(*cp && *cp != '\t')
                    cp++;
                cp++;
            }
            if(full && (wp = strchr(cp, ':')) != NULL) {
                cp = wp + 1;
                while(*cp == ' ')
                    cp++;
            }
            wp = ovrline;
            while(*cp && *cp != '\t') {
                *wp++ = *cp++;
            }
            *wp = '\0';
        }

        return(n);
    }

    return(0);
}

/*
 * $Log: oldover.c,v $
 * Revision 1.1  1997/02/13 14:03:52  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:52  oliver
 * initial revision
 *
 */
