/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char os2que_rcsid[] = "$Id: os2que.c,v 1.1 1997/02/13 14:03:52 oliver Exp $";

#define INCL_DOSQUEUES
#define INCL_DOSPROCESS
#define INCL_DOSERRORS
#include <os2.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <process.h>

#include <syslog.h>
#include "os2que.h"

/************************************************************************/
/*                                                                      */
/************************************************************************/
int create_queue(char *name)
{
    unsigned long hq;
    APIRET rc;

    rc = DosCreateQueue(&hq, QUE_FIFO, name);
    if(rc != NO_ERROR) {
        syslog(LOG_ERR, "Error %u DosCreateQueue(%s)", rc, name);
        return(-1);
    }
    return((int)hq);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int open_queue(char *name)
{
    unsigned long hq;
    APIRET rc;
    unsigned long owner;

    rc = DosOpenQueue(&owner, &hq, name);
    if(rc != NO_ERROR) {
        syslog(LOG_ERR, "Error %u DosOpenQueue(%s)", rc, name);
        return(-1);
    }
    return((int)hq);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int close_queue(int hq)
{
    APIRET rc;

    rc = DosCloseQueue(hq);
    if(rc != NO_ERROR) {
        syslog(LOG_ERR, "Error %u DosCloseQueue", rc);
        return(-1);
    }
    return(0);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
long read_queue(int hq, size_t *sizep, void **data)
{
    APIRET rc;
    char prty;
    REQUESTDATA req;

    rc = DosReadQueue(hq, &req, (PULONG)sizep, data, 0, DCWW_WAIT, &prty, 0);
    if(rc != NO_ERROR) {
        syslog(LOG_ERR, "Error %u DosReadQueue", rc);
        return(-1);
    }
    return((long)(req.ulData));
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int write_queue(int hq, size_t size, void *data, long req)
{
    APIRET rc;

    for(;;) {
        rc = DosWriteQueue(hq, req, size, data, 0);
        if(rc == ERROR_QUE_NO_MEMORY)
            DosSleep(100L);
        else
            break;
    }
    if(rc != NO_ERROR) {
        syslog(LOG_ERR, "Error %u DosWriteQueue", rc);
        return(-1);
    }
    return(0);
}

/*
 * $Log: os2que.c,v $
 * Revision 1.1  1997/02/13 14:03:52  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:52  oliver
 * initial revision
 *
 */
