/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char parsit_rcsid[] = "$Id: parsit.c,v 1.1 1997/02/13 14:03:55 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "globals.h"
#include "parsit.h"

/************************************************************************/
/*                                                                      */
/* parsit                                                               */
/*                                                                      */
/* Parse a string of words separated by spaces into an                  */
/* array of pointers to characters, just like good ol' argv[]           */
/* and argc.                                                            */
/*                                                                      */
/* Usage:                                                               */
/*                                                                      */
/* char line[132];                                                      */
/* char **argv;                                                         */
/* int argc;                                                            */
/*                                                                      */
/*      argv = (char **) NULL;                                          */
/*      argc = parsit(line, &argv);                                     */
/*                                                                      */
/* returns the number of words parsed in argc.  argv[argc] will         */
/* be (char *) NULL to indicate end of list, if you're not              */
/* happy with just knowing how many words you have.                     */
/*                                                                      */
/* Note that setting argv = (char **) NULL is only done the first       */
/* time the routine is called with a new "argv" -- it tells             */
/* parsit that "argv" is a new array, and parsit shouldn't free         */
/* up the elements (as it would do if it were an old array).            */
/*                                                                      */
/************************************************************************/
int parsit(char *line, char ***array)
{
    char **argv;
    char *word;
    char *linecp;
    int i, j, num_words;
    int longest_word;

    /*
     * Check to see if we should free the old array
     * and then free the pointer itself.
     */
    argv = *array;
    if (argv) {
        for (i = 0; argv[i]; i++)
            free(argv[i]);
        free(argv);
    }

    linecp = line;
    num_words = longest_word = 0;
    for (;;) {				/* count words in input */
        while(*linecp == ' ' || *linecp == '\t')
            ++linecp;
	if (*linecp == '\0')
	    break;
	word = linecp;
        while(*linecp != ' ' && *linecp != '\t' && *linecp != '\0')
            ++linecp;
	++num_words;
	if ((i = linecp - word) > longest_word)
	    longest_word = i;
	if (*linecp == '\0')
	    break;
    }

    /*
     * Allocate enough for that many words plus 1 for null
     */
    argv = malloc((size_t)(num_words + 1) * sizeof(char *));

    /*
     * Allocate enough to fit the longest word
     */
    word = malloc((size_t)longest_word + 1);

    j = i = 0;
    for (;;) {				/* Now build the list of words */
        while(*line == ' ' || *line == '\t')
            line++;
	if (*line == '\0')
	    break;

	i = 0;
	for (; *line != ' ' && *line != '\t' && *line != '\0'; ++line)
	    word[i++] = *line;
	word[i] = '\0';
        argv[j++] = strdup(word);
	if (*line == '\0')
	    break;
    }
    argv[j] = NULL;    /* remember null at end of list */
    *array = argv;
    free(word);
    return (j);
}

/*
 * $Log: parsit.c,v $
 * Revision 1.1  1997/02/13 14:03:55  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:54  oliver
 * initial revision
 *
 */
