const char pgpvrify_rcsid[] = "$Id$";
/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>

#include <chlimits.h>
#include <xfile.h>

#include "article.h"
#include "pgpvrify.h"

/************************************************************************/
/*                                                                      */
/*                                                                      */
/************************************************************************/
int pgp_verify(ARTICLE *artp, char *pgp_user)
{
    FILE *fpd;
    char *cp, *cp2;
    int i, j;
    int  result = 1;
    char *line;
    char *pgp_version;
    char *pgp_header;
    char *pgp_signature;
    char *tmp_name;
    char *tmp_result;
    char *tmp_dir;
    int  n_list = 0;
    char *pgp_grplst[32];

    /*
     * Search pgp signature.
     */
    for(i = ID_LASTENTRY; artp->header[i].key; i++)
        if(strcmp(artp->header[i].key, "X-PGP-Sig") == 0)
            break;
    if(artp->header[i].key == NULL) {
        syslog(LOG_ERR, "No PGP signature found");
        return(2);
    }

    /*
     * Split pgp signature header line.
     */
    cp = strcpy(line = malloc(MAX_BUFSIZ), artp->header[i].info);

    while(*cp == ' ' || *cp == '\t' || *cp == '\n')
        cp++;
    cp2 = cp;
    while(*cp && *cp != ' ' && *cp != '\t' && *cp != '\n')
        cp++;
    if(*cp == 0) {
        syslog(LOG_ERR, "Bad signature line");
        free(line);
        return(2);
    }
    *cp++ = 0;
    pgp_version = cp2;

    while(*cp == ' ' || *cp == '\t' || *cp == '\n')
        cp++;
    cp2 = cp;
    while(*cp && *cp != ' ' && *cp != '\t' && *cp != '\n')
        cp++;
    if(*cp == 0) {
        syslog(LOG_ERR, "Bad signature line");
        free(line);
        return(2);
    }
    *cp++ = 0;
    pgp_header = cp2;

    while(*cp == ' ' || *cp == '\t' || *cp == '\n')
        cp++;
    cp2 = pgp_signature = cp;
    while(*cp) {
        if(*cp != ' ' && *cp != '\t')
            *cp2++ = *cp;
        cp++;
    }
    *cp2 = 0;

    /*
     * Split pgp header list.
     */
    pgp_grplst[n_list++] = cp = pgp_header;
    while(*++cp) {
        if(*cp == ',') {
            *cp++ = 0;
            pgp_grplst[n_list++] = cp;
        }
    }

    /*
     * Create pgp signed message.
     */
    tmp_dir = getenv("CHANGITEMPDIR");
    if((tmp_name = tempnam(tmp_dir, "pgp")) == NULL) {
        syslog(LOG_ERR, "Can't create temporary file");
        free(line);
        return(3);
    }
    if((fpd = xfopen(tmp_name, "wb")) == NULL) {
        perror(tmp_name);
        free(line);
        return(3);
    }
    fputs("-----BEGIN PGP SIGNED MESSAGE-----\n\n", fpd);
    fputs("X-Signed-Headers: ", fpd);
    fputs(pgp_grplst[0], fpd);
    for(i = 1; i < n_list; i++) {
        fputc(',', fpd);
        fputs(pgp_grplst[i], fpd);
    }
    fputc('\n', fpd);

    for(i = 0; i < n_list; i++) {
        for(j = 0; artp->header[j].key; j++) {
            if(strcmp(artp->header[j].key, pgp_grplst[i]) == 0)
                break;
        }
        if(artp->header[j].info)
            fprintf(fpd, "%s: %s\n", artp->header[j].key, artp->header[j].info);
        else
            fprintf(fpd, "%s: \n", pgp_grplst[i]);
    }
    fputc('\n', fpd);

    /*
     * Transfer message body.
     */
    for(i = 0; i < artp->bodybuf->nlines; i++)
        fprintf(fpd, "%s\n", LBPTR(artp->bodybuf, i));

    fputs("\n-----BEGIN PGP SIGNATURE-----\n", fpd);
    fprintf(fpd, "Version: %s\n\n", pgp_version);
    fputs(pgp_signature, fpd);
    fputs("\n-----END PGP SIGNATURE-----\n", fpd);
    fclose(fpd);

    if((tmp_result = tempnam(tmp_dir, "pgp")) == NULL) {
        syslog(LOG_ERR, "Can't create temporary file");
        free(line);
        return(3);
    }
    sprintf(line, "pgp +batchmode -f <%s 2>%s >NUL", tmp_name, tmp_result);
    if(system(line)) {
        syslog(LOG_ERR, "pgp verification failed");
        xremove(tmp_name);
        free(line);
        return(3);
    }
    xremove(tmp_name);

    /*
     * Open temporary result file.
     */
    if((fpd = xfopen(tmp_result, "r")) == NULL) {
        perror(tmp_result);
        xremove(tmp_result);
        free(line);
        return(3);
    }

    /*
     * Find user's name
     */
    while(fgets(line, MAX_BUFSIZ, fpd)) {
        if((cp = strchr(line, '\r')) != NULL || (cp = strchr(line, '\n')) != NULL)
            *cp = 0;
        if(strnicmp(line, "Good signature from user", strlen("Good signature from user")) == 0) {
            cp = line + strlen("Good signature from user");
            while(*cp == '"' || *cp == ' ' || *cp == '\t')
                cp++;
            cp2 = cp++;
            while(*cp && *cp != '"')
                cp++;
            *cp = 0;
            syslog(LOG_DEBUG, "User '%s'-'%s'", pgp_user, cp2);
            if(stricmp(pgp_user, cp2) == 0)
                result = 0;
        }
    }
    fclose(fpd);
    xremove(tmp_result);

    free(line);

    return(result);
}
