/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Additional code had been stolen from                                 */
/*                                                                      */
/*      INN written by Rich $alz                                        */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char recvcmd_rcsid[] = "$Id: recvcmd.c,v 1.1 1997/02/13 14:03:58 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <signal.h>

#include <syslog.h>
#include <xfile.h>

#include <chlimits.h>
#include <split.h>
#include <sysdep.h>
#include <sodtgram.h>
#include <sostream.h>
#include <nactive.h>
#include <overview.h>
#include <nhistory.h>
#include <dircache.h>

#include "config.h"
#include "globals.h"
#include "serve.h"
#include "control.h"
#include "client.h"
#include "newsfeeds.h"
#include "changi.h"
#include "rescalc.h"
#include "oversync.h"

typedef struct {
    char letter;
    int  args;
    char *(*fnc)(char **, char *);
} CNTL_TBL;

static char *cc_addhist(char *argv[], char *who);
static char *cc_allow(char *argv[], char *who);
static char *cc_begin(char *argv[], char *who);
static char *cc_cancel(char *argv[], char *who);
static char *cc_changegroup(char *argv[], char *who);
static char *cc_checkfile(char *argv[], char *who);
static char *cc_drop(char *argv[], char *who);
static char *cc_flush(char *argv[], char *who);
static char *cc_flushlogs(char *argv[], char *who);
static char *cc_flushserver(char *argv[], char *who);
static char *cc_go(char *argv[], char *who);
static char *cc_hangup(char *argv[], char *who);
static char *cc_mode(char *argv[], char *who);
static char *cc_name(char *argv[], char *who);
static char *cc_newgroup(char *argv[], char *who);
static char *cc_param(char *argv[], char *who);
static char *cc_pause(char *argv[], char *who);
static char *cc_readers(char *argv[], char *who);
static char *cc_refile(char *argv[], char *who);
static char *cc_reject(char *argv[], char *who);
static char *cc_reload(char *argv[], char *who);
static char *cc_renumber(char *argv[], char *who);
static char *cc_reserve(char *argv[], char *who);
static char *cc_rmgroup(char *argv[], char *who);
static char *cc_send(char *argv[], char *who);
static char *cc_shutdown(char *argv[], char *who);
static char *cc_throttle(char *argv[], char *who);
static char *cc_trace(char *argv[], char *who);
static char *cc_xabort(char *argv[], char *who);
static char *cc_oversync(char *argv[], char *who);
static char *cc_hostentry(char *argv[], char *who);
static char *cc_userentry(char *argv[], char *who);
static char *cc_resource(char *argv[], char *who);
static char *cc_disconnect(char *argv[], char *who);
static char *cc_groupentry(char *argv[], char *who);
static char *cc_rejecting(char *argv[], char *who);

static CNTL_TBL cntl_tbl[] = {
    { 'a', 5, cc_addhist     }, /* addhist id arr exp post path...: Add history line                 */
    { 'D', 1, cc_allow       }, /* allow reason...                : Allow remote connections         */
    { 'b', 1, cc_begin       }, /* begin site                     : Start newly-added site           */
    { 'c', 1, cc_cancel      }, /* cancel id                      : Cancel message locally           */
    { 'u', 2, cc_changegroup }, /* changegroup group rest         : Change mode of group             */
    { 'd', 0, cc_checkfile   }, /* checkfile                      : Check syntax of newsfeeds file   */
    { 'e', 1, cc_drop        }, /* drop site                      : Stop feeding site                */
    { 'f', 1, cc_flush       }, /* flush site                     : Flush feed for site*             */
    { 'g', 0, cc_flushlogs   }, /* flushlogs                      : Flush log files                  */
    { 'F', 0, cc_flushserver }, /* flushserver reason...          : Flush history and active         */
    { 'h', 1, cc_go          }, /* go reason...                   : Restart after pause/throttle     */
    { 'i', 1, cc_hangup      }, /* hangup channel                 : Hangup specified connection      */
    { 's', 0, cc_mode        }, /* mode                           : Print operating mode             */
    { 'j', 1, cc_name        }, /* name nnn                       : Print name of specified channel* */
    { 'k', 3, cc_newgroup    }, /* newgroup group rest creator    : Create new group                 */
    { 'l', 2, cc_param       }, /* param letter value             : Change command-line params       */
    { 'm', 1, cc_pause       }, /* pause reason...                : Short-term pause                 */
    { 'v', 2, cc_readers     }, /* readers flag text...           : Enable/disable newsreading       */
    { 't', 2, cc_refile      }, /* refile path group              : Refile an article                */
    { 'C', 1, cc_reject      }, /* reject reason...               : Reject remote connections        */
    { 'o', 2, cc_reload      }, /* reload what reason...          : Re-read config files*            */
    { 'n', 1, cc_renumber    }, /* renumber group                 : Renumber the active file*        */
    { 'z', 1, cc_reserve     }, /* reserve reason...              : Reserve next pause/throttle      */
    { 'p', 1, cc_rmgroup     }, /* rmgroup group                  : Remove named group               */
    { 'A', 2, cc_send        }, /* send feed text...              : Send text to exploder feed       */
    { 'q', 1, cc_shutdown    }, /* shutdown reason...             : Shut down server                 */
    { 'r', 1, cc_throttle    }, /* throttle reason...             : Stop accepting articles          */
    { 'w', 1, cc_trace       }, /* trace flag                     : Turn tracing on or off           */
    { 'x', 1, cc_xabort      }, /* xabort text...                 : Abort the server                 */
    { 'Y', 1, cc_oversync    }, /* oversync speed                 : Update overview slow/fast        */
    { 'H', 1, cc_hostentry   }, /* hostentry index                : Print host by index              */
    { 'U', 1, cc_userentry   }, /* userentry index                : Print user by index              */
    { 'R', 1, cc_resource    }, /* resource type                  : Query system resource            */
    { 'E', 1, cc_disconnect  }, /* disconnect client              : Kick out a client                */
    { 'G', 1, cc_groupentry  }, /* groupentry index               : Print group by index             */
    { 'S', 0, cc_rejecting   }, /* rejecting                      : Print reject reason              */
};

#define NUMCNTLS (sizeof(cntl_tbl) / sizeof(CNTL_TBL))

volatile char *reject_reason = NULL;    /* TODO: mutex access */
volatile int servermode = SMOD_RUNNING;
volatile int in_command = 0;
volatile int server_paused = 0;     /* set (in grab_server) to
                                       hold connections. In fact this
                                       is a bad implementation of
                                       a mutex semaphore */
volatile char *modereason = NULL;
volatile int downshut = 0;

static void grab_server(void);

static int sock;
static int keep_running = 1;

/************************************************************************/
/*                                                                      */
/* receive_cmd                                                          */
/*                                                                      */
/* This thread processes incoming control datagrams. It sets up an UDP  */
/* server at port 119 by default.                                       */
/*                                                                      */
/************************************************************************/
void _Optlink receive_cmd(void *pv)
{
    int i;
    void *from = NULL;
    char *line = malloc(MAX_BUFSIZ);

    tzset();

    if((sock = SoDtgramSetServer(cfg.controlport, 119)) == -1)
        abort();

    /*
     * Receive and interpret commands, which contain
     * the following fields separated by '\001':
     *
     * <sender><sep><command>[<sep><parameter>...]
     */
    from = SoDtgramAlloc();
    while(keep_running) {
        if((i = SoDtgramGet(line, MAX_BUFSIZ, sock, from)) > 0) {
            if(downshut)
                break;
            line[i] = '\0';
            if(*line) {
                int argc = 1;
                char *argv[32];
                char *cp;

                memset(argv, 0, sizeof(argv));
                for(argv[0] = cp = line; *cp; cp++) {
                    if(*cp == '\001') {
                        *cp = '\0';
                        argv[argc++] = cp + 1;
                    }
                }
                if(argc > 1) {
                    for(i = NUMCNTLS; --i >= 0;) {
                        if(cntl_tbl[i].letter == argv[1][0]) {
                            char pms[128];
                            char *r;

                            /*
                             * Check permission.
                             */
                            if(ctrl_perms(cfg.ctrlaccessfile, from, argv[0], pms) != 0 ||
                               (*pms != '*' && strchr(pms, argv[1][0]) == NULL)) {
                                SoDtgramReply(sock, from, "1 No permission");
                                ms_sleep(1000);
                                break;
                            }

                            /*
                             * Process the command and return reply.
                             */
                            if((r = (*cntl_tbl[i].fnc)(&argv[2], argv[0])) == NULL)
                                r = "0 OK";
                            SoDtgramReply(sock, from, r);
                            if(cntl_tbl[i].letter == 'x')
                                keep_running = 0;
                            break;
                        }
                    }
                    if(i < 0) {
                        syslog(LOG_ERR, "Got unknown command");
                        SoDtgramReply(sock, from, "1 Bad command");
                    }
                }

            }
        }
        else
            syslog(LOG_ERR, "recvfrom failed");
    }
    SoDtgramFree(from);
    SoDtgramClose(sock);
    free(line);

}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void start_commander(void)
{
    _beginthread(receive_cmd, NULL, 32767, NULL);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
void stop_commmander(void)
{
    keep_running = 0;
    if(sock != -1)
        SoDtgramClose(sock);
}

/************************************************************************/
/*                                                                      */
/* cc_addhist                                                           */
/*                                                                      */
/* 'a id arr exp post path...'                                          */
/*                                                                      */
/* Appends a new history line.                                          */
/*                                                                      */
/************************************************************************/
static char *cc_addhist(char *argv[], char *who)
{
    char *result = NULL;

    if(servermode == SMOD_FLUSHED) {
        if(reopen_history(cfg.historyfile, 0)) {
            reopen_history(cfg.historyfile, 1);
            return("1 Open failed");
        }
    }
    else if(servermode != SMOD_RUNNING) {
        if(init_history(cfg.historyfile, 0))
            return("1 Open failed");
    }

    if(addhistentry(argv[0], atol(argv[1]), atol(argv[2]), atol(argv[3]), 0, argv[4]))
        result = "1 Write failed";

    if(servermode == SMOD_FLUSHED)
        reopen_history(cfg.historyfile, 1);
    else if(servermode != SMOD_RUNNING)
        exit_history();

    return(result);
}

/************************************************************************/
/*                                                                      */
/* cc_allow                                                             */
/*                                                                      */
/* 'D reason...'                                                        */
/*                                                                      */
/* Allows remote connections.                                           */
/*                                                                      */
/************************************************************************/
static char *cc_allow(char *argv[], char *who)
{
    if(reject_reason == NULL)
        return("1 Already allowed");
    if(strcmp(argv[0], (char *)reject_reason))
        return("1 Wrong reason");
    free((void *)reject_reason);
    reject_reason = NULL;

    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_begin                                                             */
/*                                                                      */
/* 'b site'                                                             */
/*                                                                      */
/* Start newly-added site.                                              */
/*                                                                      */
/************************************************************************/
static char *cc_begin(char *argv[], char *who)
{
    /*
     * TODO: We have to read the newsfeeds file to find the
     *       name and add the site to the array of sites. If
     *       the site is already in the array, we drop it and
     *       load it with the new parameters (therefore we
     *       also allow ME).
     */
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_cancel                                                            */
/*                                                                      */
/* 'c id'                                                               */
/*                                                                      */
/* Cancels an article locallly.                                         */
/*                                                                      */
/************************************************************************/
static char *cc_cancel(char *argv[], char *who)
{
    if(servermode == SMOD_FLUSHED) {
        if(reopen_history(cfg.historyfile, 0)) {
            reopen_history(cfg.historyfile, 1);
            return("1 Open failed");
        }
    }
    else if(servermode != SMOD_RUNNING) {
        if(init_history(cfg.historyfile, 0))
            return("1 Open failed");
    }

    cancel_art(argv[0]);

    if(servermode == SMOD_FLUSHED)
        reopen_history(cfg.historyfile, 1);
    else if(servermode != SMOD_RUNNING)
        exit_history();

    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_changegroup                                                       */
/*                                                                      */
/* 'u group rest'                                                       */
/*                                                                      */
/* Changes mode of a group.                                             */
/*                                                                      */
/************************************************************************/
static char *cc_changegroup(char *argv[], char *who)
{
    char *result = NULL;

    if(test_active(argv[0])) {
        switch(add_active(argv[0], 0, 0, argv[1][0], who, NULL)) {
        case -1:
            result = "1 Illegal character in newsgroup name";
            break;
        case 2:
            result = "0 Group status unchanged";
            break;
        }
    }
    else
        result = "1 No such group";

    return(result);
}

/************************************************************************/
/*                                                                      */
/* cc_checkfile                                                         */
/*                                                                      */
/* 'd'                                                                  */
/*                                                                      */
/* Checks syntax of the newsfeeds file.                                 */
/*                                                                      */
/************************************************************************/
static char *cc_checkfile(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_drop                                                              */
/*                                                                      */
/* 'e site'                                                             */
/*                                                                      */
/* Stops feeding of one or all sites.                                   */
/*                                                                      */
/************************************************************************/
static char *cc_drop(char *argv[], char *who)
{
    SITE *sp;

    if(argv[0] && *(argv[0])) {
        if((sp = find_site(argv[0])) == NULL)
            return("1 No such site");
        syslog(LOG_NOTICE, "Drop %s", sp->sitename);
        drop_site(sp);
    }
    else {
        syslog(LOG_NOTICE, "Drop all sites");
        drop_site(NULL);
    }

    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_flush                                                             */
/*                                                                      */
/* 'f site'                                                             */
/*                                                                      */
/* Flushes feed for one or all sites.                                   */
/*                                                                      */
/************************************************************************/
static char *cc_flush(char *argv[], char *who)
{
    SITE *sp;

    if(argv[0] && *(argv[0])) {
        if((sp = find_site(argv[0])) == NULL)
            return("1 No such site");
        syslog(LOG_NOTICE, "Flush %s", sp->sitename);
        flush_site(sp);
    }
    else {
        syslog(LOG_NOTICE, "Flush all sites");
        flush_site(NULL);
    }

    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_flushlogs                                                         */
/*                                                                      */
/* 'g'                                                                  */
/*                                                                      */
/* Flushes log files.                                                   */
/*                                                                      */
/************************************************************************/
static char *cc_flushlogs(char *argv[], char *who)
{
    if(servermode != SMOD_RUNNING)
        return("1 Already flushed");
    save_active(cfg.activefile, 1);
    if(reopen_history(cfg.historyfile, 0))
        return("1 Open failed");
    /*
     * 09/01/1996 harald
     * Additionally flush overview
     */
    flush_overview();
    flush_dircache();
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_flushserver                                                       */
/*                                                                      */
/* 'F reason...'                                                        */
/*                                                                      */
/* History and active files are flushed and will not change during      */
/* flushed state. Incoming articles are accepted, but buffered for      */
/* later processing.                                                    */
/*                                                                      */
/************************************************************************/
static char *cc_flushserver(char *argv[], char *who)
{
    char *result = "1 Unknown mode";

    switch(servermode) {
    case SMOD_RUNNING:
        grab_server();
        servermode = SMOD_FLUSHED;
        if(modereason)
            free((void *)modereason);
        modereason = strdup(argv[0]);
        syslog(LOG_NOTICE, "%s, flushing server", modereason);

        /*
         * Flush history and save active file
         */
        reopen_history(cfg.historyfile, 1);
        save_active(cfg.activefile, 0);
        result = NULL;
        server_paused = 0;
        break;

    case SMOD_FLUSHED:
        result = "1 Already flushed";

    case SMOD_PAUSED:
        result = "1 Already paused";

    case SMOD_THROTTLED:
        result = "1 Already throttled";
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/* cc_go                                                                */
/*                                                                      */
/* 'h reason...'                                                        */
/*                                                                      */
/* Restart server after pause/throttle.                                 */
/*                                                                      */
/************************************************************************/
static char *cc_go(char *argv[], char *who)
{
    if(servermode == SMOD_RUNNING)
        return("1 Already running");

    grab_server();
    switch(servermode) {
    case SMOD_RUNNING:
        syslog(LOG_DEBUG, "Restart from running");
        server_paused = 0;
        return("1 Already running");
    case SMOD_FLUSHED:
        syslog(LOG_DEBUG, "Restart from flushed");
        if(stricmp(argv[0], (char *)modereason)) {
            server_paused = 0;
            return("1 Wrong reason");
        }
        reopen_history(cfg.historyfile, 0);
        load_active(cfg.activefile);
        break;
    case SMOD_PAUSED:
        syslog(LOG_DEBUG, "Restart from paused");
        if(stricmp(argv[0], (char *)modereason)) {
            server_paused = 0;
            return("1 Wrong reason");
        }
        init_history(cfg.historyfile, 0);
        load_active(cfg.activefile);
        break;
    case SMOD_THROTTLED:
        syslog(LOG_DEBUG, "Restart from throttled");
        if(stricmp(argv[0], (char *)modereason)) {
            server_paused = 0;
            return("1 Wrong reason");
        }
        init_history(cfg.historyfile, 0);
        load_active(cfg.activefile);
        break;
    }
    if(modereason) {
        free((char *)modereason);
        modereason = NULL;
    }
    servermode = SMOD_RUNNING;
    server_paused = 0;
    syslog(LOG_NOTICE, "Server restarted");

    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_hangup                                                            */
/*                                                                      */
/* 'i channel'                                                          */
/*                                                                      */
/* Hangs up specified connection.                                       */
/*                                                                      */
/************************************************************************/
static char *cc_hangup(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_mode                                                              */
/*                                                                      */
/* 's'                                                                  */
/*                                                                      */
/* Prints current operating mode.                                       */
/*                                                                      */
/************************************************************************/
static char *cc_mode(char *argv[], char *who)
{
    static char buf[128];

    strcpy(buf, "0 Server ");
    switch(servermode) {
    case SMOD_RUNNING:
        strcat(buf, "running");
        return(buf);
    case SMOD_FLUSHED:
        strcat(buf, "flushed");
        break;
    case SMOD_PAUSED:
        strcat(buf, "paused");
        break;
    case SMOD_THROTTLED:
        strcat(buf, "throttled");
        break;
    default:
        return("1 Unknown mode");
    }
    if(modereason) {
        strcat(buf, ", ");
        strcat(buf, (char *)modereason);
    }
    return(buf);
}

/************************************************************************/
/*                                                                      */
/* cc_name                                                              */
/*                                                                      */
/* 'j nnn'                                                              */
/*                                                                      */
/* Prints name of a specified channel.                                  */
/*                                                                      */
/************************************************************************/
static char *cc_name(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_newgroup                                                          */
/*                                                                      */
/* 'k group rest creator'                                               */
/*                                                                      */
/* Creates a new newsgroup.                                             */
/*                                                                      */
/************************************************************************/
static char *cc_newgroup(char *argv[], char *who)
{
    char *result = NULL;

    switch(add_active(argv[0], 1, 0, argv[1][0], argv[2], cfg.active_times)) {
    case -1:
        result = "1 Illegal character in newsgroup name";
        break;
    case 2:
        result = "0 Group status unchanged";
        break;
    }
    return(result);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
static char *set_dbgflag(char *name, char *cp, unsigned long flag)
{
    switch(*(cp + 1)) {
    case '0':
        cp++;
        cfg.logflg &= ~flag;
        syslog(LOG_INFO, "%s debug off", name);
        break;
    case '1':
        cp++;
    default:
        cfg.logflg |= flag;
        syslog(LOG_INFO, "%s debug on", name);
        break;
    }
    return(cp);
}

/************************************************************************/
/*                                                                      */
/* Parse debug options                                                  */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsedbgopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
         case 'a':
             if (*(opt + 1) == 0) {
                 opt++;
                 debug_active(0);
                 syslog(LOG_INFO, "Active debug off");
             }
             else if (*(opt + 1) == 1)
                 opt++;
             else {
                 debug_active(1);
                 syslog(LOG_INFO, "Active debug on");
             }
             break;
         case 'c':
             opt = set_dbgflag("Command", opt, LOG_CMDFAIL);
             break;
         case 'd':
             opt = set_dbgflag("Dupe", opt, LOG_DUPES);
             break;
         case 'D':
             syslog(LOG_INFO, "Dumping environment");
             dump_cfg();
             break;
         case 'f':
             opt = set_dbgflag("Feed", opt, LOG_FEED);
             break;
         case 'h':
             if (*(opt + 1) == 0) {
                 opt++;
                 debug_history(0);
                 syslog(LOG_INFO, "History debug off");
             }
             else if (*(opt + 1) == 1)
                 opt++;
             else {
                 debug_history(1);
                 syslog(LOG_INFO, "History debug on");
             }
             break;
         case 'j':
             opt = set_dbgflag("Junk", opt, LOG_JUNK);
             break;
         case 'm':
             opt = set_dbgflag("Heap", opt, LOG_HEAP);
             break;
         case 'n':
             opt = set_dbgflag("Accounting", opt, LOG_ACCOUNT);
             break;
         case 'P':
             opt = set_dbgflag("Permission", opt, LOG_PERM);
             break;
         case 't':
             opt = set_dbgflag("Telegram", opt, LOG_TCPFAIL);
         case 's':
         case 'r':
             if (*(opt + 1) == 0) {
                 opt++;
                 SoStreamDebug(0);
             }
             else if (*(opt + 1) == 1)
                 opt++;
             else
                 SoStreamDebug(1);
             break;
         case 'v':
             if (*(opt + 1) == 0) {
                 opt++;
                 debug_overview(0);
                 debug_oversync(0);
                 syslog(LOG_INFO, "Overview debug off");
             }
             else if (*(opt + 1) == 1)
                 opt++;
             else {
                 debug_overview(1);
                 debug_oversync(1);
                 syslog(LOG_INFO, "Overview debug on");
             }
             break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Parse list options                                                   */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parselstopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
         case 'c':
             opt++;
             if(isdigit(*opt)) {
                 cfg.max_clients = atoi(opt);
                 syslog(LOG_INFO, "Max clients: %d", cfg.max_clients);
                 while(isdigit(*opt))
                     opt++;
             }
             break;
         case 'f':
             opt++;
             if(isdigit(*opt)) {
                 cfg.min_free = atol(opt);
                 syslog(LOG_INFO, "Min diskspace: %ld", cfg.min_free);
                 while(isdigit(*opt))
                     opt++;
             }
             break;
         case 'm':
             opt++;
             if(isdigit(*opt)) {
                 cfg.min_core = atol(opt);
                 syslog(LOG_INFO, "Min available core: %ld", cfg.min_core);
                 while(isdigit(*opt))
                     opt++;
             }
             break;
         case 's':
             opt++;
             if(isdigit(*opt)) {
                 cfg.max_load = atol(opt);
                 syslog(LOG_INFO, "Max system load: %ld", cfg.max_load);
                 while(isdigit(*opt))
                     opt++;
             }
             break;
         case 't':
             opt++;
             if(isdigit(*opt)) {
                 cfg.max_threads = atoi(opt);
                 syslog(LOG_INFO, "Max threads: %d", cfg.max_threads);
                 while(isdigit(*opt))
                     opt++;
             }
             break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* Parse exclude options                                                */
/*                                                                      */
/* This has been moved to an own procedure in order to overcome trouble */
/* with VisualAge for Windows.                                          */
/*                                                                      */
/************************************************************************/
static void parsexclopt(char *opt)
{
   while(*opt) {
      switch(*opt) {
         case 'l':
             switch(*(opt + 1)) {
             case '0':
                 opt++;
                 cfg.emulate_links = 1;
                 break;
             case '1':
                 opt++;
             default:
                 cfg.emulate_links = 0;
                 break;
             }
             syslog(LOG_INFO, "Linked file emulation o%s",
                    cfg.emulate_links ? "n" : "ff");
             break;
         case 'q':
             switch(*(opt + 1)) {
             case '0':
                 opt++;
                 cfg.queue_incoming = 1;
                 break;
             case '1':
                 opt++;
             default:
                 cfg.queue_incoming = 0;
                 break;
             }
             syslog(LOG_INFO, "Queueing %sabled",
                    cfg.queue_incoming ? "en" : "dis");
             break;
         case 'Q':
             switch(*(opt + 1)) {
             case '0':
                 opt++;
                 cfg.auto_dequeue = 1;
                 break;
             case '1':
                 opt++;
             default:
                 cfg.auto_dequeue = 0;
                 break;
             }
             syslog(LOG_INFO, "Automatic queue processing %sabled",
                    cfg.auto_dequeue ? "en" : "dis");
             break;
         case 'r':
             switch(*(opt + 1)) {
             case '0':
                 opt++;
                 cfg.local_replygroup = 1;
                 break;
             case '1':
                 opt++;
             default:
                 cfg.local_replygroup = 0;
                 break;
             }
             syslog(LOG_INFO, "Local reply group %sabled",
                    cfg.local_replygroup ? "en" : "dis");
             break;
      }
   opt++;
   }
}

/************************************************************************/
/*                                                                      */
/* cc_param                                                             */
/*                                                                      */
/* 'l letter value'                                                     */
/*                                                                      */
/* Changes command line parameters.                                     */
/*                                                                      */
/************************************************************************/
static char *cc_param(char *argv[], char *who)
{
    grab_server();
    switch(argv[0][0]) {
    case 'a':
        strcpy(cfg.hostaccessfile, argv[1]);
        syslog(LOG_INFO, "Host access file: %s", cfg.hostaccessfile);
        break;
    case 'A':
        strcpy(cfg.useraccessfile, argv[1]);
        syslog(LOG_INFO, "User access file: %s", cfg.useraccessfile);
        break;
    case 'D':
        strcpy(cfg.distribution, argv[1]);
        syslog(LOG_INFO, "Distribution: %s", cfg.distribution);
        break;
    case 'd':
        parsedbgopt(argv[1]);
        break;
    case 'F':
        cfg.force_followup = atoi(argv[1]);
        syslog(LOG_INFO, "Followup check: %d", cfg.force_followup);
        break;
    case 'l':
        parselstopt(argv[1]);
        break;
    case 'O':
        strcpy(cfg.organization, argv[1]);
        syslog(LOG_INFO, "Organization: %s", cfg.organization);
        break;
    case 'Q':
        cfg.max_quotelines = atoi(argv[1]);
        syslog(LOG_INFO, "Quote check: %d lines", cfg.max_quotelines);
        break;
    case 'S':
        strcpy(cfg.signfile, argv[1]);
        syslog(LOG_INFO, "Signature file: %s", cfg.signfile);
        break;
    case 't':       /* Define temporary directory */
        strcpy(cfg.tempdir, argv[1]);
        syslog(LOG_INFO, "Temporary file dir: %s", cfg.tempdir);
        break;
    case 'T':   /* character translation */
        strcpy(cfg.chartranfile, argv[1]);
        syslog(LOG_INFO, "Character translation file: %s", cfg.chartranfile);
        break;
    case 'u':
        strcpy(cfg.logname, argv[1]);
        syslog(LOG_INFO, "Logged user: %s", cfg.logname);
        break;
    case 'x':
        parsexclopt(argv[1]);
        break;
    default:
        server_paused = 0;
        return("1 Unknown parameter");
    }
    server_paused = 0;
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_pause                                                             */
/*                                                                      */
/* 'm reason...'                                                        */
/*                                                                      */
/* Pauses server for a short time. History and active files are flushed */
/* and will not change during paused state. Incoming articles are       */
/* accepted, but buffered for later processing.                         */
/*                                                                      */
/************************************************************************/
static char *cc_pause(char *argv[], char *who)
{
    switch(servermode) {
    case SMOD_RUNNING:
        grab_server();
        servermode = SMOD_PAUSED;
        if(modereason)
            free((char *)modereason);
        modereason = strdup(argv[0]);
        syslog(LOG_NOTICE, "%s, pausing server", modereason);
        exit_history();
        save_active(cfg.activefile, 0);
        server_paused = 0;
        return(NULL);
    case SMOD_FLUSHED:
        grab_server();
        if(stricmp(argv[0], (char *)modereason)) {
            server_paused = 0;
            return("1 Wrong reason");
        }
        servermode = SMOD_PAUSED;
        syslog(LOG_NOTICE, "%s, pausing server", modereason);
        exit_history();
        server_paused = 0;
        return(NULL);
    case SMOD_PAUSED:
        return("1 Already paused");
    case SMOD_THROTTLED:
        return("1 Already throttled");
    }
    return("1 Unknown mode");
}

/************************************************************************/
/*                                                                      */
/* cc_readers                                                           */
/*                                                                      */
/* 'v flag text...'                                                     */
/*                                                                      */
/* Enables/disables newsreading.                                        */
/*                                                                      */
/************************************************************************/
static char *cc_readers(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_refile                                                            */
/*                                                                      */
/* 't path group'                                                       */
/*                                                                      */
/* Refiles an article.                                                  */
/*                                                                      */
/************************************************************************/
static char *cc_refile(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_reject                                                            */
/*                                                                      */
/* 'C reason...'                                                        */
/*                                                                      */
/* Rejects remote connections.                                          */
/*                                                                      */
/************************************************************************/
static char *cc_reject(char *argv[], char *who)
{
    if(reject_reason)
        return("1 Already rejecting");
    reject_reason = strdup(argv[0]);

    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_reload                                                            */
/*                                                                      */
/* 'o what reason...'                                                   */
/*                                                                      */
/* Rereads config files.                                                */
/*                                                                      */
/************************************************************************/
static char *cc_reload(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_renumber                                                          */
/*                                                                      */
/* 'n group'                                                            */
/*                                                                      */
/* Renumbers the active file.                                           */
/*                                                                      */
/************************************************************************/
static char *cc_renumber(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_reserve                                                           */
/*                                                                      */
/* 'z reason...'                                                        */
/*                                                                      */
/* Reserves next pause/throttle.                                        */
/*                                                                      */
/************************************************************************/
static char *cc_reserve(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_rmgroup                                                           */
/*                                                                      */
/* 'p group'                                                            */
/*                                                                      */
/* Removes the specified group.                                         */
/*                                                                      */
/************************************************************************/
static char *cc_rmgroup(char *argv[], char *who)
{
    if(del_active(argv[0], who))
        return("1 Group not found");
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_send                                                              */
/*                                                                      */
/* 'A feed text'                                                        */
/*                                                                      */
/* Sends text to an exploder feed.                                      */
/*                                                                      */
/************************************************************************/
static char *cc_send(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_shutdown                                                          */
/*                                                                      */
/* 'q reason...'                                                        */
/*                                                                      */
/* Shuts down the server.                                               */
/*                                                                      */
/************************************************************************/
static char *cc_shutdown(char *argv[], char *who)
{
    /* 08/29/1996 harald
     * Crash on shutdown. Changed argv[1] to argv[0].
     */
    syslog(LOG_NOTICE, "Server going down, %s", argv[0]);
    grab_server();
    modereason = strdup(argv[0]);
    servermode = SMOD_THROTTLED;
    server_paused = 0;
    ms_sleep(5000);
    downshut = 1;
    ms_sleep(15000);
    exit_history();
    save_active(cfg.activefile, 0);

    raise(SIGTERM);
    _endthread();
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_throttle                                                          */
/*                                                                      */
/* 'r reason...'                                                        */
/*                                                                      */
/* Stops accepting articles, closes all connection and rejects new      */
/* connections as well. History will be closed.                         */
/*                                                                      */
/************************************************************************/
static char *cc_throttle(char *argv[], char *who)
{
    switch(servermode) {
    case SMOD_FLUSHED:
    case SMOD_PAUSED:
        if(stricmp((char *)modereason, argv[0]))
            return("1 Wrong reason");
    case SMOD_RUNNING:
        grab_server();
        servermode = SMOD_THROTTLED;
        if(modereason)
            free((char *)modereason);
        modereason = strdup(argv[0]);
        exit_history();
        syslog(LOG_NOTICE, "Server throttled");
        server_paused = 0;
        return(NULL);
    case SMOD_THROTTLED:
        return("1 Already throttled");
    }
    return("1 Unknown mode");
}

/************************************************************************/
/*                                                                      */
/* cc_trace                                                             */
/*                                                                      */
/* 'w flag'                                                             */
/*                                                                      */
/* Turns tracing on/off                                                 */
/*                                                                      */
/************************************************************************/
static char *cc_trace(char *argv[], char *who)
{
    return("1 Not implemented");
}

/************************************************************************/
/*                                                                      */
/* cc_xabort                                                            */
/*                                                                      */
/* 'x text...'                                                          */
/*                                                                      */
/* Aborts the server.                                                   */
/*                                                                      */
/************************************************************************/
static char *cc_xabort(char *argv[], char *who)
{
    syslog(LOG_NOTICE, "Aborting server, %s", argv[0]);
    abort();
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_oversync                                                          */
/*                                                                      */
/* 'Y speed'                                                            */
/*                                                                      */
/* Update overview.                                                     */
/*                                                                      */
/************************************************************************/
static char *cc_oversync(char *argv[], char *who)
{
    if(stricmp(argv[0], "fast") == 0)
        init_oversync(1);
    else if(stricmp(argv[0], "slow") == 0)
        init_oversync(-1);
    else if(stricmp(argv[0], "stop") == 0)
        exit_oversync();
    else if(stricmp(argv[0], "normal") == 0)
        init_oversync(0);
    else
        return("1 bad mode");
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_hostentry                                                         */
/*                                                                      */
/* 'H index'                                                            */
/*                                                                      */
/* Print host by index.                                                 */
/*                                                                      */
/************************************************************************/
static char *cc_hostentry(char *argv[], char *who)
{
    static char buf[128];

    strcpy(buf, "0 ");
    if(get_hostentry(atoi(argv[0]), buf))
        return("1 not available");

    return(buf);
}

/************************************************************************/
/*                                                                      */
/* cc_userentry                                                         */
/*                                                                      */
/* 'U index'                                                            */
/*                                                                      */
/* Print user by index.                                                 */
/*                                                                      */
/************************************************************************/
static char *cc_userentry(char *argv[], char *who)
{
    static char buf[128];

    strcpy(buf, "0 ");
    if(get_userentry(atoi(argv[0]), buf))
        return("1 not available");

    return(buf);
}

/************************************************************************/
/*                                                                      */
/* cc_resource                                                          */
/*                                                                      */
/* 's type'                                                             */
/*                                                                      */
/* Print available system resource                                      */
/*                                                                      */
/************************************************************************/
static char *cc_resource(char *argv[], char *who)
{
    static char buf[128];
    long avail = -1;

    if(stricmp(argv[0], "memory") == 0)
        avail = mbytes_avail(NULL);
    else if(stricmp(argv[0], "news") == 0)
        avail = mbytes_avail(cfg.newsdir);
    else if(stricmp(argv[0], "overview") == 0)
        avail = mbytes_avail(cfg.overdir);
    else if(stricmp(argv[0], "history") == 0)
        avail = mbytes_avail(cfg.historyfile);
    else if(stricmp(argv[0], "sysload") == 0) {
        if((avail = system_load()) != -1) {
            sprintf(buf, "0 %lu", avail);
            return(buf);
        }
    }

    if(avail == -1)
        return("1 failed");
    sprintf(buf, "0 %lu MByte%s", avail, avail == 1 ? "" : "s");
    return(buf);
}

/************************************************************************/
/*                                                                      */
/* cc_disconnect                                                        */
/*                                                                      */
/* 'E client'                                                           */
/*                                                                      */
/* Kick out a client                                                    */
/*                                                                      */
/************************************************************************/
static char *cc_disconnect(char *argv[], char *who)
{
    if(kick_entries(argv[0]))
        return("1 failed");
    return(NULL);
}

/************************************************************************/
/*                                                                      */
/* cc_groupentry                                                        */
/*                                                                      */
/* 'G index'                                                            */
/*                                                                      */
/* Print group by index.                                                */
/*                                                                      */
/************************************************************************/
static char *cc_groupentry(char *argv[], char *who)
{
    static char buf[128];
    int result;

    strcpy(buf, "0 ");
    if((result = get_groupentry(atoi(argv[0]), buf)) < 0)
        return("1 not available");
    else if(result)
        return("2 Go on");

    return(buf);
}


/************************************************************************/
/*                                                                      */
/* cc_rejecting                                                         */
/*                                                                      */
/* 'S'                                                                  */
/*                                                                      */
/* Print reject reason.                                                 */
/*                                                                      */
/************************************************************************/
static char *cc_rejecting(char *argv[], char *who)
{
    static char buf[128];

    if(reject_reason == NULL)
        return("1 Connections allowed");
    strcpy(buf, "0 ");
    strcat(buf, (char *)reject_reason);

    return(buf);
}


/************************************************************************/
/*                                                                      */
/************************************************************************/
static void grab_server(void)
{
    int retries = 15;

    while(server_paused && retries--)
        ms_sleep(1000);
    if(server_paused)
        syslog(LOG_ERR, "Grabbing server by brute force");
    server_paused = 1;

    retries = 30;
    while(in_command && retries--) {
        server_paused = 1;
        ms_sleep(1000);
    }
    if(in_command) {
        syslog(LOG_ERR, "%d client%s left in current mode",
               in_command, (in_command == 1) ? "" : "s");
        in_command = 0;
    }
}

/*
 * $Log: recvcmd.c,v $
 * Revision 1.1  1997/02/13 14:03:58  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:57  oliver
 * initial revision
 *
 */
