/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char serve_rcsid[] = "$Id: serve.c,v 1.1 1997/02/13 14:04:01 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <process.h>

#include <sostream.h>

#include <syslog.h>
#include <nactive.h>
#include <sysdep.h>

#include "config.h"
#include "globals.h"
#include "nntp.h"
#include "parsit.h"
#include "client.h"
#include "serve.h"

static struct cmdent {
    char *cmd_name;
    int readauthreq;        /* 0=none,1=userpass */
    int postauthreq;        /* 0=none,1=userpass */
    int minargs;
    int maxargs;
    int read_access;
    int xfer_access;
    int post_access;
    void (*cmd_fctn) (PNEWSCLIENT, int, char **);
    char *usage;
} cmdtbl[]= {
/*----------------authr-nargs-permishn-------------------*/
/*-----cmdname----rd-po-mn-mx-rp-xp-pp-function-usage----*/
    { "ARTICLE"  , 1, 0, 1, 2, 0, 0, 0, ahbs, " <message-id>|article_number" },
    { "AUTHINFO" , 0, 0, 3, 3, 0, 0, 0, doxauthinfo, " USER|PASS user|password." },
    { "BODY"     , 1, 0, 1, 2, 0, 0, 0, ahbs, " <message-id>|article_number" },
    { "DATE"     , 0, 0, 1, 1, 0, 0, 0, dodate, "" },
    { "GROUP"    , 1, 0, 2, 2, 1, 0, 0, group, " newsgroup" },
    { "HEAD"     , 1, 0, 1, 2, 0, 0, 0, ahbs, " <message-id>|article_number" },
    { "HELP"     , 0, 0, 1,99, 0, 0, 0, help, "" },
    { "IHAVE"    , 1, 0, 2, 2, 0, 1, 0, ihave, " <message-id>" },
    { "LAST"     , 1, 0, 1, 1, 1, 0, 0, nextlast, "" },
    { "LIST"     , 1, 0, 1, 3, 0, 0, 0, list, " [ACTIVE|NEWSGROUPS|DISTRIBUTIONS|SUBSCRIPTIONS|OVERVIEW.FMT|ACTIVE.TIMES] [wildmat]" },
    { "LISTGROUP", 1, 0, 1, 2, 1, 0, 0, listgroup, " [newsgroup]" },
    { "MODE"     , 0, 0, 2, 2, 0, 0, 0, domode, " [READER|QUERY]" },
    { "NEWGROUPS", 1, 0, 3, 5, 0, 0, 0, newgroups, " yymmdd hhmmss [\"GMT\"] [<distributions>]" },
    { "NEWNEWS"  , 1, 0, 4, 6, 1, 0, 0, newnews, " newsgroups yymmdd hhmmss [\"GMT\"] [<distributions>]" },
    { "NEXT"     , 1, 0, 1, 1, 1, 0, 0, nextlast, "" },
    { "POST"     , 0, 1, 1, 1, 0, 0, 1, post, "" },
    { "SLAVE"    , 0, 0, 1,99, 0, 0, 0, slave, "" },
    { "STAT"     , 1, 0, 1, 2, 0, 0, 0, ahbs, " <message-id>|article_number" },
    { "UWANT"    , 0, 0, 2, 2, 0, 0, 0, uwant, " <message-id>" },
    { "XBATCH"   , 1, 0, 2, 2, 0, 1, 0, xbatch, " size" },
    { "XGTITLE"  , 1, 0, 1, 2, 0, 0, 0, xgtitle, " [wildmat]" },
    { "XHDR"     , 1, 0, 2, 3, 1, 0, 0, xhdr, " headerfield [range|<message-id>]" },
    { "XOVER"    , 1, 0, 1, 2, 1, 0, 0, doxover, " [range]" },
    { "XPAT"     , 1, 0, 4,99, 0, 0, 0, xpat, " header range|<message-id> pat [pat...]" },
    { "XPATH"    , 1, 0, 2, 2, 0, 1, 0, xpath, " <message-id>" },
    { "XROVER"   , 1, 0, 1, 2, 1, 0, 0, xrover, " [range|<message-id>]" }
/*-----cmdname----rd-po-mn-mx-rp-xp-pp-function-usage----*/
/*----------------authr-nargs-permishn-------------------*/
};

#define NUMCMDS (sizeof(cmdtbl) / sizeof(struct cmdent))

static void cleanthread(int sig);

/************************************************************************/
/*                                                                      */
/* ClientThread                                                         */
/*                                                                      */
/* This thread is running for each client connection. When called the   */
/* first time it will wait for a client and respawn itself after a      */
/* client connected.                                                    */
/*                                                                      */
/************************************************************************/
void _Optlink ClientThread(void *pv)
{
    int *ps = (int *)pv;    /* server socket */
    int sock;               /* client socket */
    PNEWSCLIENT pnc;
    char buf[BUFSIZ];       /* buffer for sending and receiving data */
    char **argp = NULL;
    int argnum, i;
    char remotehost[MAX_DOMAINNAME];    /* domain name this host    */
    long remoteaddr;                    /* host's ip address        */
    unsigned long clock_start;
    char *cp;
    int pause_count;

    tzset();
    /*
     * Register a signal handler to avoid crashing the
     * complete program.
     */
    signal(SIGSEGV, cleanthread);

    if((sock = SoStreamAccept(*ps, remotehost, &remoteaddr)) == -1)
        return;
    if(stricmp(remotehost, "localhost") == 0)
        strcpy(remotehost, cfg.mydomain);
    else if(strchr(remotehost, '.') == NULL &&
            (cp = strchr(cfg.mydomain, '.')) != NULL)
            strcat(remotehost, cp);


    /*
     * Reject connections
     */
    if(servermode == SMOD_THROTTLED) {
        SoStreamPrintf(sock, "%d %s\r\n", ERR_GOODBYE, modereason);
        syslog(LOG_WARNING, "Reject connecting %s: %s", remotehost, modereason);
        _beginthread(ClientThread, NULL, 49136, ps);
    }
    else if(reject_reason) {
        SoStreamPrintf(sock, "%d %s\r\n", ERR_GOODBYE, reject_reason);
        syslog(LOG_WARNING, "Reject connecting %s: %s", remotehost, reject_reason);
        _beginthread(ClientThread, NULL, 49136, ps);
    }

    /*
     * Create a local client structure, send greeting banner and
     * process client commands in a loop.
     */
    else if((pnc = create_client(sock, remotehost, remoteaddr)) != NULL) {

        _beginthread(ClientThread, NULL, 49136, ps);

        /*
         * Loop for command input and processing
         */
        clock_start = clock();
        in_command++;
        for (;;) {
            pnc->usr_clock += clock() - clock_start;
            if(in_command)
                in_command--;
            if(SoStreamBufRecv(buf, sizeof(buf), pnc->rb, cfg.sockwait) <= 0) {
                in_command++;
                break;
            }
            if(downshut) {
                SoStreamPrintf(pnc->s, "%d Server going down. Goodbye.\r\n", ERR_GOODBYE);
                break;
            }
            pause_count = 0;
            while(servermode == SMOD_PAUSED || server_paused) {
                ms_sleep(100);
                if(pause_count++ > 600) {
                    pause_count = 0;
                    syslog(LOG_WARNING, "%s waiting, server paused, %s", remotehost, modereason);
                }
                if(pnc->kickout)
                    break;
            }
            in_command++;
            if(pnc->kickout) {
                SoStreamPrintf(sock, "%d Can't talk to you\r\n", ERR_GOODBYE);
                syslog(LOG_NOTICE, "Kicked out %s", remotehost);
                break;
            }
            if(servermode == SMOD_THROTTLED) {
                SoStreamPrintf(sock, "%d %s\r\n", ERR_GOODBYE, modereason);
                syslog(LOG_NOTICE, "Drop connecting %s: %s", remotehost, modereason);
                break;
            }
            clock_start = clock();
            if(dolog(LOG_CMDFAIL))
                syslog(LOG_DEBUG, "%03u> '%s'", sock, buf);
            if((argnum = parsit(buf, &argp)) == 0)
		continue;
            strupr(argp[0]);
            if(!strcmp(argp[0], "QUIT")) {
                SoStreamPrintf(pnc->s, "%d Closing connection. Goodbye.\r\n", OK_GOODBYE);
                break;
            }

	    for (i = 0; i < NUMCMDS; ++i)
                if (!strcmp(cmdtbl[i].cmd_name, argp[0]))
		    break;

            if (i < NUMCMDS) {
                /*
                 * Globally check for correct number of arguments.
                 * More specific checks may be made inside command
                 * handling routines.
                 */
                if(argnum < cmdtbl[i].minargs || argnum > cmdtbl[i].maxargs) {
                    if(dolog(LOG_CMDFAIL))
                        syslog(LOG_INFO, "%s: Bad syntax in %s",
                               pnc->remotehost, cmdtbl[i].cmd_name);
                    if(SoStreamPrintf(pnc->s, "%d Usage: %s%s.\r\n",
                                      ERR_CMDSYN, argp[0], cmdtbl[i].usage) == -1)
                        break;
                }

                /*
                 * Check access permissions.
                 */
                else if(cmdtbl[i].read_access && !pnc->perms.canread) {
                    syslog(LOG_INFO, "%s: Illegal read access", pnc->remotehost);
                    if(SoStreamPrintf(pnc->s, "%d No permission to read, sorry.\r\n",
                                   ERR_ACCESS) == -1)
                        break;
                }
                else if(cmdtbl[i].xfer_access && !pnc->perms.canxfer) {
                    syslog(LOG_INFO, "%s: Illegal xfer access", pnc->remotehost);
                    if(SoStreamPrintf(pnc->s, "%d No permission to transfer, sorry.\r\n",
                                   ERR_ACCESS) == -1)
                        break;
                }
                else if(cmdtbl[i].post_access && !pnc->perms.canpost) {
                    syslog(LOG_INFO, "%s: Illegal post access", pnc->remotehost);
                    if(SoStreamPrintf(pnc->s, "%d No permission to post, sorry.\r\n",
                                   ERR_NOPOST) == -1)
                        break;
                }

                /*
                 * Check if authentication is needed.
                 */
                else if(cmdtbl[i].postauthreq && pnc->perms.needpostauth)  {
                    syslog(LOG_INFO, "%s: Posting requires authentication",
                           pnc->remotehost);
                    if(SoStreamPrintf(pnc->s, "%d Authentication required for command.\r\n",
                                   ERR_NOAUTH) == -1)
                        break;
                }
                else if(cmdtbl[i].readauthreq && pnc->perms.needreadauth)  {
                    syslog(LOG_INFO, "%s: Reading requires authentication",
                           pnc->remotehost);
                    if(SoStreamPrintf(pnc->s, "%d Authentication required for command.\r\n",
                                   ERR_NOAUTH) == -1)
                        break;
                }

                /*
                 * Perform command.
                 */
                else
                    (*cmdtbl[i].cmd_fctn) (pnc, argnum, argp);
            }
            else {
                if(dolog(LOG_CMDFAIL))
                    syslog(LOG_INFO, "%s: Command %s not implemented.", pnc->remotehost, argp[0]);
                if(SoStreamPrintf(pnc->s, "%d Command not implemented.\r\n", ERR_COMMAND) == -1)
                    break;
            }
        }
        if(in_command)
            in_command--;
        destroy_client(pnc);
    }
    else
        _beginthread(ClientThread, NULL, 49136, ps);

    SoStreamClose(sock);

    /*
     * Release argument vector memory.
     */
    if(argp) {
        for(i = 0; argp[i]; i++)
            free(argp[i]);
        free(argp);
    }

    /*
     * This is a nice place for our heap cleaner
     */
    _heapmin();
}


/************************************************************************/
/*                                                                      */
/* Some NNTP commands are recognized but not available.                 */
/*                                                                      */
/************************************************************************/
void ignore(PNEWSCLIENT pnc, int argc, char *argv[])
{
    if(dolog(LOG_CMDFAIL))
        syslog(LOG_INFO, "Command ignored: %s %s %s", argv[0],
                argc > 1 ? argv[1] : "",
                argc > 2 ? argv[2] : "");
    SoStreamPuts(pnc->s, "500 command recognized but not applicable.\r\n");
}

/************************************************************************/
/*                                                                      */
/* HELP                                                                 */
/*                                                                      */
/* Provide a naive user with a brief help message.                      */
/*                                                                      */
/************************************************************************/
void help(PNEWSCLIENT pnc, int argc, char *argv[])
{
    register int i;

    SoStreamPrintf(pnc->s, "%d This server accepts the following commands:\r\n",
                   INF_HELP);
    for(i = 0; i < NUMCMDS; ++i) {
        SoStreamPuts(pnc->s, cmdtbl[i].cmd_name);
        SoStreamPuts(pnc->s, cmdtbl[i].usage);
        SoStreamPuts(pnc->s, "\r\n");
    }
    SoStreamPuts(pnc->s, "\r\nBugs to Harald Kipp (harald@os2point.ping.de)\r\n");
    SoStreamPuts(pnc->s, ".\r\n");
}

/************************************************************************/
/*                                                                      */
/* MODE                                                                 */
/*                                                                      */
/* We don't need this command and simply treat it for compatibility.    */
/*                                                                      */
/************************************************************************/
void domode(PNEWSCLIENT pnc, int argc, char *argv[])
{
    SoStreamPrintf(pnc->s, "%d Hello, you can%s post.\r\n",
                  pnc->perms.canpost ? OK_CANPOST : OK_NOPOST,
                  pnc->perms.canpost ? "" : "'t");
}

/************************************************************************/
/*                                                                      */
/* Signal handler                                                       */
/*                                                                      */
/************************************************************************/
static void cleanthread(int sig)
{
    NEWSCLIENT *pnc;
    int tid = *_threadid;
    char msg[] = "Fatal program bug, process died";
    int sock;

    fputs(msg, stderr);
    fputc('\n', stderr);
    syslog(LOG_ERR, msg);
    if((pnc = find_client(tid)) != NULL) {
        sock = pnc->s;
        destroy_client(pnc);
        SoStreamClose(sock);
    }
    _endthread();
}

/*
 * $Log: serve.c,v $
 * Revision 1.1  1997/02/13 14:04:01  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:58  oliver
 * initial revision
 *
 */
