/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* and                                                                  */
/*      INEWS written by Kai Uwe Rommel.                                */
/*                                                                      */
/* Some parts are                                                       */
/*                                                                      */
/*       Copyright (c) 1989-1994 by Kendra Electronic Wonderworks.      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char spool_rcsid[] = "$Id: spool.c,v 1.1 1997/02/13 14:04:02 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <syslog.h>
#include <jobid.h>
#include <xfile.h>
#include <pipe.h>
#include <sysdep.h>
#include <domnpath.h>
#include <uupcname.h>

#include "globals.h"
#include "config.h"
#include "misc.h"
#include "spool.h"

/************************************************************************/
/*                                                                      */
/* make_spool                                                           */
/*                                                                      */
/* Renames an existing file to a data file and creates corresponding    */
/* command files following UUCP conventions.                            */
/*                                                                      */
/************************************************************************/
int make_spool(char *filename, char *sysname, int mungle, char *command, char grade, int compress)
{
    FILE *fp;
    char wkfile[15];
    char idfile[15];
    char rdfile[15];
    char ixfile[15];
    char rxfile[15];
    char *pathname = malloc(_MAX_PATH);
    char *spooldir = malloc(_MAX_PATH);
    char localpath[22];
    char *job;
    char *cp;
    char *syshost = strdup(sysname);
#ifndef __OS2__
    char *tname;
#endif

    strcpy(spooldir, cfg.spooldir);
    if(!mungle) {
        domain_path(spooldir, sysname, NULL);
        if((cp = strrchr(spooldir, '\\')) != NULL)
            *cp = '\0';
        if((cp = strchr(syshost, '.')) != NULL)
            *cp = '\0';
    }

    job = job_id(cfg.seqf, 1);
    if(dolog(LOG_FEED))
        syslog(LOG_DEBUG, "Spooling job %s for %s", job, sysname);
    sprintf(wkfile, "C.%.8s%c%3.3s", syshost, grade, job);
    sprintf(idfile, "D.%.7s0%sd", cfg.mynode , job);
    sprintf(rdfile, "D.%.7s0%sr", cfg.mynode , job);
    sprintf(ixfile, "D.%.7s0%se", cfg.mynode , job);
    sprintf(rxfile, "X.%.7s0%sr", cfg.mynode , job);

    /*
     * Create remote X.file
     */
    file_glue(pathname, spooldir, '\\', mungle ?
              uupc_path(cfg.mynode, localpath, ixfile, syshost) : ixfile);

    if((fp = xfopen(pathname, "wb")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(pathname));
        free(pathname);
        free(spooldir);
        free(syshost);
        return(-1);
    }
    if(dolog(LOG_FEED))
        syslog(LOG_DEBUG, "Writing %s", pathname);
    fprintf(fp, "R news %s\n", cfg.mydomain);
    fprintf(fp, "U news %s\n", cfg.mynode);
    fprintf(fp, "F %s\n", rdfile);
    fprintf(fp, "I %s\n", rdfile);
    fprintf(fp, "C %s\n", command);
    if(fclose(fp)) {
        syslog(LOG_ERR, "%s", _strerror(pathname));
        free(pathname);
        free(spooldir);
        free(syshost);
        return(-1);
    }

    /*
     * Rename D.file
     */
    file_glue(pathname, spooldir, '\\', mungle ?
            uupc_path(cfg.mynode, localpath, idfile, syshost) : idfile);
    if(dolog(LOG_FEED))
        syslog(LOG_DEBUG, "Moving %s", pathname);
    if(compress) {
        FILE *fpp;
        FILE *fpc;

        compress = 0;
        if((fpc = xfopen(pathname, "wb")) != NULL) {
            char *cmd = malloc(_MAX_PATH);

#ifdef __OS2__
            sprintf(cmd, "%s %s", cfg.compresscall, filename);
            if((fpp = popen(cmd, "rb")) == NULL)
                syslog(LOG_ERR, "Failed to run %s", cmd);
#else
            tname = tempnam(cfg.tempdir, "GZP");
            sprintf(cmd, "%s %s > %s", cfg.compresscall, filename, tname);
            if(system(cmd))
                syslog(LOG_ERR, "'%s' returned error", cmd);
            else if((fpp = xfopen(tname, "r")) == NULL)
                syslog(LOG_ERR, "%s", _strerror(tname));
#endif
            else {
                char *buf = malloc(512);
                int nr, rc;
                char *ubkey;

                /*
                 * 08/02/97 harald
                 * Check a new config item COMPRESSINTRO. Otherwise
                 * use the first character of the basic program name,
                 * not the first one of the path.
                 */
                if(cfg.compressintro[0])
                    fprintf(fpc, "%s\n", cfg.compressintro);
                else if((ubkey = strrchr(cfg.compresscall, '\\')) != NULL ||
                        (ubkey = strrchr(cfg.compresscall, ':')) != NULL)
                    fprintf(fpc, "#! %cunbatch\n", tolower(*(ubkey + 1)));
                else
                    fprintf(fpc, "#! %cunbatch\n", tolower(cfg.compresscall[0]));
                while((nr = fread(buf, 1, 512, fpp)) != 0)
                    fwrite(buf, 1, nr, fpc);
                if(fclose(fpc) == 0) {
#ifdef __OS2__
                    if((rc = pclose(fpp)) != 0)
                        syslog(LOG_ERR, "%s returned %u", cmd, (unsigned int)rc >> 8);
#else
                    if(fclose(fpp))
                        syslog(LOG_ERR, "%s", _strerror(tname));
                    if(xremove(tname))
                        syslog(LOG_WARNING, "%s", _strerror(tname));
#endif
                    else {
                        if(xremove(filename))
                            syslog(LOG_WARNING, "%s", _strerror(filename));
                        compress = 1;
                    }
                }
                else
                    syslog(LOG_ERR, "%s", _strerror(pathname));
                free(buf);
            }
            free(cmd);
#ifndef __OS2__
            free(tname);
#endif
        }
        else
            syslog(LOG_ERR, "%s", _strerror(pathname));
    }
    if(compress == 0 && xrename(filename, pathname)) {
        syslog(LOG_ERR, "%s", _strerror(pathname));
        free(pathname);
        free(spooldir);
        free(syshost);
        return(-1);
    }

    /*
     * Create local C.file
     */
    file_glue(pathname, spooldir, '\\', mungle ?
            uupc_path(cfg.mynode, localpath, wkfile, syshost) : wkfile);
    free(syshost);
    free(spooldir);

    if((fp = xfopen(pathname, "wb")) == NULL) {
        syslog(LOG_ERR, "%s", _strerror(pathname));
        free(pathname);
        return(-1);
    }
    if(dolog(LOG_FEED))
        syslog(LOG_DEBUG, "Writing %s", pathname);
    fprintf(fp, "S %s %s %s - %s 0666\n", idfile, rdfile, "news", idfile);
    fprintf(fp, "S %s %s %s - %s 0666\n", ixfile, rxfile, "news", ixfile);
    if(fclose(fp)) {
        syslog(LOG_ERR, "%s", _strerror(pathname));
        free(pathname);
        return(-1);
    }

    free(pathname);
    return(0);
}

/*
 * $Log: spool.c,v $
 * Revision 1.1  1997/02/13 14:04:02  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:05:59  oliver
 * initial revision
 *
 */
