/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char xbatch_rcsid[] = "$Id$";
/*
 * $Log$
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>

#include <chlimits.h>
#include <xfile.h>

#include <sostream.h>
#include "article.h"
#include "artqueue.h"


#include "config.h"
#include "serve.h"
#include "client.h"
#include "nntp.h"
#include "artbatch.h"

/************************************************************************/
/*                                                                      */
/* XBATCH size                                                          */
/*                                                                      */
/* This command is an extension, and not included in RFC 977.           */
/*                                                                      */
/************************************************************************/
void xbatch(PNEWSCLIENT pnc, int argc, char *argv[])
{
    long ngot;
    size_t size = atol(argv[1]);
    char *binbuf = malloc(size);
    FILE *fp;

    if(servermode != SMOD_RUNNING && !cfg.queue_incoming) {
        SoStreamPrintf(pnc->s, "%d No articles currently accepted. Try later.\r\n", ERR_XFERFAIL);
        syslog(LOG_NOTICE, "Article rejected, server flushed.");
        pnc->rejected++;
        return;
    }

    SoStreamPrintf(pnc->s, "%d Continue to send batch.\r\n", CONT_XBATCH);

    if((ngot = SoStreamBinRecv(binbuf, size, pnc->s, 600)) <= 0) {
        free(binbuf);
        return;
    }
    if(ngot != size) {
        SoStreamPrintf(pnc->s, "%d Not enough.\r\n", ERR_XFERFAIL);
        syslog(LOG_ERR, "%d Not enough.", ERR_XFERFAIL);
    }
    else {
        char *tname = tempnam(cfg.workdir, "XBT");

        if(tname == NULL) {
            SoStreamPrintf(pnc->s, "%d Can't store.\r\n", ERR_XFERFAIL);
            syslog(LOG_ERR, "Can't create temporary file");
            free(binbuf);
            return;
        }

        /*
         * 03/27/98 harald
         * Changed to binary ("w" -> "wb").
         */
        if((fp = xfopen(tname, "wb")) != NULL) {
            if(fwrite(binbuf, 1, ngot, fp) == ngot) {
                if(fclose(fp) != EOF) {
                    /*
                     * 03/27/98 harald
                     * Changed to binary ("r" -> "rb").
                     */
                    if((fp = xfopen(tname, "rb")) != NULL) {
                        SoStreamPrintf(pnc->s, "%d Thanks.\r\n", OK_XBATCHED);
                        artfileproc(pnc, fp);
                        if(fclose(fp) == EOF)
                            syslog(LOG_ERR, "%s (xbatch reclose)", _strerror(tname));
                    }
                    else {
                        SoStreamPrintf(pnc->s, "%d Can't store.\r\n", ERR_XFERFAIL);
                        syslog(LOG_ERR, "%s (xbatch reopen)", _strerror(tname));
                    }
                }
                else {
                    SoStreamPrintf(pnc->s, "%d Can't store.\r\n", ERR_XFERFAIL);
                    syslog(LOG_ERR, "%s (xbatch close)", _strerror(tname));
                }
            }
            else {
                SoStreamPrintf(pnc->s, "%d Can't store.\r\n", ERR_XFERFAIL);
                syslog(LOG_ERR, "%s (xbatch write)", _strerror(tname));
                fclose(fp);
            }
        }
        else {
            SoStreamPrintf(pnc->s, "%d Can't store.\r\n", ERR_XFERFAIL);
            syslog(LOG_ERR, "%s (xbatch open)", _strerror(tname));
        }
        if(xremove(tname))
            syslog(LOG_ERR, "%s (xbatch remove)", _strerror(tname));
        free(tname);
    }
    free(binbuf);
}
