/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char xhdr_rcsid[] = "$Id: xhdr.c,v 1.1 1997/02/13 14:04:05 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <sostream.h>

#include <syslog.h>
#include <sysdep.h>
#include <xfile.h>
#include <overvfmt.h>
#include <overview.h>

#include "globals.h"
#include "nntp.h"
#include "config.h"
#include "misc.h"
#include "artfile.h"
#include "client.h"

/************************************************************************/
/*                                                                      */
/* XHDR header [<messageid>|articlerange]                               */
/*                                                                      */
/* header is a case-insensitive header field, minus any colons.         */
/*                                                                      */
/* articlerange is one of:                                              */
/*      an article number                                               */
/*      an article number followed by a dash to indicate all following  */
/*      an article number followed by a dash followed by another        */
/*              article number.                                         */
/* e.g.,                                                                */
/* XHDR subject                 retrieve subject of current article     */
/* XHDR subject 5589-6325       retrieve subject of arts 5589 to 6325   */
/* XHDR subject 5589-           retrieve subject of arts 5589 and up    */
/* XHDR subject 5589            retrieve subject of art 5589 only       */
/* XHDR subject -6325           retrieve subject of arts upto 6325      */
/* XHDR subject <123@ucbvax>    retrieve subject of art <123@ucbvax>    */
/*                                                                      */
/* This command is an extension, and not included in RFC 977.           */
/*                                                                      */
/************************************************************************/
void xhdr(PNEWSCLIENT pnc, int argc, char *argv[])
{
    register int i;
    long lo_art, hi_art;
    long artnum;
    FILE *fp;
    int  overitem;
    char *overline = NULL;
    char *artpath = NULL;
    char *subpath;
    long *art_array = NULL;

    /*
     * Handle message-id requests
     */
    if (argc == 3 && *argv[2] == '<') {
        if((fp = openartbyid(argv[2])) == NULL) {
            SoStreamPrintf(pnc->s, "%d No article by message-id %s, sorry.\r\n",
		      ERR_NOART, argv[2]);
	    return;
	}
        if(SoStreamPrintf(pnc->s, "%d 0 %s header of article %s.\r\n",
                     OK_HEAD, argv[1], argv[2]) == -1)
            return;
        overline = malloc(MAX_BUFSIZ);
        if(get_headeritem(fp, argv[1], overline) == 0)
            SoStreamPrintf(pnc->s, "0 %s\r\n", overline);
        else
            SoStreamPrintf(pnc->s, "0 (none)\r\n");
        fclose(fp);
        free(overline);
        SoStreamPuts(pnc->s, ".\r\n");
	return;
    }

    /*
     * The routine 'getrange' will check if we are in
     * a group, if any article is currently selected
     * and if the range is acceptable. In case of errors
     * it will send a proper response to the client too.
     */
    if(getrange(pnc, argv[2], &lo_art, &hi_art, &art_array))
        return;

    /*
     * Check if the requested header line is part of our overview base.
     */
    if((overitem = overview_index(argv[1])) != 0)
        overline = malloc(MAX_OVERLINE);
    else {
        overline = malloc(MAX_BUFSIZ);
        strcpy(artpath = malloc(_MAX_PATH), pnc->mydir);
        subpath = artpath + strlen(artpath);
        *subpath++ = '\\';
    }

    /*
     * Return the desired data.
     */
    SoStreamPrintf(pnc->s, "%d %s fields follow\r\n", OK_HEAD, argv[1]);
    for(i = 0; i < pnc->num_arts; i++) {
        if((artnum = art_array[i]) < lo_art) {
            continue;
        }
        if(artnum > hi_art) {
            break;
        }

        if(overitem) {
            if(get_overline(pnc->group_name, artnum, overline, overitem) >= 0
               && SoStreamPrintf(pnc->s, "%lu %s\r\n", artnum, overline) == -1)
                break;
            continue;
        }

        /*
         * If the requested header line is not part of the overview,
         * then get it directly from the file.
         */
        ltoa(artnum, subpath, 10);
        if((fp = openartlink(artpath)) != NULL) {
            int result = get_headeritem(fp, argv[1], overline);
            fclose(fp);
            if(result == 0) {
                if(SoStreamPrintf(pnc->s, "%lu %s\r\n", artnum, overline) == -1)
                    break;
            }
            else if(SoStreamPrintf(pnc->s, "%lu (none)\r\n", artnum) == -1)
                break;
        }
        else
            perror(artpath);
    }
    SoStreamPrintf(pnc->s, ".\r\n");
    if(artpath)
        free(artpath);
    if(art_array)
        free(art_array);
    free(overline);
}

/*
 * $Log: xhdr.c,v $
 * Revision 1.1  1997/02/13 14:04:05  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:06:01  oliver
 * initial revision
 *
 */
