/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char xover_rcsid[] = "$Id: xover.c,v 1.1 1997/02/13 14:04:05 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <sostream.h>

#include <syslog.h>
#include <xfile.h>

#include "globals.h"
#include "nntp.h"
#include "config.h"
#include "misc.h"
#include <overview.h>
#include "artfile.h"
#include "client.h"

/************************************************************************/
/*                                                                      */
/* XOVER [range]                                                        */
/*                                                                      */
/* This command is an extension, and not included in RFC 977.           */
/*                                                                      */
/************************************************************************/
void doxover(PNEWSCLIENT pnc, int argc, char *argv[])
{
    long lo_art, hi_art;
    char *overline;
    long artnum;
    int i;
    long *art_array = NULL;

    /*
     * The routine 'getrange' will check if we are in
     * a group, if any article is currently selected
     * and if the range is acceptable. In case of errors
     * it will send a proper response to the client too.
     */
    if(getrange(pnc, argv[1], &lo_art, &hi_art, &art_array))
        return;

    /*
     * Try to open the overview file. We will ignore
     * failures and fake overview records directly
     * from the article headers.
     */
    overline = malloc(MAX_OVERLINE);

    /*
     * Return the desired data.
     */
    SoStreamPrintf(pnc->s, "%d overview data follows\r\n", OK_OVER);
    for(i = 0; i < pnc->num_arts; i++) {
        if((artnum = art_array[i]) < lo_art)
            continue;
        if(artnum > hi_art)
            break;

        if(get_overline(pnc->group_name, artnum, overline, 0) >= 0
           && SoStreamPrintf(pnc->s, "%ld\t%s\r\n", artnum, overline) == -1)
            break;
    }
    SoStreamPrintf(pnc->s, ".\r\n");
    free(overline);
    if(art_array)
        free(art_array);
}

/*
 * $Log: xover.c,v $
 * Revision 1.1  1997/02/13 14:04:05  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:06:01  oliver
 * initial revision
 *
 */
