/*----------------------------------------------------------------------*/
/* Copyright (c) 1996 by Harald Kipp. All rights reserved.              */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/* This software was originally based on the                            */
/*                                                                      */
/*      Network News Transfer Protocol server                           */
/*                                                                      */
/*      Phil Lapsley                                                    */
/*      University of California, Berkeley                              */
/*      Stan Barber                                                     */
/*      Baylor College of Medicine                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@os2point.ping.de                                         */
/*                                                                      */
/*----------------------------------------------------------------------*/

const char xpat_rcsid[] = "$Id: xpat.c,v 1.1 1997/02/13 14:04:06 oliver Exp $";

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <sostream.h>

#include <syslog.h>
#include <xfile.h>
#include <match.h>
#include <overvfmt.h>

#include "globals.h"
#include "nntp.h"
#include "config.h"
#include "misc.h"
#include <overview.h>
#include "artfile.h"
#include "client.h"

/************************************************************************/
/*                                                                      */
/* XPAT header <messageid>|articlerange pattern [pattern...]            */
/*                                                                      */
/* Header is a case-insensitive header field, minus any colons.         */
/*                                                                      */
/* This command is an extension, and not included in RFC 977.           */
/*                                                                      */
/************************************************************************/
void xpat(PNEWSCLIENT pnc, int argc, char *argv[])
{
    register int ia;
    int  i;
    long lo_art, hi_art;
    long artnum;
    int  result;
    FILE *fp;
    int  overitem;
    char *overline = NULL;
    char *artpath = NULL;
    char *subpath;
    long *art_array = NULL;

    /*
     * Handle message-id requests
     */
    if(*argv[2] == '<') {
        if((fp = openartbyid(argv[2])) == NULL) {
            SoStreamPrintf(pnc->s, "%d No article by message-id %s, sorry.\r\n",
		      ERR_NOART, argv[2]);
	    return;
	}
        if(SoStreamPrintf(pnc->s, "%d 0 %s header of article %s.\r\n",
                     OK_HEAD, argv[1], argv[2]) == -1)
            return;
        overline = malloc(MAX_BUFSIZ);
        if(get_headeritem(fp, argv[1], overline) == 0)
            for(i = 3; i < argc; i++)
                if(wildmat(overline, argv[i])) {
                    SoStreamPrintf(pnc->s, "0 %s\r\n", overline);
                    break;
                }
        fclose(fp);
        free(overline);
        SoStreamPuts(pnc->s, ".\r\n");
	return;
    }

    /*
     * The routine 'getrange' will check if we are in
     * a group, if any article is currently selected
     * and if the range is acceptable. In case of errors
     * it will send a proper response to the client too.
     */
    if(getrange(pnc, argv[2], &lo_art, &hi_art, &art_array))
        return;

    /*
     * Check if the requested header line is part of our overview base.
     */
    if((overitem = overview_index(argv[1])) != 0)
        overline = malloc(MAX_OVERLINE);
    else {
        overline = malloc(MAX_BUFSIZ);
        strcpy(artpath = malloc(_MAX_PATH), pnc->mydir);
        subpath = artpath + strlen(artpath);
        *subpath++ = '\\';
    }

    /*
     * Return the desired data.
     */
    SoStreamPrintf(pnc->s, "%d %s fields follow\r\n", OK_HEAD, argv[1]);
    for(ia = 0; ia < pnc->num_arts; ia++) {
        if((artnum = art_array[ia]) < lo_art)
            continue;
        if(artnum > hi_art)
            break;

        if(overitem) {
            if(get_overline(pnc->group_name, artnum, overline, overitem) >= 0) {
                for(i = 3; i < argc; i++)
                    if(wildmat(overline, argv[i]))
                        break;
                if(i < argc &&
                   SoStreamPrintf(pnc->s, "%lu %s\r\n", artnum, overline) == -1)
                    break;
            }
            continue;
        }

        /*
         * If the requested header line is not part of the overview,
         * then get it directly from the file.
         */
        ltoa(artnum, subpath, 10);
        if((fp = openartlink(artpath)) != NULL) {
            result = get_headeritem(fp, argv[1], overline);
            fclose(fp);
            if(result == 0) {
                for(i = 3; i < argc; i++)
                    if(wildmat(overline, argv[i]))
                        break;
                if(i < argc &&
                   SoStreamPrintf(pnc->s, "%lu %s\r\n", artnum, overline) == -1)
                    break;
            }
        }
    }
    SoStreamPrintf(pnc->s, ".\r\n");
    if(artpath)
        free(artpath);
    if(art_array)
        free(art_array);
    free(overline);
}

/*
 * $Log: xpat.c,v $
 * Revision 1.1  1997/02/13 14:04:06  oliver
 * initial revision
 *
 * Revision 1.1  1997/02/13  10:06:01  oliver
 * initial revision
 *
 */
