const char sod_tran_rcsid[] = "$Id: sod_tran.c,v 1.1 1997/02/13 11:27:32 oliver Exp $";

#ifdef __OS2__
#define BSD_SELECT
#include <types.h>
#include <netinet\in.h>
#include <sys\select.h>
#include <sys\socket.h>
#include <netdb.h>
#include <nerrno.h>
#ifdef min
#undef min
#endif
#else
#include <winsock.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <chlimits.h>
#include <syslog.h>
#include <sostream.h>
#include <sodtgram.h>

int sodebug_on = 0;

/************************************************************************/
/*                                                                      */
/* SoDtgramDebug                                                        */
/*                                                                      */
/* Switches debugging on or off.                                        */
/*                                                                      */
/* Parameter    Description                                             */
/* ------------ ------------------------------------------------------- */
/* enable       A value of 0 will switch off debugging, otherwise it    */
/*              is switched on.                                         */
/* ------------ ------------------------------------------------------- */
/* Return value None.                                                   */
/*                                                                      */
/************************************************************************/
void _Export _System SoDtgramDebug(int enable)
{
    sodebug_on = enable;
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int _Export _System SoDtgramTransact(const char *host, const char *service,
                                     unsigned short defport,
                                     const char *question, char *answer,
                                     long maxsecs)
{
    struct sockaddr_in server;
    struct sockaddr_in from;
    int fromlen;
    int sock;
    int n;
    unsigned short port;
    fd_set rdfds;
    struct timeval timeout;
    int nsocks;

    memset(&server, 0, sizeof(server));
    server.sin_family = AF_INET;
    if((server.sin_addr.s_addr = SoGetHostAddr((char *)host)) == -1)
        return(-1);
    if((port = SoGetServPort((char *)service, "udp")) == -1) {
        port = defport;
        syslog(LOG_INFO, "Unknown service: %s/udp, using port %u", service, port);
    }
    server.sin_port = htons(port);

    if((sock = SoDtgramOpen()) < 0)
        return(-1);

    if(sendto(sock, (char *)question, strlen(question), 0, (struct sockaddr * )&server, sizeof(server)) < 0) {
        syslog(LOG_ERR, "sendto failed");
        return(-1);
    }
    if(sodebug_on)
        syslog(LOG_DEBUG, "%s< '%s'", inet_ntoa(server.sin_addr), question);

    if(maxsecs == 0) {
        SoDtgramClose(sock);
        strcpy(answer, "0 OK");
        return(0);
    }

    FD_ZERO(&rdfds);
    FD_SET(sock, &rdfds);
    nsocks = sock + 1;
    timeout.tv_sec = maxsecs;
    timeout.tv_usec = 0;

    if((nsocks = select(nsocks, &rdfds, 0, 0, &timeout)) <= 0) {
        if(nsocks == 0)
            syslog(LOG_DEBUG, "Connection to %s timed out", host);
        return(-1);
    }
    fromlen = sizeof(from);
    n = recvfrom(sock, answer, MAX_BUFSIZ, 0, (struct sockaddr * )&from, &fromlen);
    SoDtgramClose(sock);
    if(n > 0) {
        answer[n] = '\0';
        if(sodebug_on)
            syslog(LOG_DEBUG, "%s> '%s'", inet_ntoa(from.sin_addr), answer);
        return(0);
    }
    syslog(LOG_ERR, "recvfrom failed");
    return(-1);
}

/************************************************************************/
/*                                                                      */
/************************************************************************/
int _Export _System SoDtgramReply(int sock, void *remote, const char *line)
{
    int rc;

    if(sodebug_on)
         syslog(LOG_DEBUG, "%s:%d< '%s'",
               inet_ntoa(((struct sockaddr_in *)(remote))->sin_addr),
               ntohs(((struct sockaddr_in *)(remote))->sin_port),
                line);

    rc = sendto(sock, (char *)line, strlen(line), 0, remote, sizeof(struct sockaddr_in));
    if(rc < 0)
        syslog(LOG_ERR, "sendto failed");
    return(rc);
}

/*
 * $Log: sod_tran.c,v $
 * Revision 1.1  1997/02/13 11:27:32  oliver
 * initial revision
 *
 */
