/*----------------------------------------------------------------------*/
/* Copyright (c) 1995-98 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@egnite.com                                               */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

const char sos_open_rcsid[] = "$Id$";

#ifdef __OS2__

#include <types.h>
#include <netinet\in.h>
#include <sys\socket.h>

#else

#include <winsock.h>

#define soclose(x)    closesocket(x)
#define sock_errno    WSAGetLastError
#define SOCEINTR      WSAEINTR

#endif

#include <string.h>
#include <stdio.h>

#include <syslog.h>
#include <sostream.h>

/************************************************************************\
*
* SoStreamOpen
*
* Creates a socket, either for connecting a server or to be bound
* to a server port.
*
* Returns the handle of the created socket or -1 in case of any error.
*
\************************************************************************/
int _Export _System SoStreamOpen(void)
{
    int sock;

    /*
     * Check, if we are initialized.
     */
    if(SoInitialize())
        return -1;

    if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1)
        syslog(LOG_ERR, "socket() error: %s", SoErrorString(sock_errno()));

    return(sock);
}

/************************************************************************\
*
* SoStreamClose
*
* Closes a socket no longer used.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* sock         Handle of the socket.
* ------------ -------------------------------------------------------
* Return value 0 on success, -1 otherwise.
*
\************************************************************************/
int _Export _System SoStreamClose(int sock)
{
    int result;

    if((result = soclose(sock)) == -1)
        syslog(LOG_NOTICE, "close() error: %s", SoErrorString(sock_errno()));

    return(result);
}


/************************************************************************\
*
* SoStreamAccept
*
* Wait for a client to connect.
*
* Parameter    Description
* ------------ ---------------------------------------------------------
* sock         Handle of the server socket created with
*              'SoStreamSetServer'.
*
* remotename   Pointer to a string buffer, in which the name of the
*              connected host will be copied. The buffer should be at
*              least 'MAX_DOMAINNAMESIZ' bytes in size. This pointer
*              may be NULL, if the caller is not interested in the
*              name of the connected host.
*
*              If the name could not be determined by a reverse
*              name lookup, then the routine will return the decimal
*              dotted ASCII representation of the ip address.
*
* remoteaddr   Pointer to a long value, in which the ip of the
*              connected host will be returned. The pointer may be
*              NULL, if the caller is not interested in the ip address.
* ------------ ---------------------------------------------------------
* Return value Handle of the client socket or -1 in case of any error.
*
\************************************************************************/
int _Export _System SoStreamAccept(int sock, char *remotename, long *remoteaddr)
{
    struct sockaddr_in client;
    int namelen = sizeof(client);
    int csock;
    int on = 1;

    /*
     * Accept a connecting client. Ignore interrupts.
     */
    for(;;) {
        if((csock = accept(sock, (void *) &client, &namelen)) != -1)
           break;
        if(sock_errno() != SOCEINTR) {
            syslog(LOG_ERR, "accept() error: %s", SoErrorString(sock_errno()));
            return -1;
        }
    }

    /*
     * Set 'KEEPALIVE' to recognize broken connections.
     */
    if(setsockopt(csock, SOL_SOCKET, SO_KEEPALIVE, (char *)&on, sizeof(on)) < 0)
        syslog(LOG_NOTICE, "setsockopt(SO_KEEPALIVE) error: %s", SoErrorString(sock_errno()));

    /*
     * Retrieve informations about the connected client.
     */
    if(remotename || remoteaddr)
        SoGetRemoteHost(&client, remotename, remoteaddr);

    return(csock);
}
