/*----------------------------------------------------------------------*/
/* Copyright (c) 1995-98 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@egnite.com                                               */
/*                                                                      */
/*----------------------------------------------------------------------*/
const char sos_recv_rcsid[] = "$Id$";

/*
 * $Log$
 */


#ifdef __OS2__

#define BSD_SELECT
#include <types.h>
#include <sys\select.h>
#include <sys\socket.h>

#ifdef min
#undef min
#endif

#else

#include <winsock.h>

#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <syslog.h>
#include <sostream.h>

typedef struct {
    int  sock;
    int  ibuf;
    int  pbuf;
    char buffer[MAX_SOSTREAMBUFSIZ];
} STRECBUF;

extern int sodebug_on;

/************************************************************************\
*
* SoStreamBufCreate
*
* Creates a socket input buffer.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* sock         Handle of the socket.
* ------------ -------------------------------------------------------
* Return value Returns a handle of the stream buffer or NULL if we
*              run out of memory.
*
\************************************************************************/
void * _Export _System SoStreamBufCreate(int sock)
{
    STRECBUF *rb = malloc(sizeof(STRECBUF));

    if(rb) {
        rb->sock = sock;
        rb->ibuf = 0;
        rb->pbuf = 0;
    }
    return(rb);
}

/************************************************************************\
*
* SoStreamBufDestroy
*
* Releases all memory used by a socket input buffer. Note, that the
* socket should be expicitly closed before calling this routine.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* rbv          Handle of a stream buffer.
* ------------ -------------------------------------------------------
* Return value None.
*
\************************************************************************/
void _Export _System SoStreamBufDestroy(void *rbv)
{
    if(rbv)
        free(rbv);
}

/************************************************************************\
*
* getb_char
*
* Retrieves a single byte from the input buffer. If the buffer is
* empty, it will be refilled with new characters from the net.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* rb           Pointer to a stream buffer structure.
*
* ch           Pointer to a one byte buffer, in which the routine
*              will return the next character from the buffer.
*
* maxsecs      Timeout in seconds.
* ------------ -------------------------------------------------------
* Return value 1 on success, 0 if the partner dropped the connection,
*              or -1 in case of an error.
*
\************************************************************************/
static int getb_char(STRECBUF *rb, char *ch, long maxsecs)
{
    fd_set rdfds;
    struct timeval timeout;
    int nsocks;

    if(rb->ibuf == rb->pbuf) {
        rb->pbuf = 0;

        if(maxsecs != -1) {
            FD_ZERO(&rdfds);
            FD_SET(rb->sock, &rdfds);
            nsocks = rb->sock + 1;
            timeout.tv_sec = maxsecs;
            timeout.tv_usec = 0;
            if((nsocks = select(nsocks, &rdfds, 0, 0, &timeout)) <= 0) {
                if(nsocks == 0)
                    syslog(LOG_INFO, "Connection %3u timed out", rb->sock);
                return -1;
            }
        }
        if((rb->ibuf = recv(rb->sock, rb->buffer, sizeof(rb->buffer), 0)) <= 0)
            return(rb->ibuf);
    }
    *ch = rb->buffer[rb->pbuf++];
    return(1);
}

/************************************************************************\
*
* SoStreamBufRecv
*
* Input a line with timeout.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* pline        Pointer to a string buffer, in which this routine will
*              copy the received line. The carriage return / linefeed
*              delimiter will not be included.
*
* n            Number of bytes the buffer can take as a maximum.
*
* rbv          Handle of a stream buffer.
*
* maxsecs      Number of seconds until this routine will return a
*              timeout error.
* ------------ -------------------------------------------------------
* Return value Number of bytes actually received or -1 in case of any
*              error.
*
\************************************************************************/
int _Export _System SoStreamBufRecv(char *pline, size_t n, void *rbv, long maxsecs)
{
    STRECBUF *rb = rbv;
    register int rc;
    register char *cp = pline;
    char cl = '\0';
    int cc = 0;

    while ((rc = getb_char(rb, cp, maxsecs)) > 0) {
        if (*cp == '\n') {
            if(cl == '\r')
                cp--;
            else
                cc++;
            *cp = '\0';
            break;
        }
        cl = *cp++;
        if(++cc >= n) {
            *cp = '\0';
            break;
        }
    }
    if(rc > 0 && sodebug_on)
        syslog(LOG_DEBUG, "%03u> '%s'", rb->sock, pline);
    return((rc > 0) ? cc : rc);
}

/************************************************************************\
*
* SoStreamBufGets
*
* Classic input, not using timeout
*
* Parameter    Description
* ------------ -------------------------------------------------------
* pline        Pointer to a string buffer, in which this routine will
*              copy the received line. The carriage return / linefeed
*              delimiter will not be included.
*
* n            Number of bytes the buffer can take as a maximum.
*
* rbv          Handle of a stream buffer.
* ------------ -------------------------------------------------------
* Return value Number of bytes actually received or -1 in case of any
*              error.
*
\************************************************************************/
int _Export _System SoStreamBufGets(char *pline, size_t n, void *rbv)
{
    return SoStreamBufRecv(pline, n, rbv, -1);
}

/************************************************************************\
*
* SoStreamBinRecv
*
* Input a fixed length binary block with timeout.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* pline        Pointer to a buffer, in which this routine will copy
*              the received data.
*
* n            Number of bytes to be received.
*
* sock         Handle of the socket.
*
* maxsecs      Number of seconds until this routine will return a
*              timeout error. Note, that every single byte received
*              will reset the timeout counter.
* ------------ -------------------------------------------------------
* Return value Number of bytes actually received or -1 in case of any
*              error.
*
\************************************************************************/
size_t _Export _System SoStreamBinRecv(char *pblock, size_t n, int sock, long maxsecs)
{
    fd_set rdfds;
    struct timeval timeout;
    int nsocks;
    long nr;
    size_t ngot = 0;
    size_t chunk;

    while(ngot < n) {
        if(maxsecs != -1) {
            FD_ZERO(&rdfds);
            FD_SET(sock, &rdfds);
            nsocks = sock + 1;
            timeout.tv_sec = maxsecs;
            timeout.tv_usec = 0;

            if((nsocks = select(nsocks, &rdfds, 0, 0, &timeout)) <= 0) {
                if(nsocks == 0)
                    syslog(LOG_INFO, "Connection %3u timed out", sock);
                return -1;
            }
        }
        if((chunk = n - ngot) > MAX_SOSTREAMCHUNK)
            chunk = MAX_SOSTREAMCHUNK;
        if((nr = recv(sock, pblock, chunk, 0)) <= 0)
            return(ngot);
        ngot += nr;
        pblock += nr;
    }
    return(ngot);
}
