/*----------------------------------------------------------------------*/
/* Copyright (c) 1995-98 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@egnite.com                                               */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

const char sos_tran_rcsid[] = "$Id$";

#ifdef __OS2__

#include <types.h>
#include <sys\socket.h>

#ifdef min
#undef min
#endif

#else

#include <winsock.h>
#define sock_errno    WSAGetLastError

#endif

#include <stdlib.h>
#include <string.h>

#include <syslog.h>
#include <sostream.h>

extern int sodebug_on;

/************************************************************************\
*
* SoStreamPuts
*
* Sends a string.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* sock         Handle of the socket.
*
* line         Pointer to a buffer containing the string to be sent.
* ------------ -------------------------------------------------------
* Return value The number of bytes having been sent or -1 in case of
*              any error.
*
\************************************************************************/
int _Export _System SoStreamPuts(int sock, const char *line)
{
    int result;

    if(sodebug_on) {
        char *buf = strdup(line);
        const char *cps = line;
        while(*cps) {
            char *cpd = buf;
            while(*cps && *cps != '\r' && *cps != '\n')
                *cpd++ = *cps++;
            *cpd = '\0';
            if(*buf)
                syslog(LOG_DEBUG, "%03u< '%s'", sock, buf);
            while(*cps == '\r' || *cps == '\n')
                cps++;
        }
        free(buf);
    }
    if((result = send(sock, (char *)line, (int)strlen(line), 0)) < 0)
        syslog(LOG_ERR, "send() error: %s", SoErrorString(sock_errno()));

    return result;
}

/************************************************************************\
*
* SoStreamTransact
*
* Sends a string to the connected partner and collects a response.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* rbv          Handle of a stream buffer.
*
* question     Pointer to a buffer containing the string to be sent.
*              The routine will append a carriage return and a
*              linefeed character.
*
* answer       Pointer to a string buffer, in which this routine will
*              copy the response. The response is expected as a
*              single, carriage and/or linefeed delimited line.
*              However, the delimiter(s) will be removed by this
*              routine.
*
* maxsecs      Number of seconds until this routine will return a
*              timeout error while waiting for a response.
* ------------ -------------------------------------------------------
* Return value 0 on success, -1 otherwise.
*
\************************************************************************/
int _Export _System SoStreamTransact(void *rbv, const char *question,
                                     char *answer, long maxsecs)
{
    int sock = *((int *)rbv);

    if(SoStreamPuts(sock, question) < 0)
        return -1;
    if(SoStreamPuts(sock, "\r\n") < 0)
        return -1;
    if(SoStreamBufRecv(answer, MAX_SOSTREAMBUFSIZ, rbv, maxsecs) <= 0)
        return -1;

    return 0;
}

/************************************************************************\
*
* SoStreamBinSend
*
* Transparently sends a block of binary data to the connected
* partner.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* pblock       Pointer to the binary data block.
*
* n            Number of bytes to be sent.
*
* sock         Handle of the socket.
* ------------ -------------------------------------------------------
* Return value The number of bytes having been sent or -1 in case of
*              any error.
*
\************************************************************************/
int _Export _System SoStreamBinSend(const char *pblock, size_t n, int sock)
{
    int result = 0;
    size_t chunk;

    while(n) {
        if((chunk = n) > MAX_SOSTREAMCHUNK)
            chunk = MAX_SOSTREAMCHUNK;
        if((result = send(sock, (char *)pblock, chunk, 0)) < 0) {
            syslog(LOG_ERR, "send() error: %s", SoErrorString(sock_errno()));
            break;
        }
        n -= chunk;
        pblock += chunk;
    }
    return result;
}
