/*----------------------------------------------------------------------*/
/* Copyright (c) 1995-98 by Harald Kipp. All rights reserved.           */
/*                                                                      */
/* Redistribution and use in source and binary forms, with or without   */
/* modification, are permitted provided that the following conditions   */
/* are met:                                                             */
/*                                                                      */
/* 1. Redistributions of source code must retain the above copyright    */
/*    notice, this list of conditions and the following disclaimer.     */
/*                                                                      */
/* 2. Redistributions in binary form must reproduce the above copyright */
/*    notice, this list of conditions and the following disclaimer in   */
/*    the documentation and/or other materials provided with the        */
/*    distribution.                                                     */
/*                                                                      */
/* 3. All advertising materials mentioning features or use of this      */
/*    software must display the following acknowledgment:               */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* 4. The name "Changi Server" must not be used to endorse or promote   */
/*    products derived from this software without prior written         */
/*    permission.                                                       */
/*                                                                      */
/* 5. Redistributions of any form whatsoever must retain the following  */
/*    acknowledgment:                                                   */
/*    "This product includes software developed by Harald Kipp          */
/*    for use in the Changi news server project."                       */
/*                                                                      */
/* ==================================================================== */
/* THIS SOFTWARE IS PROVIDED BY HARALD KIPP ``AS IS'' AND ANY EXPRESSED */
/* OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED    */
/* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   */
/* ARE DISCLAIMED. IN NO EVENT SHALL HARALD KIPP BE LIABLE FOR ANY      */
/* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL   */
/* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE    */
/* GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS        */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER */
/* IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR      */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN  */
/* IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                        */
/* ==================================================================== */
/*                                                                      */
/*                                                                      */
/* Send bug reports to                                                  */
/*                                                                      */
/*      harald@egnite.com                                               */
/*                                                                      */
/*----------------------------------------------------------------------*/

/*
 * $Log$
 */

const char sos_util_rcsid[] = "$Id$";

#include <stdlib.h>
#include <ctype.h>

#ifdef __OS2__

#include <types.h>
#include <netdb.h>
#include <sys\socket.h>

#ifdef min
#undef min
#endif

#else

#include <stdio.h>
#include <winsock.h>

#define WINSOCK_VERSION (1 | (1 << 8))
#define sock_init() \
   {                \
   WSADATA wsadata; \
   WSAStartup(WINSOCK_VERSION, &wsadata); \
   }

#define SOCEINTR            WSAEINTR
#define SOCEBADF            WSAEBADF
#define SOCEACCES           WSAEACCES
#define SOCEFAULT           WSAEFAULT
#define SOCEINVAL           WSAEINVAL
#define SOCEMFILE           WSAEMFILE
#define SOCEWOULDBLOCK      WSAEWOULDBLOCK
#define SOCEINPROGRESS      WSAEINPROGRESS
#define SOCEALREADY         WSAEALREADY
#define SOCENOTSOCK         WSAENOTSOCK
#define SOCEDESTADDRREQ     WSAEDESTADDRREQ
#define SOCEMSGSIZE         WSAEMSGSIZE
#define SOCEPROTOTYPE       WSAEPROTOTYPE
#define SOCENOPROTOOPT      WSAENOPROTOOPT
#define SOCEPROTONOSUPPORT  WSAEPROTONOSUPPORT
#define SOCESOCKTNOSUPPORT  WSAESOCKTNOSUPPORT
#define SOCEOPNOTSUPP       WSAEOPNOTSUPP
#define SOCEPFNOSUPPORT     WSAEPFNOSUPPORT
#define SOCEAFNOSUPPORT     WSAEAFNOSUPPORT
#define SOCEADDRINUSE       WSAEADDRINUSE
#define SOCEADDRNOTAVAIL    WSAEADDRNOTAVAIL
#define SOCENETDOWN         WSAENETDOWN
#define SOCENETUNREACH      WSAENETUNREACH
#define SOCENETRESET        WSAENETRESET
#define SOCECONNABORTED     WSAECONNABORTED
#define SOCECONNRESET       WSAECONNRESET
#define SOCENOBUFS          WSAENOBUFS
#define SOCEISCONN          WSAEISCONN
#define SOCENOTCONN         WSAENOTCONN
#define SOCESHUTDOWN        WSAESHUTDOWN
#define SOCETOOMANYREFS     WSAETOOMANYREFS
#define SOCETIMEDOUT        WSAETIMEDOUT
#define SOCECONNREFUSED     WSAECONNREFUSED
#define SOCELOOP            WSAELOOP
#define SOCENAMETOOLONG     WSAENAMETOOLONG
#define SOCEHOSTDOWN        WSAEHOSTDOWN
#define SOCEHOSTUNREACH     WSAEHOSTUNREACH
#define SOCENOTEMPTY        WSAENOTEMPTY


#endif

#include <syslog.h>
#include <mutex.h>
#include <sostream.h>

#define MUTEXWAIT   (96000 + __LINE__)

static long mutex = -1;
static int initialized = 0;

int sodebug_on = 0;

/************************************************************************\
*
* SoInitialize
*
* Initializes the socket interface. Should be called prior to any
* other routine of this DLL.
*
* Returns 0 on success, -1 otherwise.
*
\************************************************************************/
int _Export _System SoInitialize(void)
{
    if(initialized == 0) {
        sock_init();
        if((mutex = create_mutex()) == -1)
            return -1;
        initialized = 1;
    }
    return(0);
}

/************************************************************************\
*
* SoStreamDebug
*
* Switches debugging on or off.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* enable       A value of 0 will switch off debugging, otherwise it
*              is switched on.
* ------------ -------------------------------------------------------
* Return value None.
*
\************************************************************************/
void _Export _System SoStreamDebug(int enable)
{
    sodebug_on = enable;
}

/************************************************************************\
*
* SoGetLocalHost
*
* Queries name and ip of the local host.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* localname    Pointer to a string buffer, in which the FQDN of the
*              local host will be copied. The buffer should be at
*              least 'MAX_DOMAINNAMESIZ' bytes in size.
*
* localaddr    Pointer to a long value, in which the ip of the local
*              host will be returned. The pointer may be NULL, if
*              the caller is only interested in the name.
* ------------ -------------------------------------------------------
* Return value 0 on success, -1 otherwise.
*
\************************************************************************/
int _Export _System SoGetLocalHost(char *localname, long *localaddr)
{
    int result = -1;
    struct hostent *hostnm;
    char *temp;

    /*
     * Check, if we are initialized.
     */
    if(SoInitialize())
        return -1;

    if(request_mutex(&mutex, MUTEXWAIT) != 0)
        return -1;

    temp = malloc(MAX_DOMAINNAMESIZ);
    gethostname(temp, MAX_DOMAINNAMESIZ);
    if((hostnm = gethostbyname(temp)) != NULL) {
        if(localaddr)
            memcpy(localaddr, hostnm->h_addr_list[0], sizeof(long));
        if(strchr(hostnm->h_name, '.') != NULL) {
            if(localname)
                strcpy(localname, hostnm->h_name);
            result = 0;
        }
        else {
            int i;
            int l = strlen(hostnm->h_name);

            for(i = 0; hostnm->h_aliases[i]; ++i) {
                if(strchr(hostnm->h_aliases[i], '.') != NULL &&
                   strncmp(hostnm->h_aliases[i], hostnm->h_name, l) == 0) {
                    if(localname)
                        strcpy(localname, hostnm->h_aliases[i]);
                    result = 0;
                    break;
                }
            }
        }
    }
    free(temp);
    release_mutex(mutex);
    return(result);
}

/************************************************************************\
*
* SoGetHostAddr
*
* Resolves the hostname and returns the corresponding ip address.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* hostname     Pointer to a string containing either the name of
*              the host or the dotted decimal ASCII representation
*              of the host to connect. This pointer may be NULL,
*              in which case the routine will return the ip of
*              the local host.
* ------------ -------------------------------------------------------
* Return value ip address as a long value or -1, if the host is
*              unknown.
*
\************************************************************************/
long _Export _System SoGetHostAddr(const char *hostname)
{
    long result;
    struct hostent *hostnm;

    /*
     * Check, if we are initialized.
     */
    if(SoInitialize())
        return -1;

    if(hostname == NULL || *hostname == '\0') {
        SoGetLocalHost(NULL, &result);
        return result;
    }

    if(request_mutex(&mutex, MUTEXWAIT) != 0)
        return -1;

    if((result = inet_addr((char *)hostname)) == -1) {
        if((hostnm = gethostbyname((char *)hostname)) == NULL) {
            char *err_txt;
            static char buf[80];

            switch(h_errno) {
            case HOST_NOT_FOUND:
                err_txt = "Authoritative Answer Host not found";
                break;
            case TRY_AGAIN:
                err_txt = "Non-Authoritive Host not found, or SERVERFAIL";
                break;
            case NO_RECOVERY:
                err_txt = "Non recoverable errors, FORMERR, REFUSED, NOTIMP";
                break;
            case NO_DATA:
                err_txt = "Valid name, no data record of requested type";
                break;
            default:
                sprintf(err_txt = buf, "Unexpected error %d", hostname, h_errno);
                break;
            }
            syslog(LOG_ERR, "%s: %s", hostname, err_txt);
        }
        else
            memcpy(&result, hostnm->h_addr_list[0], sizeof(long));
    }
    release_mutex(mutex);
    return(result);
}

/************************************************************************\
*
* SoGetServPort
*
* Retrieves the port number of a specified service name.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* service      Pointer to a string, either containing the name of the
*              service port or the ASCII representation of the port
*              number.
*
* proto        Pointer to a string specifying the protocol type.
*              Should contain either "tcp" or "udp". The first one is
*              used, if the pointer is NULL.
* ------------ -------------------------------------------------------
* Return value The unsigned short port number or 0, if the port is
*              unknown.
*
\************************************************************************/
unsigned short _Export _System SoGetServPort(const char *service, const char *proto)
{
    unsigned short result = 0;
    struct servent *sp;

    /*
     * Check, if we are initialized.
     */
    if(SoInitialize())
        return 0;

    if(request_mutex(&mutex, MUTEXWAIT) != 0)
        return 0;

    if(isdigit(*service))
        result = (unsigned short)atoi(service);
    else if ((sp = getservbyname((char *)service, (char *)proto)) != NULL)
        result = ntohs(sp->s_port);

    release_mutex(mutex);
    return result;
}

/************************************************************************\
*
* SoGetRemoteHost
*
* Retrieves name and ip address of a connected host from a given
* socket address structure.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* from         Pointer to the socket address structure.
*
* remotename   Pointer to a string buffer, in which the name of the
*              connected host will be copied. The buffer should be at
*              least 'MAX_DOMAINNAMESIZ' bytes in size. This pointer
*              may be NULL, if the caller is only interested in the
*              ip address.
*              If the name could not be determined by a reverse
*              name lookup, then the routine will return the decimal
*              dotted ASCII representation of the ip address.
*
* remoteaddr   Pointer to a long value, in which the ip of the
*              connected host will be returned. The pointer may be
*              NULL, if the caller is only interested in the name.
* ------------ -------------------------------------------------------
* Return value None.
*
\************************************************************************/
void _Export _System SoGetRemoteHost(void *from, char *remotename, long *remoteaddr)
{
    struct hostent *hp;
    struct sockaddr_in *client = from;

    /*
     * Check for valid parameters.
     */
    if(from == NULL) {
        if(remotename)
            *remotename = '\0';
        if(remoteaddr)
            *remoteaddr = -1;
        return;
    }

    if(request_mutex(&mutex, MUTEXWAIT) != 0)
        return;

    hp = gethostbyaddr((char *)&(client->sin_addr), sizeof(struct in_addr), AF_INET);
    if(remotename)
        strcpy(remotename, hp ? (hp->h_name) : inet_ntoa(client->sin_addr));
    if(remoteaddr)
        memcpy(remoteaddr, &(client->sin_addr), sizeof(long));

    release_mutex(mutex);
}

/************************************************************************\
*
* SoErrorString
*
* Returns a corresponding error text to a given TCP/IP error code.
*
* Parameter    Description
* ------------ -------------------------------------------------------
* ec           TCP/IP error code
* ------------ -------------------------------------------------------
* Return value Pointer to a static buffer containing the error text.
*
\************************************************************************/
char * _Export _System SoErrorString(int ec)
{
    char *result;

    switch(ec) {
    case SOCEINTR:
        result = "Interrupted system call";
        break;
    case SOCEBADF:
        result = "Bad file number";
        break;
    case SOCEACCES:
        result = "Permission denied";
        break;
    case SOCEFAULT:
        result = "Bad address";
        break;
    case SOCEINVAL:
        result = "Invalid argument";
        break;
    case SOCEMFILE:
        result = "Too many open files";
        break;
    case SOCEWOULDBLOCK:
        result = "Operation would block";
        break;
    case SOCEINPROGRESS:
        result = "Operation now in progress";
        break;
    case SOCEALREADY:
        result = "Operation already in progress";
        break;
    case SOCENOTSOCK:
        result = "Socket operation on non-socket";
        break;
    case SOCEDESTADDRREQ:
        result = "Destination address required";
        break;
    case SOCEMSGSIZE:
        result = "Message too long";
        break;
    case SOCEPROTOTYPE:
        result = "Protocol wrong type for socket";
        break;
    case SOCENOPROTOOPT:
        result = "Protocol not available";
        break;
    case SOCEPROTONOSUPPORT:
        result = "Protocol not supported";
        break;
    case SOCESOCKTNOSUPPORT:
        result = "Socket type not supported";
        break;
    case SOCEOPNOTSUPP:
        result = "Operation not supported on socket";
        break;
    case SOCEPFNOSUPPORT:
        result = "Protocol family not supported";
        break;
    case SOCEAFNOSUPPORT:
        result = "Address family not supported by protocol family";
        break;
    case SOCEADDRINUSE:
        result = "Address already in use";
        break;
    case SOCEADDRNOTAVAIL:
        result = "Can't assign requested address";
        break;
    case SOCENETDOWN:
        result = "Network is down";
        break;
    case SOCENETUNREACH:
        result = "Network is unreachable";
        break;
    case SOCENETRESET:
        result = "Network dropped connection on reset";
        break;
    case SOCECONNABORTED:
        result = "Software caused connection abort";
        break;
    case SOCECONNRESET:
        result = "Connection reset by peer";
        break;
    case SOCENOBUFS:
        result = "No buffer space available";
        break;
    case SOCEISCONN:
        result = "Socket is already connected";
        break;
    case SOCENOTCONN:
        result = "Socket is not connected";
        break;
    case SOCESHUTDOWN:
        result = "Can't send after socket shutdown";
        break;
    case SOCETOOMANYREFS:
        result = "Too many references: can't splice";
        break;
    case SOCETIMEDOUT:
        result = "Connection timed out";
        break;
    case SOCECONNREFUSED:
        result = "Connection refused";
        break;
    case SOCELOOP:
        result = "Too many levels of symbolic links";
        break;
    case SOCENAMETOOLONG:
        result = "File name too long";
        break;
    case SOCEHOSTDOWN:
        result = "Host is down";
        break;
    case SOCEHOSTUNREACH:
        result = "No route to host";
        break;
    case SOCENOTEMPTY:
        result = "Directory not empty";
        break;

#ifdef __OS2__

    case SOCEPERM:
        result = "Not owner";
        break;
    case SOCESRCH:
        result = "No such process";
        break;
    case SOCENXIO:
        result = "No such device or address";
        break;
    case SOCEPIPE:
        result = "Broken pipe";
        break;
    case SOCEOS2ERR:
        result = "OS/2 system error";
        break;
#else

    case WSAEPROCLIM:
        result = "Too many processes";
        break;
    case WSAEUSERS:
        result = "Users";
        break;
    case WSAEDQUOT:
        result = "Quotation";
        break;
    case WSAESTALE:
        result = "Stale";
        break;
    case WSAEREMOTE:
        result = "Remote";
        break;
    case WSAEDISCON:
        result = "Disconnected";
        break;
    case WSASYSNOTREADY:
        result = "System not ready";
        break;
    case WSAVERNOTSUPPORTED:
        result = "Version not supported";
        break;
    case WSANOTINITIALISED:
        result = "Not initialized";
        break;

#endif

    default:
        if(ec) {
            static char buf[80];
            sprintf(result = buf, "Socket interface error %d", ec);
        }
        else
            result = "Socket interface error";
        break;
    }
    return result;
}
